unit dMDK_Data;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 2/22/08 11:14a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  Contains the Application Client Data Set variables
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/dMDK_Data.pas $
  *
  *       $ History: dMDK_Data.pas $
}

interface

uses
  SysUtils, Classes, DB, DBClient, Dialogs, Variants, DBCtrls, Controls,
  Graphics;

{ ===============================================================================
  dmData providers ClienDataSets and DataSources objects and links CDSs with DSs.
  CDSs structure supposed to be defined dynamically on dmData object creation
  CDSs are not supposed to be saved in the files or retrieve data from files
  CDSs wil be populated by another module
  =============================================================================== }
type
  TCDSCreate = procedure(ACDS: TClientDataSet);

  TdmData = class(TDataModule)
    DsEvents: TDataSource;
    CdsEvents: TClientDataSet;
    CdsSummary: TClientDataSet;
    DsSummary: TDataSource;
    CdsFlowSheet: TClientDataSet;
    DsFlowSheet: TDataSource;
    CdsAP: TClientDataSet;
    CdsAPHistory: TClientDataSet;
    DsAP: TDataSource;
    DsAPHistory: TDataSource;
    CdsVAInfection: TClientDataSet;
    DsVAInfection: TDataSource;
    CdsInfo: TClientDataSet;
    DsInfo: TDataSource;
    CdsStudyHistory: TClientDataSet;
    DsStudyHistory: TDataSource;
    CdsStudyHistoryPreBPStanding: TStringField;
    CdsStudyHistoryPreBPSeated: TStringField;
    CdsStudyHistoryPostBPStanding: TStringField;
    CdsStudyHistoryPostBPSeated: TStringField;
    CdsMedList: TClientDataSet;
    DsMedList: TDataSource;
    CdsSummaryNotes: TStringField;
    CdsInfoAdvDir: TStringField;
    CdsDict: TClientDataSet;
    DsDict: TDataSource;
    CdsSource: TClientDataSet;
    DsSource: TDataSource;
    CdsTimeline: TClientDataSet;
    DsTimeline: TDataSource;
    CdsStatus: TClientDataSet;
    CdsStatusPage: TStringField;
    CdsStatusStatusID: TIntegerField;
    CdsStatusComments: TStringField;
    DsStatus: TDataSource;
    DsStudyEvents: TDataSource;
    CdsStudyEvents: TClientDataSet;
    DsNotes: TDataSource;
    CdsNotes: TClientDataSet;
    CdsNotesAuthorName: TStringField;
    CdsSubmitHistory: TClientDataSet;
    DsSubmitHistory: TDataSource;
    CdsWorkflow: TClientDataSet;
    DsWorkflow: TDataSource;
    procedure DataModuleCreate(Sender: TObject);
    procedure CdsStudyHistoryCalcFields(DataSet: TDataSet);
    procedure DataModuleDestroy(Sender: TObject);
    procedure CdsInfoCalcFields(DataSet: TDataSet);
    procedure CdsMedListAfterInsert(DataSet: TDataSet);
    procedure CdsSummaryBeforeInsert(DataSet: TDataSet);
    procedure CdsNotesAfterInsert(DataSet: TDataSet);
    procedure CdsNotesAuthorNameGetText(Sender: TField; var Text: string; DisplayText: Boolean);
    procedure DsNotesDataChange(Sender: TObject; Field: TField);
    procedure DsStudyHistoryDataChange(Sender: TObject; Field: TField);
    procedure DsWorkflowDataChange(Sender: TObject; Field: TField);
    procedure DsInfoDataChange(Sender: TObject; Field: TField);
    procedure DsMedListDataChange(Sender: TObject; Field: TField);
  private
    { Private declarations }
    FIgnoreDebug: Boolean;
    FSessionID: string;
    FPatientID: string;
    FActor: string;
    FAction: string;
    FResult: string;
    FParams: string;
    SID: string;
    FID: Integer; // internal sequence number
    FStart: TDateTime;
    procedure CreateCDS(ACDS: TClientDataSet; ACDSCreator: TCDSCreate; ALabel: string);
    procedure CreateEvents;
    procedure CreateDictionary;
    procedure CreateFlowSheet;
    procedure CreateSummary;
    procedure CreateSummaryHistory;
    procedure CreateMedList;
    procedure CreateInfo;
    procedure CreateVA;
    procedure CreateAssessHistory;
    procedure CreateVAInfection;
    procedure CreateStatus;
    procedure CreateNotes;
    procedure CreateStudyEvents;
    procedure CreateSubmitHistory;
    function GetNextID: string;
    function GetDuration: TDateTime;
    procedure AddStudyEvent(AnID, AKind, AComment, ASession: string; DateTime: TDateTime; ADuration: TDateTime = 0);
    procedure EventAdd(AnID, AKind, AComment, ASession: string; DateTime: TDateTime; ADuration: TDateTime = 0;
      ALocal: Boolean = False);
  public
    BIgnore: Boolean;
    { Public declarations }
    property PatientID: string read FPatientID write FPatientID;
    property SessionID: string read FSessionID write FSessionID;
    property Actor: string read FActor write FActor;
    property Action: string read FAction write FAction;
    property Results: string read FResult write FResult;
    property Params: string read FParams write FParams;
    property NextID: string read GetNextID;
    property Duration: TDateTime read GetDuration;
    property Start: TDateTime write FStart;
    procedure RegisterEvent(AKind, AComment, ASession: string; ADuration: TDateTime = 0; ALocal: Boolean = False);
    procedure RecreateCDS(ADS: TClientDataSet);
    function GetFieldData(AName: string): string;
    function GetCDSByName(AName: string): TClientDataSet;
    function GetMedicineAdministrated: string;
    function GetFlowsheetReport: string;
    function GetMedicineReport: string;
    function GetScheduleReport: string;
    function GetPainReport(PreTreatment: Boolean): string;
    function GetFlowNotesReport: string;
    function GetCommentsReport(AContextFilter: string): string;
    function GetGeneralInfoReport: string;
    procedure ChangeCurrentNoteValidStatus;
    procedure SetCurrentNoteStatus(AStatus: string);
    procedure LockCurrentNote;
    procedure LockNotes;
    procedure DeleteCurrentNote;
    procedure PatchDictionary(AString: string);
    function CheckCDSIDField(ADS: TClientDataSet; AIDField, AIDValue: string; var BadValue: string): Boolean;
    procedure CreateStudyEventsCDS(ADS: TClientDataSet);
    function ValidDataSetXML(XML: string): Boolean;
    function UpdateFlowsheetIndex(ADescending: Boolean): Boolean;
    function UpdateMedlistIndex(ADescending: Boolean): Boolean;
    procedure SaveErrorsLog;
  end;

var
  DmData: TdmData;

const
  FnEvents_ID        = 'ID';
  FnEvents_WS        = 'WS';
  FnEvents_Type      = 'Kind';
  FnEvents_DateTime  = 'DateTime';
  FnEvents_Duration  = 'Duration';
  FnEvents_StudyID   = 'StudyID';
  FnEvents_PatientID = 'PatientID';
  FnEvents_Comments  = 'Comments';
  FnEvents_Action    = 'Action';
  FnEvents_Actor     = 'Actor';
  FnEvents_Params    = 'Params';
  FnEvents_Result    = 'Result';
  // Study List
  FnSL_StudyID      = 'StudyID';
  FnSL_PatientID    = 'PatientID';
  FnSL_Sensitive    = 'SENSITIVE';
  FnSL_Status       = 'Status';
  FnSL_StatusTime   = 'StatusTime';
  FnSL_StatusBy     = 'StatusBy';
  FnSL_StatusAt     = 'StatusAt';
  FnSL_Comments     = 'Comments';
  FnSL_LockStatus   = 'LockStatus';
  FnSL_LockedBy     = 'LockedStatus';
  FnSL_LastModified = 'LastModified';
  FnSL_CheckedIn    = 'CheckedIn';
  FnSL_Name         = 'Name';
  FnSL_DOB          = 'DOB';
  FnSL_SSN          = 'SSN';
  FnSL_Sex          = 'Sex';
  FnSL_Source       = 'Source';
  FnSL_UserID       = 'UserID';
  FnSL_SessionID    = 'SessionID';
  FnSL_Workstation  = 'Workstation';
  FnSL_RegisteredAt = 'RegisteredAt';
  FnSL_Result       = 'Result';
  FnSL_StatusOld    = 'StatusOld';
  FnWL_Enabled      = 'Enabled';
  FnWL_ItemType     = 'ItemType';
  FnWL_ItemID       = 'ItemID';
  FnWL_RegisteredBy = 'RegisteredBy';
  FnWL_RegisteredAt = 'RegisteredAt';
  FnWL_Comments     = 'Comments';
  FnWL_StatusBy     = 'StatusBy';
  FnWL_StatusAt     = 'StatusAt';
  WL_USER           = 'USER';
  WL_WORKSTATION    = 'WORKSTATION';
  WL_STUDY          = 'STUDY';
  WL_PATIENT        = 'PATIENT';
  WL_STATUS_NEW     = 'STATUS_NEW';
  WL_STATUS_OLD     = 'STATUS_OLD';

const // Event Types
  EtDebug   = 'DEBUG';
  EtError   = 'ERROR';
  EtWarning = 'WARNING';
  EtInfo    = 'INFO';

function ValidDateTime(AValue: string): Boolean;

implementation

uses
  UMDK_Common, UMDK_CDSCommon, UMDK_VADataSet, UMDK_Summary, UMDK_Info, UMDK_MedList, UMDK_Engine, UMDK_Patients,
  Windows, UMDK_Options, UROR_CommonUtils, UMDK_Notes, UMDK_SubmitHistory, UMDK_Dictionary, UMDK_Flowsheet,
  FMDK_Flowsheet, UMDK_Timeline, System.UITypes, FMDK_Summary, FMDK_Pre, FMDK_Post, FMDK_RxLab, UMDK_Workflow,
  FMDK_Cover, FMDK_Main, FGMV_AboutDlg;
{$R *.dfm}

/// /////////////////////////////////////////////////////////////////////////////
const
  _CR_       = #13#10;
  FlComments = 4000;

function ValidDateTime(AValue: string): Boolean;
var
  Dt: TDateTime;
begin
  if (Trim(AValue) = '') then Result := DevOptions.BValue(DonIgnoreBlankDateFields)
  else
    try
      Dt := StrToDateTime(AValue);
      if Dt > 0 then Result := True
      else Result := False;
    except
      on E: EConvertError do Result := False;
    end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TdmData.DataModuleCreate(Sender: TObject);
begin
  BIgnore := False;
  FID := 0;
  FIgnoreDebug := False;
  FSessionID := '';
  FPatientID := '';
  // Create Data dictionary
  CreateEvents;
  CreateDictionary;
  CreateSummary;
  CreateSummaryHistory;
  CreateMedList;
  CreateFlowSheet;
  CreateInfo;
  CreateVA;
  CreateAssessHistory;
  CreateVAInfection;
  CreateStatus;
  CreateNotes;
  CreateStudyEvents;
  CreateSubmitHistory;
  CreateCDS(CdsTimeline, CreateTimelineCDS, 'Timeline Table');
  SetupWorkflowCDS(CdsWorkflow);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TdmData.GetDuration: TDateTime;
begin
  Result := 24 * 3600 * (Now - FStart);
end;

procedure TdmData.CreateStatus;
begin
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnCover, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnRxLab, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnPre, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnAccess, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnFlow, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnPost, 0, '']);
  CdsStatus.Edit;
  CdsStatus.AppendRecord([PnSummary, 0, '']);
end;

procedure TdmData.RecreateCDS(ADS: TClientDataSet);
var
  AField: TFieldDef;

  procedure ClearFields(ANum: Integer);
  begin
    try
      while ADS.FieldDefs.Count > ANum do begin
        AField := ADS.FieldDefs[ANum];
        ADS.FieldDefs.Delete(ANum);
      end;
    except
    end;
  end;

begin
  FStart := Now;
  if ADS = DmData.CdsEvents then CreateEvents;
  if ADS = DmData.CdsSummary then begin
    ClearFields(1);
    CreateSummary;
  end;
  if ADS = DmData.CdsStudyHistory then begin
    ClearFields(4);
    CreateSummaryHistory;
  end;
  if ADS = DmData.CdsSubmitHistory then CreateSubmitHistory;
  if ADS = DmData.CdsFlowSheet then CreateFlowSheet;
  if ADS = DmData.CdsInfo then begin
    ClearFields(1);
    CreateInfo;
  end;
  if ADS = DmData.CdsAP then CreateVA;
  if ADS = DmData.CdsAPHistory then CreateAssessHistory;
  if ADS = DmData.CdsVAInfection then CreateVAInfection;
  Action := 'RECREATE TABLE';
  Params := ADS.Name;
  RegisterEvent(EtInfo, 'Table <' + ADS.Name + '> ReCreated', 'Startup', GetDuration);
end;
/// ////////////////////////////////////////////////////////////////// FLOW SHEET

procedure TdmData.CreateCDS(ACDS: TClientDataSet; ACDSCreator: TCDSCreate; ALabel: string);
begin
  Start := Now;
  try
    ACDSCreator(ACDS);
    ACDS.Open;
    Action := 'Open ' + ALabel;
  except
    RegisterEvent(EtError, 'Open ' + ALabel + ' Failed', '', Duration);
  end;
end;

procedure TdmData.CreateFlowSheet;
begin
  CreateCDS(CdsFlowSheet, CreateFlowsheetDS, 'Flowsheet Table');
end;

procedure TdmData.CreateSummary;
begin
  CreateCDS(CdsSummary, CreateSummaryDS, 'Summary Table');
end;

procedure TdmData.CreateSummaryHistory;
begin
  CreateCDS(CdsStudyHistory, CreateSummaryDS, 'Summary History Table');
end;

procedure TdmData.CreateSubmitHistory;
begin
  CreateCDS(CdsSubmitHistory, CreateSubmitHistoryDS, 'Submit Table');
end;

procedure TdmData.CreateMedList;
begin
  Start := Now;
  try
    CreateMedListCDS(CdsMedList, DsSummary);
    CdsMedList.Open;
    Action := 'Open Med List Table';
  except
    RegisterEvent(EtError, 'Open Med List Table Failed', 'Startup', Duration);
  end;
  Action := '';
end;

procedure TdmData.CreateInfo;
begin
  CreateCDS(CdsInfo, CreateInfoDS, 'Info Table');
end;

procedure TdmData.CreateNotes;
begin
  Start := Now;
  try
    CreateNoteListCDS(CdsNotes);
    CdsNotes.Open;
    Action := 'Open Notes Table';
  except
    RegisterEvent(EtError, 'Open Notes Table Failed', 'Startup', Duration);
  end;
  Action := '';
end;

procedure TdmData.CreateStudyEvents;
begin
  Start := Now;
  try
    Action := 'Create Study Events Table (Log)';
    CreateStudyEventsCDS(CdsStudyEvents);
    CdsStudyEvents.Open;
    Action := 'Open Study Events Table';
  except
    RegisterEvent(EtError, 'Open Study Events Table Failed', 'Startup', Duration);
  end;
  Action := '';
end;
/// ///////////////////////////////////////////////////////////// VASCULAR ACCESS

procedure TdmData.CreateVA;
begin
  if not FileExists(CdsAP.FileName) then CreateVASiteDS(CdsAP);
  CdsAP.Open;
end;

procedure TdmData.CreateAssessHistory;
begin
  if not FileExists(CdsAPHistory.FileName) then CreateVAHistoryDS(CdsAPHistory, DsAP);
  CdsAPHistory.Open;
end;

procedure TdmData.CreateVAInfection;
begin
  if not FileExists(CdsVAInfection.FileName) then CreateVAInfectionDS(CdsVAInfection, DsAP);
  CdsVAInfection.Open;
end;
/// ////////////////////////////////////////////////////////////////////// EVENTS

procedure TdmData.CreateEvents;
begin
  NewStringField(CdsEvents, FnEvents_ID, 'ID', 9, 40);
  NewStringField(CdsEvents, FnEvents_DateTime, 'Time', 12, 22);
  NewStringField(CdsEvents, FnEvents_Type, 'Event Type', 8, 20);
  NewStringField(CdsEvents, FnEvents_StudyID, 'Study #', 6, 8);
  NewStringField(CdsEvents, FnEvents_PatientID, 'Patient DFN', 12, 12);
  NewStringField(CdsEvents, FnEvents_Action, 'Action', 16, 80);
  NewStringField(CdsEvents, FnEvents_Comments, 'Comments', 80, FlComments);
  NewFloatField(CdsEvents, FnEvents_Duration, 'Duration', '#0.0###');
  NewStringField(CdsEvents, FnEvents_Actor, 'User', 8, 40);
  NewStringField(CdsEvents, FnEvents_Params, 'Params', 8, 80);
{$IFDEF AANTEST}CdsEvents.FieldList.Fields[CdsEvents.FieldList.Count - 1].Visible := True;
{$ELSE}CdsEvents.FieldList.Fields[CdsEvents.FieldList.Count - 1].Visible := False;
{$ENDIF}
  NewStringField(CdsEvents, FnEvents_Result, 'Result', 8, 20);
{$IFDEF AANTEST}CdsEvents.FieldList.Fields[CdsEvents.FieldList.Count - 1].Visible := True;
{$ELSE}CdsEvents.FieldList.Fields[CdsEvents.FieldList.Count - 1].Visible := False;
{$ENDIF}
  CdsEvents.IndexFieldNames := 'ID';
  CdsEvents.CreateDataSet;
  CdsEvents.Open;
end;

procedure TdmData.CreateStudyEventsCDS(ADS: TClientDataSet);
begin
  NewStringField(ADS, FnEvents_ID, 'ID', 9, 40);
  NewStringField(ADS, FnEvents_WS, 'Workstation', 9, 40);
  NewStringField(ADS, FnEvents_DateTime, 'Time', 12, 22);
  NewStringField(ADS, FnEvents_Type, 'Type', 8, 20);
  NewStringField(ADS, FnEvents_StudyID, 'Study', 6, 8);
  NewStringField(ADS, FnEvents_PatientID, 'Patient', 4, 8);
  NewStringField(ADS, FnEvents_Actor, 'Actor', 8, 40);
  NewStringField(ADS, FnEvents_Action, 'Action', 16, 80);
  NewStringField(ADS, FnEvents_Params, 'Params', 8, 80);
  NewStringField(ADS, FnEvents_Result, 'Result', 8, 20);
  NewFloatField(ADS, FnEvents_Duration, 'Duration', '#0.0###');
  NewStringField(ADS, FnEvents_Comments, 'Comments', 80, FlComments);
  ADS.IndexFieldNames := 'ID';
  ADS.CreateDataSet;
  ADS.Open;
end;

procedure TdmData.CreateDictionary;
begin
  PrepareDictionaryCDS(CdsDict);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TdmData.GetNextID: string;
var
  S: string;
begin
  try
    if Assigned(Engine) then S := FormatDateTime('yymmdd hh:mm:ss.zzz', Engine.GetCurrentServerDateTime)
    else S := FormatDateTime('yymmdd hh:nn:ss.zzz', Now);
  except
    S := FormatDateTime('yymmdd hh:nn:ss.zzz', Now);
  end;
  if S = SID then begin
    Inc(FID);
    Result := SID + ' ' + Format('%-3.3d', [FID]);;
  end else begin
    SID := S;
    FID := 0;
    Result := SID;
  end;
end;

procedure TdmData.AddStudyEvent(AnID, AKind, AComment, ASession: string; DateTime: TDateTime; ADuration: TDateTime = 0);
var
  PBuffer: PChar;
  Buffer: array [0 .. 200] of Char;
  IBuffer: DWord;
  S: string;
begin
  if not CdsStudyEvents.Active then
    try
      CdsStudyEvents.Open;
    except
      Exit;
    end;
  if PatientID = '' then Exit;
  PBuffer := @Buffer;
  IBuffer := 200;
  try
    GetComputerName(@Buffer, IBuffer);
    S := Copy(PBuffer, 1, IBuffer);
  except
  end;
  try
    CdsStudyEvents.Insert;
    CdsStudyEvents.FieldByName(FnEvents_ID).AsString := AnID;
    CdsStudyEvents.FieldByName(FnEvents_WS).AsString := S;
    CdsStudyEvents.FieldByName(FnEvents_Type).AsString := AKind;
    CdsStudyEvents.FieldByName(FnEvents_StudyID).AsString := ASession;
    CdsStudyEvents.FieldByName(FnEvents_PatientID).AsString := PatientID;
    CdsStudyEvents.FieldByName(FnEvents_Comments).AsString := AComment;
    CdsStudyEvents.FieldByName(FnEvents_Duration).AsFloat := ADuration;
    CdsStudyEvents.FieldByName(FnEvents_DateTime).AsString := FormatDateTime(GMDK_DateTimeFMT, DateTime);
    CdsStudyEvents.FieldByName(FnEvents_Action).AsString := Action;
    CdsStudyEvents.FieldByName(FnEvents_Actor).AsString := Actor;
    CdsStudyEvents.FieldByName(FnEvents_Result).AsString := Results;
    CdsStudyEvents.FieldByName(FnEvents_Params).AsString := Params;
    CdsStudyEvents.Post;
  except
  end;
end;

procedure TdmData.EventAdd(AnID, AKind, AComment, ASession: string; DateTime: TDateTime; ADuration: TDateTime = 0;
  ALocal: Boolean = False);
begin
  try
    CdsEvents.Insert;
    CdsEvents.FieldByName(FnEvents_ID).AsString := AnID;
    CdsEvents.FieldByName(FnEvents_Type).AsString := AKind;
    CdsEvents.FieldByName(FnEvents_StudyID).AsString := ASession;
    CdsEvents.FieldByName(FnEvents_PatientID).AsString := PatientID;
    CdsEvents.FieldByName(FnEvents_Comments).AsString := AComment;
    CdsEvents.FieldByName(FnEvents_Duration).AsFloat := ADuration;
    CdsEvents.FieldByName(FnEvents_DateTime).AsDateTime := DateTime;
    CdsEvents.FieldByName(FnEvents_Action).AsString := Action;
    CdsEvents.FieldByName(FnEvents_Actor).AsString := Actor;
    CdsEvents.FieldByName(FnEvents_Result).AsString := Results;
    CdsEvents.FieldByName(FnEvents_Params).AsString := Params;
    CdsEvents.Post;
  except
  end;
  if ALocal then AddStudyEvent(AnID, AKind, AComment, ASession, DateTime, ADuration);
end;

procedure TdmData.RegisterEvent(AKind, AComment, ASession: string; ADuration: TDateTime = 0; ALocal: Boolean = False);
var
  IProcessed: Integer;
  SLabel, SID: string;
  B: Boolean;
begin
  if FIgnoreDebug and (AKind = EtDebug) then Exit;
  IProcessed := 1;
  SLabel := ASession;
  if ASession = 'Startup' then begin
    Actor := ASession;
    SLabel := '';
  end;
  repeat
    SID := NextID;
    B := ALocal and DevOptions.BValue(DonRegisterStudyEvents);
    EventAdd(SID, AKind, Copy(AComment, IProcessed, FlComments), SLabel, Now, // Engine.getCurrentServerDateTime,
      ADuration, B);
    SLabel := CToBeContinued;
    Inc(IProcessed, FlComments);
  until IProcessed >= Length(AComment);
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TdmData.CdsStudyHistoryCalcFields(DataSet: TDataSet);
  function CalculateBP(ASys, ADias: Real): string;
  var
    S, Ss: string;
  begin
    Result := '---';
    S := '---';
    Ss := '---';
    try
      S := Format('%3.0f', [ASys]);
      while (S <> '') and (Pos(' ', S) = 1) do S := Copy(S, 2, Length(S));
      if S = '' then S := '-';
      Ss := Format('%3.0f', [ADias]);
      while (Ss <> '') and (Pos(' ', Ss) = 1) do Ss := Copy(Ss, 2, Length(Ss));
      if Ss = '' then Ss := '-';
      Result := S + '/' + Ss;
    except
      Result := '***';
    end;
  end;

begin
  try
    CdsStudyHistoryPreBPSeated.Value := AnsiString(CalculateBP(CdsStudyHistory.FieldValues[FnS_PreBPSysSitting],
      CdsStudyHistory.FieldValues[FnS_PreBPDiasSitting]));
    CdsStudyHistoryPreBPStanding.Value := AnsiString(CalculateBP(CdsStudyHistory.FieldValues[FnS_PreBPSysStanding],
      CdsStudyHistory.FieldValues[FnS_PreBPDiasStanding]));
    CdsStudyHistoryPostBPSeated.Value := AnsiString(CalculateBP(CdsStudyHistory.FieldValues[FnS_PostBPSysSitting],
      CdsStudyHistory.FieldValues[FnS_PostBPDiasSitting]));
    CdsStudyHistoryPostBPStanding.Value := AnsiString(CalculateBP(CdsStudyHistory.FieldValues[FnS_PostBPSysStanding],
      CdsStudyHistory.FieldValues[FnS_PostBPDiasStanding]));
  except
  end;
end;

procedure TdmData.DataModuleDestroy(Sender: TObject);
begin
  CdsSummary.Close;
  CdsInfo.Close;
  CdsFlowSheet.Close;
  CdsStudyHistory.Close;
  CdsAP.Close;
  CdsAPHistory.Close;
  CdsVAInfection.Close;
  CdsMedList.Close;
  try
    if Assigned(CdsEvents) then CdsEvents.Close;
  except
  end;
  CdsEvents.Close;
  CdsDict.Close;
  CdsStatus.Close;
  CdsNotes.Close;
  CdsStudyEvents.Close;
  CdsSubmitHistory.Close;
end;

procedure TdmData.CdsInfoCalcFields(DataSet: TDataSet);
begin
  try
    if CdsInfo.FieldByName(FnI_AD).AsString = 'True' then CdsInfoAdvDir.Value := 'Yes'
    else if CdsInfo.FieldByName(FnI_AD).AsString = 'False' then CdsInfoAdvDir.Value := 'No'
    else CdsInfoAdvDir.Value := '';
  except
    CdsInfoAdvDir.Value := '';
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TdmData.CdsMedListAfterInsert(DataSet: TDataSet);
begin
  DataSet.FieldByName(FnML_ID).AsString := DmData.NextID;
  DataSet.FieldByName(FnML_PatientID).AsString := Engine.CurrentPatient.ID;
  DataSet.FieldByName(FnML_StudyID).AsString := Engine.CurrentPatient.SessionID;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TdmData.GetFieldData(AName: string): string;
var
  ADataType, AFormat, ATable, AColumn: string;
  function GetCDSFieldValue(ACDS: TClientDataSet; AField: string): string;
  var
    S: string;
  begin
    try
      if (ADataType = '11') and (AFormat <> '') then // Date and Time
          S := FormatDateTime(AFormat, ACDS.FieldByName(AField).AsDateTime)
      else if (ADataType = '5') then // boolean
      begin
        S := ACDS.FieldByName(AField).AsString;
        if S = 'True' then S := 'Yes';
        if S = 'False' then S := 'No';
      end
      else S := ACDS.FieldByName(AField).AsString;
    except
      S := '';
    end;
    if S = '' then S := Preferences.SValue(OnBlanksPlaceholder);
    Result := S;
  end;

begin
  if CdsDict.Locate(FnDict_Tag, VarArrayOf([AName]), []) then begin
    ATable := CdsDict.FieldByName(FnDict_Table).AsString;
    AColumn := CdsDict.FieldByName(FnDict_Column).AsString;
    AFormat := CdsDict.FieldByName(FnDict_Format).AsString;
    ADataType := CdsDict.FieldByName(FnDict_Type).AsString;
    // anEditMask := cdsDict.FieldByName(fnDict_EditMask).AsString;
    // TheUnit := cdsDict.FieldByName(fnDict_Units).AsString;
  end;
  if ATable = 'cdsSummary' then Result := GetCDSFieldValue(CdsSummary, AColumn)
  else if ATable = 'cdsInfo' then Result := GetCDSFieldValue(CdsInfo, AColumn)
  else if ATable = 'cdsFlowSheet' then Result := GetCDSFieldValue(CdsFlowSheet, AColumn)
  else Result := '<' + AName + '>';
end;
/// /////////////////////////////////////////////////////////////////////////////

function TdmData.GetCDSByName(AName: string): TClientDataSet;
var
  I: Integer;
begin
  Result := nil;
  if AName = '' then Exit;
  Result := nil;
  for I := 0 to ComponentCount - 1 do
    if (Components[I] is TClientDataSet) and (TClientDataSet(Components[I]).Name = AName) then begin
      Result := TClientDataSet(Components[I]);
      Break;
    end;
end;

procedure TdmData.CdsSummaryBeforeInsert(DataSet: TDataSet);
begin
  if not ValidDateTime(DataSet.FieldByName(FnS_rxTxLength).AsString) then begin
    ShowMessage('Invalid format' + DataSet.FieldByName(FnS_rxTxLength).AsString);
  end;
end;

function TdmData.GetFlowsheetReport: string;
var
  ILength: Integer;
  BM: TBookmark;
  Zz, Z: string;
  Ss, S: string;
  Dt: TDateTime;
  Bb, Cc: string;
  procedure ProcessRecord;
  var
    Ll: string;
  begin
    Cc := '|';
    Bb := ' ';
    if CdsFlowSheet.FieldByName(FnFS_Status).AsString <> FsValid then begin
      Ll := ' The next entry was marked as "INVALID" ';
      // ll := ' The next entry was marked as "'+cdsFlowSheet.FieldByName(fnFS_Status).AsString+'" ';
      while Length(Ll) < ILength - 6 do Ll := '.' + Ll;
      Ss := Ss + Ll + #13#10;
      Zz := Zz + Ll + #13#10;
    end;
    Dt := CdsFlowSheet.FieldByName(FnFS_Time).AsDateTime;
    S := FormatDateTime('hh:nn:ss ', Dt);
    if S = '' then S := '         ';
    Ss := Ss + S;
    Zz := Zz + S;
{$IFDEF FLOWEVENTS}
    Z := CdsFlowSheet.FieldByName(FnFS_Event).AsString;
    if Z <> '' then begin
      Ll := Cc + Format('%-60s', [CdsFlowSheet.FieldByName(FnFS_Event).AsString + ' ' +
        CdsFlowSheet.FieldByName(FnFS_Comments).AsString]) + #13#10;
      Ss := Ss + Ll;
      if Preferences.BValue(OnShowFlowsheetEventsCopies) then Zz := Zz + Ll
      else Zz := Zz + #13#10;
    end else begin
{$ENDIF}
      Ss := Ss + Cc + Format('%7.7s', [CdsFlowSheet.FieldByName(FnFS_BP).AsString]);
      Ss := Ss + Cc + Format('%3.3s', [CdsFlowSheet.FieldByName(FnFS_HR).AsString]);
      Ss := Ss + Cc + Format('%5.5s', [CdsFlowSheet.FieldByName(FnFS_BFR).AsString]);
      Ss := Ss + Cc + Format('%5.5s', [CdsFlowSheet.FieldByName(FnFS_DFR).AsString]);
      Ss := Ss + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_AP).AsString]);
      Ss := Ss + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_VP).AsString]);
      Ss := Ss + Cc + Format('%4.4s', [CdsFlowSheet.FieldByName(FnFS_TMP).AsString]);
      Ss := Ss + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_UFR).AsString]);
      Ss := Ss + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_Fluid).AsString]);
      Ss := Ss + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_HEP).AsString]);
      Zz := Zz + Cc + Format('%6.6s ', [CdsFlowSheet.FieldByName(FnFS_MAP).AsString]);
      Zz := Zz + Cc + Format('%6.6s ', [CdsFlowSheet.FieldByName(FnFS_COND).AsString]);
      Zz := Zz + Cc + Format('%6.6s ', [CdsFlowSheet.FieldByName(FnFS_BIC).AsString]);
      Zz := Zz + Cc + Format('%6.6s ', [CdsFlowSheet.FieldByName(FnFS_DIAL).AsString]);
      Zz := Zz + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_IONI).AsString]);
      Zz := Zz + Cc + Format('%6.6s', [CdsFlowSheet.FieldByName(FnFS_KTV).AsString]);
      Zz := Zz + #13#10;
      Ss := Ss + #13#10;
{$IFDEF FLOWEVENTS}
    end;
{$ENDIF}
  end;

begin
  ILength := DevOptions.IValue(DonReportLineLength);
  Ss := '';
  Zz := '';
  Result := '';
  if not CdsFlowSheet.Active then begin
    Result := 'Flowsheet is not availabe';
    Exit;
  end;
  BM := CdsFlowSheet.GetBookmark;
  Ss := Ss + '   Time  |   BP  | HR| BFR | DFR |  AP  |  VP  | TMP|  UFR | Fluid| HEP  ' + #13#10;
  Ss := Ss + '-------------------------------------------------------------------------' + #13#10;
  Zz := Zz + '   Time  |  MAP  | COND  |  BIC  | DIAL  | IONI  |  Kt/V ' + #13#10;
  Zz := Zz + '---------------------------------------------------------' + #13#10;
  if not Preferences.BValue(OnReverseFlowsheetOrder) then begin
    CdsFlowSheet.First;
    if not CdsFlowSheet.Eof then begin
      while not CdsFlowSheet.Eof do begin
        ProcessRecord;
        CdsFlowSheet.Next;
      end;
    end;
  end else begin
    CdsFlowSheet.Last;
    while not CdsFlowSheet.Bof do begin
      ProcessRecord;
      CdsFlowSheet.Prior;
    end;
  end;
  CdsFlowSheet.GotoBookmark(BM);
  CdsFlowSheet.FreeBookmark(BM);
  if CdsFlowSheet.RecordCount > 0 then Result := Ss + #13#10#13#10 + Zz
  else Result := 'Flowsheet contains no data' + #13#10;
end;

function TdmData.GetMedicineAdministrated: string;
var
  BM: TBookmark;
const
  T  = Char(VK_TAB);
  TT = '   ';
begin
  BM := CdsMedList.GetBookmark;
  Result := '';
  CdsMedList.First;
  if CdsMedList.Eof then Result := 'No Medication Administered'
  else begin
    while not CdsMedList.Eof do begin
      Result := Result + FormatDateTime(GMDK_DateFMT, CdsMedList.FieldByName(FnML_Date).AsDateTime) + '  ';
      Result := Result + FormatDateTime(GMDK_TimeFMT, CdsMedList.FieldByName(FnML_Time).AsDateTime) + #13#10;
      Result := Result + T + 'Medication: ' + CdsMedList.FieldByName(FnML_Medication).AsString + #13#10;
      Result := Result + T + 'Dose:       ' + CdsMedList.FieldByName(FnML_Dose).AsString + #13#10;
      Result := Result + T + 'Rout:       ' + CdsMedList.FieldByName(FnML_Rout).AsString + #13#10;
      Result := Result + T + 'Given By:   ' + CdsMedList.FieldByName(FnML_GivenBy).AsString + #13#10;
      Result := Result + T + 'Reason:     ' + CdsMedList.FieldByName(FnML_Reason).AsString + #13#10;
      Result := Result + T + 'Outcome:    ' + CdsMedList.FieldByName(FnML_Outcome).AsString + #13#10;
      if (CdsMedList.FieldByName(FnML_Status).AsString <> 'V') and
        (CdsMedList.FieldByName(FnML_Status).AsString <> 'VALID') then begin
        Result := Result + T + 'Status:     ' + CdsMedList.FieldByName(FnML_Status).AsString + #13#10;
        Result := Result + T + 'Set By:     ' + CdsMedList.FieldByName(FnML_StatusBy).AsString + #13#10;
        Result := Result + T + 'Set On:     ' + CdsMedList.FieldByName(FnML_StatusDate).AsString + #13#10;
      end;
      Result := Result + #13#10;
      CdsMedList.Next;
    end;
  end;
  CdsMedList.GotoBookmark(BM);
  CdsMedList.FreeBookmark(BM);
end;

function TdmData.GetMedicineReport: string;
var
  BFIlter: Boolean;
  BM: TBookmark;
  SItem,
  // sMargin,
  Ss: string;
  ILen: Integer;
  Sl: TStringList;
  SlMedStatuses: TStringList;
  function GetMedListStatus(ACode: string): string;
  begin
    Result := SlMedStatuses.Values[ACode];
    if Result = '' then Result := ACode;
  end;
  function GetUserName(ACode: string): string;
  begin
    Result := Engine.GetUserNameByID(ACode);
    if Result = '' then Result := ACode;
  end;
  function FormatLines(ATopMargin, AMargin, AValue: string): string;
  var
    I: Integer;
    S: string;
  begin
    Result := '';
    Sl.Text := AValue;
    for I := 0 to Sl.Count - 1 do begin
      S := Sl[I];
      if I = 0 then S := ATopMargin + S
      else S := AMargin + S;
      repeat
        Result := Result + Copy(S, 1, ILen) + _CR_;
        S := Copy(S, ILen + 1, Length(S));
        if S = '' then Break;
      until False;
    end;
  end;
  function StatusHeader: string;
  var
    SStatus: string;
  begin
    Result := '';
    with CdsMedList do begin
      SStatus := FieldByName(FnML_Status).AsString;
      Result := Format('%-12.12s', [SStatus]) +
        Format('%s', ['Set By ' + GetUserName(FieldByName(FnML_StatusBy).AsString) + ' on ' +
        FieldByName(FnML_StatusDate).AsString]) + _CR_;
      Result := Result + '            Record ID   ' + FieldByName(FnML_ID).AsString + _CR_;
      Result := Result + '            Replaces    ' + FieldByName(FnML_ParentID).AsString + _CR_;
    end;
  end;
  function ItemMargin: string;
  var
    SStatus: string;
  begin
    Result := '';
    SStatus := CdsMedList.FieldByName(FnML_Status).AsString;
    if SStatus = StML_DELETED then Result := '            ';
    if SStatus = StML_REPLACED then Result := '            ';
  end;

begin
  Ss := '';
  ILen := DevOptions.IValue(DonReportLineLength);
  Result := ''; // 'Flowsheet report';
  if not CdsMedList.Active then Exit;
  SlMedStatuses := TStringList.Create;
  SlMedStatuses.Text := Engine.GetSystemOptionByName(OnMedicationStatuses);
  Sl := TStringList.Create;
  BM := CdsMedList.GetBookmark;
  BFIlter := CdsMedList.Filtered;
  CdsMedList.Filtered := False;
  CdsMedList.First;
  if not CdsMedList.Eof then begin
    with CdsMedList do begin
      while not Eof do begin
        SItem := '';
        SItem := SItem + FormatLines('', '', StatusHeader);
        SItem := SItem + FormatLines('', '', FormatDateTime(GMDK_DateFMT, CdsMedList.FieldByName(FnML_Date).AsDateTime)
          + Format('  %s', [FieldByName(FnML_Medication).AsString + ',  ' + FieldByName(FnML_Dose).AsString + ' (' +
          FieldByName(FnML_Units).AsString + ') ' + FieldByName(FnML_Rout).AsString]));
        SItem := SItem + FormatLines('', '', FormatDateTime('hh:nn:ss    ', FieldByName(FnML_Time).AsDateTime) +
          'given by ' + GetUserName(FieldByName(FnML_GivenBy).AsString));
        SItem := SItem + FormatLines('Reason:     ', '              ', FieldByName(FnML_Reason).AsString);
        SItem := SItem + FormatLines('Outcome:    ', '              ', FieldByName(FnML_Outcome).AsString);
        SItem := SItem + _CR_;
        Ss := Ss + FormatLines(ItemMargin, ItemMargin, SItem);
        Next;
      end;
    end;
  end;
  Sl.Free;
  CdsMedList.Filtered := BFIlter;
  CdsMedList.GotoBookmark(BM);
  CdsMedList.FreeBookmark(BM);
  SlMedStatuses.Free;
  Result := Ss;
end;

function TdmData.GetScheduleReport: string;
begin
  Result := '';
  if CdsInfo.FieldByName(FnI_Mon).AsBoolean then Result := Result + 'Monday, ';
  if CdsInfo.FieldByName(FnI_Tue).AsBoolean then Result := Result + 'Tuesday, ';
  if CdsInfo.FieldByName(FnI_Wed).AsBoolean then Result := Result + 'Wednesday, ';
  if CdsInfo.FieldByName(FnI_Thu).AsBoolean then Result := Result + 'Thursday, ';
  if CdsInfo.FieldByName(FnI_Fri).AsBoolean then Result := Result + 'Friday, ';
  if CdsInfo.FieldByName(FnI_Sat).AsBoolean then Result := Result + 'Saturday, ';
  if CdsInfo.FieldByName(FnI_Sun).AsBoolean then Result := Result + 'Sunday, ';
  if Result = '' then Result := 'No schedule data'
  else Result := Copy(Result, 1, Length(Result) - 2);
end;

function TdmData.GetPainReport(PreTreatment: Boolean): string;
var
  S: string;
  I: Integer;
  function FieldValue(AName: string): string;
  begin
    if CdsSummary.FieldByName(AName).AsString = '' then Result := Preferences.SValue(OnBlanksPlaceholder)
    else Result := CdsSummary.FieldByName(AName).AsString;
  end;

begin
  Result := '';
  if PreTreatment then begin
    S := 'PRE-TREATMENT PAIN ASSESSMENT' + _CR_;
    if CdsSummary.FieldByName(FnS_PrePain).AsString = '' then begin
      S := S + 'Pain evaluation was not performed' + _CR_;
    end else begin
      I := CdsSummary.FieldByName(FnS_PrePain).AsInteger;
      if I = 0 then begin
        S := S + 'No pain (pain level - 0)' + _CR_;
        Result := S;
        Exit;
      end;
      S := S + 'How bad is the pain? ' + FieldValue(FnS_PrePain) + _CR_;
      if Preferences.BValue(OnConsiderPainTolerance) then begin
        S := S + 'Can patient tolerate pain? ' + FieldValue(FnS_PrePainTolerance) + _CR_;
        if Uppercase(FieldValue(FnS_PrePainTolerance)) = 'YES' then begin
          Result := S;
          Exit;
        end;
      end;
      S := S + 'Where is The Pain Located? ' + FieldValue(FnS_PrePainLocation) + _CR_;
      S := S + 'Pain Form? ' + FieldValue(FnS_PrePainAcute) + _CR_;
      S := S + 'Pain Treatment? ' + FieldValue(FnS_PrePainTreatment) + _CR_;
      if I < StrToInt(Preferences.SValue(OnPainLevel)) then begin
        Result := S;
        Exit;
      end;
      S := S + _CR_;
      S := S + 'Detailed Pain Assessment ' + _CR_;
      // S := S + '1. Location of pain: ' + FieldValue(fnS_PrePainIASP) + _CR_;
      // S := S + '2. Intensity: ' + FieldValue(fnS_PrePainIntensity) + _CR_;
      S := S + '1. Frequency text: ' + FieldValue(FnS_PrePainFrequencyText) + _CR_;
      S := S + '2. Onset: ' + FieldValue(FnS_PrePainOnset) + _CR_;
      S := S + '3. Duration of episodes: ' + FieldValue(FnS_PrePainDuration) + _CR_;
      S := S + '4. Quality of pain: ' + FieldValue(FnS_PrePainQuality) + _CR_;
      S := S + 'The patient indicated that:' + _CR_;
      S := S + '1. The impact of the pain was: ' + FieldValue(FnS_PrePainImpact) + _CR_;
      S := S + '2. The following makes the pain worse: ' + FieldValue(FnS_PrePainWorse) + _CR_;
      S := S + '3. The following relieves the pain: ' + FieldValue(FnS_PrePainRelieve) + _CR_;
      S := S + '4. The medications that help the pain are: ' + FieldValue(FnS_PrePainHelpMed) + _CR_;
      S := S + '5. Other treatments used for pain are: ' + FieldValue(FnS_PrePainOtherTreatments) + _CR_;
      S := S + '6. Other comments: ' + FieldValue(FnS_PrePainOtherComments) + _CR_;
    end;
  end else begin
    S := 'POST-TREATMENT PAIN ASSESSMENT' + _CR_;
    if CdsSummary.FieldByName(FnS_PostPain).AsString = '' then begin
      S := S + 'Pain evaluation was not performed' + _CR_;
    end else begin
      I := CdsSummary.FieldByName(FnS_PostPain).AsInteger;
      // s := 'POST-PAIN ASSESSMENT' + _CR_;
      if I = 0 then begin
        S := S + 'No pain (pain level - 0)' + _CR_;
        Result := S;
        Exit;
      end;
      S := S + 'How bad is the pain? ' + FieldValue(FnS_PostPain) + _CR_;
      if Preferences.BValue(OnConsiderPainTolerance) then begin
        S := S + 'Can patient tolerate pain? ' + FieldValue(FnS_PostPainTolerance) + _CR_;
        if Uppercase(FieldValue(FnS_PrePainTolerance)) = 'YES' then begin
          Result := S;
          Exit;
        end;
      end;
      S := S + 'Where is The Pain Located? ' + FieldValue(FnS_PostPainLocation) + _CR_;
      S := S + 'Pain Form? ' + FieldValue(FnS_PostPainAcute) + _CR_;
      S := S + 'Pain Treatment? ' + FieldValue(FnS_PostPainTreatment) + _CR_;
      if I < StrToInt(Preferences.SValue(OnPainLevel)) then begin
        Result := S;
        Exit;
      end;
      S := S + _CR_;
      S := S + 'Detailed Pain Assessment ' + _CR_;
      // S := S + '1. Location of pain: ' + FieldValue(fnS_PostPainIASP) + _CR_;
      // S := S + '2. Intensity: ' + FieldValue(fnS_PostPainIntensity) + _CR_;
      S := S + '1. Frequency text: ' + FieldValue(FnS_PostPainFrequencyText) + _CR_;
      S := S + '2. Onset: ' + FieldValue(FnS_PostPainOnset) + _CR_;
      S := S + '3. Duration of episodes: ' + FieldValue(FnS_PostPainDuration) + _CR_;
      S := S + '4. Quality of pain: ' + FieldValue(FnS_PostPainQuality) + _CR_;
      S := S + 'The patient indicated that:' + _CR_;
      S := S + '1. The impact of the pain was: ' + FieldValue(FnS_PostPainImpact) + _CR_;
      S := S + '2. The following makes the pain worse: ' + FieldValue(FnS_PostPainWorse) + _CR_;
      S := S + '3. The following relieves the pain: ' + FieldValue(FnS_PostPainRelieve) + _CR_;
      S := S + '4. The medications that help the pain are: ' + FieldValue(FnS_PostPainHelpMed) + _CR_;
      S := S + '5. Other treatments used for pain are: ' + FieldValue(FnS_PostPainOtherTreatments) + _CR_;
      S := S + '6. Other comments: ' + FieldValue(FnS_PostPainOtherComments) + _CR_;
    end;
  end;
  Result := S;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TdmData.GetCommentsReport(AContextFilter: string): string;
var
  SFilter: string;
  BFiltered: Boolean;
begin
  BFiltered := CdsNotes.Filtered;
  SFilter := CdsNotes.Filter;
  if AContextFilter <> '' then begin
    SetCDSFilter(CdsNotes, AContextFilter);
    try
      CdsNotes.Filtered := True;
    except
    end;
  end
  else Engine.NoteList_CDS.Filtered := False;
  Result := GetNotesReport('', CdsNotes);
  if BFiltered then begin
    CdsNotes.Filter := SFilter;
    CdsNotes.Filtered := True;
  end;
end;

function TdmData.GetFlowNotesReport: string;
begin
  Result := '';
  if not CdsNotes.Active then Exit;
  Result := GetCommentsReport(GetNotesContextFilter(4));
end;

procedure TdmData.CdsNotesAfterInsert(DataSet: TDataSet);
begin
  CdsNotes.FieldByName(FnNL_ID).AsString := IntToStr(CdsNotes.RecordCount);
  CdsNotes.FieldByName(FnNL_Lock).AsString := FsUnlocked;
  CdsNotes.FieldByName(FnNL_StudyID).AsString := Engine.CurrentPatient.SessionID;
  CdsNotes.FieldByName(FnNL_PatientID).AsString := Engine.CurrentPatient.ID;
  CdsNotes.FieldByName(FnNL_AuthorID).AsString := Engine.CurrentOperatorID;
end;

procedure TdmData.CdsNotesAuthorNameGetText(Sender: TField; var Text: string; DisplayText: Boolean);
var
  S: string;
  I: Integer;
begin
  try
    S := CdsNotes.FieldByName(FnNL_AuthorID).AsString;
    I := StrToIntDef(Trim(S), -1);
    if (I > 0) and (S <> '') then S := Engine.GetUserNameByID(S);
    if (S = '') or (S = #$D#$A) then S := CdsNotes.FieldByName(FnNL_Tag).AsString;
    Text := S;
  except
  end;
end;

procedure TdmData.DsNotesDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(CdsNotes) and (CdsNotes.Active) and Assigned(FrmMain) then FrmMain.RefreshCurrentWindow;
end;

procedure TdmData.SetCurrentNoteStatus(AStatus: string);
begin
  try
    CdsNotes.Edit;
    CdsNotes.FieldByName(FnNL_Status).AsString := AStatus;
    CdsNotes.FieldByName(FnNL_StatusBy).AsString := Engine.CurrentOperatorID;
    CdsNotes.FieldByName(FnNL_StatusDate).AsDateTime := Engine.GetCurrentServerDateTime;
    CdsNotes.Post;
  except
  end;
end;

procedure TdmData.ChangeCurrentNoteValidStatus;
begin
  if (CdsNotes.FieldByName(FnNL_Status).AsString = FsValid) then SetCurrentNoteStatus(FsInvalid)
  else SetCurrentNoteStatus(FsValid)
end;

procedure TdmData.LockCurrentNote;
begin
  if CdsNotes.FieldByName(FnNL_Lock).AsString = FsLocked then Exit;
  try
    CdsNotes.Edit;
    CdsNotes.FieldByName(FnNL_Lock).AsString := FsLocked;
    CdsNotes.Post;
  except
  end;
end;

procedure TdmData.LockNotes;
var
  BM: TBookmark;
  SFilter: string;
  BFIlter: Boolean;
begin
  BM := CdsNotes.GetBookmark;
  SFilter := CdsNotes.Filter;
  BFIlter := CdsNotes.Filtered;
  CdsNotes.Filtered := False;
  CdsNotes.DisableControls;
  CdsNotes.First;
  while not CdsNotes.Eof do begin
    LockCurrentNote;
    CdsNotes.Next;
  end;
  if Assigned(BM) then
    try
      CdsNotes.GotoBookmark(BM);
      CdsNotes.FreeBookmark(BM);
    except
      // on E: Exception do
      // MessageDLG('Lock Notes:'+#13#10+E.Message,mtWarning,[mbOK],0);
    end;
  CdsNotes.EnableControls;
  if BFIlter then begin
    CdsNotes.Filter := SFilter;
    CdsNotes.Filtered := True;
  end;
end;

procedure TdmData.DeleteCurrentNote;
begin
  if CdsNotes.FieldByName(FnNL_Lock).AsString = FsLocked then Exit;
  try
    CdsNotes.Edit;
    CdsNotes.Delete;
    CdsNotes.Post;
  except
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TdmData.PatchDictionary(AString: string);
var
  SOldDictionaryXML, ATable, AColumn, SOldValue, SCopyValue, SValue: string;
  CdsCopy: TClientDataSet;
  Fld: TFieldDef;
  Fn: string;
  ICase, I: Integer;
begin
  CdsDict.MergeChangeLog;
  SOldDictionaryXML := CdsDict.XMLData;
  SValue := AString;
  while Pos(#13 + #10, SValue) = 1 do SValue := Copy(SValue, 3, Length(SValue));
  while Pos(#10 + #13, SValue) = 1 do SValue := Copy(SValue, 3, Length(SValue));
  if (AString = '') or (AString = #13 + #10) or (Pos('-1^', SValue) = 1) then begin
    ShowMessage('No data to process');
    Exit;
  end;
  CdsCopy := TClientDataSet.Create(nil);
  try
    CdsCopy.XMLData := SValue;
    CdsCopy.Open;
    while not CdsCopy.Eof do begin
      ATable := CdsCopy.FieldByName(FnDict_Table).AsString;
      AColumn := CdsCopy.FieldByName(FnDict_Column).AsString;
      if not CdsDict.Locate(FnDict_Table + ';' + FnDict_Column, VarArrayOf([ATable, AColumn]), []) then CdsDict.Insert;
      for I := 0 to CdsDict.FieldCount - 1 do begin
        if I >= CdsDict.FieldDefs.Count then Break;
        Fld := CdsDict.FieldDefs[I];
        Fn := Fld.Name;
        SOldValue := CdsDict.FieldByName(Fn).AsString;
        try
          try
            SCopyValue := CdsCopy.FieldByName(Fn).AsString;
          except
            on E: Exception do SCopyValue := '';
          end;
          if (SOldValue = '') then CdsDict.FieldByName(Fn).AsString := SCopyValue
          else if (SOldValue <> SCopyValue) then begin
            ICase := MessageDlg('Do you want to replace the value of the <' + Fn + '> column?' + #13#10#13#10 +
              'Old Value: "' + SOldValue + '" ' + #13#10 + 'New Value: "' + SCopyValue + '"' + #13#10 + #13#10 +
              '(To stop updates and restore the original dictionary select "Cancel")', MtInformation,
              [MbOk, MbNo, MbCancel], 0);
            case ICase of
              MrOK: CdsDict.FieldByName(Fn).AsString := SCopyValue;
              MrCancel: begin
                  CdsCopy.Last;
                  try
                    CdsDict.Close;
                    CdsDict.XMLData := SOldDictionaryXML;
                    CdsDict.Open;
                  except
                  end;
                  Break;
                end;
            end;
          end;
        except
        end;
      end;
      try
        CdsDict.Post;
      except
      end;
      CdsCopy.Next;
    end;
  except
  end;
  CdsCopy.Free;
end;

function TdmData.CheckCDSIDField(ADS: TClientDataSet; AIDField, AIDValue: string; var BadValue: string): Boolean;
var
  B: Boolean;
  BM: TBookmark;
begin
  B := True;
  BM := ADS.GetBookmark;
  ADS.First;
  while not ADS.Eof do begin
    BadValue := ADS.FieldByName(AIDField).AsString;
    B := BadValue = AIDValue;
    if not B then Break;
    ADS.Next;
  end;
  ADS.GotoBookmark(BM);
  ADS.FreeBookmark(BM);
  Result := B;
end;

function TdmData.ValidDataSetXML(XML: string): Boolean;
var
  CDS: TClientDataSet;
begin
  CDS := nil;
  try
    CDS := TClientDataSet.Create(nil);
    try
      CDS.XMLData := XML;
      CDS.Open;
      Result := CDS.Active;
    except
      on E: Exception do Result := False;
    end;
  finally
    CDS.Free;
  end;
end;

function TdmData.UpdateFlowsheetIndex(ADescending: Boolean): Boolean;
begin
  Result := False;
  with CdsFlowSheet do begin
    try
      if not Active then
        try
          Open;
        except
          on E: Exception do Exit;
        end;
      try
        if (IndexName = 'Flowsheet_Index') then DeleteIndex('Flowsheet_Index');
      except
        on E: Exception do Exit;
      end;
      if Preferences.BValue(OnReverseFlowsheetOrder) then
          AddIndex('Flowsheet_Index', FnFS_Date + ';' + FnFS_Time, [IxDescending])
      else AddIndex('Flowsheet_Index', FnFS_Date + ';' + FnFS_Time, []);
      // IndexFieldNames := fnFS_Date + ';' + fnFS_Time;
      IndexDefs.Update;
      IndexName := 'Flowsheet_Index';
    except
      on E: Exception do ShowMessage('Flowsheet Index update error:' + #13#10 + E.Message);
    end;
  end;
end;

procedure TdmData.DsStudyHistoryDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(FrmCover) then FrmCover.RefreshTIU;
end;

procedure TdmData.DsWorkflowDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(FrmFlow) then FrmFlow.SetWorkflowStatus;
end;

function TdmData.GetGeneralInfoReport: string;
var
  // i: Integer;
  S: string;
  // sName: String;
begin
  Result := '';
  S := '';
  S := S + 'Current Treatment Date: ' + CdsInfo.FieldByName(FnI_StudyDate).AsString + #13;
  S := S + 'Diagnosis:              ' + CdsInfo.FieldByName(FnI_Diagnosis).AsString + #13;
  S := S + 'Diagnosis Date:         ' + CdsInfo.FieldByName(FnI_DiagnosisDate).AsString + #13;
  S := S + 'Initial Therapy Date:   ' + CdsInfo.FieldByName(FnI_TherapyDate).AsString + #13;
  S := S + 'Modalities:             ' + CdsInfo.FieldByName(FnI_Modality).AsString + #13;
  S := S + 'Code Status:            ' + CdsInfo.FieldByName(FnI_CodeStatus).AsString + #13;
  S := S + 'Attending Nephrologist: ' + CdsInfo.FieldByName(FnI_Nephrologist).AsString + #13;
  S := S + #13 + 'Visit Schedule' + #13;
  if CdsInfo.FieldByName(FnI_Mon).AsBoolean then S := S + '      Monday' + #13;
  if CdsInfo.FieldByName(FnI_Tue).AsBoolean then S := S + '      Tuesday' + #13;
  if CdsInfo.FieldByName(FnI_Wed).AsBoolean then S := S + '      Wednesday' + #13;
  if CdsInfo.FieldByName(FnI_Thu).AsBoolean then S := S + '      Thursday' + #13;
  if CdsInfo.FieldByName(FnI_Fri).AsBoolean then S := S + '      Friday' + #13;
  if CdsInfo.FieldByName(FnI_Sat).AsBoolean then S := S + '      Saturday' + #13;
  if CdsInfo.FieldByName(FnI_Sun).AsBoolean then S := S + '      Sunday' + #13;
  S := S + #13 + 'Transplant status' + #13;
  if not CdsSummary.FieldByName(FnS_TrnCandidate).AsBoolean then S := S + '      Non Candidate'
  else begin
    S := S + '      Candidate' + #13;
    if CdsSummary.FieldByName(FnS_TrnWIP).AsBoolean then S := S + '                Work in progress' + #13;
    if CdsSummary.FieldByName(FnS_TrnReferred).AsBoolean then
        S := S + '                Referred to Transplant Center' + #13;
  end;
  Result := S;
end;

procedure TdmData.SaveErrorsLog;
begin
{$IFDEF SAVE_ERRORS_LOG}
  CdsEvents.MergeChangeLog;
  CdsEvents.First;
  while not CdsEvents.Eof do
    if CdsEvents.FieldByName(FnEvents_Type).AsString <> RtError then CdsEvents.Delete
    else CdsEvents.Next;
  CdsEvents.MergeChangeLog;
  Engine.FRenalController.SetSystemOption(OnErrorLog, CdsEvents.XMLData);
{$ENDIF}
end;

procedure TdmData.DsInfoDataChange(Sender: TObject; Field: TField);
begin
  if not Assigned(FrmMain) then Exit;
  if not Assigned(Field) then Exit;
  if Field.FieldName = FnI_StudyDate then FrmMain.RefreshStudyStatus;
end;

function TdmData.UpdateMedlistIndex(ADescending: Boolean): Boolean;
const
  INDX_NAME = 'MedList_Index';
begin
  Result := False;
  with CdsMedList do begin
    try
      if not Active then
        try
          Open;
        except
          on E: Exception do Exit;
        end;
      try
        if (IndexName = INDX_NAME) then DeleteIndex(INDX_NAME);
      except
        on E: Exception do Exit;
      end;
      if Preferences.BValue(OnReverseFlowsheetOrder) then
          AddIndex(INDX_NAME, FnML_StudyID + ';' + FnML_Date + ';' + FnML_Time, [IxDescending])
      else AddIndex(INDX_NAME, FnML_StudyID + ';' + FnML_Date + ';' + FnML_Time, []);
      IndexDefs.Update;
      IndexName := INDX_NAME;
    except
      on E: Exception do ShowMessage('Medlist Index update error:' + #13#10 + E.Message);
    end;
  end;
end;

procedure TdmData.DsMedListDataChange(Sender: TObject; Field: TField);
begin
  if Assigned(FrmFlow) then FrmFlow.SetActionStatus;
end;

end.
