unit fGMV_AboutDlg;
{
  ================================================================================
  *
  *        Application:  Hemodialysis
  *        Revision:     $Revision: 1 $  $Modtime: 4/23/08 6:59a $
  *        Developer:    doma.user@domain.ext
  *                     modified from BCMA Application by BKurtze@Dallas CIOFO
  *        Site:         Hines OIFO
  *
  *         Description:  This is an About Dialog which displays data from the project
  *                     VersionInfo block for the application.
  *
  *        Notes:        Delphi developer must add Keys VAReleaseDate and VANamespace
  *                     Under Project|Options on the Version Info tab for the version,
  *                     patch, and release date information to show up on the form.
  *
  *
  ================================================================================
  *        $ Archive: /CP Version 1.0 $
  *        $ History: fGMV_AboutDlg.pas $
  *
  ================================================================================
  *       2007-06-20 zzzzzzandria         Execute method updated b. 312 (#13)
}

interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls, Buttons, ExtCtrls, Jpeg;

type
  TfrmGMV_AboutDlg = class(TForm)
    Panel1: TPanel;
    LblProductNameInset: TLabel;
    LblClientVersion: TLabel;
    LblCopyright: TLabel;
    LblReleaseDate: TLabel;
    Label2: TLabel;
    LblReleaseDateLabel: TLabel;
    Label4: TLabel;
    LblPatchLabel: TLabel;
    LblPatch: TLabel;
    LblProductNameShadow: TLabel;
    PnlImage: TPanel;
    ApplicationImage: TImage;
    BtnOK: TButton;
    Label1: TLabel;
    LblHelpFile: TLabel;
    Panel3: TPanel;
    PnlComments: TPanel;
    LblSecurity: TLabel;
    Label3: TLabel;
    LblServerVersion: TLabel;
    LblCompileDate: TLabel;
    LblCRCValue: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    LblSourceInfo: TLabel;
    Image: TImage;
    procedure FormCreate(Sender: TObject);
    procedure LblCommentsMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    (*
      Uses TVersionInfo to read the Version Info for application.ExeName into
      the form's display fields.
    *)
  private
    FClose: Boolean;
    FComments: TStringList;
    procedure ShowInfo;
    procedure HideInfo;
    procedure Splash;
  public
    procedure Execute;
  end;

var
  GMVAboutDlg: TfrmGMV_AboutDlg;
  GMVAboutSplash: TfrmGMV_AboutDlg;
procedure SplashOn;
procedure SplashOff;

implementation

{$R *.DFM}

uses
  Dialogs, Math, UGMV_CRC32, UGMV_VersionInfo, UMDK_Utils, FMDK_Main;

procedure SplashOn;
begin
  GMVAboutSplash := TfrmGMV_AboutDlg.Create(Application);
  GMVAboutSplash.Splash;
end;

procedure SplashOff;
begin
  if Assigned(GMVAboutSplash) then GMVAboutSplash.Free;
end;

function GetResourceString(AName: string): string;
var
  TmpStream: TResourceStream;
  SL: TStringList;
begin
  Result := '';
  if AName = '' then Exit;
  SL := TStringList.Create;
  TmpStream := nil;
  try
    TmpStream := TResourceStream.Create(HInstance, AName, 'TXT');
    SL.LoadFromStream(TmpStream);
  finally
    TmpStream.Free;
  end;
  Result := SL.Text;
  SL.Free;
end;
(* === AboutDlg Methods ====================================================== *)

procedure TfrmGMV_AboutDlg.Splash;
begin
  HideInfo;
  PnlComments.Align := AlClient;
  Image.Align := AlClient;
  BtnOK.Visible := False;
  Height := 260;
  BorderStyle := BsNone;
  Panel1.BevelWidth := 3;
  Panel1.BevelOuter := BvRaised;
  Show;
  Update;
end;

procedure TfrmGMV_AboutDlg.Execute;
begin
  HideInfo;
  PnlComments.Align := AlClient;
  Image.Align := AlClient;
  ShowModal;
end;

// Copy from FileCtrl

procedure CutFirstDirectory(var S: TFileName);
var
  Root: Boolean;
  P: Integer;
begin
  if S = '\' then S := ''
  else begin
    if S[1] = '\' then begin
      Root := True;
      Delete(S, 1, 1);
    end
    else Root := False;
    if S[1] = '.' then Delete(S, 1, 4);
    P := AnsiPos('\', S);
    if P <> 0 then begin
      Delete(S, 1, P);
      S := '...\' + S;
    end
    else S := '';
    if Root then S := '\' + S;
  end;
end;

function MinimizeName(const Filename: TFileName; Canvas: TCanvas; MaxLen: Integer): TFileName;
var
  Drive: TFileName;
  Dir: TFileName;
  Name: TFileName;
begin
  Result := FileName;
  Dir := ExtractFilePath(Result);
  name := ExtractFileName(Result);
  if (Length(Dir) >= 2) and (Dir[2] = ':') then begin
    Drive := Copy(Dir, 1, 2);
    Delete(Dir, 1, 2);
  end
  else Drive := '';
  while ((Dir <> '') or (Drive <> '')) and (Canvas.TextWidth(Result) > MaxLen) do begin
    if Dir = '\...\' then begin
      Drive := '';
      Dir := '...\';
    end else if Dir = '' then Drive := ''
    else CutFirstDirectory(Dir);
    Result := Drive + Dir + name;
  end;
end;

procedure TfrmGMV_AboutDlg.FormCreate(Sender: TObject);
begin
  FClose := False;
  FComments := TStringList.Create;
  FComments.Text := GetResourceString('TEAM');
  LblProductNameInset.Font.Color := ClWindowText; // clActiveCaption;
  LblProductNameShadow.Font.Color := ClBtnHighlight;
  LblProductNameShadow.Top := LblProductNameInset.Top + 1;
  LblProductNameShadow.Left := LblProductNameInset.Left + 1;
  LblHelpFile.Caption := MinimizeName(Application.HelpFile, LblHelpFile.Canvas, LblHelpFile.Width);
  LblHelpFile.Hint := 'Help file name: ' + Application.HelpFile;
  LblHelpFile.ShowHint := (Pos('...', LblHelpFile.Caption) > 0);
  LblCRCValue.Caption := GetFileCRC32(Application.Exename);
  with TVersionInfo.Create(Self) do
    try
      LblProductNameInset.Caption := 'CP ' + ProductName;
      LblProductNameShadow.Caption := 'CP ' + ProductName;
      Self.Caption := 'About ' + 'CP ' + ProductName;
      LblSourceInfo.Caption := VASourceInformation;
      LblClientVersion.Caption := FileVersion;
      LblServerVersion.Caption := VAServerVersion;
      LblPatch.Caption := VAPatchNumber;
      LblPatchLabel.Visible := (VAPatchNumber <> '');
      LblCompileDate.Caption := FormatDateTime('mm/dd/yy hh:mm:ss', CompileDateTime);
      LblReleaseDate.Caption := VAReleaseDate;
      LblCopyright.Caption := LegalCopyRight;
    finally
      Free;
    end;
end;

procedure TfrmGMV_AboutDlg.LblCommentsMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState;
  X, Y: Integer);
begin
  if (Button = MbRight) and (SsShift in Shift) and (SsCtrl in Shift) and (SsAlt in Shift) and (SsDouble in Shift) then
      ShowInfo;
end;

procedure TfrmGMV_AboutDlg.HideInfo;
begin
  PnlComments.Visible := False;
end;

procedure TfrmGMV_AboutDlg.ShowInfo;
var
  IStep, IMargin, J, I: Integer;
  SLeft, SCenter, SRight: string;
const
  ClText = ClNavy;

  procedure GrowRightCentered(ACanvas: TCanvas; AWait: Integer; AXMargin, Y, AWidth: Integer; AFColor: TColor;
    AText: string);
  var
    Ii, I: Integer;
  begin
    if FClose then Exit;
    with ACanvas do begin
      Font.Style := [FsBold];
      Ii := TextWidth(AText);
      Ii := (AWidth - Ii) div 2;
      AXMargin := Ii;
      Ii := TextHeight(AText);
      Font.Color := AFColor;
      for I := AXMargin to AWidth - AXMargin do begin
        FillRect(Rect(AXMargin, Y, I, Y + Ii));
        TextRect(Rect(AXMargin, Y, I, Y + Ii), AXMargin, Y, AText);
        Application.ProcessMessages;
        if FClose then Exit;
        Sleep(AWait);
      end;
    end;
  end;

  procedure GrowRight(ACanvas: TCanvas; AWait: Integer; AXMargin, Y, AWidth: Integer; AFColor: TColor; AText: string);
  var
    ILen, IWidth, IHeight, I: Integer;
  begin
    if FClose then Exit;
    with ACanvas do begin
      Font.Style := [FsBold];
      IWidth := TextWidth(AText);
      IHeight := TextHeight(AText);
      Font.Color := AFColor;
      ILen := Min(IWidth, AWidth - AXMargin);
      for I := AXMargin to ILen do begin
        FillRect(Rect(AXMargin, Y, I, Y + IHeight));
        TextRect(Rect(AXMargin, Y, I, Y + IHeight), AXMargin, Y, AText);
        Application.ProcessMessages;
        if FClose then Exit;
        Sleep(AWait);
      end;
    end;
  end;

  procedure SlideRight(ACanvas: TCanvas; AWait: Integer; AXMargin, Y, AwIDTH: Integer; AFColor: TColor; AText: string);
  var
    IWidth, IHeight, I: Integer;
  begin
    if FClose then Exit;
    with ACanvas do begin
      Font.Style := [FsBold];
      IWidth := TextWidth(AText);
      IHeight := TextHeight(AText);
      Font.Color := AFColor;
      for I := AXMargin to AXMargin + IWidth do begin
        TextRect(Rect(AXMargin, Y, I, Y + IHeight), AXMargin, Y, AText);
        Application.ProcessMessages;
        if FClose then Exit;
        Sleep(AWait);
      end;
    end;
  end;

  procedure SlideLeft(ACanvas: TCanvas; AWait: Integer; AXMargin, Y, AWidth: Integer; AFColor: TColor; AText: string);
  var
    IX, ILen, IWidth, IHeight, IPos, I: Integer;
  begin
    if FClose then Exit;
    with ACanvas do begin
      IPos := AWidth - AXMargin;
      ILen := 0;
      Font.Style := [];
      IWidth := TextWidth(AText);
      IHeight := TextHeight(AText);
      Font.Color := AFColor;
      IX := Max(AXMargin, AWidth - IWidth - AXMargin);
      for I := AWidth - AXMargin downto IX do begin
        IPos := I;
        if Font.Color > $0 then Font.Color := Font.Color - $00030303;
        ILen := Min(IWidth, AWidth - AXMargin - I);
        FillRect(Rect(IPos, Y, IPos + ILen + 2, Y + IHeight));
        TextRect(Rect(IPos, Y, IPos + ILen, Y + IHeight), I, Y, AText);
        Application.ProcessMessages;
        if FClose then Exit;
        if (AWidth - AXMargin - I) > 0 then begin
          Sleep(AWait);
          Application.ProcessMessages;
        end;
      end;
      FillRect(Rect(IPos, Y, IPos + ILen + 2, Y + IHeight));
      Font.Style := [];
      TextRect(Rect(IPos, Y, IPos + ILen, Y + IHeight), IPos, Y, AText);
      Application.ProcessMessages;
    end;
  end;

begin
  FClose := False;
  PnlComments.Visible := True;
  Image.Canvas.Brush.Color := PnlComments.Color;
  Image.Canvas.FillRect(Rect(0, 0, Image.Width, Image.Height));
  I := 0;
  for J := 0 to FComments.Count - 1 do begin
    IStep := StrToIntDef(Piece(FComments[J], '^', 1), 0);
    // iWait := StrToIntDef(piece(fComments[j],'^',2),0);
    IMargin := StrToIntDef(Piece(FComments[J], '^', 3), 30);
    SLeft := Piece(FComments[J], '^', 4);
    SCenter := Piece(FComments[J], '^', 5);
    SRight := Piece(FComments[J], '^', 6);
    Inc(I, IStep);
    if SCenter <> '' then GrowRightCentered(Image.Canvas, 0, IMargin, I, Image.Width, ClBlack, SCenter);
    if SLeft <> '' then SlideRight(Image.Canvas, 10, IMargin, I, Image.Width, ClText, SLeft);
    if SRight <> '' then SlideLeft(Image.Canvas, 30, IMargin, I, Image.Width, ClBlack, SRight);
  end;
  if FClose then Exit;
  Sleep(750);
  PnlComments.Visible := False;
end;

procedure TfrmGMV_AboutDlg.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then begin
    FClose := True;
    Close;
  end;
end;

procedure TfrmGMV_AboutDlg.FormDestroy(Sender: TObject);
begin
  FComments.Free;
  inherited;
end;

end.
