unit fMDK_DictionaryEdit;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/22/08 11:00a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     client data set record editor
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_DictionaryEdit.pas $
  *
  *       $ History: fMDK_DictionaryEdit.pas $
}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Buttons, Grids, ValEdit, ExtCtrls, Mask, DBCtrls, DB, DBClient, ActnList,
  UMDK_Vitals, System.Actions;

type
  TfrmMDK_DictionaryEdit = class(TForm)
    PnlEdit: TPanel;
    Panel2: TPanel;
    BbOK: TBitBtn;
    BbCancel: TBitBtn;
    Panel3: TPanel;
    Vle: TValueListEditor;
    Panel4: TPanel;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    ActionList1: TActionList;
    AcEditDomain: TAction;
    AcSaveToDB: TAction;
    procedure VleDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure AcEditDomainExecute(Sender: TObject);
    procedure AcSaveToDBExecute(Sender: TObject);
    procedure VleSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: string);
    procedure VleEnter(Sender: TObject);
    procedure VleGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: string);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VleMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure FormCloseQuery(Sender: TObject; var CanClose: Boolean);
    procedure BbCancelClick(Sender: TObject);
  private
    OriginalLabel: string;
    procedure SetDomainEditStatus(ARow, ACol: Integer);
    procedure Prepare(ATable, AField: string; AVitalType: TVitalType = VtUnknown);
    procedure SetUp(ATable, AField: string; AStrings: TStrings);
    function GetDisplayFormat: string;
    function GetEditMask: string;
    function GetMaxValue: Double;
    function GetMinValue: Double;
  public
    { Public declarations }
  end;

const
  DOMAIN_PICK_LIST = 'Anticoagulants' + #13#10 + 'Attending Nephrologists' + #13#10 + 'Code Statuses' + #13#10 +
    'Dialyzer List' + #13#10 + 'Education Codes' + #13#10 + 'ESRD Diagnoses' + #13#10 + 'ESRD ICD10' + #13#10
  // added 60/12/12 Joe Carney
  // +'LAB Codes'+#13#10
    + 'Medical Facilities' + #13#10 + 'Medications' + #13#10 + 'Medication Routes' + #13#10 + 'Medication Units' +
    #13#10 + 'Medication Statuses' + #13#10 + 'Medication Reasons' + #13#10 + 'Modalities' + #13#10 + 'Surgeons' +
    #13#10 + 'TIU Note Titles' + #13#10 + 'Transportation Methods' + #13#10;
  TRUE_FALSE = 'True' + #13#10 + 'False' + #13#10;
function EditRecord(ATable, AField: string; AVitalType: TVitalType = VtUnknown): Boolean;
procedure UpdateDBEditField(AField: TDBedit; AVitalType: TVitalType = VtUnknown);
procedure UpdateColumnField(AField: TField; DsName, DsFName: string; AVitalType: TVitalType = VtUnknown);

var
  FrmMDK_DictionaryEdit: TfrmMDK_DictionaryEdit;

implementation

uses
  DMDK_Data, UMDK_Engine, UMDK_Options, FMDK_Options, UMDK_Dictionary, System.UITypes,
  UMDK_CDSCommon, UMDK_Common;
{$R *.dfm}

function EditRecord(ATable, AField: string; AVitalType: TVitalType = VtUnknown): Boolean;
var
  SOld: string;
begin
  Result := False;
  if not Assigned(FrmMDK_DictionaryEdit) then FrmMDK_DictionaryEdit := TfrmMDK_DictionaryEdit.Create(Application);
  with FrmMDK_DictionaryEdit do begin
    Prepare(ATable, AField, AVitalType);
    if Vle.RowCount < 2 then Exit;
    Refresh;
    SOld := Vle.Strings.Text;
    if (ShowModal <> MrCancel) and (SOld <> Vle.Strings.Text) then begin
      SetUp(ATable, AField, Vle.Strings);
      Result := True;
    end;
  end;
  FreeAndNil(FrmMDK_DictionaryEdit);
end;

procedure SetDictionaryRecord(AField: TField);
var
  BbIgnore: Boolean;
  S: string;
  I: Integer;
begin
  if not Assigned(AField) then Exit;
  BbIgnore := DmData.BIgnore;
  DmData.BIgnore := True;
  with FrmMDK_DictionaryEdit do
    try
      case AField.DataType of
        FtDateTime: begin
            TDateTimeField(AField).DisplayFormat := GetDisplayFormat;
            TDateTimeField(AField).EditMask := GetEditMask;
          end;
        FtInteger: begin
            TintegerField(AField).DisplayFormat := GetDisplayFormat;
            TintegerField(AField).EditMask := GetEditMask;
            TintegerField(AField).MaxValue := Round(GetMaxValue);
            TintegerField(AField).MinValue := Round(GetMinValue);
          end;
        FtFloat: begin
            TFloatField(AField).DisplayFormat := GetDisplayFormat;
            TFloatField(AField).EditMask := GetEditMask;
            TFloatField(AField).MaxValue := GetMaxValue;
            TFloatField(AField).MinValue := GetMinValue;
          end;
        FtString: begin
            I := TStringField(AField).DisplayWidth;
            S := Vle.Values[FnDict_Width];
            if S <> '' then
              try
                TStringField(AField).DisplayWidth := StrToInt(S);
                TStringField(AField).Size := I;
              except
                begin
                  TStringField(AField).DisplayWidth := I;
                  TStringField(AField).Size := I;
                end;
              end;
          end;
      end;
    except
    end;
  DmData.BIgnore := BbIgnore;
end;

procedure UpdateDBEditField(AField: TDBedit; AVitalType: TVitalType = VtUnknown);
begin
  if not DevOptions.BValue(OnAllowDictionaryEdit) then Exit;
  if Engine.CurrentOperatorMode <> RnAdmin then Exit;
  if EditRecord(AField.DataSource.DataSet.Name, AField.Field.FieldName, AVitalType) then
      SetDictionaryRecord(AField.Field);
end;

procedure UpdateColumnField(AField: TField; DsName, DsFName: string; AVitalType: TVitalType = VtUnknown);
begin
  if Engine.CurrentOperatorMode <> RnAdmin then Exit;
  if not DmData.CdsDict.Locate(FnDict_Table + ';' + FnDict_Column, VarArrayOf([DsName, DsFName]), []) then begin
    if MessageDlg('Dictionary does not contain information about' + #13#10 + 'Table <' + DsName + '> Column <' + DsFName
      + '>' + #13#10#13#10 + 'Add new description?', MtConfirmation, [MbYes, MbNo], 0) = MrYes then begin
      AddFieldToDictionary(DsName, AField);
      Engine.SaveDictionaryToDB; // zzzzzzandria 060608
    end
    else Exit;
  end;
  if EditRecord(DsName, DsFName, AVitalType) then SetDictionaryRecord(AField);
end;
/// /////////////////////////////////////////////////////////////////////////////
// populates VLE with dictionary data
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmMDK_DictionaryEdit.Prepare(ATable, AField: string; AVitalType: TVitalType = VtUnknown);
var
  ADS: TClientDataSet;
  I, J: Integer;
begin
  Vle.Strings.Clear;
  ADS := DmData.CdsDict;
  if not ADS.Locate(FnDict_Table + ';' + FnDict_Column, VarArrayOf([ATable, AField]), []) then Exit
  else begin
    J := 0;
    for I := 0 to ADS.FieldCount - 1 do begin
{$IFNDEF AANTEST}
      if ADS.Fields.Fields[I].FieldName = FnDict_ID then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Tag then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_iTag then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Table then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Column then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Type then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Length then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Label then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Format then Continue; // zzzzzzandria 2007-06-29 hide fiels
      if ADS.Fields.Fields[I].FieldName = FnDict_editMask then Continue; // zzzzzzandria 2007-06-29 hide field
      if ADS.Fields.Fields[I].FieldName = FnDict_Width then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Units then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Required then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Overwrite then
      // Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Section then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Domain then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Group then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Min then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Max then Continue;
      if ADS.Fields.Fields[I].FieldName = FnDict_Default then Continue;
      // if aDS.Fields.Fields[i].FieldName = fnDict_Comments then Continue;
{$ENDIF}
      Vle.Strings.Add(ADS.Fields.Fields[I].FieldName + '=' + ADS.Fields.Fields[I].AsString);
      Inc(J);
    end;
    I := J * (Vle.DefaultRowHeight + Vle.GridLineWidth) + Vle.GridLineWidth;
    Height := I + 50;
    Vle.ItemProps[FnDict_Tag].ReadOnly := True; // zzzzzzandria 2007-06-29
    // disable edit of the tag
    // zzzzzzandria 2007-08-23 - to simplify documentation
    // vle.ItemProps[fnDict_Domain].EditStyle := esPickList;
    // vle.ItemProps[fnDict_Domain].PickList.Text := DOMAIN_PICK_LIST;
    Vle.ItemProps[FnDict_Domain].ReadOnly := True;
    Vle.ItemProps[FnDict_Required].EditStyle := EsPickList;
    Vle.ItemProps[FnDict_Required].PickList.Text := TRUE_FALSE;
    Vle.ItemProps[FnDict_Required].ReadOnly := True;
    Vle.ItemProps[FnDict_Overwrite].EditStyle := EsPickList;
    Vle.ItemProps[FnDict_Overwrite].PickList.Text := TRUE_FALSE;
    // zzzzzzandria 2007-08-10
    if AVitalType <> VtUnknown then // Only Vitals are Readonly
    begin
      Vle.Values[FnDict_MaxValue] := FloatToStr(GMVVitalHiRange[AVitalType]);
      Vle.Values[FnDict_MinValue] := FloatToStr(GMVVitalLoRange[AVitalType]);
      // vle.ItemProps[fnDict_MinValue].ReadOnly := True;
      // vle.ItemProps[fnDict_MaxValue].ReadOnly := True;
    end else begin
      if Vle.Values[FnDict_MaxValue] <> '' then Vle.Values[FnDict_MaxValue] := FloatToStr(GetMaxValue);
      if Vle.Values[FnDict_MinValue] <> '' then Vle.Values[FnDict_MinValue] := FloatToStr(GetMinValue);
    end;
    Vle.ItemProps[FnDict_MinValue].ReadOnly := True;
    Vle.ItemProps[FnDict_MaxValue].ReadOnly := True;
    OriginalLabel := ADS.FieldByName(FnDict_Label).AsString;
  end;
end;

procedure TfrmMDK_DictionaryEdit.SetUp(ATable, AField: string; AStrings: TStrings);
var
  ADS: TClientDataSet;
  I: Integer;
  AName: string;
  procedure UpdateField(AName: string);
  var
    S: string;
  begin
{$IFNDEF AANTEST}
    if (AName = FnDict_Table) or // 'TableName';
      (AName = FnDict_Column) or // 'ColumnName';
      (AName = FnDict_Type) { // 'DataType'; } then Exit;
{$ENDIF}
    try
      ADS.Edit;
      S := Vle.Values[AName];
      ADS.FieldByName(AName).AsString := S;
      ADS.Post;
    except
    end;
  end;

begin
  ADS := DmData.CdsDict;
  if ADS.Locate(FnDict_Table + ';' + FnDict_Column, VarArrayOf([ATable, AField]), []) then begin
    for I := 0 to Vle.RowCount - 1 do begin
      AName := Vle.Keys[I];
      if ADS.FieldDefs.Find(AName) <> nil then UpdateField(Vle.Keys[I]);
    end;
  end;
  Engine.SaveDictionaryToDB; // save dictionary in DB
end;

function TfrmMDK_DictionaryEdit.GetDisplayFormat: string;
begin
  Result := Vle.Values[FnDict_Format];
end;

function TfrmMDK_DictionaryEdit.GetEditMask: string;
begin
  Result := Vle.Values[FnDict_editMask];
end;

function TfrmMDK_DictionaryEdit.GetMinValue: Double;
begin
  Result := StrToFloat(Vle.Values[FnDict_MinValue]);
end;

function TfrmMDK_DictionaryEdit.GetMaxValue: Double;
begin
  Result := StrToFloat(Vle.Values[FnDict_MaxValue]);
end;

procedure TfrmMDK_DictionaryEdit.VleDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  SName: string;
begin
  with Vle.Canvas do begin
    if ACol <> 0 then begin
      Brush.Color := ClWindow;
      SName := Vle.Cells[0, ARow];
      if Vle.ItemProps[SName].ReadOnly = True then begin
        Font.Color := ClNavy;
        if Vle.ItemProps[SName].EditStyle <> EsPickList then Brush.Color := Preferences.IValue(OnReadOnlyFieldColor);
      end
      else Font.Color := ClWindowText;
      Font.Style := [];
    end else begin
      Brush.Color := ClBtnFace;
      Font.Style := [FsBold];
    end;
    FillRect(Rect);
    TextOut(Rect.Left + 2, Rect.Top + 2, Vle.Cells[ACol, ARow]);
  end;
end;

procedure TfrmMDK_DictionaryEdit.AcEditDomainExecute(Sender: TObject);
var
  S: string;
begin
  S := Vle.Values[FnDict_Domain];
  EditDomain(S);
end;

procedure TfrmMDK_DictionaryEdit.AcSaveToDBExecute(Sender: TObject);
begin
  Engine.SaveDictionaryToDB;
end;

procedure TfrmMDK_DictionaryEdit.VleSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: string);
begin
  AcEditDomain.Enabled := (Vle.Cells[0, ARow] = FnDict_Domain) and (Trim(Value) <> '');
end;

procedure TfrmMDK_DictionaryEdit.SetDomainEditStatus(ARow, ACol: Integer);
begin
  AcEditDomain.Enabled := (Vle.Cells[0, ARow] = FnDict_Domain) and (Trim(Vle.Cells[1, ARow]) <> '');
end;

procedure TfrmMDK_DictionaryEdit.VleEnter(Sender: TObject);
begin
  SetDomainEditStatus(Vle.Row, Vle.Col);
end;

procedure TfrmMDK_DictionaryEdit.VleGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: string);
begin
  SetDomainEditStatus(ARow, ACol);
end;

procedure TfrmMDK_DictionaryEdit.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then begin
    if (Vle.Values[FnDict_Label] = '') then
      (Vle.Values[FnDict_Label] := ' ');
    ModalResult := MrCancel;
  end;
end;

procedure TfrmMDK_DictionaryEdit.VleMouseDown(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  if (SsCTRL in Shift) and (SsALT in Shift) and (Button = MbRight) and (Engine.CurrentOperatorMode = RnAdmin) then begin
{$IFDEF DICTIONARY_RANGE}
    Vle.ItemProps[FnDict_MinValue].ReadOnly := False;
    Vle.ItemProps[FnDict_MaxValue].ReadOnly := False;
{$ENDIF}
  end;
end;

procedure TfrmMDK_DictionaryEdit.FormCloseQuery(Sender: TObject; var CanClose: Boolean);
begin
  if Sender = BbCancel then Exit;
  if (Vle.Values[FnDict_Label] = '') then begin
    MessageDlg(FnDict_Label + ' field cannot be blank.' + #13 + 'The original label will be restored.', MtWarning,
      [MbOK], 0);
    Vle.Values[FnDict_Label] := OriginalLabel;
    CanClose := False;
  end;
end;

procedure TfrmMDK_DictionaryEdit.BbCancelClick(Sender: TObject);
begin
  if (Vle.Values[FnDict_Label] = '') then
    (Vle.Values[FnDict_Label] := ' ');
  ModalResult := MrCancel;
end;

end.
