unit fMDK_HospitalSelector;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:37a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  This unit contains the Hemodialysis application
  *                     Hospital selection dialog
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Forms/fMDK_HospitalSelector.pas $
  *
  *       $ History: fMDK_HospitalSelector.pas $
}
interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  Buttons, ExtCtrls, StdCtrls, ComCtrls;

type
  TfrmMDK_HospitalSelector = class(TForm)
    GbHospital: TGroupBox;
    Panel1: TPanel;
    TmSearch: TTimer;
    LblLeft: TLabel;
    LvH: TListView;
    BtnOK: TButton;
    Button2: TButton;
    CmbTarget: TComboBox;
    Label1: TLabel;
    procedure TmSearchTimer(Sender: TObject);
    procedure LvHChange(Sender: TObject; Item: TListItem; Change: TItemChange);
    procedure CmbTargetChange(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormActivate(Sender: TObject);
    procedure CmbTargetKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure CmbTargetEnter(Sender: TObject);
    procedure CmbTargetExit(Sender: TObject);
    procedure LvHExit(Sender: TObject);
    procedure LvHEnter(Sender: TObject);
    procedure LvHKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure LvHDblClick(Sender: TObject);
  private
    ICycle: Integer;
    FID, FName: string;
    procedure Search;
  public
    property HospitalID: string read FID;
    property HospitalName: string read FName;
  end;

var
  FrmMDK_HospitalSelector: TfrmMDK_HospitalSelector;

function SelectLocation(ADFN: string; var AnID: string; var AName: string): Boolean;

implementation

uses
  UMDK_Engine, UROR_CommonUtils, System.UITypes;

{$R *.DFM}

const
  ClTabIn  = ClWindow;
  ClTabOut = ClInfoBk;

function SelectLocation(ADFN: string; var AnID: string; var AName: string): Boolean;
begin
  Result := False;
  if not Assigned(FrmMDK_HospitalSelector) then
      Application.CreateForm(TfrmMDK_HospitalSelector, FrmMDK_HospitalSelector);
  if FrmMDK_HospitalSelector = nil then Exit;
{$IFDEF AANTEST}
  FrmMDK_HospitalSelector.LblLeft.Visible := True;
{$ELSE}
  FrmMDK_HospitalSelector.LblLeft.Visible := False;
{$ENDIF}
  if FrmMDK_HospitalSelector.ShowModal = MrOK then begin
    FrmMDK_HospitalSelector.TmSearch.Enabled := False;
    AName := FrmMDK_HospitalSelector.HospitalName;
    AnID := FrmMDK_HospitalSelector.HospitalID;
    Result := True;
  end;
end;

procedure TfrmMDK_HospitalSelector.TmSearchTimer(Sender: TObject);
begin
  if ICycle > 1 then begin
    ICycle := ICycle - 1;
    LblLeft.Caption := IntToStr(ICycle);
  end else
    Search;
end;

procedure TfrmMDK_HospitalSelector.Search;
var
  TmpList: TStringList;
  I: Integer;
  LI: TListItem;
begin
  if (CmbTarget.Items.IndexOf(CmbTarget.Text) = -1) and (CmbTarget.Text <> '') then CmbTarget.Items.Add(CmbTarget.Text);
  TmSearch.Enabled := False;
  LblLeft.Caption := '';
  BtnOK.Enabled := False;
  try
    TmpList := Engine.FRenalController.GetLookupEntries('44', CmbTarget.Text);
    if TmpList.Count < 1 then begin
      MessageDlg('No entries found matching ''' + CmbTarget.Text + '''.', MtInformation, [Mbok], 0);
      CmbTarget.SetFocus;
      CmbTarget.SelectAll;
      FreeAndNil(TmpList);
      Exit;
    end;
    if Piece(TmpList[0], '^', 1) = '-1' then begin
      MessageDlg(Piece(TmpList[0], '^', 2), MtError, [Mbok], 0);
      CmbTarget.SelectAll;
      CmbTarget.SetFocus;
    end else if Piece(TmpList[0], '^', 1) = '0' then begin
      MessageDlg('No entries found matching ''' + CmbTarget.Text + '''.', MtInformation, [Mbok], 0);
      CmbTarget.SetFocus;
      CmbTarget.SelectAll;
    end else begin
      TmpList.Delete(0); { Contains the header count record }
      LvH.Items.Clear;
      for I := 0 to TmpList.Count - 1 do begin
        LI := LvH.Items.Add;
        LI.Caption := Piece(TmpList[I], '^', 2);
        LI.SubItems.Add(Piece(Piece(TmpList[I], '^', 1), ';', 2));
      end;
      LvH.SetFocus;
      LvH.ItemFocused := LvH.Items[0];
      LvH.ItemIndex := 0;
    end;
  finally
    FreeAndNil(TmpList);
  end;
end;

procedure TfrmMDK_HospitalSelector.LvHChange(Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  if not Assigned(LvH.Selected) then Exit;
  try
    FID := LvH.Selected.SubItems[0];
    FName := LvH.Selected.Caption;
    BtnOK.Enabled := True;
    LblLeft.Caption := FName + '(' + FID + ')';
  except
  end;
end;

procedure TfrmMDK_HospitalSelector.CmbTargetChange(Sender: TObject);
begin
  if CmbTarget.Text = '' then Exit;
  TmSearch.Enabled := True;
  ICycle := 3;
  LblLeft.Caption := IntToStr(ICycle);
  TmSearch.Interval := 1000;
end;

procedure TfrmMDK_HospitalSelector.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = Char(VK_ESCAPE) then ModalResult := MrCancel;
end;

procedure TfrmMDK_HospitalSelector.FormActivate(Sender: TObject);
begin
  try
    CmbTarget.SetFocus;
  except
  end;
end;

procedure TfrmMDK_HospitalSelector.CmbTargetKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_Return then Search;
end;

procedure TfrmMDK_HospitalSelector.CmbTargetEnter(Sender: TObject);
begin
  CmbTarget.Color := ClTabIn;
end;

procedure TfrmMDK_HospitalSelector.CmbTargetExit(Sender: TObject);
begin
  CmbTarget.Color := ClTabOut;
end;

procedure TfrmMDK_HospitalSelector.LvHExit(Sender: TObject);
begin
  LvH.Color := ClTabOut;
end;

procedure TfrmMDK_HospitalSelector.LvHEnter(Sender: TObject);
begin
  LvH.Color := ClTabIn;
end;

procedure TfrmMDK_HospitalSelector.LvHKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  case Key of
    VK_Return: if (LvH.ItemIndex > -1) then begin
        Key := 0;
        FID := LvH.Selected.SubItems[0];
        FName := LvH.Selected.Caption;
        ModalResult := MrOK;
      end;
  end;
end;

procedure TfrmMDK_HospitalSelector.LvHDblClick(Sender: TObject);
begin
  FID := LvH.Selected.SubItems[0];
  FName := LvH.Selected.Caption;
  ModalResult := MrOK;
end;

end.
