unit fMDK_Medication;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/23/08 12:26p $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Medication record dialog
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_Medication.pas $
  *
  *       $ History: fMDK_Medication.pas $
  *
}
interface

uses
  Windows, SysUtils, Classes, Graphics, Forms, Controls, StdCtrls,
  Buttons, ExtCtrls, DBCtrls, Mask, DBClient, ComCtrls;

type
  TdlgMEdit = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    GrbMedication: TGroupBox;
    Label9: TLabel;
    EdDate: TEdit;
    Label1: TLabel;
    EdTime: TEdit;
    Label2: TLabel;
    CmbMedications: TComboBox;
    Label3: TLabel;
    EdDose: TEdit;
    Label4: TLabel;
    CmbUnits: TComboBox;
    Label5: TLabel;
    CmbRouts: TComboBox;
    Label6: TLabel;
    EdGivenBy: TEdit;
    Label8: TLabel;
    EdReason: TEdit;
    Label7: TLabel;
    MmOutcome: TMemo;
    GrbStatus: TGroupBox;
    Label10: TLabel;
    CmbStatusName: TComboBox;
    Label11: TLabel;
    EdStatusDate: TEdit;
    Label12: TLabel;
    EdStatusBy: TEdit;
    PnlReplacementDetails: TPanel;
    PnlReplacement: TPanel;
    GroupBox1: TGroupBox;
    Label13: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    Label19: TLabel;
    Label20: TLabel;
    Label21: TLabel;
    EdReasonR: TEdit;
    EdGivenByR: TEdit;
    CmbRoutsR: TComboBox;
    CmbUnitsR: TComboBox;
    EdDoseR: TEdit;
    CmbMedicationsR: TComboBox;
    EdTimeR: TEdit;
    EdDateR: TEdit;
    Panel5: TPanel;
    OKbtn: TBitBtn;
    CancelBtn: TBitBtn;
    GrbStatusR: TGroupBox;
    Label22: TLabel;
    Label23: TLabel;
    Label24: TLabel;
    CmbStatusNameR: TComboBox;
    EdStatusDateR: TEdit;
    EdStatusByR: TEdit;
    PnlSource: TPanel;
    Splitter1: TSplitter;
    EdUpdatedBy: TEdit;
    CkbWrap: TCheckBox;
    MmOutcomeR: TRichEdit;
    LblCharCount: TLabel;
    Label25: TLabel;
    BitBtn3: TBitBtn;
    procedure FormCreate(Sender: TObject);
    procedure DbmOutcomesChange(Sender: TObject);
    procedure DbeGivenByChange(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure FormDestroy(Sender: TObject);
    procedure CkbWrapClick(Sender: TObject);
    procedure MmOutcomeRKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure MmOutcomeRChange(Sender: TObject);
  private
    FLocked: Integer;
    FMode: string;
    FIgnore: Boolean;
    FStatusList: TStringList;
    function GetValidStatus: string;
    procedure SetValidStatus(AValue: string);
    procedure ActivateDLG;
    procedure SetComboByName(ACombo: TComboBox; AName: string);
  public
    property ValidStatus: string read GetValidStatus write SetValidStatus;
    procedure ClearStringValues;
    procedure SetStringValues;
    procedure SetStatusValues;
    procedure UpdateValues;
    procedure GetStringValues;
    function VerifyStringValues: string;
    procedure SetStatus(AReadOnly: Boolean);
  end;

var
  DlgMEdit: TdlgMEdit;

procedure EditMedicationStatus;
function AddMedication: string;

implementation

uses
  DMDK_Data, UMDK_MedList, UMDK_Engine, UMDK_CDSCommon, Dialogs, System.UITypes,
  UMDK_Common, DB, FMDK_Flowsheet, UMDK_Options, UMDK_Flowsheet, UMDK_ReportEngine, UMDK_Utils,
  UMDK_Dictionary;
{$R *.dfm}

procedure EditMedicationStatus;
var
  BStatus: Boolean;
  Dt: TDateTime;
begin
  if not Assigned(DlgMEdit) then Application.CreateForm(TdlgMEdit, DlgMEdit);
  with DlgMEdit do begin
    ActivateDLG;
    Caption := 'Edit Medication';
    OKbtn.Caption := 'Update';
    FMode := 'UPDATE';
    GetStringValues;
    EdStatusBy.Enabled := False;
    EdStatusDate.Enabled := False;
    SetStatus(True);
    MmOutcome.Enabled := True;
    MmOutcome.ReadOnly := True;
    MmOutcome.Font.Color := ClGrayText;
    GrbStatus.Visible := False;
    GrbStatusR.Visible := False;
    PnlReplacementDetails.Visible := True;
    Width := 800;
    PnlReplacementDetails.Width := Width div 2;
    PnlSource.Color := ClSilver;
    Dt := Engine.GetCurrentServerDateTime;
    EdStatusDateR.Text := FormatDateTime('mm/dd/yyyy hh:nn', Dt);
    EdStatusByR.Text := Engine.CurrentOperatorID;
    EdUpdatedBy.Text := Engine.CurrentOperatorID;
    EdStatusByR.Enabled := False;
    EdStatusDateR.Enabled := False;
    SetComboByName(CmbStatusNameR, 'Valid');
    MmOutcomeR.Text := MmOutcome.Text + #13#10 + ' [' + EdStatusDateR.Text + '  ' + EdUpdatedBy.Text + '] ';
    FLocked := Length(MmOutcomeR.Text);
    ActiveControl := EdDateR;
    if ShowModal = MrOK then
      try
        BStatus := FrmFlow.TmrRefresh.Enabled;
        FrmFlow.TmrRefresh.Enabled := False;
        UpdateValues;
        FrmFlow.TmrRefresh.Enabled := BStatus;
      except
      end;
  end;
end;

function AddMedication: string;
var
  BStatus: Boolean;
  Dt: TDateTime;
begin
  Result := 'OK';
  if not Assigned(DlgMEdit) then Application.CreateForm(TdlgMEdit, DlgMEdit);
  with DlgMEdit do begin
    ActivateDLG;
    Dt := Engine.GetCurrentServerDateTime;
    Caption := 'Add Medication';
    OKbtn.Caption := 'Add';
    Width := 400;
    PnlReplacementDetails.Visible := False;
    PnlSource.Color := ClBtnFace;
    FMode := 'ADD';
    ClearStringValues;
    EdDate.Text := FormatDateTime('mm/dd/yyyy', Dt);
    EdTime.Text := FormatDateTime('hh:nn', Dt);
    EdGivenBy.Text := Engine.CurrentOperatorID;
    MmOutcome.ReadOnly := False;
    MmOutcome.Font.Color := ClWindowText;
    GrbMedication.Enabled := True;
    SetStatus(False);
    EdStatusBy.Enabled := True;
    EdStatusDate.Enabled := True;
    ActiveControl := CmbMedications;
    GrbStatus.Visible := False;
    if ShowModal = MrCancel then Result := 'CANCEL'
    else
      try
        BStatus := FrmFlow.TmrRefresh.Enabled;
        FrmFlow.TmrRefresh.Enabled := False;
        with DmData.CdsMedList do begin
          Insert;
          // FieldByName(fnML_Status).AsString := fsValid; // 2007-08-01
          FieldByName(FnML_Status).AsString := 'NEW';
          FieldByName(FnML_StatusDate).AsDateTime := Now;
          FieldByName(FnML_StatusBy).AsString := Engine.CurrentOperatorID;
          SetStringValues;
          Post;
        end;
        FrmFlow.TmrRefresh.Enabled := BStatus;
      except
      end;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TdlgMEdit.FormCreate(Sender: TObject);
begin
  FIgnore := False;
  FStatusList := TStringList.Create;
end;

procedure TdlgMEdit.SetStatus(AReadOnly: Boolean);
var
  I: Integer;
begin
  for I := 0 to GrbMedication.ControlCount - 1 do begin
    if GrbMedication.Controls[I] is TEdit then begin
      TEdit(GrbMedication.Controls[I]).Enabled := not AReadOnly;
    end else if GrbMedication.Controls[I] is TComboBox then begin
      TComboBox(GrbMedication.Controls[I]).Enabled := not AReadOnly;
    end else if GrbMedication.Controls[I] is TMemo then begin
      TMemo(GrbMedication.Controls[I]).Enabled := not AReadOnly;
    end
  end;
end;

procedure TdlgMEdit.ActivateDLG;
var
  I: Integer;
begin
  FStatusList.Text := Engine.GetSystemOptionByName(OnMedicationStatuses);
  CmbStatusName.Items.Clear;
  CmbStatusNameR.Items.Clear;
  for I := 0 to FStatusList.Count - 1 do begin
    CmbStatusName.Items.Add(FStatusList.Values[FStatusList.Names[I]]);
    CmbStatusNameR.Items.Add(FStatusList.Values[FStatusList.Names[I]]);
  end;
  AssignList(CmbMedications.Items, Engine.GetSystemOptionByName(OnMedications));
  AssignList(CmbRouts.Items, Engine.GetSystemOptionByName(OnMedicationRoutes));
  AssignList(CmbUnits.Items, Engine.GetSystemOptionByName(OnMedicationUnits));
  AssignList(CmbMedicationsR.Items, Engine.GetSystemOptionByName(OnMedications));
  AssignList(CmbRoutsR.Items, Engine.GetSystemOptionByName(OnMedicationRoutes));
  AssignList(CmbUnitsR.Items, Engine.GetSystemOptionByName(OnMedicationUnits));
end;

procedure TdlgMEdit.SetValidStatus(AValue: string);
var
  S: string;
  I: Integer;
begin
  CmbStatusName.ItemIndex := -1;
  S := FStatusList.Values[AValue];
  for I := 0 to CmbStatusName.Items.Count - 1 do begin
    if CmbStatusName.Items[I] <> S then Continue;
    CmbStatusName.ItemIndex := I;
    Break;
  end;
end;

function TdlgMEdit.GetValidStatus: string;
var
  I: Integer;
  S: string;
begin
  Result := '';
  for I := 0 to FStatusList.Count - 1 do begin
    S := FStatusList.Names[I];
    if FStatusList.Values[S] <> CmbStatusName.Text then Continue;
    Result := FStatusList.Names[I];
    Break;
  end;
end;

procedure TdlgMEdit.ClearStringValues;
begin
  with DmData.CdsMedList do begin
    EdStatusBy.Text := '';
    EdStatusDate.Text := '';
    CmbStatusName.ItemIndex := -1;
    EdDate.Text := '';
    EdTime.Text := '';
    EdDose.Text := '';
    EdReason.Text := '';
    EdGivenBy.Text := '';
    try
      CmbMedications.Text := '';
    except
    end;
    CmbMedications.ItemIndex := -1;
    CmbRouts.ItemIndex := -1;
    CmbUnits.ItemIndex := -1;
    MmOutcome.Text := '';
    EdStatusByR.Text := '';
    EdStatusDateR.Text := '';
    CmbStatusNameR.ItemIndex := -1;
    EdDateR.Text := '';
    EdTimeR.Text := '';
    EdDoseR.Text := '';
    EdReasonR.Text := '';
    EdGivenByR.Text := '';
    CmbMedicationsR.ItemIndex := -1;
    CmbRoutsR.ItemIndex := -1;
    CmbUnitsR.ItemIndex := -1;
    MmOutcomeR.Text := '';
  end;
end;

procedure TdlgMEdit.SetComboByName(ACombo: TComboBox; AName: string);
var
  I: Integer;
begin
  ACombo.ItemIndex := -1;
  for I := 0 to ACombo.Items.Count - 1 do begin
    if ACombo.Items[I] <> AName then Continue;
    ACombo.ItemIndex := I;
    Break;
  end;
  if I = ACombo.Items.Count then begin
    if ACombo.Style = CsDropDown then ACombo.Text := AName;
  end;
end;

procedure TdlgMEdit.GetStringValues;
var
  DValue: TDateTime;
  SValue: string;
begin
  with DmData.CdsMedList do begin
    EdStatusBy.Text := FieldByName(FnML_StatusBy).AsString;
    EdStatusDate.Text := FieldByName(FnML_StatusDate).AsString;
    ValidStatus := FieldByName(FnML_Status).AsString;
    EdDate.Text := FieldByName(FnML_Date).AsString;
    EdTime.Text := FieldByName(FnML_Time).AsString;
    SetComboByName(CmbMedications, FieldByName(FnML_Medication).AsString);
    EdDose.Text := FieldByName(FnML_Dose).AsString;
    EdReason.Text := FieldByName(FnML_Reason).AsString;
    EdGivenBy.Text := FieldByName(FnML_GivenBy).AsString;
    SetComboByName(CmbRouts, FieldByName(FnML_Rout).AsString);
    SetComboByName(CmbUnits, FieldByName(FnML_Units).AsString);
    MmOutcome.Text := FieldByName(FnML_Outcome).AsString;
    EdDateR.Text := FieldByName(FnML_Date).AsString;
    DValue := FieldByName(FnML_Time).AsDateTime;
    DValue := DValue + 1 / 24 / 60 / 60;
    SValue := DateTimeToStr(DValue);
    EdTimeR.Text := SValue;
    SetComboByName(CmbMedicationsR, FieldByName(FnML_Medication).AsString);
    EdDoseR.Text := FieldByName(FnML_Dose).AsString;
    EdReasonR.Text := FieldByName(FnML_Reason).AsString;
    EdGivenByR.Text := FieldByName(FnML_GivenBy).AsString;
    SetComboByName(CmbRoutsR, FieldByName(FnML_Rout).AsString);
    SetComboByName(CmbUnitsR, FieldByName(FnML_Units).AsString);
  end;
end;

procedure TdlgMEdit.SetStatusValues;
begin
  with DmData.CdsMedList do begin
    FieldByName(FnML_StatusDate).AsDateTime := Engine.GetCurrentServerDateTime;
    FieldByName(FnML_StatusBy).AsString := EdStatusBy.Text;
    FieldByName(FnML_Status).AsString := ValidStatus;
  end;
end;

procedure TdlgMEdit.UpdateValues;
var
  SInitialTime, SOutcome, SID: string;
  function FieldsChanged: Boolean;
  begin
    Result := (EdDate.Text <> EdDateR.Text) or (EdTime.Text <> EdTimeR.Text) or
      (CmbMedications.Text <> CmbMedicationsR.Text) or (EdDose.Text <> EdDoseR.Text) or
      (CmbUnits.Text <> CmbUnitsR.Text) or (CmbRouts.Text <> CmbRoutsR.Text) or (EdGivenBy.Text <> EdGivenByR.Text) or
      (EdReason.Text <> EdReasonR.Text);
  end;
  procedure UpdateOutcome;
  begin
    if Length(MmOutcomeR.Text) = FLocked then Exit;
    with DmData.CdsMedList do
      try
        if State <> DsEdit then Edit;
        FieldByName(FnML_Outcome).AsString := MmOutcomeR.Text;
      except
        on E: Exception do ShowMessage(E.Message);
      end;
  end;
  procedure SetReplacementValues;
  begin
    with DmData.CdsMedList do begin
      FieldByName(FnML_Date).AsDateTime := StrToDateTime(EdDateR.Text);
      FieldByName(FnML_Time).AsDateTime := StrToDateTime(EdTimeR.Text);
      FieldByName(FnML_Medication).AsString := CmbMedicationsR.Text;
      FieldByName(FnML_Rout).AsString := CmbRoutsR.Text;
      FieldByName(FnML_Dose).AsString := EdDoseR.Text;
      FieldByName(FnML_Reason).AsString := EdReasonR.Text;
      FieldByName(FnML_GivenBy).AsString := EdGivenByR.Text;
      FieldByName(FnML_Units).AsString := CmbUnitsR.Text;
    end;
  end;

begin
  // was it only OUTCOME that had been changed?
  if (Trim(MmOutcomeR.Text) <> '') and not FieldsChanged then begin
    // yes
    // - update OUTCOME field only
    UpdateOutcome;
    DmData.CdsMedList.Post;
  end else begin
    // no
    with DmData.CdsMedList do begin
      Disablecontrols;
      // - change status of the current record to REPLACED
      SOutcome := FieldByName(FnML_Outcome).AsString;
      if State <> DsEdit then Edit;
      SID := FieldByName(FnML_ID).AsString;
      if SID = '' then begin
        SID := DmData.NextID;
        FieldByName(FnML_ID).AsString := SID;
      end;
      FieldByName(FnML_StatusDate).AsDateTime := Engine.GetCurrentServerDateTime;
      FieldByName(FnML_StatusBy).AsString := EdStatusByR.Text;
      FieldByName(FnML_Status).AsString := 'REPLACED';
      SInitialTime := FieldByName(FnML_Time).AsString;
      Post;
      // - create new record
      try
        Insert;
        // - link to  original record
        FieldByName(FnML_ParentID).AsString := SID;
        // - set status to VALID
        FieldByName(FnML_StatusDate).AsDateTime := Engine.GetCurrentServerDateTime;
        FieldByName(FnML_StatusBy).AsString := EdStatusByR.Text;
        FieldByName(FnML_Status).AsString := 'NEW';
        // - populate fields
        SetReplacementValues;
        UpdateOutcome;
        // - save data
        Post;
      except
        on E: Exception do ShowMessage(E.Message);
      end;
      EnableControls;
    end;
  end;
end;

procedure TdlgMEdit.SetStringValues;
begin
  with DmData.CdsMedList do begin
    FieldByName(FnML_Date).AsDateTime := StrToDateTime(EdDate.Text);
    FieldByName(FnML_Time).AsDateTime := StrToDateTime(EdTime.Text);
    FieldByName(FnML_Medication).AsString := CmbMedications.Text;
    FieldByName(FnML_Rout).AsString := CmbRouts.Text;
    FieldByName(FnML_Dose).AsString := EdDose.Text;
    FieldByName(FnML_Reason).AsString := EdReason.Text;
    FieldByName(FnML_GivenBy).AsString := EdGivenBy.Text;
    FieldByName(FnML_Units).AsString := CmbUnits.Text;
    FieldByName(FnML_Outcome).AsString := MmOutcome.Text;
  end;
end;

function TdlgMEdit.VerifyStringValues: string;
var
  S: string;
  Ac: TWinControl;

  function TestLength(AString, AMessage: string; AControl: TWinControl; ALimit: Integer): string;
  begin
    if Length(AString) > ALimit then begin
      Result := AMessage + ' value is longer then ' + IntToStr(ALimit) + ' characters' + #13#10;
      if not Assigned(Ac) then Ac := AControl;
    end
    else Result := '';
  end;

  function TestFloat(AString, AMessage: string; AControl: TWinControl): string;
  begin
    Result := '';
    try
      StrToFloat(AString);
    except
      begin
        Result := AMessage + ' "' + AString + '"' + #13#10;
        if not Assigned(Ac) then Ac := AControl;
      end;
    end;
  end;

  function TestDateTime(AString, AMessage: string; AControl: TWinControl): string;
  begin
    Result := '';
    if not IsValidDateString(AString) then begin
      if not Assigned(Ac) then Ac := AControl;
      Result := AMessage + ' "' + AString + '"' + #13#10;
    end;
  end;

  function TestBlank(AString, AMessage: string; AControl: TWinControl; AColumn: string = ''): string;
  var
    IRequired: Integer;
  begin
    Result := '';
    IRequired := 1;
    if AColumn <> '' then IRequired := GetDictionaryRequired('cdsMedList', AColumn);
    if IRequired < 0 then Exit;
    if Trim(AString) = '' then begin
      Result := AMessage + #13#10;
      if not Assigned(Ac) then Ac := AControl;
    end
    else Result := '';
  end;

begin
  S := '';
  Ac := nil;
  if FMode = 'ADD' then begin
    S := S + TestDateTime(EdDate.Text, 'Invalid Date value of ', EdDate);
    S := S + TestDateTime(EdTime.Text, 'Invalid Time value of ', EdTime);
    S := S + TestFloat(EdDose.Text, 'Invalid Dose value of ', EdDose);
    S := S + TestBlank(CmbMedications.Text, 'Field "Medication" should be populated',
      CmbMedications { ,fnML_Medication } );
    S := S + TestBlank(CmbUnits.Text, 'Field "Units" should be populated', CmbUnits { ,fnML_Units } );
    S := S + TestBlank(CmbRouts.Text, 'Field "Route" should be populated', CmbRouts { ,fnML_Rout } );
    S := S + TestBlank(EdGivenBy.Text, 'Field "Given By" should be populated', EdGivenBy { , fnML_GivenBy } );
    S := S + TestBlank(EdReason.Text, 'Field "Reason" should be populated', EdReason, FnML_Reason);
    S := S + TestLength(MmOutcome.Text, 'Outcome', MmOutcome, 200);
  end else begin
    S := S + TestDateTime(EdDateR.Text, 'Invalid Date value of ', EdDateR);
    S := S + TestDateTime(EdTimeR.Text, 'Invalid Time value of ', EdTimeR);
    S := S + TestFloat(EdDoseR.Text, 'Invalid Dose value of ', EdDoseR);
    S := S + TestBlank(CmbMedicationsR.Text, 'Field "Medication" should be populated',
      CmbMedicationsR { , fnML_Medication } );
    S := S + TestBlank(CmbUnitsR.Text, 'Field "Units" should be populated', CmbUnitsR { , fnML_Units } );
    S := S + TestBlank(CmbRoutsR.Text, 'Field "Route" should be populated', CmbRoutsR { , fnML_Rout } );
    S := S + TestBlank(EdGivenByR.Text, 'Field "Given By" should be populated', EdGivenByR { , fnML_GivenBy } );
    S := S + TestBlank(EdReasonR.Text, 'Field "Reason" should be populated', EdReasonR, FnML_Reason);
    S := S + TestLength(MmOutcomeR.Text, 'Outcome', MmOutcomeR, 200);
  end;
  Result := S;
  if Assigned(Ac) then
    try
      ActiveControl := Ac;
    except
    end;
end;

procedure TdlgMEdit.DbmOutcomesChange(Sender: TObject);
begin
  if FIgnore then Exit;
  FIgnore := True;
  UpdateMemo(Sender, Engine.Signature);
  FIgnore := False;
end;

procedure TdlgMEdit.DbeGivenByChange(Sender: TObject);
begin
  if FIgnore then Exit;
  FIgnore := True;
  UpdateEdit(Sender, Engine.CurrentOperatorID);
  FIgnore := False;
end;

procedure TdlgMEdit.FormClose(Sender: TObject; var Action: TCloseAction);
var
  SMsg: string;
begin
  if (ModalResult = MrOK) then begin
    SMsg := VerifyStringValues;
    if SMsg <> '' then begin
      MessageDlg('Update data and try again' + #13#10#13#10 + SMsg, MtWarning, [MbOK], 0);
      Action := CaNone;
      Exit;
    end
  end;
end;

procedure TdlgMEdit.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then ModalResult := MrCancel;
end;

procedure TdlgMEdit.FormDestroy(Sender: TObject);
begin
  FStatusList.Free;
  inherited;
end;

procedure TdlgMEdit.CkbWrapClick(Sender: TObject);
begin
  MmOutcome.WordWrap := CkbWrap.Checked;
  MmOutcomeR.WordWrap := CkbWrap.Checked;
  if CkbWrap.Checked then begin
    MmOutcome.ScrollBars := SsVertical;
    MmOutcomeR.ScrollBars := SsVertical;
  end else begin
    MmOutcome.ScrollBars := SsBoth;
    MmOutcomeR.ScrollBars := SsBoth;
  end;
end;

procedure TdlgMEdit.MmOutcomeRKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  MmOutcomeR.ReadOnly := MmOutcomeR.SelStart < FLocked;
  if (MmOutcomeR.SelStart = FLocked) and (Key = VK_BACK) then Key := 0
end;

procedure TdlgMEdit.MmOutcomeRChange(Sender: TObject);
begin
  LblCharCount.Caption := IntToStr(Length(MmOutcomeR.Text));
  if Length(MmOutcomeR.Text) > 1000 then LblCharCount.Font.Color := ClRed
  else LblCharCount.Font.Color := ClWindowText;
end;

end.
