unit fMDK_Options;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/23/08 3:29p $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     options definition form
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Forms/fMDK_Options.pas $
  *
  * $ History: fMDK_Options.pas $
  *
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ActnList, Grids, ValEdit, Buttons, ExtCtrls, UMDK_Options, ImgList, StdCtrls, ComCtrls, AppEvnts,
  UMDK_Common, System.Actions;

type
  TfrmMDK_Options = class(TForm)
    PnlOptionTools: TPanel;
    PnlOptionActions: TPanel;
    Panel1: TPanel;
    VleOptions: TValueListEditor;
    ActionList1: TActionList;
    AcOptionAdd: TAction;
    AcOptionDelete: TAction;
    AcOptionLoad: TAction;
    AcOptionSave: TAction;
    AcSaveToDB: TAction;
    ImageList1: TImageList;
    SbSaveToFile: TBitBtn;
    SbLoadFromFile: TBitBtn;
    SbDelete: TBitBtn;
    SbAdd: TBitBtn;
    SbSaveToDB: TBitBtn;
    ColorDialog1: TColorDialog;
    AcSetDefaults: TAction;
    AcAssign: TAction;
    PnlOptionsAdmin: TPanel;
    Panel5: TPanel;
    BbtnAdd: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    BitBtn5: TBitBtn;
    PnlUserPreferencesAdmin: TPanel;
    PnlUserAdminButtons: TPanel;
    BitBtn8: TBitBtn;
    BitBtn9: TBitBtn;
    BitBtn10: TBitBtn;
    BitBtn11: TBitBtn;
    PnlSystemPreferencesAdmin: TPanel;
    Panel11: TPanel;
    BitBtn15: TBitBtn;
    BitBtn16: TBitBtn;
    BitBtn17: TBitBtn;
    BitBtn18: TBitBtn;
    BitBtn6: TBitBtn;
    PnlOptionsUser: TPanel;
    Panel8: TPanel;
    BitBtn14: TBitBtn;
    BitBtn19: TBitBtn;
    BitBtn20: TBitBtn;
    PnlUserPreferencesUser: TPanel;
    Panel12: TPanel;
    BitBtn21: TBitBtn;
    BitBtn23: TBitBtn;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    BitBtn24: TBitBtn;
    Label7: TLabel;
    BbDefaults: TBitBtn;
    AcDefaults: TAction;
    BtnDefaults: TBitBtn;
    AcCleanup: TAction;
    PnlComments: TPanel;
    MmComments: TMemo;
    DlgOpen: TOpenDialog;
    DlgSave: TSaveDialog;
    procedure AcOptionAddExecute(Sender: TObject);
    procedure AcOptionLoadExecute(Sender: TObject);
    procedure AcOptionSaveExecute(Sender: TObject);
    procedure AcOptionDeleteExecute(Sender: TObject);
    procedure VleOptionsGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: string);
    procedure AcSaveToDBExecute(Sender: TObject);
    procedure VleOptionsExit(Sender: TObject);
    procedure VleOptionsDblClick(Sender: TObject);
    procedure VleOptionsMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
    procedure VleOptionsDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure AcSetDefaultsExecute(Sender: TObject);
    procedure VleOptionsSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: string);
    procedure AcAssignExecute(Sender: TObject);
    procedure AcDefaultsExecute(Sender: TObject);
    procedure AcCleanupExecute(Sender: TObject);
    procedure VleOptionsSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
    procedure VleOptionsEditButtonClick(Sender: TObject);
    procedure VleOptionsValidate(Sender: TObject; ACol, ARow: Integer; const KeyName, KeyValue: string);
    procedure VleOptionsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure VleOptionsEnter(Sender: TObject);
  private
    FIgnore: Boolean;
    FUpdated: Boolean;
    FLastName: string;
    FOptionsType: string;
    FReadOnly: Boolean;
    FOwnerTreeNodes: TTreeNodes;
    FOwnerNode: TTreeNode;
    FOptionName: string;
    procedure SetLastName(AName: string);
    procedure SetOptionsType(AName: string);
    procedure SaveOptions;
    procedure LoadOptions;
    procedure SetReadOnly(ARO: Boolean);
    procedure RefreshOwnerNode;
  public
    property OwnerTreeNode: TTreeNodes read FOwnerTreeNodes write FOwnerTreeNodes;
    property OwnerNode: TTreeNode read FOwnerNode write FOwnerNode;
    property OptionsType: string read FOptionsType write SetOptionsType;
    property LastName: string read FLastName write SetLastName;
    property Ignore: Boolean read FIgnore write FIgnore;
    property ROnly: Boolean read FReadOnly write SetReadOnly;
    property ValueUpdated: Boolean read FUpdated write FUpdated;
    procedure SaveOptionList;
    procedure SetRole(ARole: string);
    procedure SetValueList(AValue: string);
    function AllowChange(var Reason: string): Boolean;
    function CheckValue(AnOption: string; var Reason: string): Boolean;
    procedure SetComments(ARow: Integer);
    function CheckValues: string;
  end;

var
  FrmMDK_Options: TfrmMDK_Options;

function NewOptionForm(ALastName: string; OType: string = OtSystem): TfrmMDK_Options;
procedure EditDomain(ADomainName: string);

implementation

uses
  UMDK_Engine, FMDK_OptionName, System.UITypes,
  FMDK_HospitalSelector, FMDK_SelectProvider, FMDK_AddOptions, FMDK_Select,
  UMDK_Utils, FMDK_ListDLG;

{$R *.dfm}

function NewOptionForm(ALastName: string; OType: string = OtSystem): TfrmMDK_Options;
begin
  Application.CreateForm(TfrmMDK_Options, Result);
  Result.FOptionsType := OType;
  Result.LastName := ALastName;
  Result.Ignore := False;
end;

procedure EditDomain(ADomainName: string);
var
  SOld: string;
begin
  if ADomainName = '' then Exit;
  with NewOptionForm(ADomainName) do begin
    BorderStyle := BsSizeable;
    Caption := 'Edit Domain <' + ADomainName + '>';
    SOld := VleOptions.Strings.Text;
    KeyPreview := True;
    SetRole(RnAdmin);
    ShowModal;
    if SOld <> VleOptions.Strings.Text then AcSaveToDB.Execute;
    Free;
  end;
end;

function SelectDate(AValue: string): Boolean;
begin
  Result := True;
end;

(* ============================================================================ *)

procedure TfrmMDK_Options.AcOptionAddExecute(Sender: TObject);
var
  BIgnore: Boolean;
  Opt: TUserOptions;
  SName, SValue: string;
  I: Integer;
begin
  if LastName = OnAdministrators then begin
    SName := SelectProvider('', Engine.FRenalController, 2);
    if SName = '' then Exit
    else begin
      SValue := Piece(SName, '^', 1);
      SName := Piece(SName, '^', 2);
    end;
  end else if Pos(OnTIUNoteTitles, LastName) <> 0 then begin
    SName := SelectProvider('', Engine.FRenalController, 4);
    if SName = '' then Exit
    else begin
      SValue := Piece(SName, '^', 1);
      SName := Piece(SName, '^', 2);
      // 2007-07-27 zzzzzzandria          sName := '';
    end;
  end else if Pos('User Options', LastName) <> 0 then begin
    AddOptions;
    VleOptions.Strings.Assign(UserOptions.Values);
    Exit;
  end else if (EditOption('?', '') <> MrCancel) then begin
    SName := DlgOption.EdName.Text;
    SValue := DlgOption.EdValue.Text;
  end
  else Exit;
  begin
    if SName = '' then SName := IntToStr(VleOptions.Strings.Count);
    Opt := nil;
    if (LastName = OnSysOptions) then Opt := SysOptions;
    if (LastName = OnUserOptions) then Opt := UserOptions;
    if ((Opt <> nil) and (Opt.SValue(SName) = YoUnKnown)) then begin
      Opt.AddNameValue(SName, SValue);
      VleOptions.Strings.Add(SName + '=' + SValue);
      ValueUpdated := True;
    end;
    I := 0;
    while I < VleOptions.Strings.Count do begin
      try
        if Pos(SName + '=' + SValue, VleOptions.Strings[I]) = 1 then begin
          MessageDLG('Record:' + #13 + SName + ' = ' + SValue + #13 + 'Already included in the list', MtInformation,
            [MbOK], 0);
          Break;
        end;
      except
      end;
      Inc(I);
    end;
    if I > VleOptions.Strings.Count - 1 then begin
      VleOptions.Strings.Add(SName + '=' + SValue);
    end else begin
      BIgnore := FIgnore;
      FIgnore := True;
      try
        VleOptions.Row := I;
      except
      end;
      FIgnore := BIgnore;
    end;
  end;
  RefreshOwnerNode;
  AcSaveToDBExecute(nil);
end;

procedure TfrmMDK_Options.AcOptionLoadExecute(Sender: TObject);
var
  I: Integer;
begin
  if DlgOpen.Execute then begin
    VleOptions.Strings.LoadFromFile(DlgOpen.FileName);
    ValueUpdated := True;
    for I := 0 to VleOptions.Strings.Count - 1 do
      if VleOptions.Strings.Names[I] = '' then
          VleOptions.Strings[I] := Format('%4.4d =%s', [I + 1, VleOptions.Strings[I]]);
  end;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.AcOptionSaveExecute(Sender: TObject);
begin
  if DlgSave.Execute then VleOptions.Strings.SaveToFile(DlgSave.FileName);
end;

procedure TfrmMDK_Options.SaveOptions;
begin
  CheckValues;
  if not Assigned(Engine) then Exit;
  if (OptionsType = OtUser) then begin
    Engine.SetUserOptionByName(LastName, VleOptions.Strings.Text);
    UserOptions.Overwrite(VleOptions.Strings.Text); // zzzzzzandria 060629
    ValueUpdated := False;
  end else if (OptionsType = OtSystem)
{$IFNDEF AANTEST} and (Engine.CurrentOperatorMode = RnAdmin){$ENDIF} then begin
    Engine.SetSystemOptionByName(LastName, VleOptions.Strings.Text);
    // if (LastName = onAdministrators) and Assigned(Administrators) then
    // Administrators.Values.Text := vleOptions.Strings.Text;
    ValueUpdated := False;
  end;
  if ((LastName = OnUserOptions) or (LastName = OnSysOptions))
  // {$IFNDEF AANTEST} and (Engine.CurrentOperatorMode = rnADMIN){$ENDIF}  // zzzzzzandria 2008-04-02
  then begin
    if LastName = OnUserOptions then UserOptions.ValueList.Text := VleOptions.Strings.Text;
    if LastName = OnSysOptions then SysOptions.ValueList.Text := VleOptions.Strings.Text;
    try
      Preferences.ValueList.Text := '';
      Preferences.Overwrite(SysOptions.ValueList.Text);
      Preferences.Overwrite(UserOptions.ValueList.Text);
      Preferences.Overwrite(UserSettings.ValueList.Text);
    except
    end;
  end;
end;

procedure TfrmMDK_Options.SaveOptionList;
begin
  if not ValueUpdated then Exit;
  if (VleOptions.Strings.Count = 0) then Exit; // we are not saving blank lists
  if LastName = OnPreferences then Exit;       // zzzzzzandria 061011
  SaveOptions;
end;

procedure TfrmMDK_Options.SetLastName(AName: string);
var
  _Ignore: Boolean;
  I: Integer;
  SName, S: string;
  function SortStringList(AString: string): string;
  var
    _sl: TStringList;
  begin
    Result := '';
    if AString = '' then Exit
    else begin
      _sl := TStringList.Create;
      _sl.Text := AString;
      _sl.Sort;
      Result := _sl.Text;
      _sl.Free;
    end;
  end;

begin
  PnlComments.Visible := (AName = OnPreferences) or (AName = OnSysOptions) or (AName = OnUserOptions);
  if not Assigned(Engine) then Exit;
  _Ignore := FIgnore;
  FIgnore := True;
  try
    if (FLastName <> AName) then SaveOptionList;
    VleOptions.Strings.Clear;
    FUpdated := False;
    FLastName := AName;
    if OptionsType = OtSystem then S := Engine.GetSystemOptionByName(FLastName)
    else S := Engine.GetUserOptionByName(FLastName);
    if Pos('-1', S) <> 1 then begin
      S := SortStringList(S);
      if AName = OnSysOptions then begin
        SysOptions.Update(S);
        S := SysOptions.ValueList.Text;
      end else if AName = OnUserOptions then begin
        UserOptions.Update(S);
        S := UserOptions.ValueList.Text;
      end;
      VleOptions.Strings.Text := SortStringList(S);
      while (VleOptions.Strings.Count > 0) and
        ((Trim(VleOptions.Strings[0]) = '') or (Trim(VleOptions.Strings[0]) = '=')) do VleOptions.Strings.Delete(0);
      for I := 0 to VleOptions.Strings.Count - 1 do begin
        S := Uppercase(Trim(Piece(VleOptions.Strings[I], '=', 2)));
        SName := Uppercase(Trim(Piece(VleOptions.Strings[I], '=', 1)));
        if (S = 'TRUE') or (S = 'FALSE') then begin
          VleOptions.ItemProps[I].EditStyle := EsPickList;
          VleOptions.ItemProps[I].PickList.CommaText := 'TRUE,FALSE';
        end else if Pos('PAIN LEVEL', SName) > 0 then begin
          VleOptions.ItemProps[I].EditStyle := EsPickList;
          VleOptions.ItemProps[I].PickList.CommaText := '0,1,2,3,4,5,6,7,8,9,99';
          VleOptions.ItemProps[I].EditMask := '!9';
        end;
        // zzzzzzandria add value edit for DATE options. 1.0.29.10
        // esElipses
        if (FLastName = OnSysOptions) then begin
{$IFDEF STUDY_STATUS_LIST}
          if
          // (Uppercase(onAllowUserResetStudyStatus)=sName) or
            (Uppercase(OnAllowUserResetActiveStudyList) = SName) or (Uppercase(OnAllowUserReleaseSelectedStudy) = SName)
            or // zzzzzzandria 2007-10-04
            (Uppercase(OnAllowUserReleaseStudySelectedBefore) = SName) // zzzzzzandria 2007-10-04
          then begin
            VleOptions.ItemProps[I].ReadOnly := True;
            VleOptions.ItemProps[I].EditStyle := EsPickList;
            VleOptions.ItemProps[I].PickList.CommaText := '';
          end;
{$ENDIF}
        end;
{$IFDEF ALLOWTRUE}
        if AName = OnUserOptions then begin
          if Pos('ALLOW USER', SName) = 1 then begin
            VleOptions.Values[SName] := 'TRUE';
            VleOptions.ItemProps[I].ReadOnly := True;
            VleOptions.ItemProps[I].EditStyle := EsSimple;
          end;
        end;
{$ENDIF}
      end;
    end;
  except
  end;
  if VleOptions.Strings.Count > 0 then
    try
      VleOptions.Row := 1;
    except
    end;
  FIgnore := _Ignore;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.AcOptionDeleteExecute(Sender: TObject);
var
  AN, AV: string;
begin
  if VleOptions.Strings.Count < 1 then Exit;
  AN := Piece(VleOptions.Strings[VleOptions.Row - 1], '=', 1);
  AV := Piece(VleOptions.Strings[VleOptions.Row - 1], '=', 2);
  if MessageDLG('You are about to delete ' + #13#10#13#10 + 'Item:  "' + AN + '"' + #13#10 + 'Value: "' + AV + '"' +
    #13#10#13#10 + 'From the list of  "' + LastName + '"' + #13#10 + 'Do you want to continue?', MtConfirmation,
    [MbYes, MbNo], 0) = MrYes then begin
    VleOptions.Strings.Delete(VleOptions.Row - 1);
    if OptionsType = OtSystem then Engine.DeleteSystemOption(AN)
    else Engine.DeleteUserOption(AN);
    ValueUpdated := True;
  end;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.VleOptionsGetEditText(Sender: TObject; ACol, ARow: Integer; var Value: string);
begin
  if Ignore then Exit;
  ValueUpdated := True;
  RefreshOwnerNode;
end;

procedure TfrmMDK_Options.SetOptionsType(AName: string);
begin
  if (AName <> OtUser) and (AName <> OtSystem) and (AName <> OtDeveloper) then Exit;
  FOptionsType := AName;
end;

procedure TfrmMDK_Options.LoadOptions;
begin
  SetLastName(FLastName);
end;

procedure TfrmMDK_Options.AcSaveToDBExecute(Sender: TObject);
var
  S: string;
begin
  if ((LastName = OnUserOptions) or (LastName = OnSysOptions)) and not AllowChange(S) then
      MessageDLG('Please, fix errors prior to saving values.' + #13#10#13#10 + S, MtWarning, [MbOK], 0)
  else SaveOptions;
end;

procedure TfrmMDK_Options.SetReadOnly(ARO: Boolean);
begin
  try
    FReadOnly := ARO;
    if not ARO then VleOptions.Color := ClWindow
    else VleOptions.Color := ClInfoBk;
    if not ARO then VleOptions.Options := VleOptions.Options + [GoEditing]
    else VleOptions.Options := VleOptions.Options - [GoEditing]
  except
  end;
end;

procedure TfrmMDK_Options.SetRole(ARole: string);
begin
  try
{$IFDEF AANTEST}
    ROnly := False;
    PnlOptionTools.Visible := True;
{$ELSE}
    ROnly := (ARole <> RnAdmin) and (LastName <> 'User Options');
    PnlOptionTools.Visible := False;
{$ENDIF}
    if LastName = 'User Options' then begin
      PnlOptionsAdmin.Visible := False;
      PnlOptionsUser.Visible := False;
      PnlSystemPreferencesAdmin.Visible := False;
      PnlUserPreferencesUser.Visible := ARole <> RnAdmin;
      PnlUserPreferencesAdmin.Visible := ARole = RnAdmin;
    end else if LastName = 'System Options' then begin
      PnlOptionsAdmin.Visible := False;
      PnlOptionsUser.Visible := ARole <> RnAdmin;
      PnlSystemPreferencesAdmin.Visible := ARole = RnAdmin;
      PnlUserPreferencesUser.Visible := False;
      PnlUserPreferencesAdmin.Visible := False;
    end else if LastName = 'Preferences' then begin
      ROnly := True;
      PnlOptionsAdmin.Visible := False;
      PnlOptionsUser.Visible := True;
      PnlSystemPreferencesAdmin.Visible := False;
      PnlUserPreferencesUser.Visible := False;
      PnlUserPreferencesAdmin.Visible := False;
    end else begin
      PnlOptionsAdmin.Visible := ARole = RnAdmin;
      PnlOptionsUser.Visible := ARole <> RnAdmin;
      PnlSystemPreferencesAdmin.Visible := False;
      PnlUserPreferencesUser.Visible := False;
      PnlUserPreferencesAdmin.Visible := False;
      BtnDefaults.Visible := (ARole = RnAdmin) and ((LastName = OnAntiCoagulants) or (LastName = OnNephrologiests) or
        (LastName = OnCodeStatuses) or (LastName = OnDialyzerList) or (LastName = OnEducationCodes) or
        (LastName = OnESRD_Diagnosys) or (LastName = OnESRD_ICD10) or (LastName = OnMedicalFacilities) or
        (LastName = OnMedicationRoutes) or (LastName = OnMedicationUnits) or (LastName = OnModalities) or
        (LastName = OnSurgeons) or (LastName = OnTransType) or (LastName = OnTIUNoteTitles) or
        (LastName = OnMedications));
      Panel5.Visible := not((LastName = OnMedicationStatuses) or (LastName = OnMedicationReasons));
      VleOptions.Visible := not((LastName = OnMedicationStatuses) or (LastName = OnMedicationReasons));
    end;
  except
    on E: Exception do
        MessageDLG('Error setting role of <' + ARole + '>:'#13#10#13#10 + E.Message, MtWarning, [MbOK], 0);
  end;
end;

procedure TfrmMDK_Options.VleOptionsExit(Sender: TObject);
begin
  if FIgnore then Exit;
  SaveOptions;
end;

procedure TfrmMDK_Options.SetValueList(AValue: string);
begin
  VleOptions.Strings.Text := AValue;
end;

procedure TfrmMDK_Options.RefreshOwnerNode;
var
  Tn0: TTreeNode;
  I: Integer;
  Tn: TTreeNode;
  BEx: Boolean;
begin
  if FOwnerNode = nil then Exit;
  if FOwnerTreeNodes = nil then Exit;
  Tn0 := FOwnerNode;
  BEx := Tn0.Expanded;
  Tn0.DeleteChildren;
  if VleOptions.Strings.Count = 0 then Exit;
  for I := 0 to VleOptions.Strings.Count - 1 do begin
    Tn := FOwnerTreeNodes.AddChild(Tn0, VleOptions.Strings.Names[I]);
    Tn.ImageIndex := Tn0.ImageIndex;
    Tn.SelectedIndex := Tn0.SelectedIndex;
  end;
  Tn0.Expanded := BEx;
end;

procedure TfrmMDK_Options.VleOptionsDblClick(Sender: TObject);
var
  Tn: TTreeNode;
  AValue: string;
begin
{$IFDEF STUDY_STATUS_LIST}
  // 2007-10-02 zzzzzzandria --------------------------------------------- begin
  if (FLastName = OnSysOptions) and ((OnAllowUserResetActiveStudyList = FOptionName) or
    (OnAllowUserReleaseSelectedStudy = FOptionName) or (OnAllowUserReleaseStudySelectedBefore = FOptionName)) then begin
    MessageDLG('The value of "' + FOptionName + '" can be set at the User Preference level only.', MtInformation,
      [MbOK], 0);
    Exit;
  end;
  // 2007-10-02 zzzzzzandria ----------------------------------------------- end
{$ENDIF}
  if (FOptionName = '') and (VleOptions.Strings.Count > 1) and (VleOptions.Row > 0) then
      FOptionName := VleOptions.Keys[VleOptions.Row];
  // if (pos('Allow USER',fOptionName) = 1) and (fOptionsType =otUser) then Exit;
  if Pos('COLOR', Uppercase(FOptionName)) <> 0 then begin
    if FReadOnly then Exit;
    if ColorDialog1.Execute then VleOptions.Values[FOptionName] := IntToStr(ColorDialog1.Color);
    Exit;
  end;
  if Pos('DATE', Uppercase(FOptionName)) <> 0 then begin
    if FReadOnly then Exit;
    if SelectDate(VleOptions.Values[FOptionName]) then VleOptions.Values[FOptionName] := IntToStr(ColorDialog1.Color);
    Exit;
  end;
  AValue := VleOptions.Values[FOptionName];
  if AValue = 'TRUE' then begin
    if FReadOnly then Exit;
    VleOptions.Values[FOptionName] := 'FALSE';
    Exit;
  end;
  if AValue = 'FALSE' then begin
    if FReadOnly then Exit;
    VleOptions.Values[FOptionName] := 'TRUE';
    Exit;
  end;
  if FOwnerNode = nil then Exit;
  if FOwnerTreeNodes = nil then Exit;
  if FOptionName <> '' then begin
    Tn := FOwnerNode.GetFirstChild;
    repeat
      if Tn = nil then Break;
      Tn := Tn.GetNextSibling;
    until (Tn = nil) or (Tn.Text = FOptionName);
  end;
end;

procedure TfrmMDK_Options.VleOptionsMouseUp(Sender: TObject; Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
  Column, Row: Longint;
begin
  try
    VleOptions.MouseToCell(X, Y, Column, Row);
    FOptionName := VleOptions.Cells[0, Row];
  except
    FOptionName := '';
  end;
end;

procedure TfrmMDK_Options.VleOptionsDrawCell(Sender: TObject; ACol, ARow: Integer; Rect: TRect; State: TGridDrawState);
var
  S: string;
  I: Integer;
begin
  with VleOptions.Canvas do begin
    Font.Color := ClWindowText;
    // if aCol <> 0 then
    // Font.Color := clNavy;
    Font.Style := [];
    if ARow = 0 then Brush.Color := ClBtnFace
    else begin
      if FReadOnly then begin
        Brush.Color := ClInfoBk;
      end else begin
        if (ACol = 0) then Brush.Color := ClWindow
        else begin
          S := VleOptions.Cells[0, ARow];
          if Pos('Color', S) > 0 then I := StrToIntDef(VleOptions.Cells[1, ARow], ClWindow)
          else if FReadOnly then I := ClInfoBk
          else I := ClWindow;
          Brush.Color := I;
        end
      end;
    end;
    FillRect(Rect);
    TextOut(Rect.Left + 2, Rect.Top + 2, VleOptions.Cells[ACol, ARow]);
  end;
end;

procedure TfrmMDK_Options.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
begin
  if Key = VK_ESCAPE then ModalResult := MrCancel;
end;

procedure TfrmMDK_Options.VleOptionsSetEditText(Sender: TObject; ACol, ARow: Integer; const Value: string);
begin
  ValueUpdated := True;
end;

procedure TfrmMDK_Options.AcAssignExecute(Sender: TObject);
begin
  AddOptions(True, Self);
  LoadOptions;
end;

procedure TfrmMDK_Options.AcDefaultsExecute(Sender: TObject);
var
  B: Boolean;
  SMSG, S: string;
begin
  S := GetDefaultResource(LastName); // jwc
  if S = '' then begin
    SMSG := 'The default list of values is blank' + #13#10 + 'Do you want to clear the current list?';
    B := MessageDLG(SMSG, MtConfirmation, [MbYes, MbNo], 0) = MrYes;
  end
  else B := IsReplacementNeeded(S);
  if B then
    try
      VleOptions.Strings.Text := S;
    except
    end;
end;

procedure TfrmMDK_Options.AcSetDefaultsExecute(Sender: TObject);
begin
  if MessageDLG('Do you want to replace the current list with the default one?', MtConfirmation, [MbYes, MbNo], 0) <> MrYes
  then Exit;
  SetSystemDefaults;
  Preferences.Overwrite(SysOptions.ValueList.Text);
  Preferences.Overwrite(UserOptions.ValueList.Text);
  VleOptions.Strings.Text := SysOptions.ValueList.Text;
  CheckSummaryReport; // zzzzzzandria 2008-04-01
end;

procedure TfrmMDK_Options.AcCleanupExecute(Sender: TObject);
begin
  Engine.CleanUpOptions;
end;

procedure TfrmMDK_Options.VleOptionsSelectCell(Sender: TObject; ACol, ARow: Integer; var CanSelect: Boolean);
begin
  SetComments(ARow);
end;

procedure TfrmMDK_Options.VleOptionsEditButtonClick(Sender: TObject);
var
  S: string;
begin
  try
    S := Engine.GetSystemOptionByName(OnTIUNoteTitles);
    if S = '' then
        MessageDLG('To select a default TIU title' + #13 + #10 + 'Create the titles list first.', MtWarning, [MbOK], 0)
    else begin
      S := SelectListItem(S);
      if S <> '' then VleOptions.Cells[1, VleOptions.Row] := S;
    end;
  except
  end;
end;

procedure TfrmMDK_Options.VleOptionsKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
var
  SName, SValue, SDefault: string;
begin
  if (Key = VK_RETURN) and (SsCtrl in Shift) and (SsShift in Shift) then begin
    SName := VleOptions.Cells[0, VleOptions.Row];
    if (LastName <> OnSystemPreferences) and (LastName <> OnUserPreferences) and (LastName <> 'System Options') and
      (LastName <> 'User Options') then Exit;
    SDefault := GetSystemDefault(SName);
    SValue := VleOptions.Values[SName];
    if (SDefault = '') then Exit;
    if SDefault <> SValue then begin
      if MessageDLG('Option selected: ' + SName + #13#10#13#10 + 'Current Value: ' + SValue + #13#10 + 'Default Value: '
        + SDefault + #13#10#13#10 + 'Do you want to replace the current value with the default one?', MtConfirmation,
        [MbNo, MbOK], 0) = MrOK then VleOptions.Values[SName] := SDefault;
    end
    else MessageDLG('Option selected: ' + SName + #13#10#13#10 + 'Current Value: ' + SValue + #13#10 + 'Default Value: '
        + SDefault, MtInformation, [MbOK], 0);
  end;
end;

const
  ValidPainLevel = '0,1,2,3,4,5,6,7,8,9,99,';

function TfrmMDK_Options.CheckValue(AnOption: string; var Reason: string): Boolean;
var
  Sl: TStringList;
  I: Integer;
  SV, SMin, SMax, UpValue, UpName: string;
begin
  SV := VleOptions.Values[AnOption];
  UpValue := Uppercase(SV);
  UpName := Uppercase(AnOption);
  if (Pos(OnSummaryReportName, AnOption) <> 0) then begin
    UpValue := Engine.GetSystemOptionByName(SV);
    Result := UpValue <> '';
    Reason := 'Report name <' + SV + '> is not found';
    (* *)
    UpValue := Engine.GetSystemOptionByName(OnReportList);
    Sl := TStringList.Create;
    Sl.Text := UpValue;
    for I := 0 to Sl.Count - 1 do begin
      if Pos(SV, Sl[I]) = 1 then Break;
    end;
    // Result := i < sl.Count;
    if I = Sl.Count then Engine.SetSystemOptionByName(OnReportList, SV + '=' + SV + #13#10 + UpValue);
    Sl.Free;
    (* *)
  end else begin
    SMin := PreferenceComments.SMinValue(AnOption);
    SMax := PreferenceComments.SMaxValue(AnOption);
    Result := CheckOption(UpName, UpValue, SMin, SMax, Reason);
  end;
end;

procedure TfrmMDK_Options.VleOptionsValidate(Sender: TObject; ACol, ARow: Integer; const KeyName, KeyValue: string);
var
  SReason: string;
begin
  if not CheckValue(KeyName, SReason) then begin
    VleOptions.Values[KeyName] := GetSystemDefault(KeyName);
    raise Exception.Create(SReason + #13#10#13#10 + 'The invalid value has been replaced' + #13#10 +
      'with the default of "' + GetSystemDefault(KeyName) + '"');
  end
end;

function TfrmMDK_Options.AllowChange(var Reason: string): Boolean;
var
  I: Integer;
  KeyValue, KeyName: string;
begin
  Result := True;
  for I := 1 to VleOptions.RowCount - 1 do begin
    KeyValue := VleOptions.Cells[1, I];
    KeyName := VleOptions.Cells[0, I];
    Result := CheckValue(KeyName, Reason);
    if not Result then Break;
  end;
end;

procedure TfrmMDK_Options.VleOptionsEnter(Sender: TObject);
begin
  SetComments(VleOptions.Row);
end;

procedure TfrmMDK_Options.SetComments(ARow: Integer);
var
  S, SOutput, SName, SValue: string;
  I: Integer;
begin
  if not PnlComments.Visible then Exit;
  // if aRow < 0 then Exit;
  SName := VleOptions.Cells[0, ARow];
  SValue := PreferenceComments.StringValue(SName);
  if SValue = YoUnKnown then MmComments.Lines.Text := ''
  else begin
    SOutput := '';
    for I := 1 to Length(SValue) do begin
      S := Copy(SValue, I, 1);
      if S = '^' then SOutput := SOutput + #13#10
      else SOutput := SOutput + S;
    end;
    SOutput := Piece(SOutput, '<MIN>', 1);
    SOutput := Piece(SOutput, '<MAX>', 1);
    MmComments.Lines.Text := SOutput;
  end;
end;

function TfrmMDK_Options.CheckValues: string;
var
  SName, SDefault, SReason: string;
  I: Integer;
begin
  Result := '';
  if ((LastName <> OnUserOptions) and (LastName <> OnSysOptions)) then Exit;
  for I := 0 to VleOptions.Strings.Count - 1 do begin
    SName := VleOptions.Keys[I];
    if not CheckValue(SName, SReason) then begin
      Result := Result + VleOptions.Values[SName];
      SDefault := GetSystemDefault(SName);
      VleOptions.Values[SName] := SDefault;
      Engine.Report(RtWarning, SReason + #13#10 + 'Value set to "' + SDefault + '"', '', 0, '', LastName);
      Result := Result + '  set to default of "' + SDefault + '"' + #13#10
    end;
  end;
end;

end.
