unit frMDK_Comments;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 12/04/07 9:13a $
  *       Site Name:    Hines OIFO
  *       Date Created: 3/28/07
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     frMDK_Comments frame definition
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Common.pas $
  *
  *       $ History: uMDK_Common.pas $
  *
}
interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ExtCtrls, ActnList, DBClient, DB, StdCtrls, Grids, DBGrids, FrMDK_StudyNoteSection, Buttons, ImgList,
  ComCtrls, ToolWin, DMDK_Data, System.Actions;

type
  TFrameMDK_Comments = class(TFrame)
    PnlTools: TPanel;
    ActionList1: TActionList;
    AcRefresh: TAction;
    AcAdd: TAction;
    SbxCanvas: TScrollBox;
    AcValid: TAction;
    AcShowReport: TAction;
    Panel1: TPanel;
    AcLock: TAction;
    AcEdit: TAction;
    AcDelete: TAction;
    AcCleanUp: TAction;
    AcUpdateItem: TAction;
    ImageList1: TImageList;
    PnlToolbar: TPanel;
    SpeedButton6: TSpeedButton;
    SpeedButton5: TSpeedButton;
    SpeedButton2: TSpeedButton;
    SpeedButton1: TSpeedButton;
    PnlButtons2DH: TPanel;
    Label2: TLabel;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    BitBtn3: TBitBtn;
    BitBtn4: TBitBtn;
    PnlButtons3DH: TPanel;
    BitBtn5: TBitBtn;
    BitBtn6: TBitBtn;
    BitBtn7: TBitBtn;
    BitBtn8: TBitBtn;
    Panel2: TPanel;
    SpbNCnew: TSpeedButton;
    SpbNCEdit: TSpeedButton;
    SpbNCReport: TSpeedButton;
    SpbNCDelete: TSpeedButton;
    procedure AcRefreshExecute(Sender: TObject);
    procedure AcAddExecute(Sender: TObject);
    procedure AcValidExecute(Sender: TObject);
    procedure AcShowReportExecute(Sender: TObject);
    procedure AcLockExecute(Sender: TObject);
    procedure AcDeleteExecute(Sender: TObject);
    procedure AcEditExecute(Sender: TObject);
    procedure AcCleanUpExecute(Sender: TObject);
    procedure AcUpdateItemExecute(Sender: TObject);
    procedure Label2Click(Sender: TObject);
  private
    FItemTag: string;
    FGroupTag: string;
    FCDS: TClientDataSet;
    FReadOnly: Boolean;
    FGlobalCollapsed: Boolean;
    FNL_ID: string; // 'ID';
    FNL_Lock: string;
    FNL_PatientID: string;  // 'PatientID';
    FNL_StudyID: string;    // 'StudyID';
    FNL_ItemID: string;     // 'ItemID';
    FNL_Tag: string;        // 'Tag';
    FNL_DateTime: string;   // 'DateTime';
    FNL_AuthorID: string;   // 'Author';
    FNL_AuthorName: string; // 'Author';
    FNL_Text: string;       // 'NoteText';
    FNL_Status: string;     // 'Status';
    FNL_StatusBy: string;   // 'StatusBy';
    FNL_StatusDate: string; // 'StatusDate';
    FCurrentFrame: TObject;
    procedure SetCommentFields;
    function ProcessRecord(STag: string; ITag: Integer): TFrameStudyNoteSection;
    function LocateNoteByID(AnID: string): Boolean;
    procedure SetReadOnly(AValue: Boolean);
  public
    ReportTitle: string;
    property GroupTag: string read FGroupTag write FGroupTag;
    property ItemTag: string read FItemTag write FItemTag;
    property readonly: Boolean read FReadOnly write SetReadOnly;
    procedure FindNext(Up: Boolean; ASender: TObject);
    procedure FindCurrent(AnID: string; ASender: TObject);
    procedure SetUpCanvas(ACDS: TClientDataSet; AStringTag: string);
    procedure SetUpGUI(AValue: string);
    procedure CleanSBX;
    procedure ResetSBX;
    function SetupScrollbox: string;
    procedure ReleaseCurrentFrame;
    procedure MarkCurrentFrame;
    procedure SetActionStatus;
    procedure MoveComment(AFieldName: string; AnID: string; ACDS: TClientDataSet = nil);
    function AddNewComment(AText: string; AnID: string = ''; AReadOnly: Boolean = False): string;
    procedure ActivateFrame;
  end;

implementation

uses
  UMDK_Notes, UMDK_Options, UMDK_Common, System.UITypes,
  UMDK_CDSCommon, UMDK_Engine, FMDK_Comments, UMDK_Flowsheet,
  FMDK_Info;

{$R *.dfm}

procedure TFrameMDK_Comments.SetUpCanvas(ACDS: TClientDataSet; AStringTag: string);
begin
  FCDS := ACDS;
  FGroupTag := AStringTag;
  PnlButtons3DH.Visible := DevOptions.BValue(DonWindowsXPTheme);
  PnlButtons2DH.Visible := not DevOptions.BValue(DonWindowsXPTheme);
end;

procedure TFrameMDK_Comments.SetCommentFields;
var
  ACDS: TClientDataSet;
  function GetField(AName, ADefault: string): string;
  begin
    try
      Result := ACDS.FieldByName(AName).AsString;
    except
      Result := ADefault;
    end;
  end;

begin
  ACDS := FCDS;
  if not Assigned(ACDS) then Exit;
  try
    FNL_ID := GetField(FnNL_ID, '');     // 'ID';
    FNL_Lock := GetField(FnNL_Lock, ''); // 'ID';
    FNL_PatientID := GetField(FnNL_PatientID, '');   // 'PatientID';
    FNL_StudyID := GetField(FnNL_StudyID, '');       // 'StudyID';
    FNL_ItemID := GetField(FnNL_ItemID, '');         // 'ItemID';
    FNL_Tag := GetField(FnNL_Tag, '');               // 'Tag';
    FNL_DateTime := GetField(FnNL_DateTime, '');     // 'DateTime';
    FNL_AuthorID := GetField(FnNL_AuthorID, '');     // 'Author';
    FNL_AuthorName := GetField(FnNL_AuthorName, ''); // 'Author';
    FNL_Text := GetField(FnNL_Text, '');             // 'NoteText';
    FNL_Status := GetField(FnNL_Status, '');         // 'Status';
    FNL_StatusBy := GetField(FnNL_StatusBy, '');     // 'StatusBy';
    FNL_StatusDate := GetField(FnNL_StatusDate, ''); // 'StatusDate';
  except
  end;
end;

function TFrameMDK_Comments.ProcessRecord(STag: string; ITag: Integer): TFrameStudyNoteSection;
var
  ASection: TFrameStudyNoteSection;
  SName: string;
begin
  SetCommentFields;
  try
    SName := Engine.GetUserNameByID(FNL_AuthorID);
    if SName = '' then SName := FNL_AuthorID;
  except
    SName := FNL_AuthorID;
  end;
  ASection := NewStudyNoteSection(SbxCanvas, FNL_ID, FNL_Lock, STag, SName, // fNL_AuthorID, //Name,
    FNL_DateTime, FNL_Text, // Text of Comment
    'NOTE',                                        // Type of Comment
    FNL_Status, FNL_StatusBy, FNL_StatusDate, 200, // Max Height
    ITag,                                          // Tab Order
    True);                                         // Collapsed
  ASection.Parent := SbxCanvas;
  ASection.PnlTypeIcon.Visible := False;
  ASection.MoveUp;
  Result := ASection;
  AcEdit.Enabled := ASection.LockStatus = FsUnlocked;
  AcDelete.Enabled := ASection.LockStatus = FsUnlocked;
end;

procedure TFrameMDK_Comments.CleanSBX;
var
  I: Integer;
  ASection: TFrameStudyNoteSection;
begin
  I := 0;
  while I < SbxCanvas.ComponentCount do begin
    if SbxCanvas.Components[I] is TFrameStudyNoteSection then begin
      ASection := TFrameStudyNoteSection(SbxCanvas.Components[I]);
      SbxCanvas.RemoveComponent(ASection);
      if Assigned(ASection) then begin
        ASection.CleanUp;
        FreeAndNil(ASection);
      end;
    end else begin
      Inc(I);
    end;
  end;
end;

procedure TFrameMDK_Comments.ResetSBX;
var
  BM: TBookmark;
  ASection: TFrameStudyNoteSection;
  ITab: Integer;
begin
  if not Assigned(FCDS) then Exit;
  if not FCDS.Active then Exit;
  ITab := 0;
  BM := FCDS.GetBookmark;
  FCDS.DisableControls;
  FCDS.Last;
  while not FCDS.BOF do begin
    ASection := ProcessRecord(IntToStr(ITab), ITab);
    if Assigned(ASection) then begin
      ASection.OnFindCurrent := FindCurrent;
      Inc(ITab);
    end;
    FCDS.Prior;
  end;
  FCDS.GotoBookmark(BM);
  FCDS.FreeBookmark(BM);
  FCDS.EnableControls;
end;

function TFrameMDK_Comments.SetupScrollbox: string;
var
  ASection: TFrameStudyNoteSection;
  ITab: Integer;
  ACDS: TClientDataSet;
  ATag: string;
  ASBX: TScrollBox;
  AFindNext: TFindNext;
begin
  Result := '';
  ACDS := FCDS;
  if not Assigned(ACDS) or not ACDS.Active then Exit;
  ATag := GroupTag;
  ASBX := SbxCanvas;
  AFindNext := FindNext;
  if not ASBX.Visible then Exit;
  CleanSBX;
  FCurrentFrame := nil;
  ITab := 0;
  ACDS.DisableControls;
  ACDS.IndexFieldNames := FnNL_DateTime;
  ACDS.Last;
  while not ACDS.BOF do begin
    ASection := ProcessRecord(IntToStr(ITab), ITab);
    if Assigned(ASection) then begin
      ASection.OnFindCurrent := FindCurrent;
      Inc(ITab);
      if not Assigned(FCurrentFrame) then FCurrentFrame := ASection;
    end;
    ACDS.Prior;
  end;
  ACDS.EnableControls;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.FindNext(Up: Boolean; ASender: TObject);
var
  J, I: Integer;
begin
  for I := 0 to SbxCanvas.ComponentCount - 1 do begin
    if SbxCanvas.Components[I] = ASender then begin
      if Up then begin
        if I = 0 then J := SbxCanvas.ComponentCount - 1
        else J := I - 1;
        TFrameStudyNoteSection(SbxCanvas.Components[J]).FocusLast;
      end else begin
        if I = SbxCanvas.ComponentCount - 1 then J := 0
        else J := I + 1;
        TFrameStudyNoteSection(SbxCanvas.Components[J]).FocusFirst;
      end;
      Break;
    end;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.ReleaseCurrentFrame;
begin
  if Assigned(FCurrentFrame) then
    try
      TFrameStudyNoteSection(FCurrentFrame).LblDate.Font.Style := TFrameStudyNoteSection(FCurrentFrame)
        .LblDate.Font.Style - [FsBold];
      TFrameStudyNoteSection(FCurrentFrame).LblTime.Font.Style := TFrameStudyNoteSection(FCurrentFrame)
        .LblTime.Font.Style - [FsBold];
      TFrameStudyNoteSection(FCurrentFrame).Lblby.Font.Style := TFrameStudyNoteSection(FCurrentFrame).Lblby.Font.Style
        - [FsBold];
      TFrameStudyNoteSection(FCurrentFrame).SectionSelected := False;
    except
    end;
end;

procedure TFrameMDK_Comments.MarkCurrentFrame;
begin
  if Assigned(FCurrentFrame) then begin
    TFrameStudyNoteSection(FCurrentFrame).LblDate.Font.Style := TFrameStudyNoteSection(FCurrentFrame).LblDate.Font.Style
      + [FsBold];
    TFrameStudyNoteSection(FCurrentFrame).LblTime.Font.Style := TFrameStudyNoteSection(FCurrentFrame).LblTime.Font.Style
      + [FsBold];
    TFrameStudyNoteSection(FCurrentFrame).Lblby.Font.Style := TFrameStudyNoteSection(FCurrentFrame).Lblby.Font.Style
      + [FsBold];
    TFrameStudyNoteSection(FCurrentFrame).SectionSelected := True;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.FindCurrent(AnID: string; ASender: TObject);
var
  SAuthor: string;
begin
  ReleaseCurrentFrame;
  if FCDS.Locate(FnNL_ID, VarArrayOf([AnID]), []) then begin
    SAuthor := FCDS.FieldByName(FnNL_AuthorID).AsString;
    AcValid.Enabled := (SAuthor = Engine.CurrentOperatorID) or (Engine.CurrentOperatorMode = RnAdmin);
  end
  else AcValid.Enabled := False;
  FCurrentFrame := ASender;
  if Assigned(FCurrentFrame) then begin
    AcAdd.Enabled := not Engine.CurrentPatient.ReadOnly;
    AcEdit.Enabled := TFrameStudyNoteSection(FCurrentFrame).LockStatus = FsUnlocked;
    AcDelete.Enabled := TFrameStudyNoteSection(FCurrentFrame).LockStatus = FsUnlocked;
  end
  else AcAdd.Enabled := not readonly;
  MarkCurrentFrame;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TFrameMDK_Comments.LocateNoteByID(AnID: string): Boolean;
begin
  Result := False;
  try
    Result := DmData.CdsNotes.Locate(FnNL_ID, VarArrayOf([AnID]), []);
  except
    on E: Exception do MessageDLG('Locate Note By ID' + #13#10 + E.Message, MtError, [MbOK], 0);
  end;
end;

procedure TFrameMDK_Comments.SetActionStatus;
begin
  SetCommentFields;
  AcEdit.Enabled := FNL_Lock = FsUnlocked;
  AcLock.Enabled := FNL_Lock = FsUnlocked;
  AcDelete.Enabled := FNL_Lock = FsUnlocked;
  readonly := FReadOnly;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TFrameMDK_Comments.AcValidExecute(Sender: TObject);
var
  SAuthor: string;
begin
  Exit;
  if Assigned(FCurrentFrame) then
    try
      SAuthor := TFrameStudyNoteSection(FCurrentFrame).Author;
      if (SAuthor = Engine.CurrentOperatorID)
      { //        or Preferences.bValue(onAllowUserInvalidateOtherUserComments) } then
          TFrameStudyNoteSection(FCurrentFrame).PnlCanvasDblClick(nil);
    except
    end;
end;

procedure TFrameMDK_Comments.AcShowReportExecute(Sender: TObject);
begin
  if DevOptions.BValue(DonShowAllComments) then ShowInfo(GetNotesReport(ReportTitle, FCDS))
  else if Assigned(FCurrentFrame) then begin
    SetCommentFields;
    ShowInfo(FNL_Text);
  end;
end;

procedure TFrameMDK_Comments.AcRefreshExecute(Sender: TObject);
var
  I: Integer;
  SID: string;
begin
  if not Visible then // zzzzzzandria 070613 8:21
      Exit;
  SID := '';
  if Assigned(FCurrentFrame) then begin
    try
      SID := TFrameStudyNoteSection(FCurrentFrame).ID;
    except
      SID := '';
    end;
  end;
  SetupScrollbox;
  if SID <> '' then
    try
      for I := 0 to SbxCanvas.ComponentCount - 1 do begin
        if (SbxCanvas.Components[I] is TFrameStudyNoteSection) and
          (TFrameStudyNoteSection(SbxCanvas.Components[I]).ID = SID) then begin
          FCurrentFrame := TFrameStudyNoteSection(SbxCanvas.Components[I]);
          MarkCurrentFrame;
          if Assigned(FCurrentFrame) and TFrameStudyNoteSection(FCurrentFrame).CanFocus then
            try
              TFrameStudyNoteSection(FCurrentFrame).SetFocus;
            except
            end;
          Break;
        end;
      end;
    except
    end;
end;

procedure TFrameMDK_Comments.AcLockExecute(Sender: TObject);
begin
  if not Assigned(FCurrentFrame) then Exit;
  SetCommentFields;
  if LocateNoteByID(FNL_ID) then begin
    DmData.LockCurrentNote;
    TFrameStudyNoteSection(FCurrentFrame).LockStatus := FsLocked;
    SetActionStatus;
  end;
end;

procedure TFrameMDK_Comments.AcDeleteExecute(Sender: TObject);
begin
  if not Assigned(FCurrentFrame) then Exit;
  SetCommentFields;
  if LocateNoteByID(FNL_ID) then begin
    if MessageDLG('Delete note: ' + #13#10#13#10 + FNL_Text, MtConfirmation, [MbYes, MbNo], 0) = MrYes then begin
      DmData.CdsNotes.Delete;
      AcRefreshExecute(nil);
      SetActionStatus;
    end;
  end;
end;

procedure TFrameMDK_Comments.AcEditExecute(Sender: TObject);
var
  S: string;
begin
  if not Assigned(FCurrentFrame) then Exit;
  SetCommentFields;
  if LocateNoteByID(FNL_ID) then begin
    S := FNL_Text;
    if EditComment(S) = MrOK then
      try
        DmData.CdsNotes.Edit;
        DmData.CdsNotes.FieldByName(FnNL_Text).AsString := S;
        DmData.CdsNotes.Post;
        AcRefresh.Execute;
        if Assigned(FCurrentFrame) then TFrameStudyNoteSection(FCurrentFrame).Collapsed := False;
      except
      end;
  end;
end;

procedure TFrameMDK_Comments.AcCleanUpExecute(Sender: TObject);
begin
  CleanSBX;
end;

procedure TFrameMDK_Comments.AcUpdateItemExecute(Sender: TObject);
begin
  if not Assigned(FCurrentFrame) then Exit;
  TFrameStudyNoteSection(FCurrentFrame).UpdateFrame;
end;

procedure TFrameMDK_Comments.AcAddExecute(Sender: TObject);
begin
  AddNewComment('');
end;

procedure TFrameMDK_Comments.MoveComment(AFieldName: string; AnID: string; ACDS: TClientDataSet = nil);
var
  S: string;
  CDS: TClientDataSet;
begin
  S := DmData.CdsSummary.FieldByName(AFieldName).AsString;
  if S = '' then Exit;
  if ACDS = nil then CDS := DmData.CdsSummary
  else CDS := ACDS;
  if MessageDLG('Field <' + AFieldName + '> contains data in old format.' + #13#10#13#10 + S + #13#10#13#10 +
    'Do you want to convert this comment to a new format?', MtConfirmation, [MbYes, MbNo], 0) = MrYes then begin
    if AddNewComment(S, AnID, True) <> '' then
      try
        CDS.Edit;
        CDS.FieldByName(AFieldName).AsString := '';
        CDS.Post;
        AcLock.Execute;
        MarkCurrentFrame;
        SetActionStatus;
      except
      end;
  end;
end;

function TFrameMDK_Comments.AddNewComment(AText: string; AnID: string = ''; AReadOnly: Boolean = False): string;
var
  ITag: Integer;
  SFilter, SDate, SAuthor, SID, SText: string;
  ASection: TFrameStudyNoteSection;
  BFiltered: Boolean;
begin
  SText := AText;
  Result := '';
  SFilter := Engine.NoteList_CDS.Filter;
  BFiltered := Engine.NoteList_CDS.Filtered;
  if ConfirmComment(SText, SAuthor, SDate, AReadOnly) = MrOK then begin
    Result := SText;
    if AnID = '' then SID := FItemTag
    else SID := AnID;
    if SAuthor = Engine.CurrentOperatorID then SAuthor := Engine.UserDUZ;
    AddNote(Engine.NoteList_CDS, SText, SAuthor, GroupTag, SID,
      FsUnlocked, StrToDateTime(SDate), Engine.GetCurrentServerDateTime);
    ITag := SbxCanvas.ComponentCount;
    ASection := ProcessRecord(IntToStr(ITag), ITag);
    ASection.OnFindCurrent := FindCurrent;
    ASection.LockStatus := FsUnlocked;
    FCurrentFrame := ASection;
    if BFiltered then Engine.NoteList_CDS.Filter := SFilter;
    AcRefresh.Execute;
  end;
end;

procedure TFrameMDK_Comments.Label2Click(Sender: TObject);
var
  I: Integer;
begin
  FGlobalCollapsed := not FGlobalCollapsed;
  try
    for I := 0 to SbxCanvas.ComponentCount - 1 do
      if (SbxCanvas.Components[I] is TFrameStudyNoteSection) then
          TFrameStudyNoteSection(SbxCanvas.Components[I]).Collapsed := FGlobalCollapsed;
  except
  end;
end;

procedure TFrameMDK_Comments.SetUpGUI(AValue: string);
begin
  PnlButtons3DH.Visible := AValue = '3D';
  PnlButtons2DH.Visible := not(AValue = '3D');
end;

procedure TFrameMDK_Comments.SetReadOnly(AValue: Boolean);
begin
  FReadOnly := AValue;
  if readonly then begin
    AcAdd.Enabled := False;
    AcEdit.Enabled := False;
    AcDelete.Enabled := False;
  end;
end;

procedure TFrameMDK_Comments.ActivateFrame;
begin
  AcRefresh.Execute;
  if Assigned(FCurrentFrame) and (TObject(FCurrentFrame) is TFrameStudyNoteSection) then begin
    LocateNoteByID(TFrameStudyNoteSection(FCurrentFrame).ID);
  end else begin
    AcEdit.Enabled := False;
    AcDelete.Enabled := False;
  end;
  AcAdd.Enabled := not Engine.CurrentPatient.ReadOnly;
end;

end.
