unit uMDK_Common;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/18/08 6:54a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     common variables/ routines
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Common.pas $
  *
  *       $ History: uMDK_Common.pas $
  *
}
interface

uses
  Classes, SysUtils,
  Windows;

const
  pnCover               = 'Cover';
  pnRxLab               = 'Rx and Lab';
  pnPre                 = 'Pre-Treatment';
  pnAccess              = 'Access';
  pnFlow                = 'Flowsheet';
  pnPost                = 'Post-Treatment';
  pnSummary             = 'Summary';
  pnSubmit              = 'Submit';
  gSaved                = char(VK_TAB) + '... Saved';
  gLoaded               = char(VK_TAB) + '... Loaded';
  gFailed               = char(VK_TAB) + '... Failed';
  gErrorBkg             = $00FEE9CB;
  msgSelectPatientFirst = 'Select patient first...';
  msgSubmitFailed = 'Pending instrument result.' + #13#10 + 'If you can not submit the result within 5 minutes,' +
    #13#10 + 'please have your IRM check to see if the CP Gateway is running.';
  msgInCompletStudy = 'Not all data records have been received from the instrument.' + #13#10 +
    'If you submit this treatment now, the unreceived data will not be saved in the data base.' + #13#10 +
    'Do you want to submit anyway?';
  rtWarning         = 'WARNING';
  rtError           = 'ERROR';
  rtInfo            = 'INFO';
  rtDebug           = 'DEBUG';
  STUDY_STATUS_DONE = 'DONE'; // StudyStatus
  /// /////////////////////////////////////////////////////////////////////////////
  sTab                    = char(VK_TAB);
  sBB                     = '<';
  sBE                     = '>';
  fDefaultServerTimeDelay = -999999.0;
  MAXIMUM_TIMEOUT_SEC     = 1000;
  MINIMUM_TIMEOUT_SEC     = 20;
  _CR_                    = #13#10;

const
  NotAplicable = '          ';

const
  yoiUnknown  = -9999;
  yoUnKnown   = '\Unknown';
  yoUndefined = '\Undefined';
  // Roles
  rnHemoDialysisOld = 'HEMO DIALYSIS';
  rnHemoDialysis    = 'DIALYSIS,HEMO';
  rnAdmin           = 'ADMIN';
  rnUser            = 'USER';
  // Option Types
  otUser         = 'OPTIONS_USER';
  otUserSettings = 'OPTIONS_SETTINGS';
  otSystem       = 'OPTIONS_SYSTEM';
  otDeveloper    = 'OPTIONS_DEVELOPER';
  // Option Names ==============================================================
  // =========== DO NOT CHANGE THEM IF YOU HAVE NO GARBAGE CLEANER ==============
  // ============================================================================
  // Tables ....................................................................
  onDictionary        = 'CDSDICT'; // option name for Dictionary
  onAccessLogTable    = 'AccessLogTable';
  onStudyListTable    = 'StudyList';
  onWatchListTable    = 'WatchListTable';
  onWatchListLogTable = 'WatchListLogTable';
  onSubmittedStudies  = 'Submitted Studies';
{$IFDEF SAVE_ERRORS_LOG}
  onErrorLog = 'Errors';
{$ENDIF}
  // Lists of names of the system options ......................................
  onReportList    = 'Report List';    // list of the report template names
  onNotes         = 'Notes';          // List of the Notes
  onNoteTemplates = 'Note Templates'; // List of the TIU NOte Templates
  onXSLTList      = 'XSLT Templates'; // List of the XSLT Templates
  // Exception lists ...........................................................
  onAdministrators     = 'Administrators';
  onAntiCoagulants     = 'Anticoagulants';
  onCodeStatuses       = 'Code Statuses';
  onCustomDataLists    = 'Custom Data Lists';
  onDatasetNames       = 'Datasets';
  onDialyzerList       = 'Dialyzer List';
  onEducationCodes     = 'Education Codes';
  onESRD_Diagnosys     = 'ESRD Diagnoses';
  onESRD_ICD10         = 'ESRD ICD10'; // added 06/12/12 Joe Carney
  onMedications        = 'Medications';
  onMedicalFacilities  = 'Medical Facilities';
  onMedicationRoutes   = 'Medication Routes';
  onMedicationUnits    = 'Medication Units';
  onMedicationStatuses = 'Medication Statuses';
  onMedicationReasons  = 'Medication Reasons';
  onModalities         = 'Modalities';
  onNephrologiests     = 'Attending Nephrologists';
  onSurgeons           = 'Surgeons';
  onSysOptions         = 'System Options';
  onTransType          = 'Transportation Methods';
  onTIUNoteTitles      = 'TIU Note Titles'; //
  // a User Option Names .......................................................
  onUserOptions  = 'User Options';  // User Option List
  onUserSettings = 'User Settings'; // User Option List
  // Node names ================================================================
  onSingleNote = 'Single Note'; //
  nnTablesInfo = 'Tables';      // node 0
  // Preferences ===============================================================
  // kept within "System Options" and "User Options"
  // ============================================================================
  onPreferences       = 'Preferences';
  onUserPreferences   = 'User Preferences';
  onSystemPreferences = 'System Preferences';
  // ...........................................................................
{$IFDEF SHOWACCESSHISTORY}
  onAccessHistoryChecked = 'Show Access History';
{$ENDIF}
{$IFDEF CLEANUP}
  onAllowUserCleanOptionsList = 'Allow USER Clean Option List'; // ADMIN only
{$ENDIF}
  onAllowUserControlStudyStatus   = 'Allow USER control Study Status';
  onAllowUserDeleteBlankFSRecords = 'Allow USER delete blank F/S records';
  // onAllowUserInvalidateOtherUserComments = 'Allow USER invalidate other users comments';
  // onAllowUserEnableMedicationRecords = 'Allow USER enable invalid medication records';
{$IFDEF SHOWACCESSHISTORY}
  onAllowUserReviewAccessHistory = 'Allow USER Review Access History';
{$ENDIF}
{$IFDEF SHOWLOG}
  onAllowUserReviewLog = 'Allow USER Review Log';
{$ENDIF}
  // onAllowUserUnlockVALists = 'Allow USER Unlock VA selection lists';
  onAllowUserResetStudyStatus = 'Allow USER Reset Study Status';
{$IFDEF STUDY_STATUS_LIST}
  onAllowUserReleaseSelectedStudy       = 'Allow USER Release Selected Study';
  onAllowUserReleaseStudySelectedBefore = 'Allow USER Release Study Selected Before';
  onAllowUserResetActiveStudyList       = 'Allow USER Reset Active Study List';
{$ENDIF}
  // onAllowUserFixActiveStudyList = 'Allow USER Fix Active Study List';
  // zzzzzzandria 03202007 update
  onAllowUserAccessDisableStudies = 'Allow USER access disabled studies';
  /// /////////////////////////////////////////////////////////////////////////////
  onAllowDictionaryEdit   = 'Allow Edit Dictionary'; // Dev option
  onAlwaysReloadRx        = 'Always Reload Rx';
  onBlanksPlaceholder     = 'Blanks Placeholder';
  onBrokerTimeout         = 'Broker Timeout (sec)';
  onCoverRO               = 'Set the new study Cover to Read Only';
  onConsiderPainTolerance = 'Pain assessment based on how patient tolerates pain';
  // onDisableReadOnlyStudy = 'Disable Read/Only Study';
  // onDefaultTIUTitle = 'Default TIU Title';
  onFallsAssessmentAsTIU   = 'Falls Assessment as Separate TIU Note';
  onFlowseetTimeFormat     = 'Flowsheet Time Format';
  onIgnoreAcceptedPain     = 'Ignore pain if accepted by patient';
  onIgnoreUnfinishedStatus = 'Ignore Unfinished Status';
  // onIgnoreNotFoundDemoFiles = 'Ignore Not Found /DEMO Files';
  onIncludeInvalidatedCommentsInReport = 'Include Invalidated comments in reports';
  // zzzzzzandria 2007-08-29 always get prescription from the last study available
  // b. 314 #14
  // onGetFirstOrder = 'Get Prescription Order from the first study';
  // onMinDate = 'Min Date Available for Selection';
  onOrdersIEN                   = 'Physician Order IEN';
  onOverwriteManualInput        = 'Overwrite Manual Input'; // manual over Device
  onPainLevel                   = 'Pain Level';
  onRefreshRate                 = 'Flowsheet Refresh Rate (min)'; // system option name
  onReportKeyword               = 'Report Keyword';
  onReverseFlowsheetOrder       = 'Reverse Flowsheet Order'; // system option name
  onRequestSignatureForComments = 'Request signature on saving the application comments';
  onSaveFlowsheetVitals         = 'Save Flowsheet Vitals';
  onSaveVitals                  = 'Save Vitals';
{$IFDEF SAVE_ERRORS_LOG}
  onSaveErrorsLog = 'Save Errors Log'; // 2007-07-20
{$ENDIF}
  // onSmartSave = 'Smart Save';
  // onShowApplicationDictionary = 'Show Application Dictionary';
  // onShowColLinesInDataGrid = 'Show column lines in data grids';
  onShowDisabledStudies = 'Show Disabled Studies to Users';
  onShowReportSignature = 'Show report signature field';
{$IFDEF IGNORE_DUPLICATES}
  onShowFlowsheetDuplicates = 'Show duplicate flowsheet records';
{$ENDIF}
  onShowFlowsheetEventsCopies   = 'Show Flowsheet Event Copies';
  onShowInfectionTree           = 'Show Infectious Diseases information as Tree';
  onShowInvalidNotesInTheReport = 'Show invalid notes in the report';
  onShowPhysicianOrders         = 'Show Physician Orders';
  onShowPatientTreatmentHistory = 'Show patient treatment history';
  onShowReportTemplates         = 'Show Additional Reports';
  onShowStatusPanel             = 'Show Treatment Status Report';
  onShowTIUNoteTemplates        = 'Show TIU Note Templates';
  onSignatureKey                = 'SignatureKey'; // developer option name
  onSummaryReportName           = 'Summary Report Name';
  // onSummaryReportName = 'Result Report Name';
  onStudyLoadLimit         = 'Study Load Limit';
  onTimeoutInterval        = 'Application Timeout Interval (sec)';
  onWebPageURL             = 'Application Web Page URL';
  onReadOnlyFieldColor     = 'Color Read Only';
  onNormalFieldColor       = 'Color Editable';
  onDisabledFieldColor     = 'Color Disabled';
  onProblemFieldColor      = 'Color Required';
  onColorReview            = 'Color Review';
  onColorEditable          = 'Color Read/Write';
  onColorUnknown           = 'Color Unknown';
  onColorToolbar           = 'Color of Toolbars';
  onColorBackground        = 'Color of Background';
  onStudyListRefreshRate   = 'Study List Refresh Rate (sec)';
  onStudyListAutorefresh   = 'Study List Autorefresh';
  onShowStudyDetailsLegend = 'Show Study Details Legend';
  onShowStudyDetails       = 'Show Study Details';
  onShowDeletedMedicine    = 'Show Deleted and Replaced Medication Records';
  onShowPatientList        = 'Show Patient List';

  // onICD10Date = 'ICD-10 implementation date';  removed 09/26/2012 zzzzzzcarnej
  // .............................................................................
  // Developer Option Names ======================================================
  // are not accessible from the GUI
  // .............................................................................
  donRPCKeyword             = 'RPC Keyword';
  donRPCParametersDelimiter = 'RPC Parameters Delimiter';
  donSMTPHost               = 'SMTP Host';
  donIgnoreBlankDateFields  = 'Ignore blank Date fields';
  donSilentDefaultsRestore  = 'Silent Defaults Restore';
  donCleanOptions           = 'Cleanup options';
  donIncludeCommentsUpdate  = 'Include Comments Update';
  donShowAllComments        = 'Show All COmments';
  donWindowsXPTheme         = 'Windows XP Theme';
  donMaxProcedureQuantity   = 'Max Procedure Quantity';
  donMinESigLength          = 'Min e-Signature Length';
{$IFDEF DICTIONARY_BACKUP}
  donDictionaryBackup = 'DictionaryBackup';
{$ENDIF}
  donGatewayRequestTimeout              = 'Gateway Request Timeout (sec)';
  donFallRiskKeyword                    = 'FallsRiskKeyword';
  donFlowsheetDateInputMask             = 'Flowsheet Date Mask';
  donFlowsheetTimeInputMask             = 'Flowsheet Time Mask';
  donKeepSubmitHistory                  = 'Keep Submit History';
  donWebUserManualURL                   = 'Application User Manual URL';
  donNotifyPCE                          = 'Notify that PCE was not updated';
  donNotifyNewStudies                   = 'Notify that new studies were added';
  donRegisterStudyAccess                = 'Register Study Access';
  donRegisterStudyEvents                = 'Register Study Events';
  donSaveInstrumentMessages             = 'Save Instrument Messages';
  donShowHorizontalScrollBarsInReports  = 'Show horizontal scroll bars in reports';
  donShowHorizontalScrollBarsInComments = 'Show horizontal scroll bars in comments';
  donReportLineLength                   = 'Report Line Length';
  donShowFlowsheetDate                  = 'Show Date in Flowsheet';
  donShowMedicineDate                   = 'Show Date in Medication List';

var
  indexBlank: integer   = 0;
  indexOK: integer      = 1;
  indexWarning: integer = 2;

implementation

end.
