unit uMDK_Engine;

{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/22/08 10:39a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Engine object
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Engine.pas $
  *
  *       $ History: uMDK_Engine.pas $
}
interface

uses
  Classes, Windows, SysUtils, uMDK_Patients,
{$IFDEF AANTEST}
  Forms,
{$ENDIF}
  Controls, StdCtrls, oDBController, oDBController_Renal, oInterfaces, XMLIntf, DBClient, DB, uMDK_Summary,
  uMDK_Info, uMDK_BillingInfo, uMDK_VitalsAgent;

type
  TMDK_Engine = class(TObject)
  private
    fVendorPtID: string;
    fUserDUZ: string;
    fUserName: string;
    fUserTitle: string;
    fUserTimeout: string;
    fServerDelay: TDateTime;
    fStart: TDateTime;
    fFirstTimeAccessedStudy: Boolean;
    fStudyInfoCopied: Boolean;
    fCurrentOperatorID: string; // VistA DUZ
    fCurrentPatient: TMDKRegisteredPatient;
    fStudyNumList: TStringList;
    fBillingInfo: TMDK_BillingInfo;
    fStudyList: string;
    fVAHistory: string;
    fProblemList: string;
    fNow: TDateTime;
    fVitals: string;
    fQualifiers: string;
    fExeNameAndVersion: string;
    // Agents provide Interfaces
    // Vitals agent supports I_MDKVitals (Vital by names, Qualifiers...)
    fVitalsAgent: TMDK_VitalsAgent;
    fRenalConnection: I_DBConnection;
    fLocationName: string;
    fLocationID: string;
    fDictionaryXML: string;
    procedure setCurrentPatient(aPatient: TMDKRegisteredPatient);
    function getVAP_CDS: TClientDataSet;
    function getVAPH_CDS: TClientDataSet;
    function getVAPIH_CDS: TClientDataSet;
    function getInfo_CDS: TClientDataSet;
    function getFlowSheet_CDS: TClientDataSet;
    function getMedList_CDS: TClientDataSet;
    function getSummary_CDS: TClientDataSet;
    function getSummaryHistory_CDS: TClientDataSet;
    function getNoteList_CDS: TClientDataSet;
    function getStudyEvents_CDS: TClientDataSet;
    function getVAPH_DS: TDataSource;
    function getVAPIH_DS: TDataSource;
    function getInfo_DS: TDataSource;
    function getFlowSheet_DS: TDataSource;
    function getSummary_DS: TDataSource;
    function getSummaryHistory_DS: TDataSource;
    procedure LoadCurrentPatientData;
    function getPatientAllergies: string;
    function getPatientDirectives: string;
    function getPatientWarnings: string;
    function getPatientLabInfo: string;
    function getPatientVaccination: string;
    function getDuration: TDateTime;
    function getStationNumber: string;
    function getStationMode: string;
    function getVendorPtID: string;
    function getSignature: string;
    function getCurrentOperatorMode: string;
    function getPCEBillingStatus: string;
    function getPCEBillingStatusDescription: string;
    function getStudyDate: string;
    function getQualifierIEN(aVitalName, aQualifier: string): string;
  public
    InvalidESigCount: Integer;
    fRenalController: TDBController_Renal;
    constructor Create;
    destructor Destroy; override;
    procedure InitDS(aDS: TClientDataSet; aString: string; Clear: Boolean = True);
    property UserDUZ: string read fUserDUZ;
    property UserName: string read fUserName;
    property UserTitle: string read fUserTitle;
    property UserTimeout: string read fUserTimeout;
    property StationNumber: string read getStationNumber;
    property StationMode: string read getStationMode;
    property StudyDate: string read getStudyDate;
    property LocationName: string read fLocationName write fLocationName;
    property LocationID: string read fLocationID write fLocationID;
    procedure getLocation;
    function setLocation(aLocation: string): Boolean;
    function ConvertToFmDate(impdate: TDateTime): string;
    property Signature: string read getSignature;
    // Report event to the event log ===========================================
    procedure Report(aType, aComment: string; aSession: string = ''; aDuration: TDateTime = 0; anActor: string = '';
      anAction: string = ''; aResult: string = ''; aLocal: Boolean = False);
    property Duration: TDateTime read getDuration;
    // =========================================================================
    procedure setUpField(aF: TField; TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage, TheLabel,
      TheMax, TheMin: string);
    procedure setUpFields;
    procedure LoadDictionaryFromDB(aName: string = '');
    function LoadDictionaryFromString(aValue: string): Boolean;
    procedure SaveDictionaryToDB(aName: string = '');
    function DictionaryOvewriteManualInput(aFN: string): Boolean;
    // Client Data Sets ========================================================
    property VAP_CDS: TClientDataSet read getVAP_CDS;
    property VAPH_DS: TDataSource read getVAPH_DS;
    property VAPH_CDS: TClientDataSet read getVAPH_CDS;
    property VAPIH_CDS: TClientDataSet read getVAPIH_CDS;
    property VAPIH_DS: TDataSource read getVAPIH_DS;
    property Info_CDS: TClientDataSet read getInfo_CDS; // uMDK_Info.pas
    property Info_DS: TDataSource read getInfo_DS;
    property Summary_DS: TDataSource read getSummary_DS;
    property Summary_CDS: TClientDataSet read getSummary_CDS;
    property SummaryHistory_DS: TDataSource read getSummaryHistory_DS;
    property SummaryHistory_CDS: TClientDataSet read getSummaryHistory_CDS;
    property FlowSheet_CDS: TClientDataSet read getFlowSheet_CDS;
    property Flowsheet_DS: TDataSource read getFlowSheet_DS;
    property MedList_CDS: TClientDataSet read getMedList_CDS;
    property NoteList_CDS: TClientDataSet read getNoteList_CDS;
    property StudyEvents_CDS: TClientDataSet read getStudyEvents_CDS;
    // =========================================================================
    function getPatientList: string;
    function getRegisteredPatientByDFN(aDFN, aStudy: string): TMDKRegisteredPatient;
    function getPatientInfo: string;
    function IsItNewInfo: Boolean;
    function getInfoData: TMDK_Info;
    procedure saveInfoData(aValue: TMDK_Info);
    function SelectHistoryStudy(aValue: Integer): string;
    function SelectHistoryStudyByDate(aValue: string): string;
    property CurrentPatient: TMDKRegisteredPatient read fCurrentPatient write setCurrentPatient;
    property CurrentOperatorID: string read fCurrentOperatorID write fCurrentOperatorID;
    property CurrentOperatorMode: string read getCurrentOperatorMode;
    // Creates controller and builds Patient List ==============================
    function StartUpRenalController(aLocal: Boolean): Boolean;
    procedure ShutDownRenalController;
    procedure SaveCurrentPatientData;
    // CoverSheet Properties ===================================================
    property VendorPtID: string read getVendorPtID;
    property VAHistory: string read fVAHistory;
    property PatientAllergies: string read getPatientAllergies;
    property PatientDirectives: string read getPatientDirectives;
    property PatientWarnings: string read getPatientWarnings;
    property PatientVaccination: string read getPatientVaccination;
    property StudyList: string read fStudyList;
    function getCurrentInfo: TMDK_Info;
    // function getPatientStudyDates(aPatientID: Integer): string;
    function getPatientStudyDates(aPatientID: Int64): string;
    // RxLab properties ========================================================
    property PatientLabInfo: string read getPatientLabInfo;
    function getPatientLabInfoForPeriod(aParam: string): string;
    procedure ReloadOrder;
    // Pre-Treatment ===========================================================
    // Flowsheet================================================================
    function getFlowsheetData(bAllRecords: Boolean; sPatientID: string = ''; sStudyID: string = ''): string;
    procedure setInstrumentRecord(aPatient, aStudy, aValue, aSource: string);
    property ProblemList: string read fProblemList write fProblemList;
    // Post-Treatment ==========================================================
    // Summary =================================================================
    function getSummary: TStringList;
    function ValidateSignature(aValue: string): Boolean;
    function ValidBillingInfo(var aComment: string; Loud: Boolean = True): Boolean;
    procedure SubmitFlowsheet;
    function SubmitVitalsData: string;
    function Submit(aStudy: Integer; aSignature, aReportString: string): string;
    function getProviderList(aTarget: string): string;
    function getProvider(aTarget: string): string;
    property BillingInfo: TMDK_BillingInfo read fBillingInfo;
    function IsPatternCorrect(apattern: string): Boolean;
    function getPatternValue(apattern: string): string;
    function getReportValue(aTemplate: string): string;
    function getReportString: string;
    function ConvertDeviceData(const aDeviceName, aData: string): string;
    procedure getColumnFormatData(aTable, aColumn: string; var TheLabel, TheUnit, TheFormat, TheEditMask: string);
    procedure setColumnFormat(aTable, aColumn, TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage,
      TheScale, TheLabel: string);
    // options ==================================================================
    function getSystemOptionByName(aName: string): string;
    function setSystemOptionByName(aName, aValue: string): Boolean;
    function getUserOptionByName(aName: string): string;
    procedure setUserOptionByName(aName, aValue: string);
    function deleteUserOption(aName: string): Boolean;
    function deleteSystemOption(aName: string): Boolean;
    // =========================================================================
    function getReportList: string;
    function getFromAddress: string;
    function getToAddress: string;
    function getSMTPHost: string;
    function LockSystemOption(aName: string): Boolean;
    function UnLockSystemOption(aName: string): Boolean;
    procedure ReportStatus(aPage: string; aCode: Integer; aComment: string);
    function SubmitNote(aUserID: string; aUserESig: string;
      // The users E-Sig in clear text, it will be encrypted before sending
      aDocumentText: TStringList; aTitle: string): string;
    function getStudyNotes: string;
    function getNotesByStudyID(anID: string): string;
    function getNoteByNum(aNoteID: string; aStudyID: string = ''): string;
    // study Lock/Unlock
    function LockStudy(aStudy: Integer): Boolean;
    function UnLockStudy(aStudy: Integer): Boolean;
    function getComputerID: string;
    /// /////////////////////////////////////////////////////////////////////////
    function getClinicProcedures(aDFN, aClinic: string; var aReturnList: string): Boolean;
    function getClinicDiagnosis(aDFN, aClinic: string; var aReturnList: string): Boolean;
    function getServiceConnections(aDFN, aClinic: string; var aReturnList: string): Boolean;
    function getCPTByTarget(aTarget: string; StudyID: string; var aReturnList: string): Boolean;
    function getICDByTarget(aTarget: string; StudyID: string; var aReturnList: string): Boolean;
    function getBillingData(aStudy: string; var aResult: string): Boolean;
    function getEncounterData(aStudy: string; var aResult: string): Boolean;
    function setEncounterData(aStudy: string; aBillingData: TStringList; var aResult: string): Boolean;
    function PCEReadyToSubmit: Boolean;
    procedure PCESetReviewed;
    procedure PCESetUpdated;
    property PCEBillingStatus: string read getPCEBillingStatus;
    property PCEBillingStatusDescription: string read getPCEBillingStatusDescription;
    function getPCEInfo: string;
    function getUserNameByID(anIEN: string): string;
    function getGatewayStatus: string;
    function getGatewayInfo: string;
    function getVistaImpDate: string;
    function CheckCDSData(aCDS: TClientDataSet; aStep, aField, aValue: string; Loud: Boolean = False): Boolean;
    property ExeNameAndVersion: string read fExeNameAndVersion;
    function getCurrentServerDateTimeString: string;
    function getCurrentServerDateTime: TDateTime;
    function getServerWDelay: TDateTime;
    procedure setServerDelay;
    property ServerDelay: TDateTime read getServerWDelay write fServerDelay;
    function getRPCResults(aParamString: string): string;
    function getStudyStatus(aStudy: string): string;
    procedure SaveUserOptions;
    procedure SaveSystemOptions;
    function RegisterAccessToSensitiveData(anID: string): string;
    function IsDataSensitive(aPatientID: string): Boolean;
    procedure xmlSavePatientData;
    function getPatientAccessInfo(aDFN: string): string;
    function getPatientBulletinInfo(aDFN: string): string;
    function setStudyStatusPending(aStudy: string): string;
    function RestoreDefaults: string;
    function RestoreOptionDefault(anOption: string): string;
    procedure CleanUpOptions;
    function getCP_TIU_TITLES(aTarget: string): string;
    function getAppointmentInfo: string;
    function VersionIsCompatible: Boolean;
    function getPatientPhysicianOrders: string;
    function isESigValid(aSig: string): Boolean;
  end;

function ValidLocationName(aName, anIEN: string): Boolean;
procedure AssignList(aStrings: TStrings; aValue: string; Len: Integer = 999);
procedure CheckSummaryReport;
// zzzzzzandria 2012-06-14 ICD-10 Remediation v. 1.0.29.10
function getICD10Date: string;
function getValues(anOptionName: string): string;

var
  Engine: TMDK_Engine;
  sSecurityWarning: string;

implementation

uses
  dMDK_Data, fMDK_Main, // to show report line in the status bar and fake IDs
  uXML, Variants, uROR_CommonUtils, Dialogs, uMDK_Common, uMDK_InstrumentRecord, uROR_RPCBroker, fMDK_Wait,
  fMDK_SelectProvider, fMDK_Summary, fMDK_Options, uMDK_Options, uMDK_CDSCommon, fROR_PCall, uMDK_VADataSet,
  fMDK_Access, uMDK_Vitals, uMDK_MedList,
{$IFDEF AANTEST}
  fMDK_Flowsheet,
{$ENDIF}
  iMDK_VitalsQualifiers, uMDK_ReportEngine, uMDK_Notes, uGMV_VersionInfo, uMDK_SubmitHistory, uMDK_Dictionary,
  System.UITypes, uMDK_Flowsheet;
/// /////////////////////////////////////////////////////////////////////////////

procedure AssignList(aStrings: TStrings; aValue: string; Len: Integer = 999);
var
  SL: TStringList;
  s: string;
  i: Integer;
begin
  if aValue = '' then Exit;
  SL := TStringList.Create;
  SL.Text := aValue;
  aStrings.Clear;
  for i := 0 to SL.Count - 1 do begin
    s := Piece(SL[i], '=', 2);
    s := Piece(s, ' ', 1, Len);
    if s <> '' then aStrings.Add(s);
  end;
  SL.Free;
end;

function ValidLocationName(aName, anIEN: string): Boolean;
var
  TheIEN: string;
  tmpList: TStringList;
begin
  {
    Example A: Legitimate hospital location
    >S OPTION="NAME",DATA="44^HEMO LAB^.01"
    >D RPC^GMVRPCHL(.RESULT,OPTION,DATA) ZW RESULT
    RESULT="^TMP("GMVHLOC",539075250)"
    >D ^%G
    Global ^TMP("GMVHLOC",$J -- NOTE: translation in effect
    ^TMP("GMVHLOC",539075250,0)=1
    1)=44;77^HEMO LAB
  }
  Result := False;
  tmpList := Engine.fRenalController.getLookupEntries('44', aName);
  if tmpList.Count = 2 then begin
    TheIEN := Piece(Piece(tmpList[1], '^', 1, 1), ';', 2);
    Result := (UpperCase(anIEN) = UpperCase(TheIEN));
  end;
  tmpList.Free;
end;

procedure CheckSummaryReport;
var
  _sValue, _sName: string;
  i: Integer;
  SL: TStringList;
begin
  _sName := Preferences.sValue(onSummaryReportName);
  if _sName <> '' then begin
    _sValue := Engine.getSystemOptionByName(_sName);
    if (_sValue = '') and (UpperCase(_sName) = 'SUMMARY REPORT') then begin
      if MessageDLG('The system report "' + _sName + '" template is blank.' + #13#10 +
        'Do you want restore the default value of template?', mtWarning, [mbOk, mbCancel], 0) = mrOK then begin
        _sValue := getDefaultResource(_sName);
        try
          Engine.setSystemOptionByName(_sName, _sValue);
          (* *)
          _sValue := Engine.getSystemOptionByName(onReportList);
          SL := TStringList.Create;
          SL.Text := _sValue;
          for i := 0 to SL.Count - 1 do begin
            if pos(_sName, SL[i]) = 1 then break;
          end;
          // Result := i < sl.Count;
          if i = SL.Count then begin
            SL.Insert(0, _sName + '=' + _sName);
            Engine.setSystemOptionByName(onReportList, SL.Text);
          end;
          SL.Free;
          (* *)
        except
        end;
      end;
    end;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

constructor TMDK_Engine.Create;
begin
  inherited;
  fStudyNumList := TStringList.Create;
  fBillingInfo := TMDK_BillingInfo.Create;
  fStart := Now;
  fCurrentPatient := TMDKRegisteredPatient.Create('', '', '', 0, '', '', '', '', '', '');
  fCurrentPatient.ReadOnly := True;
  fVitals := '';
  fQualifiers := '';
  fServerDelay := fDefaultServerTimeDelay;
  fExeNameAndVersion := CurrentExeNameAndVersion;
end;

destructor TMDK_Engine.Destroy;
begin
  ShutDownRenalController;
  FreeAndNil(fStudyNumList);
  FreeAndNil(fBillingInfo);
  FreeAndNil(fCurrentPatient);
  if fVitalsAgent <> nil then FreeAndNil(fVitalsAgent);
  inherited;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.StartUpRenalController(aLocal: Boolean): Boolean;
var
  SL: TStringList;
  procedure CreateRenalController;
  begin
    fStart := Now;
    try
      if aLocal then fRenalController := TDBController_Renal.Create(nil)
      else fRenalController := NewDBController_Renal(RPCBroker);
      dmData.Action := 'Create Renal Controller';
      Report(etDebug, 'Created', 'Startup', Duration);
      try
        FreeAndNil(fVitalsAgent);
        fVitalsAgent := TMDK_VitalsAgent.CreateByRenalController(fRenalController);
      except
        if fVitalsAgent <> nil then FreeAndNil(fVitalsAgent);
      end;
    except
      try
        if fRenalController <> nil then fRenalController.Free;
        if fVitalsAgent <> nil then FreeAndNil(fVitalsAgent);
      except
      end;
      Report(etDebug, 'Renal Controller Failed', 'Startup', Duration);
    end;
  end;

begin
  try
    if Assigned(RPCBroker) then begin
      fCurrentOperatorID := RPCBroker.User.Name;
      fUserDUZ := RPCBroker.User.DUZ;
    end
    else fUserDUZ := CurrentOperatorID;
  except
    fCurrentOperatorID := 'Unknown Connection';
  end;
  CreateRenalController;
  if fRenalController <> nil then begin
    SL := TStringList.Create;
    SL.Text := fRenalController.getUserInfo;
    if SL.Count > 6 then begin
      fUserName := SL[0];
      fUserTitle := SL[4];
      fUserTimeout := SL[7];
    end;
  end;
  Result := fRenalController <> nil;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.ShutDownRenalController;
begin
  try
    if Assigned(fRenalConnection) then fRenalConnection.Close;
  except
  end;
  RefreshPatientList(MDKPatientList);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getDuration: TDateTime;
begin
  Result := 24 * 3600 * (Now - fStart);
end;

procedure TMDK_Engine.setCurrentPatient(aPatient: TMDKRegisteredPatient);
begin
  if aPatient = nil then begin
    fCurrentPatient.ID := '';
    fCurrentPatient.Name := '';
    fCurrentPatient.Station := '';
    fCurrentPatient.SessionID := '';
    fCurrentPatient.Machine := '';
    fCurrentPatient.StudyStatus := '';
    fCurrentPatient.Sex := '';
    fCurrentPatient.SSN := '';
    fCurrentPatient.DOB := 0;
    fCurrentPatient.VendorID := '';
    dmData.PatientID := '';
    dmData.SessionID := '';
    fCurrentPatient.ReadOnly := True;
  end else begin
    fCurrentPatient.Name := aPatient.Name;
    fCurrentPatient.ID := aPatient.ID;
    fCurrentPatient.DOB := aPatient.DOB;
    fCurrentPatient.Station := aPatient.Station;
    fCurrentPatient.SessionID := aPatient.SessionID;
    fCurrentPatient.Machine := aPatient.Machine;
    fCurrentPatient.StudyStatus := aPatient.StudyStatus;
    fCurrentPatient.Sex := aPatient.Sex;
    fCurrentPatient.SSN := aPatient.SSN;
    fCurrentPatient.VendorID := aPatient.VendorID;
    dmData.PatientID := aPatient.ID;
    dmData.SessionID := aPatient.SessionID;
    fCurrentPatient.ReadOnly := aPatient.ReadOnly;
  end;
  LoadCurrentPatientData;
  InvalidESigCount := 0;
end;

function TMDK_Engine.getRegisteredPatientByDFN(aDFN, aStudy: string): TMDKRegisteredPatient;
var
  i: Integer;
begin
  Result := nil;
  if not Assigned(MDKPatientList) then Exit;
  for i := 0 to MDKPatientList.Count - 1 do
    try
      if (aDFN = TMDKRegisteredPatient(MDKPatientList.Objects[i]).ID) and
        ((aStudy = TMDKRegisteredPatient(MDKPatientList.Objects[i]).SessionID) or (aStudy = '')) then begin
        Result := TMDKRegisteredPatient(MDKPatientList.Objects[i]);
        Exit;
      end;
    except
    end;
end;

function TMDK_Engine.getPatientList: string;
var
  sReport: string;
  s: string;
  SL: TStringList;
  ss: string;
  i: Integer;
begin
  fStart := Now;
  Result := '';
  if not Assigned(fRenalController) then Report(etDebug, 'Renal Controller is not assigned')
  else begin
    sReport := 'No Active Patients found';
    if fRenalController.GetActiveRenalPatients(s) then begin
      Result := s;
      sReport := s;
      // zzzzzzandria 20070228
{$IFDEF AANTEST}
      Report(etDebug, s, '', Duration, '', 'Get Study List');
{$ENDIF}
    end;
    SL := TStringList.Create;
    SL.Text := s;
    s := '';
    for i := 0 to SL.Count - 1 do begin
      ss := SL[i];
      if pos('<PATIENT_SSN>', ss) <> 0 then ss := '<PATIENT_SSN>**********</PATIENT_SSN>';
      if pos('<PATIENT_SEX>', ss) <> 0 then ss := '<PATIENT_SEX>**********</PATIENT_SEX>';
      if pos('<PATIENT_DOB>', ss) <> 0 then ss := '<PATIENT_DOB>**********</PATIENT_DOB>';
      s := s + ss + #13#10;
    end;
    Report(etDebug, s { Report } , '', Duration, '', 'Get Study List');
  end;
end;

procedure TMDK_Engine.Report(aType, aComment: string; aSession: string = ''; aDuration: TDateTime = 0;
  anActor: string = ''; anAction: string = ''; aResult: string = ''; aLocal: Boolean = False);
begin
  dmData.Results := aResult;
  if anActor <> '' then dmData.Actor := anActor
  else dmData.Actor := fUserDUZ;
  if anAction <> '' then dmData.Action := anAction;
  try
    if Assigned(fCurrentPatient) then
        dmData.RegisterEvent(aType, aComment, fCurrentPatient.SessionID, aDuration, aLocal)
    else dmData.RegisterEvent(aType, aComment, '', aDuration, aLocal);
  except
  end;
{$IFDEF AANTEST}
  frmMain.StatusBar.Panels[0].Text := ' ' + aComment;
  Application.ProcessMessages;
  frmMain.timer.Enabled := True;
{$ENDIF}
end;

// DATA SETS //////////////////////////////////////////////////////////////////

function TMDK_Engine.getVAP_CDS: TClientDataSet;
begin
  Result := dmData.cdsAP;
end;

function TMDK_Engine.getVAPH_CDS: TClientDataSet;
begin
  Result := dmData.cdsAPHistory;
end;

function TMDK_Engine.getVAPIH_CDS: TClientDataSet;
begin
  Result := dmData.cdsVAInfection;
end;

function TMDK_Engine.getSummary_CDS: TClientDataSet;
begin
  Result := dmData.cdsSummary;
end;

function TMDK_Engine.getSummaryHistory_CDS: TClientDataSet;
begin
  Result := dmData.cdsStudyHistory;
end;

function TMDK_Engine.getInfo_CDS: TClientDataSet;
begin
  Result := dmData.cdsInfo;
end;

function TMDK_Engine.getFlowSheet_CDS: TClientDataSet;
begin
  Result := dmData.cdsFlowsheet;
end;

function TMDK_Engine.getMedList_CDS: TClientDataSet;
begin
  Result := dmData.cdsMedList;
end;

function TMDK_Engine.getNoteList_CDS: TClientDataSet;
begin
  Result := dmData.cdsNotes;
end;

function TMDK_Engine.getStudyEvents_CDS: TClientDataSet;
begin
  Result := dmData.cdsStudyEvents;
end;

// DATA SOURCES ////////////////////////////////////////////////////////////////

function TMDK_Engine.getVAPH_DS: TDataSource;
begin
  Result := dmData.dsAPHistory;
end;

function TMDK_Engine.getVAPIH_DS: TDataSource;
begin
  Result := dmData.dsVAInfection;
end;

function TMDK_Engine.getSummary_DS: TDataSource;
begin
  Result := dmData.dsSummary;
end;

function TMDK_Engine.getSummaryHistory_DS: TDataSource;
begin
  Result := dmData.dsStudyHistory;
end;

function TMDK_Engine.getInfo_DS: TDataSource;
begin
  Result := dmData.dsInfo;
end;

function TMDK_Engine.getFlowSheet_DS: TDataSource;
begin
  Result := dmData.dsFlowsheet;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.InitDS(aDS: TClientDataSet; aString: string; Clear: Boolean = True);
var
  sValue: string;
begin
  sValue := CDSInitByString(aDS, aString, Clear);
  if sValue <> '' then Report(etWarning, aString, '', 0, sValue);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getPatientAllergies: string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    fStart := Now;
    fRenalController.getPatientAllergies(CurrentPatient.ID, SL);
    if (SL.Count > 0) and (pos('Allergies', SL[0]) = 1) then SL.Delete(0);
    Report(etDebug, SL.Text, '', Duration, '', 'Get Patient Allergies');
    Result := SL.Text;
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getPatientDirectives: string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    fStart := Now;
    fRenalController.getPatientDirectives(CurrentPatient.ID, SL);
    Report(etDebug, SL.Text, '', Duration, '', 'Get Patient Advanced Directives');
    Result := SL.Text;
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getPatientWarnings: string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    fStart := Now;
    fRenalController.getPatientWarnings(CurrentPatient.ID, SL);
    Report(etDebug, SL.Text, '', Duration, '', 'Get Patient Warnings');
    Result := SL.Text;
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getPatientLabInfoForPeriod(aParam: string): string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    if aParam = '' then fRenalController.getPatientLabInfo(CurrentPatient.ID, SL)
    else fRenalController.getPatientLabInfo(CurrentPatient.ID + '^' + aParam, SL);
    Result := SL.Text;
    if (Result = '') or (Result = #13#10) then
        Report(etWarning, 'No Lab Data Found', '', Duration, ClassName, 'Get Patient Lab Info')
    else Report(etDebug, SL.Text, '', Duration, '', 'Get Patient Lab Info')
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getPatientLabInfo: string;
begin
  Result := getPatientLabInfoForPeriod('');
end;

function TMDK_Engine.getPatientVaccination: string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    fStart := Now;
    fRenalController.getPatientVaccination(CurrentPatient.ID, SL);
    Result := SL.Text;
    if Result = '' then Report(etDebug, 'No Vaccination data found', '', Duration, '', 'Get Patient Vaccination')
    else Report(etDebug, SL.Text, '', Duration, '', 'Get Patient Vaccination');
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getPatientInfo: string;
var
  SL: TStringList;
begin
  SL := nil;
  Result := '';
  if not Assigned(fRenalController) then Exit;
  try
    SL := TStringList.Create;
    fStart := Now;
    fRenalController.getPatientInfo(CurrentPatient.ID, SL);
    Result := SL.Text;
    Report(etDebug, 'Patient Information', '', Duration, '', 'Get Patient Info');
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.getUserNameByID(anIEN: string): string;
var
  SL: TStringList;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  SL := TStringList.Create;
  try
    SL.Text := fRenalController.getUserNameByIEN2(anIEN);
    Result := SL[0];
  except
    Result := '';
  end;
  SL.Free;
end;

// CURRENT PATIENT ////////////////////////////////////////////////////////////

function TMDK_Engine.CheckCDSData(aCDS: TClientDataSet; aStep, aField, aValue: string; Loud: Boolean = False): Boolean;
var
  bb: Boolean;
  s, ss: string;
begin
  bb := False;
  aCDS.DisableControls;
  try
    bb := dmData.CheckCDSIDField(aCDS, aField, aValue, ss);
    if not bb then begin
      s := 'Invalid value <' + ss + '> at the <' + aField + '> field' + #13 + 'Dataset <' + aCDS.Name + '>' + #13 +
        'Study ' + aValue;
      Report(rtError, s, '', Duration, '', aStep + ' Error in study Data', '', True);
      if Loud then
          MessageDLG(s + #13#10#13#10 + 'The error registered in the study events log' + #13#10 +
          'Press OK to continue', mtInformation, [mbOk], 0);
      try
        aCDS.MergeChangeLog;
        Report(rtError, aCDS.XMLData, '', Duration, '', 'Error in study ID for <' + aCDS.Name + '>', '', True);
      except
      end;
    end;
  except
  end;
  aCDS.EnableControls;
  Result := bb;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.SaveCurrentPatientData;
var
  intStudy: Integer;
  // intPatient: Int64;
  sPatientID: string;
  sStudyID: string;
  procedure MergeCDS(aCDS: TClientDataSet);
  begin
    try
      try
        if Assigned(aCDS) and (aCDS.State = dsEdit) then aCDS.Post;
      except
      end;
      aCDS.MergeChangeLog;
    except
{$IFDEF AANTEST}
      on E: Exception do
          Report(rtError, 'Patient ' + fCurrentPatient.ID, '', Duration, '', 'Merge Change Log <' + aCDS.Name + '>',
          'Failed', True);
{$ENDIF}
    end;
  end;
  procedure SaveStudyItem(aFile, aNode, anItemID, anItemName, anOld: string; aCDS: TClientDataSet);
  begin
    aCDS.DisableControls;
    MergeCDS(aCDS);
    fStart := Now;
    if anOld <> aCDS.XMLData then
      if fRenalController.SetStudyItem(aFile, aNode, anItemID, aCDS.XMLData, CurrentPatient.SessionID) then
          Report(rtDebug, anItemName + ' (' + anItemID + '):' + #13#10 + aCDS.XMLData, '', Duration, ClassName, 'Save')
      else Report(rtDebug, anItemName + ' (' + anItemID + '):' + #13#10 + aCDS.XMLData, '', Duration, ClassName, 'Save',
          'Failed');
    aCDS.EnableControls;
  end;

begin
  if not Assigned(fCurrentPatient) then Exit;
  if fCurrentPatient.Name = '' then Exit;
  if fCurrentPatient.ReadOnly then begin
    Report(rtDebug, 'No data  was saved for R/O study', '', Duration, '', 'No data was saved for R/O study', '', True);
    Exit;
  end;
  // Save Patient CDSs
  intStudy := fCurrentPatient.iStudy;
  sPatientID := fCurrentPatient.ID;
  sStudyID := Format('%d,', [intStudy]);
  frmAccess.SavePatientData;
  SaveStudyItem('704.201', '.1', sPatientID + ',', 'Access Points', fCurrentPatient.xmlStudyVAP, VAP_CDS);
  SaveStudyItem('704.201', '.2', sPatientID + ',', 'AP History', fCurrentPatient.xmlStudyVAPH, VAPH_CDS);
  SaveStudyItem('704.201', '.3', sPatientID + ',', 'AP Infection History', fCurrentPatient.xmlStudyVAPIH, VAPIH_CDS);
  try
    Info_CDS.Edit;
    Info_CDS.FieldByName(fnI_LastOpened).AsDateTime := Now;
    Info_CDS.FieldByName(fnI_LastUser).AsString := UserDUZ;
    Info_CDS.Post;
  except
  end;
  dmData.LockNotes;
  MergeCDS(Summary_CDS);
  MergeCDS(Info_CDS);
  MergeCDS(FlowSheet_CDS);
  MergeCDS(MedList_CDS);
  MergeCDS(NoteList_CDS);
  // check that the data contains correct patient ID and Study ID
  CheckCDSData(Info_CDS, 'Save Data: ', fnI_StudyID, fCurrentPatient.SessionID);
  CheckCDSData(Summary_CDS, 'Save Data: ', fnS_SessionID, fCurrentPatient.SessionID, True);
  CheckCDSData(FlowSheet_CDS, 'Save Data: ', fnFS_SessionID, fCurrentPatient.SessionID, True);
  CheckCDSData(MedList_CDS, 'Save Data: ', fnML_StudyID, fCurrentPatient.SessionID, True);
  CheckCDSData(NoteList_CDS, 'Save Data: ', fnNL_StudyID, fCurrentPatient.SessionID, True);
  SaveStudyItem('704.202', '.1', sStudyID, 'Info', fCurrentPatient.xmlStudyInfo, Info_CDS);
  SaveStudyItem('704.202', '.2', sStudyID, 'Summary', fCurrentPatient.xmlStudySummary, Summary_CDS);
  SaveStudyItem('704.202', '.3', sStudyID, 'Flowsheet', fCurrentPatient.xmlStudyFlowsheet, FlowSheet_CDS);
  SaveStudyItem('704.202', '.4', sStudyID, 'Med List', fCurrentPatient.xmlStudyMedList, MedList_CDS);
  SaveStudyItem('704.202', '.5', sStudyID, 'Notes', fCurrentPatient.xmlStudyNotes, NoteList_CDS);
  SaveStudyItem('704.202', '.6', sStudyID, 'Events', fCurrentPatient.xmlStudyEvents, StudyEvents_CDS);
  Report(etDebug, ' INFO: ' + IntToStr(Info_CDS.RecordCount) + ' Summary: ' + IntToStr(Summary_CDS.RecordCount) +
    ' Flowsheet: ' + IntToStr(FlowSheet_CDS.RecordCount) + ' Medlist: ' + IntToStr(MedList_CDS.RecordCount), '',
    Duration, UserDUZ, 'Save Check Point');
end;

function TMDK_Engine.getPatientStudyDates(aPatientID: Int64): string;
var
  _StudyList: string;
  function preparePatientStudyList(aList: string): string;
  var
    i: Integer;
    aDoc: IXMLDocument;
    ss, s: string;
  begin
    Result := '';
    if aList = '' then Exit;
    ss := '';
    fStudyNumList.Clear;
    try
      aDoc := XMLDocument(aList);
      with aDoc.DocumentElement do begin
        if ChildValues['STATUS'] <> 'OK' then Exit;
        for i := 0 to ChildNodes.Count - 1 do begin
          if ChildNodes[i].NodeName <> 'RECORD' then continue;
          try // reverse order update
            s := FormatDateTime(gMDK_DateFMT, FMDateToWindowsDate(StrToFloat(ChildNodes[i].ChildValues['STUDY_DATETIME']
              ))) + #13#10;
            Result := s + Result;
            fStudyNumList.Insert(0, ChildNodes[i].ChildValues['STUDY_ID']);
            ss := FormatDateTime(gMDK_DateFMT,
              FMDateToWindowsDate(StrToFloat(ChildNodes[i].ChildValues['STUDY_DATETIME']))) + ' -- ' +
              ChildNodes[i].ChildValues['STUDY_ID'] + #13#10 + ss;
          except
            on E: Exception do;
          end;
        end;
      end;
    except
    end;
    Report(rtDebug, ss, '', Duration, UserDUZ, 'Process Study List');
  end;

begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  // Study List ================================================================
  fStart := Now;
  frmWait.AddLogLine(char(VK_TAB) + 'Patient Studies:');
  if fRenalController.GetAllStudiesByPatient(aPatientID, _StudyList) then begin
    Report(rtDebug, _StudyList, '', Duration, UserDUZ, 'Study List Load', 'OK');
    frmWait.AppendLogLine(gLoaded);
  end else begin
    Report(rtDebug, IntToStr(aPatientID), '', Duration, UserDUZ, 'Study List Load', 'Failed');
    frmWait.AppendLogLine(gFailed);
  end;
  fStart := Now;
  Result := preparePatientStudyList(_StudyList);
end;

procedure TMDK_Engine.xmlSavePatientData;
begin
  if not Assigned(CurrentPatient) then Exit;
  CurrentPatient.xmlStudyInfo := Info_CDS.XMLData;
  CurrentPatient.xmlStudySummary := Summary_CDS.XMLData;
  CurrentPatient.xmlStudyFlowsheet := FlowSheet_CDS.XMLData;
  CurrentPatient.xmlStudyMedList := MedList_CDS.XMLData;
  CurrentPatient.xmlStudyNotes := NoteList_CDS.XMLData;
  CurrentPatient.xmlStudyEvents := StudyEvents_CDS.XMLData;
  CurrentPatient.xmlStudyVAP := VAP_CDS.XMLData;
  CurrentPatient.xmlStudyVAPH := VAPH_CDS.XMLData;
  CurrentPatient.xmlStudyVAPIH := VAPIH_CDS.XMLData;
end;

procedure TMDK_Engine.LoadCurrentPatientData;
var
  aSH: TStringList;
  fLatestSummary, fStudyHistory: string;
  i, j, jStudy: Integer;
  iLoadLimit: Integer;
  sBillingData, sInfo, sSummary, sFlowsheet, sMedList, sNotes, sEvents, sVAP, sVAPH, sVAPIH: string;
  OldInfo, NewInfo: TMDK_Info;
  TimeBM: TDateTime;
  procedure CleanUpDatasets;
  begin
    CleanUpCDS(VAPIH_CDS);
    CleanUpCDS(VAPH_CDS);
    CleanUpCDS(VAP_CDS);
    CleanUpCDS(Summary_CDS);
    CleanUpCDS(SummaryHistory_CDS);
    CleanUpCDS(FlowSheet_CDS);
    CleanUpCDS(Info_CDS);
    CleanUpCDS(MedList_CDS);
    CleanUpCDS(NoteList_CDS);
    CleanUpCDS(StudyEvents_CDS);
  end;
  procedure CleanUpStringBuffers;
  begin
    sMedList := '';
    sFlowsheet := '';
    sSummary := '';
    sInfo := '';
    sNotes := '';
    sEvents := '';
  end;
  procedure ReportLoad(aString: string);
  begin
    Report(rtDebug, 'Study data loaded: ' + aString + #13#10 + ' Info (' + IntToStr(Length(sInfo)) + ') Flowsheet (' +
      IntToStr(Length(sFlowsheet)) + ') Summary (' + IntToStr(Length(sSummary)) + ') MedList (' +
      IntToStr(Length(sMedList)) + ')', '', Duration, UserDUZ, 'Get Study History Data');
  end;
  function getMDK_Info: TMDK_Info;
  begin
    Result := nil;
    try
      Info_CDS.First;
      if not Info_CDS.Eof then Result := getInfoData;
    except
    end;
  end;
  procedure RestoreHistory;
  begin
    // Study List ================================================================
    fStudyList := getPatientStudyDates(fCurrentPatient.IntID);
    fLatestSummary := '';
{$IFNDEF NOHISTORY}
    // Study History =============================================================
    fStart := Now;
    frmWait.AddLogLine(char(VK_TAB) + 'Study History:');
    aSH := TStringList.Create;
    aSH.Text := fStudyList; // list of study numbers
    fStudyList := '';
    fStudyHistory := '';
    SummaryHistory_CDS.DisableControls;
    iLoadLimit := StrToIntDef(Preferences.sValue(onStudyLoadLimit), 99999);
    j := 0; // Study History
    i := 0; // Study list
    fStart := Now;
    TimeBM := fStart;
    while i < fStudyNumList.Count do begin
      jStudy := StrToInt(fStudyNumList[i]);
      if (fStudyNumList[i] = fCurrentPatient.SessionID) or (j >= iLoadLimit) or
        (StrToInt(fCurrentPatient.SessionID) <= jStudy)
      { //050610 zzzzzzandria } then begin // do not include study in the history list. We will Process it later.
        fStudyNumList.Delete(i);
        aSH.Delete(i);
        continue;
      end;
      CleanUpStringBuffers; // Clean up the buffers
      fStart := Now;
      if fRenalController.GetStudy(jStudy, sInfo, sSummary, sFlowsheet, sMedList, sNotes, sEvents) then begin
        if pos('-1', sSummary) = 1 then begin
          Report(rtDebug, 'No data found for study: ' + fStudyNumList[i], '', Duration, UserDUZ, 'Get Study History');
          fStudyNumList.Delete(i);
          aSH.Delete(i);
          continue;
        end else begin
          ReportLoad(fStudyNumList[i]);
          if fLatestSummary = '' then // zzzzzzandria 060609
              fLatestSummary := sSummary;
          if not Assigned(OldInfo) then
          // take the first not nil study Info
          begin
            InitDS(Info_CDS, sInfo);
            OldInfo := getMDK_Info;
            // will not create record if not found
            if OldInfo = nil then
                Report(rtDebug, 'No info data found for study: ' + fStudyNumList[i], '', Duration, UserDUZ,
                'Get Old Study Info')
            else Report(rtDebug, 'Info Data found for study: ' + fStudyNumList[i], '', Duration, UserDUZ,
                'Get Old Study Info');
          end;
          InitDS(SummaryHistory_CDS, sSummary, False); // get history data
          try
            if (fVendorPtID = '') and (SummaryHistory_CDS.FieldByName(fnS_VendorPTID).AsString <> '') then
                fVendorPtID := SummaryHistory_CDS.FieldByName(fnS_VendorPTID).AsString
            else if fVendorPtID <> SummaryHistory_CDS.FieldByName(fnS_VendorPTID).AsString then
                Report(rtDebug, ' Vendor Patient ID: ' + SummaryHistory_CDS.FieldByName(fnS_VendorPTID).AsString +
                ' Current Vendor Patient ID: ' + fVendorPtID, '', Duration, UserDUZ, 'Compare Vendor Pt IDs');
          except
          end;
          try
            fStudyList := fStudyList + aSH[i] + #13#10;
          except
          end;
          try
            if SummaryHistory_CDS.FieldByName(fnS_SessionDate).AsString = '' then begin
              SummaryHistory_CDS.Edit;
              SummaryHistory_CDS.FieldByName(fnS_SessionDate).AsString := aSH[i];
              SummaryHistory_CDS.Post;
            end;
            j := j + 1;
          except
          end;
          try // extract VA History Info into fVAHistory
            if (SummaryHistory_CDS.FieldByName(fnS_IABF).AsString <> '') or
              (SummaryHistory_CDS.FieldByName(fnS_AVP0).AsString <> '') or
              (SummaryHistory_CDS.FieldByName(fnS_VP0).AsString <> '') or
              (SummaryHistory_CDS.FieldByName(fnS_VP200).AsString <> '') then begin
              fVAHistory := fVAHistory + FormatDateTime(gMDK_DateFMT, SummaryHistory_CDS.FieldByName(fnS_Date)
                .AsDateTime) + '^' + SummaryHistory_CDS.FieldByName(fnS_IABF).AsString + '^' +
                SummaryHistory_CDS.FieldByName(fnS_AVP0).AsString + '^' + SummaryHistory_CDS.FieldByName(fnS_VP0)
                .AsString + '^' + SummaryHistory_CDS.FieldByName(fnS_VP200).AsString + '^' + #13#10;
            end;
          except
          end;
        end;
        inc(i);
      end else begin
        Report(rtDebug, 'Failed to load study: ' + fStudyNumList[i], '', Duration, '', 'Study History', 'Failed', True);
        fStudyNumList.Delete(i);
        aSH.Delete(i);
      end;
    end;
    frmWait.AppendLogLine(gLoaded);
    try
      if not SummaryHistory_CDS.Active then SummaryHistory_CDS.open;
      SummaryHistory_CDS.First;
    except
    end;
    SummaryHistory_CDS.EnableControls;
    fStart := TimeBM;
    if fStudyList <> '' then begin
      Report(rtDebug, fStudyList, '', Duration, '', 'Updated Study Dates');
      Report(rtDebug, fStudyNumList.Text, '', Duration, '', 'Updated Study Numbers');
    end
    else Report(rtDebug, 'No Study History Records Found', '', Duration, '', 'Load Study History');
    aSH.Free;
{$ENDIF}
  end;
  procedure UpdateComments;
    procedure CopyNote(aTag, aText, aField: string);
    begin
      AddNote(NoteList_CDS, aText, Engine.UserDUZ, // Engine.CurrentOperatorID,
        aTag, '',                                  // Item ID
        fsValid, Now,                              // Note Date is unknown
        Engine.getCurrentServerDateTime);
      try
        Summary_CDS.Edit;
        Summary_CDS.FieldByName(aField).AsString := '';
        Summary_CDS.Post;
      except
      end;
    end;

  begin
    if Summary_CDS.FieldByName(fnS_PreNotes).AsString <> '' then
        CopyNote(pnPre, Summary_CDS.FieldByName(fnS_PreNotes).AsString, fnS_PreNotes);
    if Summary_CDS.FieldByName(fnS_PostNotes).AsString <> '' then
        CopyNote(pnPost, Summary_CDS.FieldByName(fnS_PostNotes).AsString, fnS_PostNotes);
    if Summary_CDS.FieldByName(fnS_WorkflowNotes).AsString <> '' then
        CopyNote(pnFlow, Summary_CDS.FieldByName(fnS_WorkflowNotes).AsString, fnS_WorkflowNotes);
    if Summary_CDS.FieldByName(fnS_SummaryNotes).AsString <> '' then
        CopyNote(pnSummary, Summary_CDS.FieldByName(fnS_SummaryNotes).AsString, fnS_SummaryNotes);
  end;

begin
  if not Assigned(fRenalController) then Exit;
  CleanUpDatasets;
  fVAHistory := '';
  fVendorPtID := '';
  fFirstTimeAccessedStudy := True;
  OldInfo := nil;
  if fCurrentPatient.ID = '' then Exit;
  Report(rtInfo, 'Start Load By ' + ExeNameAndVersion, '', 0, '', 'Load Patient Data');
  // Vascular Access Points ====================================================
  fStart := Now;
  frmWait.AddLogLine(char(VK_TAB) + 'Access Points:');
  if fRenalController.getPatientAccessPoints(fCurrentPatient.IntID, sVAP, sVAPH, sVAPIH) then begin
    Report(rtDebug, 'Access Points: ' + #13 + #10 + sVAP + #13 + #10, '', Duration, UserDUZ, 'Load AP Info');
    Report(rtDebug, 'Access Points History: ' + #13 + #10 + sVAPH + #13 + #10, '', Duration, UserDUZ,
      'Load AP Assess History');
    Report(rtDebug, 'Access Points Infection History: ' + #13 + #10 + sVAPIH, '', Duration, UserDUZ,
      'Load AP Infection History');
    frmWait.AppendLogLine(gLoaded);
  end else begin
    Report(rtDebug, fCurrentPatient.PatientInfo, '', Duration, UserDUZ, 'AP Data Load', 'Failed');
    frmWait.AppendLogLine(gFailed);
  end;
  InitDS(VAP_CDS, sVAP);
  InitDS(VAPH_CDS, sVAPH);
  InitDS(VAPIH_CDS, sVAPIH);
  // Vascular Access Points ================================================ End
  RestoreHistory;
  // Get unfinished study ======================================================
  fStart := Now;
  frmWait.AddLogLine(char(VK_TAB) + 'Current Study Data:');
  sMedList := '';
  sFlowsheet := '';
  sSummary := '';
  sInfo := '';
  sEvents := '';
  sNotes := '';
  if fRenalController.GetStudy(fCurrentPatient.iStudy, sInfo, sSummary, sFlowsheet, sMedList, sNotes, sEvents) then
  begin
    Report(rtDebug, 'Info (' + IntToStr(Length(sInfo)) + '):' + #13 + #10 + sInfo, '', Duration, UserDUZ,
      'Load Study Info');
    Report(rtDebug, 'Summary (' + IntToStr(Length(sSummary)) + '):' + #13 + #10 + sSummary, '', Duration, UserDUZ,
      'Load Study Summary');
    Report(rtDebug, 'Flowsheet (' + IntToStr(Length(sFlowsheet)) + '):' + #13 + #10 + sFlowsheet, '', Duration, UserDUZ,
      'Load Study Flowsheet');
    Report(rtDebug, 'Medication List (' + IntToStr(Length(sMedList)) + '):' + #13 + #10 + sMedList, '', Duration,
      UserDUZ, 'Load Study Medication List');
    Report(rtDebug, 'Notes List (' + IntToStr(Length(sNotes)) + '):' + #13 + #10 + sNotes, '', Duration, UserDUZ,
      'Load Study Note List');
    Report(rtDebug, 'Events Log (' + IntToStr(Length(sEvents)) + '):' + #13 + #10 + sEvents, '', Duration, UserDUZ,
      'Load Study Event List');
    frmWait.AppendLogLine(gLoaded);
  end else begin
    Report(rtError, 'Study Load ', fCurrentPatient.PatientInfo, 0, '', 'Study Load', '', True);
    frmWait.AppendLogLine(gFailed);
  end;
  // StudyEvents ===========================================================
  fStart := Now;
  InitDS(StudyEvents_CDS, sEvents);
  Report(etDebug, 'Study Events: ' + #13 + #10 + sEvents, '', Duration, UserDUZ, 'Set Study Events');
  // Update Info ===============================================================
  fStart := Now;
  InitDS(Info_CDS, sInfo);
  try
    if Info_CDS.RecordCount > 1 then
        Report(rtError, 'Info Data too long: ' + IntToStr(Info_CDS.RecordCount) + ' Records found', '', 0, UserDUZ,
        'Get Current Study Info');
    Info_CDS.First;
    if Info_CDS.Eof then begin // the study is open for the first time
      NewInfo := TMDK_Info.Create;
      // getCurrentInfo; // will create an TMDK_Info
      fFirstTimeAccessedStudy := True;
      NewInfo.setEqualTo(OldInfo);                 // will do nothing if OldInfo is nil
      NewInfo.StudyID := CurrentPatient.SessionID; // zzzzzzandria 051228
      NewInfo.PatientID := CurrentPatient.ID;      // zzzzzzandria 051228
      fStudyInfoCopied := OldInfo <> nil;
      NewInfo.StudyDate := Now; // zzzzzzandria 050610
      Report(rtDebug, 'Study Info (First Time Accessed): ' + NewInfo.Description, '', 0, UserDUZ,
        'Get Current Study Info');
      NewInfo.saveToCDS(Info_CDS); // Will save data in the CDS
      // the next call to getCurrentInfo will return this value
      if Assigned(NewInfo) then FreeAndNil(NewInfo);
    end else begin
      fFirstTimeAccessedStudy := False;
      fStudyInfoCopied := False;
      Report(rtDebug, 'Last time saved ' { +Info_CDS.FieldByName(fnI_LastOpened).AsString } + ' by: ' +
        Info_CDS.FieldByName(fnI_LastUser).AsString + #13#10 + sInfo, '', 0, '', 'Get Current Study Info');
    end;
  except
  end;
  if Assigned(OldInfo) then FreeAndNil(OldInfo);
  // update fSummary ===========================================================
  fStart := Now;
  InitDS(Summary_CDS, sSummary);
  try
    Summary_CDS.First;
    if Summary_CDS.Eof then begin // if there is no record in Summary_CDS then insert record for the study
      try
        fStart := Now;
        AddDefaultSummary(Summary_CDS, fCurrentPatient.ID, fCurrentPatient.SessionID);
        Report(etDebug, 'Added New Summary Record', '', Duration, '', 'Add Default Summary', 'OK', True);
      except
        Report(etDebug, 'No Summary Added', '', Duration, '', 'Add Default Summary', 'Failed', True);
      end;
      ReloadOrder;
    end
    // if included then prescription is changed every time the study is selected
    else if Preferences.bValue(onAlwaysReloadRx) then ReloadOrder;
    fCurrentPatient.VendorID := Summary_CDS.FieldByName(fnS_VendorPTID).AsString;
    // 2007-08-20 b. 314 #3  Restore DONE status.
    if Summary_CDS.FieldByName(fnS_LastRecord).AsBoolean then fCurrentPatient.StudyStatus := STUDY_STATUS_DONE;
  except
  end;
  // update fFlowsheet =========================================================
  fStart := Now;
  FlowSheet_CDS.IndexName := '';
  InitDS(FlowSheet_CDS, sFlowsheet);
  FlowSheet_CDS.MergeChangeLog;
  if not FlowSheet_CDS.Active then FlowSheet_CDS.open;
  try
    try
      FlowSheet_CDS.IndexDefs.Clear;
      FlowSheet_CDS.AddIndex('TimeIndex', fnFS_Date + ';' + fnFS_Time, [ixDescending], '', '', 0);
    except
      on E: EDatabaseError do
          FlowSheet_CDS.AddIndex('TimeIndex', fnFS_Date + ';' + fnFS_Time, [ixDescending], '', '', 0);
    end;
    FlowSheet_CDS.IndexName := 'TimeIndex';
  except
    on E: Exception do Report(etDebug, 'Flowsheet Index creation failed ' + #13 + E.Message, '', 0);
  end;
  // Medication List ===========================================================
  fStart := Now;
  InitDS(MedList_CDS, sMedList);
  Report(etDebug, 'Medication list: ' + #13 + #10 + sMedList, '', Duration, '', 'Set Medication List');
  // Note List ===========================================================
  fStart := Now;
  InitDS(NoteList_CDS, sNotes);
  Report(etDebug, 'Note list: ' + #13 + #10 + sNotes, '', Duration, '', 'Set Note List');
{$IFDEF LOCK_COMMENTS}
  // Review and delete anonymous comments ======================================
  UpdateComments;
{$ENDIF}
  // Billing Info ==============================================================
  fStart := Now;
  fBillingInfo.Clear;
  fRenalController.getBillingData(fCurrentPatient.iStudy, sBillingData);
  fBillingInfo.XMLData := sBillingData;
  Engine.getEncounterData(fCurrentPatient.SessionID, sBillingData);
  fBillingInfo.Legacy := sBillingData;
  Report(etInfo, ' INFO (count): ' + IntToStr(Info_CDS.RecordCount) + ' Summary (count): ' +
    IntToStr(Summary_CDS.RecordCount) + ' Flowsheet (count): ' + IntToStr(FlowSheet_CDS.RecordCount) +
    ' Medlist (count): ' + IntToStr(MedList_CDS.RecordCount), '', Duration, '', 'Load Check Point');
  if fCurrentPatient.ReadOnly then
      Report(rtDebug, ExeNameAndVersion, '', Duration, '', 'Load R/O Patient Data', 'OK', True)
  else Report(rtDebug, ExeNameAndVersion, '', Duration, '', 'Load Patient Data', 'OK', True);
  CheckCDSData(Info_CDS, 'Load Data: ', fnI_StudyID, fCurrentPatient.SessionID);
  CheckCDSData(Summary_CDS, 'Load Data: ', fnS_SessionID, fCurrentPatient.SessionID, True);
  CheckCDSData(FlowSheet_CDS, 'Load Data: ', fnFS_SessionID, fCurrentPatient.SessionID, True);
  CheckCDSData(MedList_CDS, 'Load Data: ', fnML_StudyID, fCurrentPatient.SessionID, True);
  CheckCDSData(NoteList_CDS, 'Load Data: ', fnNL_StudyID, fCurrentPatient.SessionID, True);
  xmlSavePatientData;
{$IFDEF LOCK_COMMENTS}
  Report(etInfo, ' Info      (bytes): ' + IntToStr(Length(CurrentPatient.xmlStudyInfo)) + _CR_ + ' Summary   (bytes): '
    + IntToStr(Length(CurrentPatient.xmlStudySummary)) + _CR_ + ' Flowsheet (bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyFlowsheet)) + _CR_ + ' Med. List (bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyMedList)) + _CR_ + ' Notes     (bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyNotes)) + _CR_ + ' Events    (bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyEvents)) + _CR_ + ' Access Pts(bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyVAP)) + _CR_ + ' AP History(bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyVAPH)) + _CR_ + ' AP Infect.(bytes): ' +
    IntToStr(Length(CurrentPatient.xmlStudyVAPIH)), '', Duration, '', 'Load Check Point');
{$ENDIF}
  Report(rtInfo, 'Finish Load By ' + ExeNameAndVersion, '', 0, '', 'Load Patient Data');
end;
/// /////////////////////////////////////////////////////////////////////////////

function getInfoRecord(aCDS: TClientDataSet): TMDK_Info;
var
  aR: TMDK_Info;
begin
  Result := nil;
  try
    try
      aCDS.First;
    except
    end;
    if not aCDS.Eof then begin
      aR := TMDK_Info.Create;
      aR.PatientID := aCDS.FieldByName(fnI_PatientID).AsString;
      aR.StudyID := aCDS.FieldByName(fnI_StudyID).AsString;
      aR.StudyDate := aCDS.FieldByName(fnI_StudyDate).AsDateTime;
      aR.Diagnosis := aCDS.FieldByName(fnI_Diagnosis).AsString;
      aR.DiagnosisDate := aCDS.FieldByName(fnI_DiagnosisDate).AsDateTime;
      aR.TherapyDate := aCDS.FieldByName(fnI_TherapyDate).AsDateTime;
      aR.Modality := aCDS.FieldByName(fnI_Modality).AsString;
      aR.CodeStatus := aCDS.FieldByName(fnI_CodeStatus).AsString;
      aR.Nephrologist := aCDS.FieldByName(fnI_Nephrologist).AsString;
      aR.AD := aCDS.FieldByName(fnI_AD).AsBoolean;
      aR.Mon := aCDS.FieldByName(fnI_Mon).AsBoolean;
      aR.Tue := aCDS.FieldByName(fnI_Tue).AsBoolean;
      aR.Wed := aCDS.FieldByName(fnI_Wed).AsBoolean;
      aR.Thu := aCDS.FieldByName(fnI_Thu).AsBoolean;
      aR.Fri := aCDS.FieldByName(fnI_Fri).AsBoolean;
      aR.Sat := aCDS.FieldByName(fnI_Sat).AsBoolean;
      aR.Sun := aCDS.FieldByName(fnI_Sun).AsBoolean;
      // the next line commented on 2007-12-06 zzzzzzandria
      // aR.Sun := aCDS.FieldByName(fnI_AD).AsBoolean;
      Result := aR;
      Engine.Report(rtDebug, 'Info Object Loaded from CDS: ' + #13#10 + Result.Description, '', 0, '',
        'Get Current Info');
    end;
  except
  end;
end;

function TMDK_Engine.getCurrentInfo: TMDK_Info; // Always returns value
begin
  Result := getInfoRecord(dmData.cdsInfo); // returns nil if no records in CDS
  if Result = nil then begin
    Result := TMDK_Info.Create;
    Result.StudyID := fCurrentPatient.SessionID;
    Result.PatientID := fCurrentPatient.ID;
    Result.StudyDate := Now;
    Result.TherapyDate := Now;
    Result.DiagnosisDate := Now;
    Result.Nephrologist := ''; // 20061229 Clean up the initial value
    Report(rtDebug, 'Info Object Created: ' + #13#10 + Result.Description, '', 0, '', 'Get Current Info');
  end;
end;

function TMDK_Engine.getInfoData: TMDK_Info;
begin
  Result := getInfoRecord(dmData.cdsInfo);
  if Result = nil then Exit;
  if Result.StudyDate = 0 then begin
    Result.StudyID := fCurrentPatient.ID;
    Result.PatientID := fCurrentPatient.SessionID;
    Result.StudyDate := Now;
    Result.TherapyDate := Now;
    Result.DiagnosisDate := Now;
    Report(rtDebug, 'Info Object Updated: ' + #13#10 + Result.Description, '', 0, ClassName, 'Get Info');
  end;
end;

function TMDK_Engine.IsItNewInfo: Boolean;
begin
  Result := fFirstTimeAccessedStudy;
  if Result then Result := (not fStudyInfoCopied) or (not Preferences.bValue(onCoverRO));
end;

procedure TMDK_Engine.saveInfoData(aValue: TMDK_Info);
begin
  aValue.saveToCDS(dmData.cdsInfo);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.SelectHistoryStudy(aValue: Integer): string;
var
  i: Integer;
  sInfo, sSummary, sFlowsheet, sMedList, sNotes, sEvents: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  fStart := Now;
  i := StrToIntDef(fStudyNumList[aValue], -1);
  if fRenalController.GetStudy(i, sInfo, sSummary, sFlowsheet, sMedList, sNotes, sEvents) then begin
    InitDS(SummaryHistory_CDS, sSummary);
    Report(rtDebug, 'Info:' + #13 + #10 + sInfo, '', Duration, '', 'Get Study Info');
    Report(rtDebug, 'Summary:' + #13 + #10 + sSummary, '', Duration, '', 'Get Study Summary');
    Report(rtDebug, 'Flowsheet:' + #13 + #10 + sFlowsheet, '', Duration, '', 'Get Study Flowsheet');
    Report(rtDebug, 'Medication List:' + #13 + #10 + sMedList, '', Duration, '', 'Get Study Medication List');
  end;
  Result := IntToStr(i);
end;

function TMDK_Engine.SelectHistoryStudyByDate(aValue: string): string;
begin
  try
    SummaryHistory_CDS.Locate(fnS_SessionDate, VarArrayOf([aValue]), []);
    Result := SummaryHistory_CDS.FieldByName(fnS_SessionID).AsString;
  except
    Result := '-1';
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.ConvertDeviceData(const aDeviceName, aData: string): string;
var
  aTemplate: string;
  function ValidTemplate(const aValue: string): Boolean;
  begin
    Result := aValue <> ''; // good for now
  end;

begin
  aTemplate := getSystemOptionByName(aDeviceName);
  if not ValidTemplate(aTemplate) then begin
    if aDeviceName <> '' then
        Report(rtWarning, 'Invalid Template for device <' + aDeviceName + '>' + #13#10 + 'Template:' + #13#10 +
        aTemplate)
    else Report(rtWarning, 'Blank device name. No XSL trasformation performed.');
    Result := aData;
    Exit;
  end;
end;

procedure TMDK_Engine.setInstrumentRecord(aPatient, aStudy, aValue, aSource: string);
var
  anIR: TInstrumentRecord;
  _SSN: string;
  j: Integer;
begin
  dmData.Params := aSource; // 'File <'+FileOpen1.Dialog.FileName+'>';
  Engine.Report(etInfo, aValue, '', 0, '', 'Get Instrument Data From: ' + aSource, 'OK', True);
  ProblemList := '';
  anIR := getInstrumentRecord(aPatient, aStudy, aValue);
  if anIR <> nil then begin
    if anIR.ProblemList.Count > 0 then fProblemList := fProblemList + anIR.ProblemList.Text;
    if Trim(anIR.VendorPtID) <> '' then fVendorPtID := anIR.VendorPtID;
    frmMain.RefreshStudyStatus;
    _SSN := CurrentPatient.SSN;
    j := pos('-', _SSN);
    while j > 0 do begin
      _SSN := copy(_SSN, 1, j - 1) + copy(_SSN, j + 1, Length(_SSN));
      j := pos('-', _SSN);
    end;
    if anIR.PatientSSN <> _SSN then
        Report(etError, 'Record SSN <' + anIR.PatientSSN + '> Does not match ' + 'current patient SSN <' +
        CurrentPatient.SSN + '>' + #13#10 + aSource, '', Duration, '', 'Process Instrument Result', 'OK', True);
  end
  else Report(etDebug, 'Source: ' + aSource + #13#10 + aValue, '', Duration, '', 'Process Instrument Result',
      'Failed', True);
  anIR.Free;
  dmData.Params := '';
  if ProblemList <> '' then begin
    dmData.cdsSummary.Edit;
    dmData.cdsSummary.FieldByName(fnS_WorkflowNotes).AsString := ProblemList;
    dmData.cdsSummary.Post;
  end;
end;

function TMDK_Engine.getFlowsheetData(bAllRecords: Boolean; sPatientID: string = ''; sStudyID: string = ''): string;
var
  _PatientID, _StudyID: string;
  ss, s: string;
  xDOC: IXMLDocument;
  xNode: IXMLNode;
{$IFDEF AANTEST}
  _iStudy,
{$ENDIF}
  i: Integer;
  BM: TBookmark;
  bSaveResults, bMedication: Boolean;
  procedure ProcessInstrumentMessage(_s: string);
  var
    InstrumentRecord: TInstrumentRecord;
  begin
    try
{$IFDEF AANTEST}
      _PatientID := sPatientID;
      _StudyID := sStudyID;
      if sPatientID = '' then _PatientID := fCurrentPatient.ID;
      if sStudyID = '' then _StudyID := fCurrentPatient.SessionID;
      InstrumentRecord := getInstrumentRecord(_PatientID, _StudyID, _s);
{$ELSE}
      InstrumentRecord := getInstrumentRecord(fCurrentPatient.ID, fCurrentPatient.SessionID, _s);
{$ENDIF}
      if InstrumentRecord <> nil then begin
        if InstrumentRecord.ProblemList.Count > 0 then fProblemList := fProblemList + InstrumentRecord.ProblemList.Text;
        if Trim(InstrumentRecord.VendorPtID) <> '' then fVendorPtID := InstrumentRecord.VendorPtID;
        frmMain.RefreshStudyStatus;
        if InstrumentRecord.PatientSSN <> CurrentPatient.SSN then
            Report(etError, 'Record SSN <' + InstrumentRecord.PatientSSN + '> Does not match ' + 'current patient SSN <'
            + CurrentPatient.SSN + '>', '', Duration, '', 'Process Instrument Result', 'Result Set #: ' + ss, True);
        InstrumentRecord.Free;
      end;
    except
    end;
  end;
  function getNodeValueByName(aNode: IXMLNode; aValueName, aDefault: string): string;
  begin
    try
      Result := aNode.ChildValues[aValueName];
    except
      Result := aDefault;
    end;
  end;

begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if fCurrentPatient.ReadOnly then Exit;
  fStart := Now;
  fProblemList := '';
{$IFDEF AANTEST}
  if sStudyID = '' then _iStudy := fCurrentPatient.iStudy
  else _iStudy := StrToInt(sStudyID);
  if fRenalController.GetInstrumentResultList(_iStudy, s, bAllRecords) then
{$ELSE}
  if fRenalController.GetInstrumentResultList(fCurrentPatient.iStudy, s, bAllRecords) then
{$ENDIF}
  begin
    bSaveResults := devOptions.bValue(donSaveInstrumentMessages);
    // Save XML in Study History
    Result := s;
    if Trim(s) = '' then Exit; // 20061109
    Report(etInfo, s, '', Duration, '', 'Get Instrument Result List', 'OK');
    try
      xDOC := XMLDocument(s);
      ss := '';
      with xDOC.DocumentElement do begin
        for i := 0 to ChildNodes.Count - 1 do begin
          xNode := ChildNodes[i];
          if (xNode.NodeName = 'STATUS') and (xNode.NodeValue <> 'OK') then continue;
          if xNode.NodeName <> 'RECORD' then continue;
          ss := getNodeValueByName(xNode, 'ID', '');
          if ss <> '' then begin
            fStart := Now;
            fRenalController.GetInstrumentResult(StrToInt(ss), s);
            Report(etInfo, s, '', Duration, '', 'Get Instrument Results', 'Result Set #: ' + ss, bSaveResults);
          end;
          Result := Result + #13 + #10 + s;

          // 061007                ProcessInstrumentMessage(s);
{$IFDEF AANTEST}
          _PatientID := sPatientID;
          _StudyID := sStudyID;
          if sPatientID = '' then _PatientID := fCurrentPatient.ID;
          if sStudyID = '' then _StudyID := fCurrentPatient.SessionID;
{$ELSE}
          _PatientID := fCurrentPatient.ID;
          _StudyID := fCurrentPatient.SessionID;
{$ENDIF}
          if s <> '' then setInstrumentRecord(_PatientID, _StudyID, s, 'Get FlowsheetData');
        end;
      end;
    except
    end;
  end;
  // update checkbox on Post-Assignment tab if a medication was administered
  with MedList_CDS do begin
    if Active then // zzzzzzandria 20070227
    begin
      BM := getBookmark;
      DisableControls; // 2007-07-24
      First;
      bMedication := False;
      try
        while (not Eof) and (not bMedication) do begin
          bMedication := FieldByName(fnML_Status).AsString <> fsInvalid;
          Next;
        end;
        Summary_CDS.Edit;
        Summary_CDS.FieldByName(fnS_Medication).AsBoolean := bMedication;
        Summary_CDS.Post;
      except
      end;
      gotoBookmark(BM);
      FreeBookmark(BM);
      EnableControls; // 2007-07-24
    end;
  end;
  // Post-Assessment tab update
  try
    ss := UpperCase(Engine.ProblemList);
    Summary_CDS.Edit;
    bMedication := pos('VOMITING', ss) > 0;
    if bMedication then Summary_CDS.FieldByName(fnS_TlrVom).AsBoolean := bMedication;
    bMedication := pos('HYPOTENSION', ss) > 0;
    if bMedication then Summary_CDS.FieldByName(fnS_TlrHyp).AsBoolean := bMedication;
    bMedication := pos('CRAMPING', ss) > 0;
    if bMedication then Summary_CDS.FieldByName(fnS_TlrCram).AsBoolean := bMedication;
    Summary_CDS.Post;
  except
  end;
end;
/// ////////////////////////////////////////////////////////////////////  Billing
/// ////////////////////////////////////////////////////////////////////  Billing
/// ////////////////////////////////////////////////////////////////////  Billing

function TMDK_Engine.ValidateSignature(aValue: string): Boolean;
begin
  Result := True;
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.ValidateElectronicSignature(aValue);
  if Result then Report(rtDebug, 'Signature confirmed')
  else Report(rtDebug, 'Signature failed', '', 0, '', 'Validate Signature', 'Failed', True);
end;

procedure TMDK_Engine.PCESetReviewed;
begin
  try
    if dmData.cdsInfo.FieldByName(fnI_BStatus).AsString <> 'UPDATED' then begin
      dmData.cdsInfo.Edit;
      dmData.cdsInfo.FieldByName(fnI_BStatus).AsString := 'REVIEWED';
      dmData.cdsInfo.FieldByName(fnI_BDate).AsString := FormatDateTime(gMDK_DateTimeFMT, Now);
      dmData.cdsInfo.FieldByName(fnI_BBy).AsString := fUserDUZ;
      dmData.cdsInfo.Post;
    end;
  except
  end;
end;

procedure TMDK_Engine.PCESetUpdated;
begin
  try
    dmData.cdsInfo.Edit;
    dmData.cdsInfo.FieldByName(fnI_BStatus).AsString := 'UPDATED';
    dmData.cdsInfo.FieldByName(fnI_BDate).AsString := FormatDateTime(gMDK_DateTimeFMT, Now);
    dmData.cdsInfo.FieldByName(fnI_BBy).AsString := fUserDUZ;
    dmData.cdsInfo.Post;
  except
  end;
end;

function TMDK_Engine.PCEReadyToSubmit: Boolean;
begin
  Result := dmData.cdsInfo.Active and (dmData.cdsInfo.FieldByName(fnI_BStatus).AsString <> '');
end;

function TMDK_Engine.getPCEInfo: string;
begin
  Result := 'No Info...  ';
  if Assigned(fRenalController) then Result := fRenalController.getPCEInfo(CurrentPatient.ID, CurrentPatient.SessionID);
end;

function TMDK_Engine.getPCEBillingStatus: string;
begin
  Result := Engine.Info_CDS.FieldByName(fnI_BStatus).AsString;
  if Result = '' then Result := '  Not reviewed';
end;

function TMDK_Engine.getPCEBillingStatusDescription: string;
begin
  Result := Engine.Info_CDS.FieldByName(fnI_BStatus).AsString;
  if Result = '' then Result := '  Not reviewed'
  else Result := '  ' + Engine.Info_CDS.FieldByName(fnI_BStatus).AsString + ' On ' +
      Engine.Info_CDS.FieldByName(fnI_BDate).AsString + ' By ' + fRenalController.getUserNameByIEN
      (Engine.Info_CDS.FieldByName(fnI_BBy).AsString);
end;

function TMDK_Engine.ValidBillingInfo(var aComment: string; Loud: Boolean = True): Boolean;
var
  sOld, sNew, s: string;
  aSL: TStringList;
begin
  aSL := TStringList.Create;
  Engine.getEncounterData(CurrentPatient.SessionID, sOld);
  Report(rtInfo, sOld, '', 0, '' { CurrentOperatorID } , 'Get Encounter Data');
  aSL.Text := fBillingInfo.Legacy;
  Result := Engine.setEncounterData(CurrentPatient.SessionID, aSL, s) and (pos('-', s) <> 1);
  Report(rtInfo, aSL.Text, '', 0, '' { CurrentOperatorID } , 'Set Encounter Data');
  if Result then begin
    Report(rtDebug, aSL.Text, '', 0, '', 'Set Billing Info', 'OK');
    if Engine.getEncounterData(CurrentPatient.SessionID, sNew) then begin
      if sOld = sNew then begin
        if devOptions.bValue(donNotifyPCE) and Loud then
            ShowMessage('Sorry, the PCE data was not updated even though the data was accepted...');
        PCESetReviewed;
      end else begin
        PCESetUpdated;
        Report(rtDebug, aSL.Text, '', 0, '', 'Update Billing Info', 'OK', True);
      end;
    end
    else Report(rtDebug, 'Billing info. Study <' + CurrentPatient.SessionID + '> - can not verify saving the PCE info');
  end else begin
    Report(rtDebug, 'Billing info. Study <' + CurrentPatient.SessionID + '> - failed' + #13#10 + s);
    aComment := Piece(s, '^', 2);
  end;
  aSL.Free;
end;

procedure TMDK_Engine.getColumnFormatData(aTable, aColumn: string;
  var TheLabel, TheUnit, TheFormat, TheEditMask: string);
begin
  try
    if dmData.cdsDict.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf([aTable, aColumn]), []) then begin
      TheLabel := dmData.cdsDict.FieldByName(fnDict_Label).AsString;
      TheFormat := dmData.cdsDict.FieldByName(fnDict_Format).AsString;
      TheUnit := dmData.cdsDict.FieldByName(fnDict_Units).AsString;
      TheEditMask := dmData.cdsDict.FieldByName(fnDict_EditMask).AsString;
    end else begin
      TheUnit := '';
      TheFormat := '';
      TheEditMask := '';
      TheLabel := aColumn;
    end;
  except
  end;
end;

procedure TMDK_Engine.setColumnFormat(aTable, aColumn, TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault,
  ThePage, TheScale, TheLabel: string);
begin
  setFieldProperties(aTable, aColumn, TheDisplayFormat, TheEditMask, TheWidth);
end;

function TMDK_Engine.getSummary: TStringList;
var
  i, iLen: Integer;
  aSL: TStringList;
  s, sName, sUnit, sFormat, sLabel, sMask: string;
const
  NameFMT = ' %8s |';
  NameLen = 8;
  function FMTValue(aValue: string): string;
  begin
    if Length(aValue) > NameLen then Result := Format(NameFMT, [copy(aValue, 1, 5) + '...'])
    else Result := Format(NameFMT, [aValue]);
  end;
  function getCopy(aString: string; aCount: Integer): string;
  begin
    Result := '';
    while Length(Result) < aCount do Result := Result + aString;
  end;

begin
  aSL := TStringList.Create;
  try
    aSL.Add(' Summary:');
    aSL.Add('');
    for i := 0 to Summary_CDS.FieldCount - 1 do begin
      s := Summary_CDS.Fields[i].FieldName;
      sName := s;
      getColumnFormatData('cdsSummary', s, sLabel, sUnit, sFormat, sMask);
      s := Format(' %-20s = %s (%s)', [sLabel, Summary_CDS.FieldByName(sName).AsString, sUnit]);
      aSL.Add(s);
    end;
    aSL.Add('');
    aSL.Add('');
    aSL.Add(' Flowsheet:');
    s := '';
    for i := 0 to FlowSheet_CDS.FieldCount - 1 do s := s + FMTValue(FlowSheet_CDS.Fields[i].FieldName);
    aSL.Add(s);
    iLen := Length(s);
    aSL.Add(getCopy('-', iLen));
    FlowSheet_CDS.First;
    while not FlowSheet_CDS.Eof do begin
      s := '';
      for i := 0 to FlowSheet_CDS.FieldCount - 1 do begin
        sName := FlowSheet_CDS.Fields[i].FieldName;
        s := s + FMTValue(FlowSheet_CDS.FieldByName(sName).AsString);
      end;
      aSL.Add(s);
      FlowSheet_CDS.Next;
    end;
    aSL.Add('');
    Result := aSL;
  except
    if Assigned(aSL) then aSL.Free;
    Result := nil;
  end;
end;

procedure TMDK_Engine.SaveDictionaryToDB(aName: string = '');
var
  sName, s: string;
  i: Integer;
begin
  if not Assigned(fRenalController) then Exit;
  if aName = '' then sName := onDictionary
  else sName := aName;
  if fRenalController.GetSystemOption(sName, s) then begin
    if not LockSystemOption(sName) then begin
      ShowMessage('Lock of the dictionary <' + sName + '> Failed' + #13#10 + 'Dictionary is not saved' + #13#10#13#10 +
        Engine.fRenalController.ResultInfo);
      Exit;
    end;
    if fDictionaryXML <> s then begin
      i := MessageDLG('The application Dictionary was modified by another user' + #13 +
        'Use the <Tools|Reload Dictionary> command to get the latest dictionary' + #13 +
        'To overrite changes press <OK>' + #13, mtWarning, [mbOk, mbCancel], 0);
      if i <> mrOK then begin
        UnLockSystemOption(sName);
        Exit;
      end;
    end;
    try
      if dmData.cdsDict.State = dsEdit then dmData.cdsDict.Post;
      dmData.cdsDict.MergeChangeLog;
      fRenalController.SetSystemOption(sName, dmData.cdsDict.XMLData);
      fRenalController.GetSystemOption(sName, s);
      fDictionaryXML := s;
      UnLockSystemOption(sName);
    except
    end;
  end;
end;

procedure TMDK_Engine.LoadDictionaryFromDB(aName: string = '');
var
  sName, s: string;
begin
  if not Assigned(fRenalController) then Exit;
  if aName = '' then sName := onDictionary
  else sName := aName;
  if fRenalController.GetSystemOption(aName, s) then LoadDictionaryFromString(s);
end;

function TMDK_Engine.LoadDictionaryFromString(aValue: string): Boolean;
begin
  // fDictionaryXML := aValue;
  Result := False;
  try
    if CDSGetData(dmData.cdsSource, aValue) then begin
      fDictionaryXML := aValue;
      dmData.cdsDict.EmptyDataSet;
      CDSMerge(dmData.cdsSource, dmData.cdsDict, fnDict_ID);
      Result := True;
    end;
  except
  end;
end;

function TMDK_Engine.DictionaryOvewriteManualInput(aFN: string): Boolean;
begin
  Result := False;
  try
    if dmData.cdsDict.Locate(fnDict_Table + ';' + fnDict_Column, VarArrayOf(['cdsSummary', aFN]), []) then begin
      Result := dmData.cdsDict.FieldByName(fnDict_Overwrite).AsBoolean;
    end;
  except
  end;
end;

function TMDK_Engine.getProvider(aTarget: string): string;
var
  SL: TStringList;
  s: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if aTarget = '' then Result := SelectProvider(aTarget, fRenalController)
  else begin
    SL := TStringList.Create;
    fRenalController.LookupProvider(aTarget, s);
    SL.Text := s;
    if SL.Count = 2 then begin
      SL.Delete(0);
      if (pos('No mat', SL.Text) <> 1) and (pos('Too', SL.Text) <> 1) then
          Result := Piece(SL[0], '^', 2) + '^' + Piece(SL[0], '^', 1)
      else Result := SelectProvider(aTarget, fRenalController);
    end
    else Result := SelectProvider(aTarget, fRenalController);
    SL.Free;
  end;
end;

function TMDK_Engine.getProviderList(aTarget: string): string;
var
  s: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if aTarget = '' then Result := ''
  else begin
    fRenalController.LookupProvider(aTarget, s);
    Result := s;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getSystemOptionByName(aName: string): string;
var
  s: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if fRenalController.GetSystemOption(aName, s) then Result := s
  else Result := '';
end;

function TMDK_Engine.setSystemOptionByName(aName, aValue: string): Boolean;
begin
  Result := True;
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.SetSystemOption(aName, aValue);
end;

function TMDK_Engine.getUserOptionByName(aName: string): string;
var
  s: string;
  sDUZ: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if Trim(fUserDUZ) = '' then sDUZ := '@DUZ'
  else sDUZ := fUserDUZ;
  // if fRenalController.getUserOption(aName,s) then    Result := s
  if fRenalController.getUserOptionByDUZ(aName, sDUZ, s) then Result := s
  else Result := '';
end;

procedure TMDK_Engine.setUserOptionByName(aName, aValue: string);
var
  sDUZ: string;
begin
  if not Assigned(fRenalController) then Exit;
  // fRenalController.setUserOption(aName,aValue);
  if fUserDUZ = '' then sDUZ := '@DUZ'
  else sDUZ := fUserDUZ;
  fRenalController.setUserOptionByDUZ(aName, sDUZ, aValue);
end;

function TMDK_Engine.deleteUserOption(aName: string): Boolean;
begin
  Result := True;
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.deleteUserOption(aName);
end;

function TMDK_Engine.deleteSystemOption(aName: string): Boolean;
begin
  Result := True;
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.deleteSystemOption(aName);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getReportList: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  fRenalController.GetSystemOption(onReportList, Result);
end;

function TMDK_Engine.getFromAddress: string;
begin
  Result := ''
end;

function TMDK_Engine.getToAddress: string;
begin
  // Result := Preferences.sValue(onAddressTo);
  Result := '';
end;

function TMDK_Engine.getSMTPHost: string;
begin
  Result := devOptions.sValue(donSMTPHost); // 'zzzzzzmsg1';
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.IsPatternCorrect(apattern: string): Boolean;
begin
  Result := uMDK_ReportEngine.IsPatternCorrect(apattern);
end;

function TMDK_Engine.getPatternValue(apattern: string): string;
begin
  if apattern = kw_PtName then Result := CurrentPatient.Name
  else if apattern = kw_PtAge then Result := CurrentPatient.Age
  else if apattern = kw_PtSSN then Result := CurrentPatient.SSN
  else if apattern = kw_PtSex then Result := CurrentPatient.Sex
  else if apattern = kw_PtDOB then Result := FormatDateTime(gMDK_DateFMT, CurrentPatient.DOB)
    // FormatDateTime('MM/dd/yyyy',DOB)
  else if apattern = kw_PtID then Result := CurrentPatient.ID
  else if apattern = kw_StationID then Result := CurrentPatient.Station
  else if apattern = kw_StudyID then Result := CurrentPatient.SessionID
  else Result := uMDK_ReportEngine.getPatternValue(apattern);
end;

function TMDK_Engine.getReportValue(aTemplate: string): string;
begin
  Result := uMDK_ReportEngine.getReportValue(aTemplate);
end;

function TMDK_Engine.getReportString: string;
var
  sReportName, sTemplate: string;
begin
  Result := '';
  if CurrentPatient = nil then Exit;
  sReportName := Preferences.sValue(onSummaryReportName);
  sTemplate := getSystemOptionByName(sReportName);
  if sTemplate = '' then ShowMessage('Summary report template <' + sReportName + '> not found')
  else Result := uMDK_ReportEngine.getReportValue(sTemplate);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.Submit(aStudy: Integer; aSignature, aReportString: string): string;
var
  TheReport: TStringList;
  procedure RegisterSubmit;
  var
    s: string;
  begin
    s := getSystemOptionByName(onSubmittedStudies);
    if pos(XML_VERSION, s) = 1 then InitDS(dmData.cdsSubmitHistory, s)
    else RestoreSubmtHistory(dmData.cdsSubmitHistory, s);
    AddSubmitHistoryRecord(dmData.cdsSubmitHistory, fCurrentPatient.ID, fCurrentPatient.SessionID, fUserDUZ,
      FormatDateTime(gMDK_DateTimeFMT, Engine.getCurrentServerDateTime));
    try
      dmData.cdsSubmitHistory.MergeChangeLog;
    except
    end;
    setSystemOptionByName(onSubmittedStudies, dmData.cdsSubmitHistory.XMLData);
  end;

begin
  fStart := Now;
  TheReport := TStringList.Create;
  TheReport.Text := aReportString; // getReportString;
  if fRenalController.SubmitStudy(aStudy, aSignature, TheReport) then begin
    Report(etDebug, 'Submit Result: < OK >', '', Duration, '', 'SUBMIT', '', True);
    Result := TheReport.Text;
    if devOptions.bValue(donKeepSubmitHistory) then RegisterSubmit;
  end else begin
    Report(etWarning, 'Submit Failed:' + #13#10 + fRenalController.ResultInfo, '', Duration, '', 'SUBMIT', '', True);
    Result := fRenalController.ResultInfo;
  end;
  TheReport.Free;
end;

function TMDK_Engine.SubmitNote(aUserID: string; aUserESig: string;
  // The users E-Sig in clear text, it will be encrypted before sending
  aDocumentText: TStringList; aTitle: string): string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if fRenalController.SubmitNoteNow(fCurrentPatient.iStudy, aUserID, aUserESig, aDocumentText, aTitle) then
      Result := fRenalController.ResultInfo
  else Result := fRenalController.ResultInfo
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.setUpField(aF: TField; TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage,
  TheLabel, TheMax, TheMin: string);
const
  FLOAT_IGNORE = -999.0;
  procedure ReportIssue(aMessage: string);
  begin
    Report(rtError, 'Update table Field <' + aF.Name + '> "' + #13 + #10 + 'Format error ' + #13 + #10 + aMessage);
  end;

var
  s: string;
begin
  s := CDSsetUpField(aF, TheDisplayFormat, TheEditMask, TheWidth, TheUnit, TheDefault, ThePage, TheLabel,
    TheMax, TheMin);
  if s <> '' then ReportIssue(s);
end;

procedure TMDK_Engine.setUpFields;
var
  aFName, aTName, s: string;
  aFD: TFieldDef;
  CDS: TClientDataSet;
  sCDSList: string;
  aF: TField;
  function Needed(aName: string): Boolean;
  begin
    aTName := aName;
    Result := pos(UpperCase('=' + aName), UpperCase(sCDSList)) > 0;
  end;

begin
  fNow := Now;
  with dmData do begin
    sCDSList := getSystemOptionByName(onDatasetNames);
    CDS := nil;
    s := '';
    cdsDict.First;
    while not cdsDict.Eof do begin
      if Needed(cdsDict.FieldByName(fnDict_Table).AsString) then begin
        if (s <> cdsDict.FieldByName(fnDict_Table).AsString) then begin
          s := cdsDict.FieldByName(fnDict_Table).AsString;
          if s <> '' then CDS := dmData.getCDSByName(s);
        end;
        if CDS <> nil then begin
          aFName := cdsDict.FieldByName(fnDict_Column).AsString;
          aFD := getFieldDefByName(CDS, aFName);
          if aFD <> nil then begin
            // iNum := aFD.FieldNo; //zzzzzzandria 050413
            try
              aF := CDS.FieldByName(aFName);
              if aF <> nil then
                  CDSsetUpField(aF, cdsDict.FieldByName(fnDict_Format).AsString, cdsDict.FieldByName(fnDict_EditMask)
                  .AsString, cdsDict.FieldByName(fnDict_Width).AsString, cdsDict.FieldByName(fnDict_Units).AsString,
                  cdsDict.FieldByName(fnDict_Default).AsString, cdsDict.FieldByName(fnDict_Section).AsString,
                  cdsDict.FieldByName(fnDict_Label).AsString, cdsDict.FieldByName(fnDict_MaxValue).AsString,
                  cdsDict.FieldByName(fnDict_MinValue).AsString);
            except
              on E: Exception do
                  Report(rtError, 'Table <' + aTName + '> Column <' + aFName + '>' + #13 + #10 + E.Message)
            end;
          end;
        end;
      end;
      cdsDict.Next;
    end;
  end;
  Report(rtDebug, 'Set Up Fields', '', Now - fNow, '', 'Set up fields');
end;

function TMDK_Engine.LockSystemOption(aName: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.LockSetting(aName);
end;

function TMDK_Engine.UnLockSystemOption(aName: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.UnLockSetting(aName);
end;

function TMDK_Engine.getStationNumber: string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := Piece(fRenalController.GetCCOWContext, '^', 1);
end;

function TMDK_Engine.getStationMode: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  Result := Piece(fRenalController.GetCCOWContext, '^', 2);
  if pos('0', Result) = 1 then Result := '_test'
  else Result := '';
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.SubmitFlowsheet;
begin
  if not Assigned(fRenalController) then Exit;
  with FlowSheet_CDS do begin
    First;
    while not Eof do begin // send only not submitted and valid
      if (FieldByName(fnFS_Status).AsString = fsValid) and not FieldByName(fnFs_Submitted).AsBoolean then begin
        if not ValidBP(FieldByName(fnFS_BP).AsString, True) then
        // expecting metric value
        begin
          if FieldByName(fnFS_BP).AsString <> '' then
              Report(rtWarning, 'Invalid BP: <' + FieldByName(fnFS_BP).AsString + '>', '', 0, ClassName,
              'Save Flowsheet BP', 'Invalid Value');
        end else if not fRenalController.SetVitalsMeasurement(FieldByName(fnFS_Date).AsDateTime + FieldByName(fnFS_Time)
          .AsDateTime, CurrentPatient.ID, vtBloodPressure,
          // CurrentPatient.LocationID,
          LocationID, UserDUZ, FieldByName(fnFS_BP).AsString) then
            Report(rtDebug, 'BP submitted: <' + FieldByName(fnFS_BP).AsString + '> - Failed');
        if not ValidPulse(FieldByName(fnFS_HR).AsString) then
        // expecting metric value
        begin
          if (FieldByName(fnFS_HR).AsString <> '') and (FieldByName(fnFS_HR).AsInteger <> 0) then
              Report(rtWarning, 'Invalid Pulse: <' + FieldByName(fnFS_HR).AsString + '>', '', 0, ClassName,
              'Save Flowsheet Pulse', 'Invalid Value');
        end else if not fRenalController.SetVitalsMeasurement(FieldByName(fnFS_Date).AsDateTime + FieldByName(fnFS_Time)
          .AsDateTime, CurrentPatient.ID, oDBController_Renal.vtPulse,
          // CurrentPatient.LocationID,
          LocationID, UserDUZ, FieldByName(fnFS_HR).AsString) then
            Report(rtDebug, 'HR submitted: <' + FieldByName(fnFS_HR).AsString + '>', '', 0, ClassName,
            'Submit Flowsheet Pulse', 'Failed');
        Edit;
        FieldByName(fnFs_Submitted).AsBoolean := True;
        Post;
      end;
      FlowSheet_CDS.Next;
    end;
  end;
end;

function TMDK_Engine.SubmitVitalsData: string;
var
  StartTime, StopTime, TheTime: TDateTime;
  s: string;
  dd: Double;
  procedure SubmitValue(aDateTime: TDateTime; aValue: string; aVitalName: string; aType: TVitalType;
    aRenalType: TRenalVitalSign; aQualifierName: string);
  var
    QualifierString: string;
    function DataTypeName(aType: TRenalVitalSign): string;
    begin
      case aType of
        oDBController_Renal.vtTemp: Result := 'Temperature';
        oDBController_Renal.vtBloodPressure: Result := 'Blood Pressure';
        oDBController_Renal.vtPulse: Result := 'Pulse';
        oDBController_Renal.vtPain: Result := 'Pain';
        oDBController_Renal.vtRespiration: Result := 'Respiration';
      else Result := 'unknown (' + char(aType) + ')';
      end;
    end;

  begin
    if Trim(aValue) = '' then Exit;
    if Trim(aValue) = '/' then Exit;
    QualifierString := getQualifierIEN(aVitalName, aQualifierName);
    if QualifierString <> '' then QualifierString := '*' + QualifierString;
    if not fRenalController.SetVitalsMeasurement(aDateTime, CurrentPatient.ID, aRenalType, LocationID,
      UserDUZ + QualifierString, aValue) then
        Report(rtDebug, 'Data submitted: <' + DataTypeName(aRenalType) + ':' + aValue + '>', '', 0, ClassName,
        'Failed');
  end;

begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  Result := 'Undefined Location. No Vitals were saved.';
  if LocationID = '' then Exit;
  Result := '';
  StartTime := Summary_CDS.FieldByName(fnS_TreatmentStarted).AsDateTime;
  if StartTime < 1 then StartTime := Engine.Info_CDS.FieldByName(fnI_StudyDate).AsDateTime;
  StopTime := Summary_CDS.FieldByName(fnS_TreatmentFinished).AsDateTime;
  if StopTime < 1 then StopTime := Engine.Info_CDS.FieldByName(fnI_StudyDate).AsDateTime;
  TheTime := StopTime;
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PreTemp).AsString, 'TEMPERATURE', vtTemp,
    oDBController_Renal.vtTemp, '');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostTemp).AsString, 'TEMPERATURE', vtTemp,
    oDBController_Renal.vtTemp, '');
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PrePulseStanding).AsString, 'PULSE', vtPulse,
    oDBController_Renal.vtPulse, 'STANDING');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostPulseStanding).AsString, 'PULSE', vtPulse,
    oDBController_Renal.vtPulse, 'STANDING');
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PrePulseSeated).AsString, 'PULSE', vtPulse,
    oDBController_Renal.vtPulse, 'SITTING');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostPulseSeated).AsString, 'PULSE', vtPulse,
    oDBController_Renal.vtPulse, 'SITTING');
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PreBPSysStanding).AsString + '/' +
    Summary_CDS.FieldByName(fnS_PreBPDiasStanding).AsString, 'BLOOD PRESSURE', vtBP,
    oDBController_Renal.vtBloodPressure, 'STANDING');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostBPSysStanding).AsString + '/' +
    Summary_CDS.FieldByName(fnS_PostBPDiasStanding).AsString, 'BLOOD PRESSURE', vtBP,
    oDBController_Renal.vtBloodPressure, 'STANDING');
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PreBPSysSitting).AsString + '/' +
    Summary_CDS.FieldByName(fnS_PreBPDiasSitting).AsString, 'BLOOD PRESSURE', vtBP, oDBController_Renal.vtBloodPressure,
    'SITTING');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostBPSysSitting).AsString + '/' +
    Summary_CDS.FieldByName(fnS_PostBPDiasSitting).AsString, 'BLOOD PRESSURE', vtBP,
    oDBController_Renal.vtBloodPressure, 'SITTING');
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PreRespirations).AsString, 'RESPIRATION', vtResp,
    oDBController_Renal.vtRespiration, '');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostRespirations).AsString, 'RESPIRATION', vtResp,
    oDBController_Renal.vtRespiration, '');
  dd := Summary_CDS.FieldByName(fnS_PreWeight).AsFloat;
  dd := ConvertKgsToLbs(dd);
  if dd > 0 then begin
    s := FloatToStr(dd);
    SubmitValue(StartTime, s, 'WEIGHT', vtWeight, oDBController_Renal.vtWeight, '');
  end;
  dd := Summary_CDS.FieldByName(fnS_PostWeight).AsFloat;
  dd := ConvertKgsToLbs(dd);
  if dd > 0 then begin
    s := FloatToStr(dd);
    SubmitValue(TheTime, s, 'WEIGHT', vtWeight, oDBController_Renal.vtWeight, '');
  end;
  SubmitValue(StartTime, Summary_CDS.FieldByName(fnS_PrePain).AsString, 'PAIN', vtPain, oDBController_Renal.vtPain, '');
  SubmitValue(TheTime, Summary_CDS.FieldByName(fnS_PostPain).AsString, 'PAIN', vtPain, oDBController_Renal.vtPain, '');
end;

function TMDK_Engine.getVendorPtID: string;
begin
  Result := dmData.cdsSummary.FieldByName(fnS_VendorPTID).AsString;
  if Result <> '' then Result := 'Vendor ID: ' + Result
  else Result := 'Vendor ID: unknown';
end;

procedure TMDK_Engine.ReportStatus(aPage: string; aCode: Integer; aComment: string);
var
  i: Integer;
begin
  if dmData.cdsStatus.Locate('Page', VarArrayOf([aPage]), []) then begin
    dmData.cdsStatus.Edit;
    dmData.cdsStatusStatusID.Value := aCode;
    dmData.cdsStatusComments.Value := AnsiString(aComment);
    dmData.cdsStatus.Post;
  end;
  i := aCode + 1;
  if i > 2 then i := 2;
  if aPage = pnCover then frmMain.tsCover.ImageIndex := i;
  if aPage = pnRxLab then frmMain.tsRxLab.ImageIndex := i;
  if aPage = pnPre then frmMain.tsPreTreatment.ImageIndex := i;
  if aPage = pnFlow then frmMain.tsFlowSheet.ImageIndex := i;
  if aPage = pnAccess then frmMain.tsAccess.ImageIndex := i;
  if aPage = pnPost then frmMain.tsPostTreatment.ImageIndex := i;
  if aPage = pnSummary then frmMain.tsSummary.ImageIndex := i;
  if aPage = pnSubmit then frmMain.tsSubmit.ImageIndex := i;
end;

function TMDK_Engine.getStudyNotes: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if fRenalController.getNoteList(fCurrentPatient.iStudy) then Result := fRenalController.ResultInfo
  else Result := fRenalController.ResultInfo
end;

function TMDK_Engine.getNotesByStudyID(anID: string): string;
var
  iID: Integer;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  iID := StrToIntDef(anID, -1);
  if iID < 0 then Exit;
  if fRenalController.getNoteList(iID) then Result := fRenalController.ResultInfo
  else Result := fRenalController.ResultInfo
end;

function TMDK_Engine.getNoteByNum(aNoteID: string; aStudyID: string = ''): string;
var
  i64: Int64;
begin
  Result := '';
  if aStudyID = '' then i64 := CurrentPatient.iStudy
  else
    try
      i64 := strToInt64(aStudyID);
    except
      on E: Exception do Exit;
    end;
  if not Assigned(fRenalController) then begin
    if fRenalController.getNote(i64, aNoteID) then Result := fRenalController.ResultInfo;
  end else begin
    if fRenalController.getNote(i64, aNoteID) then Result := fRenalController.ResultInfo
    else Result := '';
  end;
end;

function TMDK_Engine.LockStudy(aStudy: Integer): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.LockStudy(aStudy);
end;

function TMDK_Engine.UnLockStudy(aStudy: Integer): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.UnLockStudy(aStudy);
end;

procedure TMDK_Engine.ReloadOrder;
begin
  if not SummaryHistory_CDS.Active then Exit;
  SummaryHistory_CDS.First;
  if SummaryHistory_CDS.Eof then begin
    Report(etDebug, 'No Summary History to Reload Order', '', Duration, '', 'Reload Rx Order', '');
    Exit;
  end;
  // zzzzzzandria 2007-08-29 always get prescription from the last study available
  // b. 314 #14
  SummaryHistory_CDS.Last;
  try
    // add code to update RxLab by data of Summary History
    // hope it is in the history table
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxDialyser);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxReuseNum);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxTxLength);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxBFR);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxDialysisFlow);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_rxTemp);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_DF_K);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_DF_HCO3);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_DF_NA);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_DF_CA);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_AC_Type);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_AC_Load);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_AC_Doses);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_AC_EndTime);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_AC_Other);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_MoD_NA);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_MoD_HCO3);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_Mod_Ultrafiltration);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_OtherOrders);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_RxUltra);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_RxEDW);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_TrnCandidate);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_TrnWIP);
    CopyIntoNullField(SummaryHistory_CDS, Summary_CDS, fnS_TrnReferred);
  except
  end;
  Report(etDebug, 'Rx Order of study <' + SummaryHistory_CDS.FieldByName(fnS_SessionID).AsString + '> reloaded', '',
    Duration, '', 'Reload Rx Order', '', True);
end;

function TMDK_Engine.getCurrentOperatorMode: string;
var
  s: string;
begin
  if Assigned(Administrators) then s := Administrators.ValueList.Text;
  if s = '' then begin
    s := getSystemOptionByName(onAdministrators);
    Administrators.Overwrite(s);
  end;
  if (pos(CurrentOperatorID, s) > 0) or (Engine.CurrentOperatorID = rnHemoDialysis) or
    (UpperCase(Engine.CurrentOperatorID) = rnHemoDialysisOld)
{$IFDEF DEMO}
    or ((UpperCase(Engine.CurrentOperatorID) = 'ADMINISTRATOR') and (cmdLineSwitch(['/DEMO'])))
{$ENDIF}
    or (pos(fUserDUZ, '(' + s + ')') > 0) then Result := rnAdmin
  else
{$IFDEF DEBUG}
    Result := rnAdmin;
{$ELSE}
      Result := rnUser; // change back to this for distribution zzzzzzcarnej
{$ENDIF}
end;

function TMDK_Engine.getComputerID: string;
var
  Buffer: array [0 .. 200] of char;
  iBuffer: DWord;
begin
  iBuffer := 200;
  Result := 'unknown';
  try
    GetComputerName(Buffer, iBuffer);
    Result := copy(Buffer, 1, iBuffer);
  except
  end;
end;

function TMDK_Engine.getClinicProcedures(aDFN, aClinic: string; var aReturnList: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getClinicProcedures(aDFN, aClinic, aReturnList);
end;

function TMDK_Engine.getClinicDiagnosis(aDFN, aClinic: string; var aReturnList: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getClinicDiagnosis(aDFN, aClinic, aReturnList);
end;

function TMDK_Engine.getServiceConnections(aDFN, aClinic: string; var aReturnList: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getServiceConnections(aDFN, aClinic, aReturnList);
end;

function TMDK_Engine.getCPTByTarget(aTarget: string; StudyID: string; var aReturnList: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getLexiconData(aTarget, 'CPT', StudyID, aReturnList);
end;

function TMDK_Engine.getICDByTarget(aTarget: string; StudyID: string; var aReturnList: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getLexiconData(aTarget, 'ICD', StudyID, aReturnList);
end;

function TMDK_Engine.getBillingData(aStudy: string; var aResult: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getBillingData(StrToInt(aStudy), aResult);
end;

function TMDK_Engine.getEncounterData(aStudy: string; var aResult: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.getEncounterData(aStudy, aResult);
end;

function TMDK_Engine.setEncounterData(aStudy: string; aBillingData: TStringList; var aResult: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.setEncounterData(aStudy, aBillingData, aResult);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getSignature;
begin
  Result := '[' + Engine.CurrentOperatorID + ': ' + FormatDateTime(gMDK_DateTimeFMT, Now) + '] ';
end;

function TMDK_Engine.getStudyDate: string;
begin
  Result := Engine.Info_CDS.FieldByName(fnI_StudyDate).AsString;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getQualifierIEN(aVitalName, aQualifier: string): string;
begin
  Result := uMDK_Vitals.getQualifierIEN(fVitalsAgent, aVitalName, aQualifier);
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getGatewayStatus: string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.getGatewayStatus;
end;

function TMDK_Engine.getGatewayInfo: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.getGatewayRunningStatus;
  if Result = 'Down' then Result := 'Gateway Status: ' + Result
  else Result := 'Gateway Status: ' + Result + #13#13 + fRenalController.getGatewayStatusInfo;
end;
/// /////////////////////////////////////////////////////////////////////////////

function TMDK_Engine.getVistaImpDate: string; // added 09/24/2012 zzzzzzcarnej
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.GetImplementationDate;
end;

function TMDK_Engine.getCurrentServerDateTimeString: string;
begin
  Result := FormatDateTime(gMDK_DateTimeFMT, Now + ServerDelay);
end;

function TMDK_Engine.getCurrentServerDateTime: TDateTime;
begin
  try
    Result := Now + ServerDelay;
  except
    on E: Exception do Result := 0;
  end;
end;

procedure TMDK_Engine.setServerDelay;
var
  sSDT: string;
begin
  if not Assigned(fRenalController) then fServerDelay := 0
  else
    try
      sSDT := fRenalController.getServerCurrentDateTime;
      if sSDT = '' then begin
        fServerDelay := 0;
        Report(etWarning, 'Can''t get server datetime. Server delay is set to 0.', 'Startup', Duration);
      end else begin
        fServerDelay := FMDateTimeToWindowsDateTime(StrToFloat(sSDT)) - Now;
        dmData.Action := 'Get Server Delay';
        Report(rtDebug, 'Server Delay: ' + FormatDateTime(gMDK_DebugTimeFMT, fServerDelay), 'Startup', 0);
      end;
    except
      begin
        fServerDelay := 0;
        Report(etError, 'Can''t get server datetime. Server delay is set to 0.', 'Startup', Duration);
      end;
    end;
end;

function TMDK_Engine.getServerWDelay: TDateTime;
var
  sSDT: string;
begin
  Result := 0;
  if not Assigned(Engine) then Exit;
  if fServerDelay = fDefaultServerTimeDelay then begin
    try
      sSDT := fRenalController.getServerCurrentDateTime;
      fServerDelay := FMDateToWindowsDate(StrToFloat(sSDT)) - Now;
    except
      begin
        fServerDelay := 0;
        Report(etError, 'Can''t get server datetime. Server delay is set to 0.', 'Startup', Duration);
      end;
    end;
  end;
  Result := fServerDelay;
end;

function TMDK_Engine.getRPCResults(aParamString: string): string;
var
  sDelim: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  sDelim := devOptions.sValue(donRPCParametersDelimiter);
  Result := fRenalController.getRPCResults(aParamString, sDelim);
end;

function TMDK_Engine.getStudyStatus(aStudy: string): string;
var
  i: Integer;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  Result := fRenalController.getStudyStatus(aStudy);
  i := pos(#13#10, Result);
  if i > 0 then Result := copy(Result, 1, i - 1);
  i := pos('^', Result);
  if i > 0 then Result := copy(Result, 1, i - 1);
end;

procedure TMDK_Engine.SaveUserOptions;
begin
  if not Assigned(fRenalController) then Exit;
  fRenalController.setUserOptionByDUZ(onUserOptions, fUserDUZ, userOptions.ValueList.Text);
  fRenalController.setUserOptionByDUZ(onUserSettings, fUserDUZ, userSettings.ValueList.Text);
end;

procedure TMDK_Engine.SaveSystemOptions;
begin
  if Assigned(fRenalController) then fRenalController.SetSystemOption(onSysOptions, sysOptions.ValueList.Text);
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TMDK_Engine.getLocation;
var
  s: string;
begin
  if not Assigned(fRenalController) then Exit;
  fLocationID := '';
  fLocationName := '';
  if CurrentPatient = nil then Exit;
  s := fRenalController.getLocation(CurrentPatient.SessionID);
  if pos('-', s) = 1 then Exit
  else begin
    fLocationID := Piece(s, '^', 1);
    fLocationName := Piece(s, '^', 2);
  end;
end;

function TMDK_Engine.setLocation(aLocation: string): Boolean;
var
  s: string;
begin
  Result := True;
  if not Assigned(fRenalController) then Exit;
  Result := True;
  s := fRenalController.setLocation(CurrentPatient.SessionID, aLocation);
  if pos('-', s) = 1 then Result := False;
end;

function TMDK_Engine.RegisterAccessToSensitiveData(anID: string): string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.RegisterAccessToSensitiveData(anID, ExeNameAndVersion);
end;

function TMDK_Engine.IsDataSensitive(aPatientID: string): Boolean;
begin
  Result := True;
  if Assigned(fRenalController) then Result := fRenalController.IsDataSensitive(aPatientID, sSecurityWarning);
end;

function TMDK_Engine.getPatientAccessInfo(aDFN: string): string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.getSensitiveRecordAccessInfo(aDFN);
end;

function TMDK_Engine.getPatientBulletinInfo(aDFN: string): string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.getSensitiveRecordBulletinInfo(aDFN);
end;

function TMDK_Engine.setStudyStatusPending(aStudy: string): string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.setStudyStatus(aStudy, '5');
end;

function TMDK_Engine.RestoreOptionDefault(anOption: string): string;
var
  s: string;
  b: Boolean;
begin
  s := getDefaultResource(anOption);
  b := fRenalController.SetSystemOption(anOption, s);
  if b then s := ''
  else s := fRenalController.DBConnection.ExecResults.Text + #13#10#13#10;
  Result := s;
end;

function TMDK_Engine.RestoreDefaults: string;
begin
  Result := '';
  // Result := Result and RestoreOptionDefault(onDictionary);
  Result := Result + RestoreOptionDefault(onAntiCoagulants);
  Result := Result + RestoreOptionDefault(onNephrologiests);
  Result := Result + RestoreOptionDefault(onCodeStatuses);
  Result := Result + RestoreOptionDefault(onDialyzerList);
  Result := Result + RestoreOptionDefault(onEducationCodes);
  Result := Result + RestoreOptionDefault(onESRD_Diagnosys);
  Result := Result + RestoreOptionDefault(onESRD_ICD10); // added 06/12/12 Joe Carney
  Result := Result + RestoreOptionDefault(onMedicalFacilities);
  Result := Result + RestoreOptionDefault(onMedicationRoutes);
  Result := Result + RestoreOptionDefault(onMedicationUnits);
  Result := Result + RestoreOptionDefault(onModalities);
  Result := Result + RestoreOptionDefault(onSurgeons);
  Result := Result + RestoreOptionDefault(onTransType);
  Result := Result + RestoreOptionDefault(onTIUNoteTitles);
  Result := Result + RestoreOptionDefault(onMedications);
  Result := Result + RestoreOptionDefault(onReportList);
  Result := Result + RestoreOptionDefault('Summary Report');
  Result := Result + RestoreOptionDefault(onDatasetNames);
  Result := Result + RestoreOptionDefault(onCustomDataLists);
  SetSystemDefaults;
  SaveSystemOptions;
end;

procedure TMDK_Engine.CleanUpOptions;
var
  i: Integer;
  sName, s: string;
  SL: TStringList;
  sReports, sNotes, sTIUTemplates: string;
  function ValidOptionItemName(aList, aName: string): Boolean;
  begin
    Result := pos(aName, aList) > 0;
  end;
  function ValidOptionName(aName: string): Boolean;
  begin
    Result := False;
    Result := Result or (UpperCase(aName) = 'SUMMARY REPORT'); // just in case
    Result := Result or (aName = onDictionary);
    Result := Result or (aName = donDictionaryBackup);
    // 'CDSDICT'; // option name for Dictionary
    Result := Result or (aName = onAccessLogTable);    // 'AccessLogTable';
    Result := Result or (aName = onStudyListTable);    // 'StudyList';
    Result := Result or (aName = onWatchListTable);    // 'WatchListTable';
    Result := Result or (aName = onWatchListLogTable); // 'WatchListLogTable';
    Result := Result or (aName = onSubmittedStudies);  // 'Submitted Studies';
{$IFDEF SAVE_ERRORS_LOG}
    Result := Result or (aName = onErrorLog); // 'Errors';
{$ENDIF}
    // Lists of names of the system options ....................................
    Result := Result or (aName = onReportList); // 'Report List';
    Result := Result or (aName = onNotes);      // 'Notes';
    Result := Result or (aName = onNoteTemplates);
    Result := Result or (aName = onXSLTList); // 'XSLT Templates';
    // Exception lists .........................................................
    Result := Result or (aName = onAdministrators);     // 'Administrators';
    Result := Result or (aName = onAntiCoagulants);     // 'Anticoagulants';
    Result := Result or (aName = onCodeStatuses);       // 'Code Statuses';
    Result := Result or (aName = onCustomDataLists);    // 'Custom Data Lists';
    Result := Result or (aName = onDatasetNames);       // 'Datasets';
    Result := Result or (aName = onDialyzerList);       // 'Dialyzer List';
    Result := Result or (aName = onEducationCodes);     // 'Education Codes';
    Result := Result or (aName = onESRD_Diagnosys);     // 'ESRD Diagnoses';
    Result := Result or (aName = onESRD_ICD10);         // 'ESRD ICD10 Diagnoses';   //added 06/12/12 Joe Carney
    Result := Result or (aName = onMedications);        // 'Medications';
    Result := Result or (aName = onMedicalFacilities);  // 'Medical Facilities';
    Result := Result or (aName = onMedicationRoutes);   // 'Medication Routes';
    Result := Result or (aName = onMedicationUnits);    // 'Medication Units';
    Result := Result or (aName = onMedicationReasons);  // 'Medication Units';
    Result := Result or (aName = onMedicationStatuses); // 'Medication Units';
    Result := Result or (aName = onModalities);         // 'Modalities';
    Result := Result or (aName = onNephrologiests);     // 'Attending Nephrologists';
    Result := Result or (aName = onSurgeons);           // 'Surgeons';
    Result := Result or (aName = onSysOptions);         // 'System Options';
    Result := Result or (aName = onTransType);          // 'Transportation Methods';
    Result := Result or (aName = onTIUNoteTitles);      // 'TIU Note Titles'; //
    // a User Option Names .....................................................
    Result := Result or (aName = onUserOptions);
    Result := Result or (aName = onUserSettings);
  end;
  procedure checkPreferencesList(aName: string);
  var
    ss, sOpt: string;
    strL: TStringList;
    j: Integer;
  begin
    strL := TStringList.Create;
    if aName = 'System' then ss := getUserOptionByName(onSysOptions)
    else ss := getUserOptionByName(onUserOptions);
    strL.Text := ss;
    j := 0;
    sOpt := '';
    while j < strL.Count - 1 do begin
      ss := strL.Names[j];
      if not sysOptions.Exists(ss) then begin
        if strL[j] <> '' then sOpt := sOpt + strL[j] + #13#10;
        strL.Delete(j);
      end
      else inc(j);
    end;
    if sOpt <> '' then begin
      Report(rtWarning, 'The next Options have been deleted:' + #13#10 + sOpt, '', 0, '', aName + ' Options Cleanup');
      if aName = 'System' then begin
        sysOptions.Values.Text := strL.Text;
        setSystemOptionByName(onSysOptions, strL.Text);
      end else begin
        userOptions.Values.Text := strL.Text;
        setUserOptionByName(onUserOptions, strL.Text);
      end;
    end;
    strL.Free;
  end;
  function checkOptionString(aName, aValue: string): string;
  begin
    SL.Text := aValue;
    Result := '';
    i := 0;
    while i < SL.Count do begin
      sName := SL[i];
      if sName = '' then inc(i)
      else if ValidOptionName(sName) then begin
        inc(i);
        if sName = onUserOptions then checkPreferencesList('User');
        if sName = onSysOptions then checkPreferencesList('System');
      end else if ValidOptionItemName(sReports, sName) then inc(i)
      else if ValidOptionItemName(sNotes, sName) then inc(i)
      else if ValidOptionItemName(sTIUTemplates, sName) then inc(i)
      else begin
        if aName = '.5' then deleteSystemOption(sName)
        else deleteUserOption(sName);
        Result := Result + sName + #13#10;
        SL.Delete(i);
      end;
    end;
    if Result <> '' then Report(rtWarning, 'Deleted Options:' + #13#10#13#10 + Result);
  end;

begin
  SL := TStringList.Create;
  sReports := getSystemOptionByName(onReportList);
  sNotes := getSystemOptionByName(onNotes);
  sTIUTemplates := getSystemOptionByName(onNoteTemplates);
  s := fRenalController.getUserOptionList('.5');
  checkOptionString('.5', s);
  s := fRenalController.getUserOptionList(Engine.fUserDUZ);
  checkOptionString(Engine.fUserDUZ, s);
  SL.Free;
end;

function TMDK_Engine.getCP_TIU_TITLES(aTarget: string): string;
begin
  Result := '';
  if Assigned(fRenalController) then Result := fRenalController.getCPTIUTitles(aTarget);
end;

function TMDK_Engine.getAppointmentInfo: string;
begin
  Result := '';
  if not Assigned(fRenalController) then Exit;
  if Assigned(CurrentPatient) then
      Result := fRenalController.getAppintmentInfo(CurrentPatient.ID, CurrentPatient.SessionID);
end;

function TMDK_Engine.VersionIsCompatible: Boolean;
var
  sUsed, sNeeded: string;
  function isCorrectVersion(aVersion: string): Boolean;
  var
    s: string;
  begin
    s := fRenalController.getVersionInfo(aVersion);
    Result := UpperCase(s) = 'YES';
  end;

begin
  Result := False;
  if not Assigned(fRenalController) then Exit;
  sUsed := CurrentExeNameAndVersion;
  Result := isCorrectVersion(sUsed);
  if not Result then begin
    sNeeded := fRenalController.getSupportedVersion;
    MessageDLG('You are using ' + sUsed + ' and it' + #13 +
      'is not listed as compatible with the server you connected to.' + #13#13 + 'The selected server needs version ' +
      sNeeded + #13 + 'installed to ensure compatability with this client version.' + #13#13 +
      'This program must be upgraded before continuing.', mtWarning, [mbOk], 0);
  end;
end;

function TMDK_Engine.getPatientPhysicianOrders: string;
var
  SL: TStringList;
  s, sID: string;
  i: Integer;
begin
  Result := '';
  SL := TStringList.Create;
  s := Preferences.sValue(onOrdersIEN);
  i := 1;
  repeat
    sID := Piece(s, ',', i);
    if sID = '' then break;
    SL.Text := '';
    SL.Text := fRenalController.getPhysicianOrders(CurrentPatient.ID, sID);
    if SL.Count > 1 then begin
      if pos('t>>', SL[1]) = 1 then Result := Result + #13#10#13#10 + SL.Text;
    end;
    break; // for now one call of RPC only
    inc(i);
  until sID = '';
  SL.Free;
  if Result = '' then Result := 'No orders found...';
end;

function TMDK_Engine.isESigValid(aSig: string): Boolean;
begin
  Result := False;
  if Assigned(fRenalController) then Result := fRenalController.ValidateElectronicSignature(aSig);
  if not Result then inc(InvalidESigCount);
end;

function getICD10Date: string;
begin
  Result := Engine.getVistaImpDate; // calls rpc proc to retrieve date in fmdate style zzzzzzcarnej
end;

function getValues(anOptionName: string): string;
var
  SL: TStringList;
  i: Integer;
begin
  SL := nil;
  try
    Result := '';
    SL := TStringList.Create;
    SL.Text := Engine.getSystemOptionByName(anOptionName);
    for i := 0 to SL.Count - 1 do Result := Result + Piece(SL[i], '=', 2) + #13#10;
  finally
    SL.Free;
  end;
end;

function TMDK_Engine.ConvertToFmDate(impdate: TDateTime): string;
var
  Yr, md: string;
  CDate: TDateTime;
begin
  CDate := impdate;
  if CDate = 0 then CDate := Now;
  Yr := IntToStr(StrToInt(FormatDateTime('yyyy', CDate)) - 1700);
  md := FormatDateTime('mmdd', CDate);
  Result := Yr + md;
end;

end.
