unit uMDK_Options;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 4/18/08 6:55a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Options object
  *       Notes:        //added ICD10 Options 06/12/12 Joe Carney
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/APP-HEMODIALYSIS/Objects/uMDK_Options.pas $
  *
  *       $ History: uMDK_Options.pas $
}

interface

uses
  Classes, SysUtils, Windows, Graphics;

type
  TUserOptions = class(TObject)
  private
    fValueList: TStringList;
  public
    LastName: string;
    constructor Create;
    destructor Destroy; override;
    function sValue(aName: string): string;
    function sMinValue(aName: string): string;
    function sMaxValue(aName: string): string;
    function stringValue(aName: string): string;
    function iValue(aName: string): Integer;
    function bValue(aName: string): Boolean;
    function Exists(aName: string): Boolean;
    procedure AddOption(aName: string);
    procedure AddNameValue(aName, aValue: string);
    property Values: TStringList read fValueList;
    procedure Update(aText: string);
    procedure Overwrite(aText: string);
    procedure CleanupString(var aText: string);
    procedure CleanupBlankLines(var aText: string);
    procedure CleanupValues(aText: string);
    function setValue(aName, aValue: string): Boolean;
    function SetFMValue(aDateTime: TDateTime): string;
    property ValueList: TStringList read fValueList;
    function checkValues(var Reason: string): Boolean;
    function taggedValue(Source, TagStart, TagStop: string): string;
  end;

var
  userOptions: TUserOptions;
  sysOptions: TUserOptions;
  devOptions: TUserOptions;
  PreferenceComments: TUserOptions;
  Preferences: TUserOptions;
  Administrators: TUserOptions;
  userSettings: TUserOptions;
  ApplicationSessionID: string;
  sysOptionsDefaults: string;
  gsESRD_ICD9: string;
  gsESRD_ICD10: string;
  dDiagnosisDate: TDateTime;

procedure ResetPreferences(aSystem, aUser: string);
procedure SetSystemDefaults;
function getDefaultResource(aName: string): string;
function getSystemDefault(aName: string): string;

implementation

uses
  uMDK_CDSCommon, uMDK_Common, Dialogs, uMDK_Engine, uMDK_Utils;

constructor TUserOptions.Create;
begin
  inherited;
  fValueList := TStringList.Create;
end;

destructor TUserOptions.Destroy;
begin
  FreeAndNil(fValueList);
  inherited;
end;

function TUserOptions.sValue(aName: string): string;
var
  i: Integer;
begin
  try
    i := fValueList.IndexOfName(aName);
    if i > -1 then Result := fValueList.Values[aName]
    else Result := yoUnknown;
  except
    Result := yoUnknown;
  end;
end;

function TUserOptions.stringValue(aName: string): string;
var
  j, i: Integer;
begin
  try
    Result := '';
    for i := 0 to fValueList.Count - 1 do
      if pos(aName, fValueList[i]) = 1 then begin
        j := pos('=', fValueList[i]);
        if j > 0 then Result := copy(fValueList[i], j + 1, Length(fValueList[i]))
        else Result := fValueList[i];
        break;
      end;
    if Result = '' then Result := yoUnknown;
  except
    Result := yoUnknown;
  end;
end;

function TUserOptions.iValue(aName: string): Integer;
begin
  try
    if fValueList.IndexOfName(aName) > -1 then Result := StrToInt(fValueList.Values[aName])
    else Result := yoiUnknown;
  except
    Result := yoiUnknown;
  end;
end;

function TUserOptions.bValue(aName: string): Boolean;
var
  s: string;
begin
  Result := False;
  s := '';
  try
    if fValueList.IndexOfName(aName) > -1 then s := fValueList.Values[aName];
    Result := UpperCase(s) = 'TRUE';
  except
  end;
end;

procedure TUserOptions.AddOption(aName: string);
begin
  fValueList.Add(aName + '=' + yoUndefined);
end;

procedure TUserOptions.AddNameValue(aName, aValue: string);
begin
  fValueList.Add(aName + '=' + aValue);
end;

procedure TUserOptions.Update(aText: string);
var
  i: Integer;
  sName, ss: string;
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := aText;
  for i := 0 to SL.Count - 1 do begin
    sName := SL.Names[i];
    if sName = '' then continue;
    ss := sValue(sName);
    if ss = yoUnknown then AddNameValue(SL.Names[i], SL.Values[SL.Names[i]])
    else fValueList.Values[SL.Names[i]] := SL.Values[SL.Names[i]];
  end;
  SL.Free;
end;

procedure TUserOptions.Overwrite(aText: string);
var
  i: Integer;
  sName, ss: string;
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := aText;
  for i := 0 to SL.Count - 1 do begin
    sName := SL.Names[i];
    if sName = '' then continue;
    ss := sValue(sName);
    if ss = yoUnknown then AddNameValue(SL.Names[i], SL.Values[SL.Names[i]])
    else fValueList.Values[sName] := SL.Values[sName];
  end;
  SL.Free;
end;

procedure TUserOptions.CleanupString(var aText: string);
var
  i, j: Integer;
  sName: string;
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := aText;
  i := 0;
  while i < SL.Count do begin
    sName := SL.Names[i];
    if sName = '' then begin
      inc(i);
      continue;
    end;
    j := ValueList.IndexOfName(sName);
    if j < 0 then SL.Delete(i)
    else inc(i);
  end;
  aText := SL.Text;
  SL.Free;
end;

procedure TUserOptions.CleanupBlankLines(var aText: string);
var
  i: Integer;
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := aText;
  i := 0;
  while i < SL.Count do begin
    if SL[i] = '' then SL.Delete(i)
    else inc(i);
  end;
  aText := SL.Text;
  SL.Free;
end;

procedure TUserOptions.CleanupValues(aText: string);
var
  i, j: Integer;
  sName: string;
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := aText;
  i := 0;
  while i < ValueList.Count do begin
    sName := ValueList.Names[i];
    if sName = '' then begin
      inc(i);
      continue;
    end;
    j := SL.IndexOfName(sName);
    if j < 0 then ValueList.Delete(i)
    else inc(i);
  end;
  aText := SL.Text;
  SL.Free;
end;

function TUserOptions.setValue(aName, aValue: string): Boolean;
begin
  try
    fValueList.Values[aName] := aValue;
    Result := true;
  except
    Result := False;
  end;
end;

function TUserOptions.taggedValue(Source, TagStart, TagStop: string): string;
begin
  Result := piece(piece(Source, TagStart, 2), TagStop, 1);
end;

function TUserOptions.sMinValue(aName: string): string;
var
  sV: string;
begin
  sV := stringValue(aName);
  Result := taggedValue(sV, '<MIN>', '</MIN>');
end;

function TUserOptions.sMaxValue(aName: string): string;
var
  sV: string;
begin
  sV := stringValue(aName);
  Result := taggedValue(sV, '<MAX>', '</MAX>');
end;

function TUserOptions.checkValues(var Reason: string): Boolean;
var
  sName, sValue, sDefault, sReason, sMin, sMax: string;
  i: Integer;
begin
  Result := true;
  for i := 0 to fValueList.Count - 1 do begin
    try
      sName := fValueList.Names[i];
      sValue := fValueList.ValueFromIndex[i];
      sMin := PreferenceComments.sMinValue(sName);
      sMax := PreferenceComments.sMaxValue(sName);
      if not CheckOption(sName, sValue, sMin, sMax, sReason) then begin
        sDefault := getSystemDefault(sName);
        fValueList.Values[sName] := sDefault;
        Result := False;
        Reason := Reason + sReason + #13#10 + 'Value set to "' + sDefault + '"' + #13#10#13#10;
      end;
    except
    end;
  end;
end;

function TUserOptions.Exists(aName: string): Boolean;
begin
  Result := fValueList.IndexOfName(aName) > -1;
end;
/// /////////////////////////////////////////////////////////////////////////////
/// /////////////////////////////////////////////////////////////////////////////
/// /////////////////////////////////////////////////////////////////////////////

procedure ResetPreferences(aSystem, aUser: string);
begin
  sysOptions.ValueList.Text := aSystem;
  userOptions.ValueList.Text := aUser;
  Preferences.ValueList.Clear;
  Preferences.Overwrite(sysOptions.ValueList.Text);
  Preferences.Overwrite(userOptions.ValueList.Text);
  Preferences.Overwrite(userSettings.ValueList.Text);
end;

procedure SetSystemDefaults;
begin
  if not Assigned(sysOptions) then Exit;
  sysOptions.ValueList.Clear;
{$IFDEF CLEANUP}
  sysOptions.AddNameValue(onAllowUserCleanOptionsList, 'FALSE');
{$ENDIF}
  sysOptions.AddNameValue(onAllowUserControlStudyStatus, 'FALSE');
  sysOptions.AddNameValue(onAllowUserDeleteBlankFSRecords, 'FALSE');
  sysOptions.AddNameValue(onAllowUserResetStudyStatus, 'FALSE');
{$IFDEF STUDY_STATUS_LIST}
{$IFDEF SHOWACCESSHISTORY}
  sysOptions.AddNameValue(onAllowUserReviewAccessHistory, 'FALSE');
{$ENDIF}
  sysOptions.AddNameValue(onAllowUserReleaseSelectedStudy, 'FALSE');
  sysOptions.AddNameValue(onAllowUserReleaseStudySelectedBefore, 'FALSE');
  sysOptions.AddNameValue(onAllowUserResetActiveStudyList, 'FALSE');
{$ENDIF}
  sysOptions.AddNameValue(onBlanksPlaceholder, '<blank>');
  sysOptions.AddNameValue(onBrokerTimeout, '30');
  sysOptions.AddNameValue(onColorEditable, intToStr(clSilver));
  sysOptions.AddNameValue(onColorReview, intToStr(clSilver));
  sysOptions.AddNameValue(onColorUnknown, intToStr(clRed));
  sysOptions.AddNameValue(onColorToolbar, intToStr(clSilver));
  sysOptions.AddNameValue(onColorBackground, intToStr(clBtnFace));
  sysOptions.AddNameValue(onConsiderPainTolerance, 'FALSE');
  sysOptions.AddNameValue(onCoverRO, 'FALSE');
  sysOptions.AddNameValue(onDisabledFieldColor, intToStr(clBtnFace));
  sysOptions.AddNameValue(onFallsAssessmentAsTIU, 'TRUE');
{$IFDEF IGNORE_DUPLICATES}
  sysOptions.AddNameValue(onShowFlowsheetDuplicates, 'TRUE');
{$ENDIF}
  sysOptions.AddNameValue(onIgnoreUnfinishedStatus, 'TRUE');
  sysOptions.AddNameValue(onNormalFieldColor, intToStr(clWindow));
  sysOptions.AddNameValue(onOverwriteManualInput, 'TRUE');
  sysOptions.AddNameValue(onPainLevel, '1');
  sysOptions.AddNameValue(onProblemFieldColor, intToStr(clInfoBk));
  sysOptions.AddNameValue(onReadOnlyFieldColor, intToStr(clCream));
  sysOptions.AddNameValue(onRefreshRate, '15'); // refresh rate seconds
  sysOptions.AddNameValue(onReportKeyword, 'TREATMENT REPORT');
  sysOptions.AddNameValue(onReverseFlowsheetOrder, 'TRUE');
  sysOptions.AddNameValue(onSaveFlowsheetVitals, 'FALSE');
{$IFDEF SAVE_ERRORS_LOG}
  sysOptions.AddNameValue(onSaveErrorsLog, 'FALSE');
{$ENDIF}
  sysOptions.AddNameValue(onSaveVitals, 'FALSE');
  sysOptions.AddNameValue(onShowDisabledStudies, 'FALSE');
  sysOptions.AddNameValue(onShowFlowsheetEventsCopies, 'TRUE');
  sysOptions.AddNameValue(onShowInfectionTree, 'TRUE');
  sysOptions.AddNameValue(onShowReportSignature, 'TRUE');
  sysOptions.AddNameValue(onShowReportTemplates, 'TRUE');
  sysOptions.AddNameValue(onShowStatusPanel, 'TRUE');
  sysOptions.AddNameValue(onShowTIUNoteTemplates, 'FALSE');
  sysOptions.AddNameValue(onStudyListRefreshRate, '60');
  sysOptions.AddNameValue(onStudyLoadLimit, '5');
  sysOptions.AddNameValue(onSummaryReportName, 'Summary Report');
//  sysOptions.AddNameValue(onWebPageURL, 'http://domaindomain.ext/ClinicalSpecialties/MD/Default.asp?PageName=OVERVIEW');
  sysOptions.AddNameValue(onWebPageURL, 'http://domain.ext/vdl/application.asp?appid=139');
  sysOptions.ValueList.Sort;
  sysOptionsDefaults := sysOptions.ValueList.Text;
end;

procedure SetUserDefaults;
begin
  if not Assigned(userOptions) then Exit;
  userOptions.ValueList.Clear;
  userOptions.AddNameValue(onShowStatusPanel, 'TRUE');
{$IFDEF SHOWACCESSHISTORY}
  userOptions.AddNameValue(onAccessHistoryChecked, 'FALSE');
{$ENDIF}
  userOptions.ValueList.Sort;
  if not Assigned(userSettings) then Exit;
  userSettings.ValueList.Clear;
  userSettings.AddNameValue(onShowStudyDetails, 'TRUE');
  userSettings.AddNameValue(onShowStudyDetailsLegend, 'TRUE');
  userSettings.AddNameValue(onStudyListAutorefresh, 'TRUE');
  userSettings.AddNameValue(onShowDeletedMedicine, 'TRUE');
{$IFDEF SHOWACCESSHISTORY}
  userSettings.AddNameValue(onAccessHistoryChecked, 'FALSE');
{$ENDIF}
end;

procedure SetDeveloperDefaults;
begin
  if not Assigned(devOptions) then Exit;
  devOptions.AddNameValue(donSMTPHost, 'zzzzzzmsg1');
  devOptions.AddNameValue(donRPCParametersDelimiter, '^');
  devOptions.AddNameValue(donIgnoreBlankDateFields, 'TRUE');
  devOptions.AddNameValue(donSilentDefaultsRestore, 'TRUE');
  devOptions.AddNameValue(donCleanOptions, 'TRUE');
  devOptions.AddNameValue(onShowPatientTreatmentHistory, 'TRUE');
  devOptions.AddNameValue(onAllowDictionaryEdit, 'TRUE');
  devOptions.AddNameValue(donIncludeCommentsUpdate, 'TRUE');
  devOptions.AddNameValue(donShowAllComments, 'TRUE');
  devOptions.AddNameValue(onOrdersIEN, '13');
  devOptions.AddNameValue(donWindowsXPTheme, 'TRUE');
  devOptions.AddNameValue(donMinESigLength, '6');
  devOptions.AddNameValue(donWebUserManualURL,
    'http://domaindomain.ext/clinicalspecialties/clinproc/getdocument.asp?ID=175');
  devOptions.AddNameValue(donGatewayRequestTimeout, '100');
  devOptions.AddNameValue(donFallRiskKeyword, 'FALLS RISK EVALUATION');
  devOptions.AddNameValue(donFlowsheetDateInputMask, '99/99/9999');
  devOptions.AddNameValue(donFlowsheetTimeInputMask, '00:00');
  devOptions.AddNameValue(donKeepSubmitHistory, 'FALSE');
  devOptions.AddNameValue(donMaxProcedureQuantity, '999');
  devOptions.AddNameValue(donNotifyPCE, 'FALSE');
  devOptions.AddNameValue(donNotifyNewStudies, 'FALSE');
  devOptions.AddNameValue(donRegisterStudyAccess, 'FALSE');
  devOptions.AddNameValue(donRegisterStudyEvents, 'FALSE');
  devOptions.AddNameValue(donSaveInstrumentMessages, 'FALSE');
  devOptions.AddNameValue(donShowHorizontalScrollBarsInReports, 'TRUE');
  devOptions.AddNameValue(donShowHorizontalScrollBarsInComments, 'FALSE');
  devOptions.AddNameValue(donReportLineLength, '78'); // Report Line Length
  devOptions.AddNameValue(donShowFlowsheetDate, 'TRUE');
  devOptions.AddNameValue(donShowMedicineDate, 'TRUE');
end;

function getResourceString(aName: string): string;
var
  tmpStream: TResourceStream;
  SL: TStringList;
begin
  Result := '';
  if aName = '' then Exit;
  SL := TStringList.Create;
  tmpStream := nil;
  try
    tmpStream := TResourceStream.Create(HInstance, aName, 'TXT');
    SL.LoadFromStream(tmpStream);
  finally
    tmpStream.Free;
  end;
  Result := SL.Text;
  SL.Free;
end;

function getDefaultResource(aName: string): string;
begin
  Result := aName;
  if aName = onAntiCoagulants then Result := getResourceString('ANTICOAGULANTS')
  else if aName = onNephrologiests then Result := ''
  else if aName = onCodeStatuses then Result := getResourceString('CODE_STATUSES')
  else if aName = onDialyzerList then Result := getResourceString('DIALYZER_LIST')
  else if aName = onEducationCodes then Result := getResourceString('EDUCATION_CODES')
  else if aName = onESRD_Diagnosys then Result := getResourceString('ESRD_Diagnoses')
  else if aName = onESRD_ICD10 then                       // added 06/12/12 Joe Carney
      Result := getResourceString('ESRD_ICD10_Diagnoses') // added 06/12/12 Joe Carney
  else if aName = onMedicalFacilities then
    // Result := getResourceString('MEDICAL_FACILITIES')
      Result := ''
  else if aName = onMedicationRoutes then Result := getResourceString('MEDICATION_ROUTES')
  else if aName = onMedicationUnits then Result := getResourceString('MEDICATION_UNITS')
  else if aName = onModalities then Result := getResourceString('MODALITIES')
  else if aName = onSurgeons then Result := ''
  else if aName = onTransType then Result := getResourceString('TRANSPORTATION_METHODS')
  else if aName = onTIUNoteTitles then Result := '01=Site Specific TIU Note Title'
  else if aName = onMedications then Result := ''
  else if aName = onReportList then Result := 'Summary Report=Default Study Summary Report'
  else if aName = 'Summary Report' then Result := getResourceString('SUMMARY_REPORT')
  else if aName = onDictionary then Result := getResourceString('DICTIONARY')
  else if aName = onDatasetNames then Result := getResourceString('DATASETS')
  else if aName = onCustomDataLists then Result := getResourceString('CUSTOM_DATA_LISTS')
end;

function getSystemDefault(aName: string): string;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := sysOptionsDefaults;
  Result := SL.Values[aName];
  SL.Free;
end;

function TUserOptions.SetFMValue(aDateTime: TDateTime): string;
begin
  Result := engine.ConvertToFmDate(aDateTime);
end;

initialization

Preferences := TUserOptions.Create; // application options
sysOptions := TUserOptions.Create;  // system options
userOptions := TUserOptions.Create; // user options
devOptions := TUserOptions.Create;  // user options
PreferenceComments := TUserOptions.Create;
userSettings := TUserOptions.Create; // user options
{$IFNDEF XMLVIEW}
PreferenceComments.ValueList.Text := getResourceString('PREFERENCE_COMMENTS');
{$ENDIF}
Administrators := TUserOptions.Create; // Administrators
SetSystemDefaults;
SetUserDefaults;
SetDeveloperDefaults;
Preferences.Overwrite(sysOptions.ValueList.Text);
Preferences.Overwrite(userOptions.ValueList.Text);
Preferences.Overwrite(userSettings.ValueList.Text);

finalization

userOptions.Free;
sysOptions.Free;
devOptions.Free;
Administrators.Free;
Preferences.Free;
userSettings.Free;

end.
