unit uMDK_Vitals;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 2/26/08 10:01a $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developers:   dddddddddomain.user@domain.ext
  *       Description:  this unit contains the Hemodialysis application
  *                     Vitals routines
  *       Notes:
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Objects/uMDK_Vitals.pas $
  *
  * $History: uMDK_Vitals.pas $
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/13/09    Time: 1:16p
  * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/APP-HEMODIALYSIS/Objects
  * Patch 19 source code
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/24/08    Time: 4:23p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/APP-HEMODIALYSIS/Objects
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/02/08    Time: 9:25a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/APP-HEMODIALYSIS/Objects
  *
  * *****************  Version 4  *****************
  * User: Zzzzzzandria Date: 8/24/07    Time: 10:49a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Objects
  *
  * *****************  Version 3  *****************
  * User: Zzzzzzandria Date: 6/20/07    Time: 3:23p
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Objects
  *
  * *****************  Version 2  *****************
  * User: Zzzzzzandria Date: 6/04/07    Time: 9:08a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Objects
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 5/22/07    Time: 9:17a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/APP-HEMODIALYSIS/Objects
}

interface

uses
  SysUtils, Classes, iMDK_VitalsQualifiers;

type
  TVitalType = (vtUnknown, vtTemp, vtPulse, vtResp, vtBP, vtHeight, vtWeight, vtPain, vtPO2, vtCVP, vtCircum);
function CalculateBMI(const WeightInLbs: Double; const HeightInInches: Double): Double;
function ConvertInToCm(const Inches: Double): Double;
function ConvertCmToIn(const Centimeters: Double): Double;
function ConvertLbsToKgs(const Pounds: Double): Double;
function ConvertKgsToLbs(const Kilograms: Double): Double;
function ConvertFToC(const Fahrenheight: Double): Double;
function ConvertCToF(const Celsius: Double): Double;
function ConvertcmH20TommHg(const cmH20: Double): Double;
function ConvertmmHgTocmH20(const mmHg: Double): Double;
function HintString(isMetric: Boolean; aVT: TVitalType): string;
function ValidVS(aVT: TVitalType; aValue: string; SIMetric: Boolean = True): Boolean;
function ValidBP(aValue: string; aMetric: Boolean = True): Boolean;
function ValidPulse(aValue: string): Boolean;
function getVitalIENByName(I_MDKVitals: I_Vitals; aName: string): string;
function IsActiveQualifier(I_MDKVitals: I_Vitals; aVitalName, aQualifierName: string): Boolean;
function getQualifierIEN(I_MDKVitals: I_Vitals; aVitalName, aQualifier: string): string;
function getStandingIEN(I_MDKVitals: I_Vitals; aVitalName: string): string;
function getSittingIEN(I_MDKVitals: I_Vitals; aVitalName: string): string;

var
  GMVVitalHiRange: array [TVitalType] of Double;
  GMVVitalLoRange: array [TVitalType] of Double;

implementation

uses
  uROR_CommonUtils, Dialogs, System.UITypes;

var
  GMVVitalTypeIEN: array [TVitalType] of string = (
    '-1',
    '2',
    '5',
    '3',
    '1',
    '8',
    '9',
    '22',
    '21',
    '19',
    '20'
  );
  GMVVitalTypeAbbv: array [TVitalType] of string = (
    'xx',
    'T',
    'P',
    'R',
    'BP',
    'HT',
    'WT',
    'PN',
    'PO2',
    'CVP',
    'CG'
  );
  fQualifiers, fVitals: string;
  OutOfRangeString: string;

function MessageDlgEnteredInvalidValue(S, ErrorS: string): Word; // AAN 07/03/2002
begin
  result := MessageDlg('The value you entered ''' + S + ''' is not valid' + #13 + #13 + ErrorS, mtError, [mbok], 0);
end;

function HintString(isMetric: Boolean; aVT: TVitalType): string;
var
  fLow, fHigh: Double;
  sVT: string;
begin
  try
    sVT := GMVVitalTypeAbbv[aVT];
  except
    sVT := 'Unknown';
  end;
  fLow := GMVVitalLoRange[aVT];
  fHigh := GMVVitalHiRange[aVT];
  if isMetric then begin
    case aVT of
      vtTemp: begin
          fLow := ConvertFToC(fLow);
          fHigh := ConvertFToC(fHigh);
        end;
      vtHeight, vtCircum: begin
          fLow := ConvertInToCm(fLow);
          fHigh := ConvertInToCm(fHigh);
        end;
      vtWeight: begin
          fLow := ConvertLbsToKgs(fLow);
          fHigh := ConvertLbsToKgs(fHigh);
        end;
      vtCVP: begin
          fLow := ConvertcmH20TommHg(fLow);
          fHigh := ConvertcmH20TommHg(fHigh);
        end;
    end;
  end;
  result := 'Range check failure.' + #13#10 + 'Vitals Type: ' + sVT + #13#10 + 'Please enter a value from  ' +
    FloatToStr(fLow) + ' to ' + FloatToStr(fHigh);
end;

procedure OutOfRange(aMetric: Boolean; aVT: TVitalType);
begin
  OutOfRangeString := 'The value you entered is out of range for this vital.' + #13#13 + HintString(aMetric, aVT);
end;

function ValidWeight(aValue: string; aMetric: Boolean = True): Boolean;
var
  tmpDouble: Double;
begin
  result := False;
  if aValue = '' then Exit;
  try
    tmpDouble := StrToFloat(aValue);
    if aMetric then tmpDouble := ConvertKgsToLbs(tmpDouble);
    if (tmpDouble >= GMVVitalLoRange[vtWeight]) and (tmpDouble <= GMVVitalHiRange[vtWeight]) then
    begin { Process valid weight Here }
      // FValueDbl := tmpDouble;
      result := True;
    end
    else OutOfRange(aMetric, vtWeight);
  except
    raise;
  end;
end;

function ValidTemperature(aValue: string; aMetric: Boolean = True): Boolean;
var
  tmpDouble: Double;
begin
  Result := False;
  if aValue = '' then Exit;
  try
    tmpDouble := StrToFloat(aValue);
    if aMetric then tmpDouble := ConvertCToF(tmpDouble);
    if (tmpDouble >= GMVVitalLoRange[vtTemp]) and (tmpDouble <= GMVVitalHiRange[vtTemp]) then begin // Process valid Temperature here
      Result := True;
    end else begin
      OutOfRange(aMetric, vtTemp);
    end;
  except
    raise;
  end;
end;

function ValidPulse(aValue: string): Boolean;
var
  tmpInt: Integer;
begin
  result := False;
  if aValue = '' then Exit;
  try
    tmpInt := StrToInt(aValue);
    // if tmpInt = 0 then // zzzzzzandria 2008-02-26 commented
    // Exit;            // zzzzzzandria 2008-02-26 commented
    if (tmpInt >= GMVVitalLoRange[vtPulse]) and (tmpInt <= GMVVitalHiRange[vtPulse]) then begin { Process valid Pulse Here }
      result := True;
    end else begin                          // zzzzzzandria 2008-02-26 uncommented
      OutOfRange(True, vtPulse); // zzzzzzandria 2008-02-26 uncommented
    end;
  except
    raise;
  end;
end;

function ValidResp(aValue: string): Boolean;
var
  tmpInt: Integer;
begin
  result := False;
  try
    tmpInt := StrToInt(aValue);
    if (tmpInt >= GMVVitalLoRange[vtResp]) and (tmpInt <= GMVVitalHiRange[vtResp]) then begin { Process valid Pulse Here }
      result := True;
    end else begin
      OutOfRange(True, vtResp);
    end;
  except
    raise;
  end;
end;

function ValidBPComponent(aValue: string): Boolean;
var
  tmpInt: Integer;
begin
  result := False;
  if aValue = '' then Exit;
  try
    tmpInt := StrToInt(aValue);
    if (tmpInt >= GMVVitalLoRange[vtBP]) and (tmpInt <= GMVVitalHiRange[vtBP]) then begin { Process valid Pulse Here }
      result := True;
    end else begin
      OutOfRange(True, vtBP);
    end;
  except
    raise;
  end;
end;

function ValidBP(aValue: string; aMetric: Boolean = True): Boolean;
var
  i, iCount: Integer;
  S, sS, sD, sM: string;
  bS, bD, bM: Boolean;
  Systolic, Diastolic, Middle: Integer;
begin
  result := False;
  try
    { AAN 2003/06/05-----------------------------------------------------------Start }
    S := aValue; // cbxInput.Text;
    sS := Piece(S, '/', 1);
    sD := Piece(S, '/', 2);
    sM := Piece(S, '/', 3);
    iCount := 0;
    for i := 1 to length(S) do
      if copy(S, i, 1) = '/' then inc(iCount);
    S := Piece(aValue { cbxInput.Text } , '/', 4);
    if (S <> '') or (iCount > 2) or (iCount < 1) or ((iCount = 2) and (sD = '') and (sM = '')) or
      ((iCount = 2) and (sM = '')) or ((iCount = 1) and (sD = '')) then begin
      Exit;
    end;
    try
      Systolic := StrToInt(sS);
      bS := (Systolic >= GMVVitalLoRange[vtBP]) and (Systolic <= GMVVitalHiRange[vtBP]);
      if sD <> '' then Diastolic := StrToInt(sD)
      else Diastolic := -1;
      bD := (sD = '') or ((Diastolic >= GMVVitalLoRange[vtBP]) and (Diastolic <= GMVVitalHiRange[vtBP]));
      if sM <> '' then Middle := StrToInt(sM)
      else Middle := -1;
      bM := (sM = '') or ((Middle >= GMVVitalLoRange[vtBP]) and (Middle <= GMVVitalHiRange[vtBP]));
      result := (bS and bD and bM);
      // 1.0.6.306 (#4) -- Dan's comment
      if sM = '' then result := result and (Diastolic < Systolic)
      else result := result and (Middle < Systolic);
    except
      on E: EConvertError do result := False;
    end;
    Exit;
    { AAN 2003/06/05-------------------------------------------------------------End }
    Systolic := StrToInt(Piece(aValue, '/', 1));
    Diastolic := StrToInt(Piece(aValue, '/', 2));
    Middle := StrToIntDef(Piece(aValue, '/', 3), 0);
    if Diastolic = 0 then begin
      if (Systolic >= GMVVitalLoRange[vtBP]) and (Systolic <= GMVVitalHiRange[vtBP]) and
        (Middle >= GMVVitalLoRange[vtBP]) and (Middle <= GMVVitalHiRange[vtBP]) then begin
        result := True;
      end
      else Exit;
    end;
    if ((Middle = 0) and (Systolic >= GMVVitalLoRange[vtBP]) and (Systolic <= GMVVitalHiRange[vtBP]) and
      (Diastolic >= GMVVitalLoRange[vtBP]) and (Diastolic <= GMVVitalHiRange[vtBP])) or
      ((Systolic >= GMVVitalLoRange[vtBP]) and (Systolic <= GMVVitalHiRange[vtBP]) and
      (Diastolic >= GMVVitalLoRange[vtBP]) and (Diastolic <= GMVVitalHiRange[vtBP]) and (Middle >= GMVVitalLoRange[vtBP]
      ) and (Middle <= GMVVitalHiRange[vtBP])) then begin
      result := True;
    end;
  except
    raise;
  end;
end;

function ValidVS(aVT: TVitalType; aValue: string; SIMetric: Boolean = True): Boolean;
begin
  result := False;
  if aValue = '' then Exit;
  case aVT of
    vtResp: result := ValidResp(aValue);
    vtWeight: result := ValidWeight(aValue, SIMetric);
    vtTemp: result := ValidTemperature(aValue, SIMetric);
    vtPain: result := True; // we select value from  the list
    vtPulse: result := ValidPulse(aValue);
    vtBP: result := ValidBPComponent(aValue);
  end;
end;

function VitalTypeByString(aType: string): TVitalType;
begin
  if aType = GMVVitalTypeIEN[vtTemp] then result := vtTemp
  else if aType = GMVVitalTypeIEN[vtPulse] then result := vtPulse
  else if aType = GMVVitalTypeIEN[vtResp] then result := vtResp
  else if aType = GMVVitalTypeIEN[vtBP] then result := vtBP
  else if aType = GMVVitalTypeIEN[vtHeight] then result := vtHeight
  else if aType = GMVVitalTypeIEN[vtWeight] then result := vtWeight
  else if aType = GMVVitalTypeIEN[vtPain] then result := vtPain
  else if aType = GMVVitalTypeIEN[vtPO2] then result := vtPO2
  else if aType = GMVVitalTypeIEN[vtCVP] then result := vtCVP
  else if aType = GMVVitalTypeIEN[vtCircum] then result := vtCircum
  else result := vtUnknown;
end;

function CalculateBMI(const WeightInLbs: Double; const HeightInInches: Double): Double;
begin
  result := Round(WeightInLbs / (HeightInInches * HeightInInches));
end;

function ConvertInToCm(const Inches: Double): Double;
begin
  result := Round(Inches * 2.54 * 10) * 0.1;
end;

function ConvertCmToIn(const Centimeters: Double): Double;
begin
  result := Round(Centimeters / 2.54 * 10) * 0.1;
end;

function ConvertLbsToKgs(const Pounds: Double): Double;
begin
  result := Round(Pounds / 2.2);
end;

function ConvertKgsToLbs(const Kilograms: Double): Double;
begin
  result := Round(Kilograms * 220.26431718) * 0.01;
end;

function ConvertFToC(const Fahrenheight: Double): Double;
begin
  result := Round(((Fahrenheight - 32) * 5.0 / 9.0) * 10) * 0.1;
end;

function ConvertCToF(const Celsius: Double): Double;
begin
  result := Round(((Celsius * 9.0 / 5.0) + 32) * 10) * 0.1;
end;

function ConvertcmH20TommHg(const cmH20: Double): Double;
begin
  result := cmH20 / 1.36; // AAN 07/03/2002
end;

function ConvertmmHgTocmH20(const mmHg: Double): Double;
begin
  result := mmHg * 1.36; // AAN 07/03/2002
end;

function iPOx(sValue: string): Integer;
var
  S: string;
begin
  if sValue = '' then begin
    result := 0;
    Exit;
  end;
  if pos('*', sValue) <> 0 then S := Piece(sValue, '*', 1)
  else S := Piece(sValue, ' ', 1);
  try
    result := StrToInt(S);
  except
    result := 0;
  end;
end;

function iVal(gStr: string): Integer;
var
  i: Integer;
begin
  i := 1;
  while StrToIntDef(copy(gStr, i, 1), -1) > -1 do inc(i);
  result := StrToIntDef(copy(gStr, 1, i - 1), 0);
end;

function BPMean(BP: string): Integer;
begin
  result := (iVal(BP) + iVal(Piece(BP, '/', 2))) div 2;
end;

function HasNumericValue(S: string): Boolean;
begin
  result := (StrToIntDef(copy(S, 1, 1), 99) <> 99);
end;

function SeriesLabel(sLbl: string): string;
begin
  while copy(sLbl, 1, 1) = ' ' do sLbl := copy(sLbl, 2, length(sLbl));
  result := copy(sLbl, 1, pos(' ', sLbl) - 1) + #13 + copy(sLbl, pos(' ', sLbl) + 1, length(sLbl));
end;
/// /////////////////////////////////////////////////////////////////////////////

function getVitalIENByName(I_MDKVitals: I_Vitals; aName: string): string;
var
  i: Integer;
  SL: TStringList;
begin
  result := '';
  if fVitals = '' then fVitals := I_MDKVitals.getVitalTypesAll;
  SL := TStringList.Create;
  SL.Text := fVitals;
  for i := 0 to SL.Count - 1 do begin
    if Piece(SL[i], '^', 2) = aName then begin
      result := Piece(Piece(SL[i], '^', 1), ';', 2);
      Break;
    end;
  end;
  SL.Free;
end;

function IsActiveQualifier(I_MDKVitals: I_Vitals; aVitalName, aQualifierName: string): Boolean;
var
  sIEN, S: string;
begin
  result := False;
  sIEN := getVitalIENByName(I_MDKVitals, aVitalName);
  if sIEN = '' then Exit;
  S := I_MDKVitals.getQualifiersByVitalType(sIEN);
  result := pos(aQualifierName, S) > 0;
end;

function getQualifierIEN(I_MDKVitals: I_Vitals; aVitalName, aQualifier: string): string;
var
  i: Integer;
  SL: TStringList;
begin
  result := '';
  if not IsActiveQualifier(I_MDKVitals, aVitalName, aQualifier) then Exit;
  if fQualifiers = '' then fQualifiers := I_MDKVitals.getQualifiersAll;
  SL := TStringList.Create;
  SL.Text := fQualifiers;
  for i := 0 to SL.Count - 1 do begin
    if Piece(SL[i], '^', 2) = aQualifier then begin
      result := Piece(Piece(SL[i], '^', 1), ';', 2);
      Break;
    end;
  end;
  SL.Free;
end;

function getSittingIEN(I_MDKVitals: I_Vitals; aVitalName: string): string;
begin
  result := getQualifierIEN(I_MDKVitals, aVitalName, 'SITTING');
end;

function getStandingIEN(I_MDKVitals: I_Vitals; aVitalName: string): string;
begin
  result := getQualifierIEN(I_MDKVitals, aVitalName, 'STANDING');
end;

initialization

begin
  fVitals := '';
  { GMVVitalHiRange[Vital] all values are US Standard }
  GMVVitalHiRange[vtUnknown] := 0;
  GMVVitalHiRange[vtTemp] := 120;
  GMVVitalHiRange[vtPulse] := 300;
  GMVVitalHiRange[vtResp] := 100;
  GMVVitalHiRange[vtBP] := 300;
  GMVVitalHiRange[vtHeight] := 100;  // AAN 07/03/2002 was 300
  GMVVitalHiRange[vtWeight] := 1500; // AAN 07/03/2002 was 500
  GMVVitalHiRange[vtPain] := 10;
  GMVVitalHiRange[vtPO2] := 100;
  GMVVitalHiRange[vtCVP] := 136;    // AAN 07/03/2002
  GMVVitalHiRange[vtCircum] := 200; // AAN 07/03/2002 was 300;
  GMVVitalLoRange[vtUnknown] := 0;
  GMVVitalLoRange[vtTemp] := 45;
  GMVVitalLoRange[vtPulse] := 0;
  GMVVitalLoRange[vtResp] := 0;
  GMVVitalLoRange[vtBP] := 0;
  GMVVitalLoRange[vtHeight] := 10;
  // zzzzzzandria 050321. was 0. See Remedy HD0000000068371
  GMVVitalLoRange[vtWeight] := 0;
  GMVVitalLoRange[vtPain] := 0;
  GMVVitalLoRange[vtPO2] := 0;
  GMVVitalLoRange[vtCVP] := -13.6; // AAN 07/03/2002    was -10;
  GMVVitalLoRange[vtCircum] := 0;
end;

end.
