unit uROR_CommonUtils;
{
  ================================================================================
  *
  *       Package:        ROR - Clinical Case Registries
  *       Date Created:   $Revision: 1 $  $Modtime: 10/15/07 1:56p $
  *       Site:           Hines OIFO
  *       Developers:
  *                       Andrey Andriyevskiy
  *                       Sergey Gavrilov
  *
  *       Description:    Common Utils
  *
  *       Notes:
  *
  ================================================================================
  *       $ Archive: /CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI/ROR/uROR_CommonUtils.pas $
  *
  *       $ History: uROR_CommonUtils.pas $
  *
  *
  ================================================================================
}

interface

uses
  Classes,  SysUtils,  Dialogs,  StdCtrls,  Forms,  Controls,  ComCtrls,  Graphics,  ExtCtrls;

type
  TFMDateTimeMode = set of (FmdtDateOnly,  // Convert date only
                            FmdtShortDate, // Short date (digits only)
                            FmdtYear2,     // 2 digit year
                            FmdtTimeOnly,  // Convert time only
                            FmdtShortTime, // Do not convert seconds
                            FmdtTime24     // Military time format
    );
function CmdLineSwitch(Swlst: array of string): Boolean; overload;
function CmdLineSwitch(Swlst: array of string; var Swval: string): Boolean; overload;
function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode = []; DfltValue: string = ''): string;
function FMDateToWindowsDate(FMDate: Double): TDate;
function FMDateTimeToWindowsDateTime(FMDateTime: Double): TDateTime;
function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
function IntVal(Str: string): Integer;
function MessageDialog(const ACaption: string; const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
  DefButton: Integer; HelpCtx: Longint): Word;
function Piece(Value, Delimiter: string; StartPiece: Integer = 1): string; overload;
function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece: Integer): string; overload;
function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;
function WindowsDateToFMDate(WinDate: TDate): Double;
function AddKeyToValue(Value, Key, Delim: string): string;
function DelKeyFromValue(Value, Key, Delim: string): string;
function FormatSSN(ASSN: string): string;
function ReFormatSSN(ASSN: string): string;
procedure DelLVSelectedLine(LVFrom: TListView);
procedure CopySelectedLVLines(LVFrom, LVTo: TListView; Mode: Boolean);
procedure CopyAllLVLine(LVFrom, LVTo: TListView);
procedure CopyLVLine(LVFrom, LVTo: TListView; Mode: Boolean);
procedure CopyLVHeaders(LVFrom, LVTo: TListView);
procedure UpdateAndNotClearListView(AStrings: TStrings; ALV: TListView; ICaptionIndex: Integer;
  AnIndexes: array of Integer);
procedure UpdateListView(AStrings: TStrings; ALV: TListView; ICaptionIndex: Integer; AnIndexes: array of Integer);
procedure ListViewToParamList(ALV: TListView; AnIndex: Integer; var AStrings: TStringList);
procedure UpdateStringList(AStrings: TStrings; ASL: TStringList);
procedure StringListToListView(ASL: TStringList; ALV: TListView; ICaptionIndex: Integer; AnIndexes: array of Integer);
procedure ListViewToStringList(ALV: TListView; AL: TStringList; AnIndexes: array of Integer; AStart, ADelim: string);
procedure LoadColumnsWidth(AName: string; AColumns: TListColumns; SL: TStringList);
procedure AddParam(AName, AValue: string; SL: TStringList);
procedure SaveColumnsWidth(AName: string; AColumns: TListColumns; SL: TStringList);
function StringByKey(AStrings: TStrings; AKey: string; APosition: Integer): string;
procedure UpdateEdit(var Key: Char; Sender: TObject);
procedure SetPanelStatus(APanel: TPanel; AStatus: Boolean; AColor: TColor);
function CtrlColor(Active: Boolean): TColor;

const
  FmdtShortDateTime = [FmdtShortDate, FmdtShortTime, FmdtTime24];

var
  FMDateFormat: array [1 .. 2, 1 .. 3] of string;

implementation

uses
  System.UITypes, Windows;

function CtrlColor(Active: Boolean): TColor;
begin
  if Active then Result := ClWindow
  else Result := ClBtnFace;
end;

function FMDateTimeStr(DateTime: string; Mode: TFMDateTimeMode; DfltValue: string): string;
var
  Buf, Format: string;
  Day, Month, Year: Word;
  Hour, Min, Sec: Word;
  Date, Time: TDate;
  DateType, DatePart: Integer;
begin
  if DateTime = '' then begin
    Result := DfltValue;
    Exit;
  end;
  Result := '';
  Buf := Piece(DateTime, '.', 1);
  if (not(FmdtTimeOnly in Mode)) and (Buf <> '') then begin
    Year := IntVal(Copy(Buf, 1, 3)) + 1700;
    Month := IntVal(Copy(Buf, 4, 2));
    Day := IntVal(Copy(Buf, 6, 2));
    if FmdtShortDate in Mode then DateType := 2
    else DateType := 1;
    DatePart := 1;
    if Day = 0 then begin
      Day := 1;
      DatePart := 2;
      if Month = 0 then begin
        Month := 1;
        DatePart := 3;
      end;
    end;
    Format := FMDateFormat[DateType][DatePart];
    if FmdtYear2 in Mode then Format := StringReplace(Format, 'YYYY', 'YY', []);
    Date := EncodeDate(Year, Month, Day);
    Result := FormatDateTime(Format, Date);
  end;
  Buf := Piece(DateTime, '.', 2);
  if (not(FmdtDateOnly in Mode)) and (Buf <> '') then begin
    Buf := Copy(Buf + '000000', 1, 6);
    Hour := IntVal(Copy(Buf, 1, 2));
    Min := IntVal(Copy(Buf, 3, 2));
    Sec := IntVal(Copy(Buf, 5, 2));
    if Hour >= 24 then begin
      Hour := 23;
      Min := 59;
      Sec := 59;
    end else if Min >= 60 then begin
      Min := 59;
      Sec := 59;
    end else if Sec >= 60 then Sec := 59;
    Time := EncodeTime(Hour, Min, Sec, 0);
    if FmdtTime24 in Mode then
      if FmdtShortTime in Mode then Format := 'HH:NN'
      else Format := 'HH:NN:SS'
    else if FmdtShortTime in Mode then Format := 'T'
    else Format := 'TT';
    if Result <> '' then Result := Result + '  ' + FormatDateTime(Format, Time)
    else Result := FormatDateTime(Format, Time);
  end;
end;

procedure FMInitFormatArray;
var
  Format: string;
begin
  Format := UpperCase(FormatSettings.ShortDateFormat);
  if Pos('M', Format) > Pos('D', Format) then begin
    FMDateFormat[1][1] := 'DD MMM YYYY';
    FMDateFormat[1][2] := 'MMM YYYY';
    FMDateFormat[1][3] := 'YYYY';
    FMDateFormat[2][1] := 'DD/MM/YYYY';
    FMDateFormat[2][2] := 'MM/YYYY';
    FMDateFormat[2][3] := 'YYYY';
  end else begin
    FMDateFormat[1][1] := 'MMM DD, YYYY';
    FMDateFormat[1][2] := 'MMM YYYY';
    FMDateFormat[1][3] := 'YYYY';
    FMDateFormat[2][1] := 'MM/DD/YYYY';
    FMDateFormat[2][2] := 'MM/YYYY';
    FMDateFormat[2][3] := 'YYYY';
  end;
end;

function CmdLineSwitch(Swlst: array of string): Boolean;
var
  Swval: string;
begin
  Result := CmdLineSwitch(Swlst, Swval);
end;

function CmdLineSwitch(Swlst: array of string; var Swval: string): Boolean;
var
  I: Integer;
begin
  Result := False;
  Swval := '';
  for I := 1 to ParamCount do begin
    if InString(ParamStr(I), Swlst, False) then begin
      Swval := Piece(ParamStr(I), '=', 2);
      Result := True;
      Break;
    end;
  end;
end;

function InString(S: string; SubStrs: array of string; CaseSensitive: Boolean = True): Boolean;
var
  I: Integer;
begin
  I := 0;
  Result := False;
  while (I <= high(SubStrs)) and (Result = False) do begin
    if CaseSensitive then begin
      if Pos(SubStrs[I], S) > 0 then Result := True
      else Inc(I)
    end else begin
      if Pos(LowerCase(SubStrs[I]), LowerCase(S)) > 0 then Result := True
      else Inc(I)
    end;
  end
end;

function IntVal(Str: string): Integer;
begin
  if Str <> '' then
    try
      Result := StrToInt(Str);
    except
      on EConvertError do Result := 0;
      else
        raise;
    end
  else Result := 0;
end;
{ -----------------------------------------------------------------------
  Source:   Torry's Delphi page
  Author:   Thomas Stutz
  Homepage: http://www.swissdelphicenter.ch
  ----------------------------------------------------------------------- }

function MessageDialog(const ACaption: string; const Msg: string; DlgType: TMsgDlgType; Buttons: TMsgDlgButtons;
  DefButton: Integer; HelpCtx: Longint): Word;
var
  I: Integer;
  Btn: TButton;
begin
  with CreateMessageDialog(Msg, DlgType, Buttons) do
    try
      Caption := ACaption;
      HelpContext := HelpCtx;
      for I := 0 to ComponentCount - 1 do begin
        if (Components[I] is TButton) then begin
          Btn := TButton(Components[I]);
          Btn.Default := Btn.ModalResult = DefButton;
          if Btn.Default then ActiveControl := Btn;
        end;
      end;
      Result := ShowModal;
    finally
      Free;
    end;
end;

function Piece(Value, Delimiter: string; StartPiece: Integer = 1): string;
begin
  Result := Piece(Value, Delimiter, StartPiece, StartPiece);
end;

function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece: Integer): string;
var
  Dlen, I, Pnum: Integer;
  Buf: string;
begin
  Result := '';
  if (Value <> '') and (StartPiece > 0) and (EndPiece >= StartPiece) then begin
    Dlen := Length(Delimiter);
    I := Pos(Delimiter, Value) - 1;
    if I >= 0 then begin
      Buf := Value;
      Pnum := 1;
      repeat
        if Pnum > EndPiece then Break;
        if I < 0 then I := Length(Buf);
        if Pnum = StartPiece then Result := Copy(Buf, 1, I)
        else if Pnum > StartPiece then Result := Result + Delimiter + Copy(Buf, 1, I);
        Delete(Buf, 1, I + Dlen);
        I := Pos(Delimiter, Buf) - 1;
        Inc(Pnum);
      until (I < 0) and (Buf = '');
    end else if StartPiece = 1 then Result := Value;
  end;
end;

function WindowsDateToFMDate(WinDate: TDate): Double;
var
  Year, Month, Day: Word;
begin
  DecodeDate(WinDate, Year, Month, Day);
  Result := ((Year - 1700) * 10000) + (Month * 100) + Day;
end;

function FMDateToWindowsDate(FMDate: Double): TDate;
var
  FMString: string;
  Year, Month, Day: Word;
begin
  FMString := FloatToStr(FMDate);
  try
    Year := StrToInt(Copy(FMString, 1, 3)) + 1700;
    Month := StrToInt(Copy(FMString, 4, 2));
    Day := StrToInt(Copy(FMString, 6, 2));
    if Month = 0 then Month := 1; // zzzzzzandria 050414
    if Day = 0 then Day := 1;     // zzzzzzandria 050414
    Result := EncodeDate(Year, Month, Day);
  except
    on E: EConvertError do begin
{$IFDEF AANTEST}
      MessageDlg('Error Converting Date ' + FMString, MtError, [Mbok], 0);
{$ENDIF}
      Result := Now;
    end;
    else
      raise;
  end;
end;

function WindowsDateTimeToFMDateTime(WinDate: TDateTime): Double;
var
  Hour, Min, Sec, Msec: Word;
begin
  Result := WindowsDateToFMDate(WinDate);
  DecodeTime(WinDate, Hour, Min, Sec, Msec);
  Result := Result + (Hour * 0.01) + (Min * 0.0001) + (Sec * 0.000001);
end;

function FMDateTimeToWindowsDateTime(FMDateTime: Double): TDateTime;
var
  FMString: string;
  Hour, Min, Sec: Word;
begin
  FMString := FloatToStr(FMDateTime);
  Result := FMDateToWindowsDate(FMDateTime);
  Hour := StrToIntDef(Copy(FMString, 9, 2), 0);
  Min := StrToIntDef(Copy(FMString, 11, 2), 0);
  Sec := StrToIntDef(Copy(FMString, 13, 2), 0);
  Result := Result + EncodeTime(Hour, Min, Sec, 0);
end;

function AddKeyToValue(Value, Key, Delim: string): string;
var
  I: Integer;
begin
  I := Pos(Delim + Key, Value);
  if I > 0 then Result := Value
  else begin
    I := Pos(Key, Value);
    if I = 1 then Result := Value
    else begin
      if Value = '' then Result := Key
      else Result := Value + Delim + Key;
    end;
  end;
end;

function DelKeyFromValue(Value, Key, Delim: string): string;
var
  I: Integer;
  S: string;
begin
  I := Pos(Key, Value);
  if I = 0 then Result := Value
  else begin
    S := Delim + Key;
    I := Pos(S, Value);
    if I <> 0 then S := Copy(Value, 1, I - 1) + Copy(Value, I + Length(S), Length(Value))
    else if Pos(Key, Value) = 1 then S := Copy(Value, Length(Key) + 1, Length(Value));
    if Pos(Delim, S) = 1 then S := Copy(S, Length(Delim) + 1, Length(S));
    Result := S;
  end;
end;

function FormatSSN(ASSN: string): string;
begin
  try
    Result := Copy(ASSN, 1, 3) + '-' + Copy(ASSN, 4, 2) + '-' + Copy(ASSN, 6, 99);
  except
    Result := ASSN
  end;
end;

function ReFormatSSN(ASSN: string): string;
var
  I: Integer;
  SResult: string;
begin
  I := Pos('-', ASSN);
  SResult := ASSN;
  while I <> 0 do begin
    SResult := Copy(SResult, 1, I - 1) + Copy(SResult, I + 1, 99);
    I := Pos('-', SResult);
  end;
  Result := SResult;
end;

procedure DelLVSelectedLine(LVFrom: TListView);
var
  IPrev, I: Integer;
begin
  if not Assigned(LVFrom) then Exit;
  if LVFrom.ItemFocused = nil then Exit;
  IPrev := LVFrom.ItemFocused.Index;
  I := 0;
  while I < LVFrom.Items.Count do begin
    if LVFrom.Items[I].Selected then LVFrom.Items.Delete(I)
    else Inc(I);
  end;
  try
    if LVFrom.Items.Count < IPrev then IPrev := LVFrom.Items.Count - 1;
    if IPrev >= 0 then LVFrom.Selected := LVFrom.Items[IPrev];
  except
    on E: Exception do MessageDlg('Error Deleting Element list' + #13 + E.Message, MtInformation, [Mbok], 0);
  end;
  Exit;
  try
    if LVFrom.ItemFocused = nil then LVFrom.Items.Delete(0)
    else LVFrom.Items.Delete(LVFrom.ItemFocused.Index);
    LVFrom.Selected := LVFrom.ItemFocused;
  except
  end;
end;

procedure CopyLVLine(LVFrom, LVTo: TListView; Mode: Boolean);
var
  I: Integer;
  LiFrom, Li: TListItem;
begin
  if LVFrom.ItemFocused = nil then LiFrom := LVFrom.Items[0]
  else LiFrom := LVFrom.ItemFocused;
  try
    for I := 0 to LVTo.Items.Count - 1 do
      if LVTo.Items[I].Caption = LiFrom.Caption then begin
        LVTo.ItemFocused := LVTo.Items[I];
        Exit;
      end;
    Li := LVTo.Items.Add;
    Li.Caption := LiFrom.Caption;
    for I := 0 to LiFrom.SubItems.Count - 1 do Li.SubItems.Add(LiFrom.SubItems[I]);
    if Mode then LVFrom.Items.Delete(LiFrom.Index)
    else if LiFrom.Index < LVFrom.Items.Count - 1 then LVFrom.ItemFocused := LVFrom.Items[LiFrom.Index + 1];
    LVFrom.Selected := LVFrom.ItemFocused;
  except
  end;
end;

procedure CopySelectedLVLines(LVFrom, LVTo: TListView; Mode: Boolean);
var
  IPrev, ICurr, I: Integer;
  LiFrom, Li: TListItem;
begin
  if LVFrom.ItemFocused = nil then Exit;
  IPrev := LVFrom.ItemFocused.Index;
  ICurr := 0;
  while ICurr < LVFrom.Items.Count do begin
    LiFrom := LVFrom.Items[ICurr];
    if not LiFrom.Selected then begin
      Inc(ICurr);
      Continue;
    end;
    LiFrom.Selected := False;
    for I := 0 to LVTo.Items.Count - 1 do
      if LVTo.Items[I].Caption = LVFrom.Items[ICurr].Caption then Break;
    if I < LVTo.Items.Count then begin
      Inc(ICurr);
      Continue;
    end;
    Li := LVTo.Items.Add;
    Li.Caption := LiFrom.Caption;
    for I := 0 to LiFrom.SubItems.Count - 1 do Li.SubItems.Add(LiFrom.SubItems[I]);
    if Mode then LVFrom.Items.Delete(ICurr)
    else Inc(ICurr);
  end;
  try
    if LVFrom.Items.Count < IPrev then IPrev := LVFrom.Items.Count - 1
    else Inc(IPrev);
    if IPrev >= 0 then begin
      LVFrom.ItemFocused := LVFrom.Items[IPrev];
      LVFrom.Selected := LVFrom.ItemFocused;
    end;
  except
    on E: Exception do MessageDlg('Error Deleting Element list' + #13 + E.Message, MtInformation, [Mbok], 0);
  end;
end;

procedure CopyAllLVLine(LVFrom, LVTo: TListView);
var
  J, I: Integer;
begin
  try
    J := LVFrom.ItemFocused.Index;
  except
    J := 0;
  end;
  for I := 0 to LVFrom.Items.Count - 1 do begin
    LVFrom.ItemFocused := LVFrom.Items[I];
    CopyLVLine(LVFrom, LVTo, False);
  end;
  LVFrom.ItemFocused := LVFrom.Items[J];
end;

procedure CopyLVHeaders(LVFrom, LVTo: TListView);
var
  Lc: TListColumn;
  I: Integer;
begin
  LVTo.Columns.Clear;
  for I := 0 to LVFrom.Columns.Count - 1 do begin
    Lc := LVTo.Columns.Add;
    Lc.Caption := LVFrom.Columns[I].Caption;
    Lc.Width := LVFrom.Columns[I].Width;
  end;
end;

procedure UpdateListView(AStrings: TStrings; ALV: TListView; ICaptionIndex: Integer; AnIndexes: array of Integer);
var
  S: string;
  Li: TListItem;
  J, I, ICount: Integer;
begin
  try
    ICount := StrToInt(Piece(AStrings[0], '^', 1));
    if ICount > AStrings.Count - 1 then ICount := AStrings.Count - 1;
  except
    ICount := 0;
  end;
  ALV.Items.Clear;
  for I := 1 to ICount do begin
    S := AStrings[I];
    Li := ALV.Items.Add;
    Li.Caption := Piece(S, '^', ICaptionIndex);
    for J := low(AnIndexes) to high(AnIndexes) do Li.SubItems.Add(Piece(S, '^', AnIndexes[J]));
  end;
  try
    if ALV.Items.Count < 1 then Exit;
    ALV.ItemFocused := ALV.Items[0];
    ALV.ItemFocused.Selected := True;
  except
  end;
end;

procedure UpdateAndNotClearListView(AStrings: TStrings; ALV: TListView; ICaptionIndex: Integer;
  AnIndexes: array of Integer);
var
  S: string;
  Li: TListItem;
  J, I, ICount: Integer;
begin
  try
    ICount := StrToInt(Piece(AStrings[0], '^', 1));
    if ICount > AStrings.Count - 1 then ICount := AStrings.Count - 1;
  except
    ICount := 0;
  end;
  // aLV.Items.Clear;
  for I := 1 to ICount do begin
    S := AStrings[I];
    Li := ALV.Items.Add;
    Li.Caption := Piece(S, '^', ICaptionIndex);
    for J := low(AnIndexes) to high(AnIndexes) do Li.SubItems.Add(Piece(S, '^', AnIndexes[J]));
  end;
  try
    if ALV.Items.Count < 1 then Exit;
    ALV.ItemFocused := ALV.Items[0];
    ALV.ItemFocused.Selected := True;
  except
  end;
end;

procedure StringListToListView(ASL: TStringList; ALV: TListView; ICaptionIndex: Integer; AnIndexes: array of Integer);
var
  Li: TListItem;
  J, I: Integer;
begin
  ALV.Items.Clear;
  for I := 0 to ASL.Count - 1 do begin
    Li := ALV.Items.Add;
    Li.Caption := Piece(ASL[I], '^', ICaptionIndex);
    for J := low(AnIndexes) to high(AnIndexes) do Li.SubItems.Add(Piece(ASL[I], '^', AnIndexes[J]));
  end;
  try
    ALV.ItemFocused := ALV.Items[0];
    ALV.ItemFocused.Selected := True;
  except
  end;
end;

procedure ListViewToParamList(ALV: TListView; AnIndex: Integer; var AStrings: TStringList);
var
  I: Integer;
begin
  for I := 0 to ALV.Items.Count - 1 do begin
    if AnIndex = 0 then AStrings.Add(ALV.Items[I].Caption)
    else AStrings.Add(ALV.Items[I].SubItems[AnIndex - 1]);
  end;
end;

procedure UpdateStringList(AStrings: TStrings; ASL: TStringList);
var
  I, ICount: Integer;
begin
  ASL.Clear;
  if AStrings.Count > 0 then begin
    ICount := IntVal(Piece(AStrings[0], '^', 1));
    for I := 1 to ICount do ASL.Add(AStrings[I]);
  end;
end;

procedure ListViewToStringList(ALV: TListView; AL: TStringList; AnIndexes: array of Integer; AStart, ADelim: string);
var
  I, J: Integer;
  S: string;
begin
  for I := 0 to ALV.Items.Count - 1 do begin
    S := AStart;
    for J := low(AnIndexes) to high(AnIndexes) do S := S + ADelim + ALV.Items[I].SubItems[J];
    AL.Add(S);
  end;
end;

procedure LoadColumnsWidth(AName: string; AColumns: TListColumns; SL: TStringList);
var
  I: Integer;
  S: string;
begin
  if AColumns = nil then Exit;
  for I := 0 to AColumns.Count - 1 do
    try
      S := SL.Values[AName + '-' + IntToStr(I)];
      AColumns[I].Width := StrToIntDef(SL.Values[S], AColumns[I].Width);
    except
    end;
end;

procedure AddParam(AName, AValue: string; SL: TStringList);
begin
  SL.Add(AName + '=' + AValue);
end;

procedure SaveColumnsWidth(AName: string; AColumns: TListColumns; SL: TStringList);
var
  I: Integer;
begin
  for I := 0 to AColumns.Count - 1 do begin
    AddParam(AName + '-' + IntToStr(I), IntToStr(AColumns[I].Width), SL);
  end;
end;

function StringByKey(AStrings: TStrings; AKey: string; APosition: Integer): string;
var
  S: string;
  I: Integer;
begin
  S := '';
  for I := 1 to AStrings.Count - 1 do
    if Piece(AStrings[I], '^', APosition) = AKey then S := AStrings[I];
  Result := S;
end;

procedure UpdateEdit(var Key: Char; Sender: TObject);
var
  S, SY, SM: string;
begin
  if ((Pos(Upcase(Key), '0123456789/') = 0)) then begin
    if Key <> Char(#13) then Key := Char(#8);
    Exit;
  end;
  S := TEdit(Sender).Text;
  SY := Piece(S, '/', 2);
  SM := Piece(S, '/', 1);
  if ((S = '') and (Key = '/')) then begin
    Key := Char(#8);
    Exit;
  end else if (Pos('/', S) <> 0) and (Key = '/') then begin
    Key := Char(#0);
    Exit;
  end else if SY = '' then begin
    case Length(SM) of
      0:;
      1: begin
          if (SM <> '0') and (SM <> '1') then begin
            if Key <> '/' then begin
              TEdit(Sender).Text := '0' + SM + '/';
            end
            else TEdit(Sender).Text := '0' + SM;
            TEdit(Sender).SelStart := 3;
          end else if (SM = '0') and ((Key = '/') or (Key = '0')) then begin
            Key := Char(#0);
            Exit;
          end;
        end;
      2: if (SM <> '11') and (SM <> '12') then begin
          if Pos('0', SM) = 0 then begin
            SM := '0' + Copy(SM, 1, 1);
            SY := Copy(SM, 2, 4);
          end;
          TEdit(Sender).Text := SM + '/' + SY;
          TEdit(Sender).SelStart := 4;
          if Key = '/' then Key := Char(#0);
        end else begin
          if Key <> '/' then begin
            if Pos('/', S) <> 0 then S := S + Key
            else S := S + '/' + Key
          end
          else S := S + '/';
          TEdit(Sender).Text := S;
          TEdit(Sender).SelStart := 4;
          Key := Char(#0);
        end;
    end;
  end else begin
    if (Key = Char(#8)) and (TEdit(Sender).SelStart = 3) then Key := Char(#0);
  end;
end;

procedure SetPanelStatus(APanel: TPanel; AStatus: Boolean; AColor: TColor);
var
  I: Integer;
begin
  APanel.Enabled := AStatus;
  for I := 0 to APanel.ControlCount - 1 do begin
    if APanel.Controls[I] is TPanel then SetPanelStatus(TPanel(APanel.Controls[I]), AStatus, AColor)
    else if APanel.Controls[I] is TEdit then TEdit(APanel.Controls[I]).Color := AColor
    else if APanel.Controls[I] is TListView then TListView(APanel.Controls[I]).Color := AColor
    else if APanel.Controls[I] is TTreeView then TTreeView(APanel.Controls[I]).Color := AColor
    else if APanel.Controls[I] is TCheckBox then TCheckBox(APanel.Controls[I]).Enabled := AStatus
    else if APanel.Controls[I] is TRadioButton then TRadioButton(APanel.Controls[I]).Enabled := AStatus
    else if APanel.Controls[I] is TComboBox then TComboBox(APanel.Controls[I]).Color := AColor;
  end;
end;

initialization

FMInitFormatArray;

end.
