unit uGMV_VersionInfo;
{
  ================================================================================
  *
  *       Package:      MDK - Clinical Procedures
  *       Revision:     $Revision: 1 $  $Modtime: 10/07/08 2:57p $
  *       Site Name:    Hines OIFO
  *       Date Created: 7/29/05
  *       Developer:    doma.user@domain.ext
  *       Description:  This unit contains the Hemodialysis application
  *                     Version info object
  *       Notes:        the original code was modified by
  *                     dddddddddomain.user@domain.ext
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/UTILS/uGMV_VersionInfo.pas $
  *
  *       $History: uGMV_VersionInfo.pas $
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/13/09    Time: 1:16p
  * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/UTILS
  * Patch 19 source code
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/24/08    Time: 4:24p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/UTILS
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/02/08    Time: 9:25a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/UTILS
  *
  * *****************  Version 4  *****************
  * User: Zzzzzzandria Date: 8/24/07    Time: 10:54a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/UTILS
  *
  * *****************  Version 3  *****************
  * User: Zzzzzzandria Date: 6/20/07    Time: 3:23p
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/UTILS
  *
  * *****************  Version 2  *****************
  * User: Zzzzzzandria Date: 6/04/07    Time: 9:08a
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/UTILS
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 5/22/07    Time: 9:21a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/UTILS
}

interface

uses
  Windows,
  SysUtils,
  Classes,
  Graphics,
  Forms,
  Controls,
  StdCtrls,
  Buttons,
  ExtCtrls;

const
  VersionInfoKeys: array [1 .. 12] of string = ('CompanyName', 'FileDescription', 'FileVersion', 'InternalName',
    'LegalCopyRight', 'OriginalFileName', 'ProductName', 'ProductVersion', 'Comments', 'VAReleaseDate', 'VANamespace',
    'VASourceInformation');
  USEnglish = $040904E4;

type
  TVersionInfo = class(TComponent)
    (*
      Retrieves Version Info data about a given binary file.
    *)
  private
    FFileName: string;
    FLanguageID: DWord;
    FInfo: Pointer;
    FInfoSize: Longint;
    FCtlCompanyName: TControl;
    FCtlFileDescription: TControl;
    FCtlFileVersion: TControl;
    FCtlInternalName: TControl;
    FCtlLegalCopyRight: TControl;
    FCtlOriginalFileName: TControl;
    FCtlProductName: TControl;
    FCtlProductVersion: TControl;
    FCtlComments: TControl;
    FCtlVAReleaseDate: TControl;
    FCtlVANamespace: TControl;
    FCtlVASourceInformation: TControl;
    procedure SetFileName(Value: string);
    procedure SetVerProp(Index: Integer; Value: TControl);
    function GetVerProp(Index: Integer): TControl;
    function GetIndexKey(Index: Integer): string;
    function GetKey(const KName: string): string;
    function GetCompileDateTime: TDateTime;
    function GetVAPatchNumber: string;
    function GetVAServerVersion: string;
    procedure Refresh;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    property FileName: string read FFileName write SetFileName;
    property LanguageID: DWord read FLanguageID write FLanguageID;
    property CompileDateTime: TDateTime read GetCompileDateTime;
    property CompanyName: string index 1 read GetIndexKey;
    property FileDescription: string index 2 read GetIndexKey;
    property FileVersion: string index 3 read GetIndexKey;
    property InternalName: string index 4 read GetIndexKey;
    property LegalCopyRight: string index 5 read GetIndexKey;
    property OriginalFileName: string index 6 read GetIndexKey;
    property ProductName: string index 7 read GetIndexKey;
    property ProductVersion: string index 8 read GetIndexKey;
    property Comments: string index 9 read GetIndexKey;
    property VAReleaseDate: string index 10 read GetIndexKey;
    property VANamespace: string index 11 read GetIndexKey;
    property VASourceInformation: string index 12 read GetIndexKey;
    // AAN 08/05/2002
    property VAPatchNumber: string read GetVAPatchNumber;
    property VAServerVersion: string read GetVAServerVersion;
    constructor Create(AOwner: TComponent); override;
    (*
      Allocates memory and initializes variables for the object.
    *)
    destructor Destroy; override;
    (*
      Releases all memory allocated for the object.
    *)
    procedure OpenFile(FName: string);
    (*
      Uses method GetFileVersionInfo to retrieve version information for file
      FName.  If FName is blank, version information is obtained for the
      current executable (Application.ExeName).
    *)
    procedure Close;
    (*
      Releases memory allocated and clears all storage variables.
    *)
  published
    property CtlCompanyName: TControl index 1 read GetVerProp write SetVerProp;
    property CtlFileDescription: TControl index 2 read GetVerProp write SetVerProp;
    property CtlFileVersion: TControl index 3 read GetVerProp write SetVerProp;
    property CtlInternalName: TControl index 4 read GetVerProp write SetVerProp;
    property CtlLegalCopyRight: TControl index 5 read GetVerProp write SetVerProp;
    property CtlOriginalFileName: TControl index 6 read GetVerProp write SetVerProp;
    property CtlProductName: TControl index 7 read GetVerProp write SetVerProp;
    property CtlProductVersion: TControl index 8 read GetVerProp write SetVerProp;
    property CtlComments: TControl index 9 read GetVerProp write SetVerProp;
    property CtlVAReleaseDate: TControl index 10 read GetVerProp write SetVerProp;
    property CtlVANamespace: TControl index 11 read GetVerProp write SetVerProp;
    property CtlVASourceInfo: TControl index 12 read GetVerProp write SetVerProp;
  end;

function CurrentExeNameAndVersion: string;
function CurrentExeVersion: string;
function RequiredServerVersion: string;
function CurrentTitle: string;
function CurrentComments: string;

implementation

uses
  Dialogs, Math, TypInfo;

function RequiredServerVersion: string;
begin
  with TVersionInfo.Create(Application) do
    try
      OpenFile(Application.ExeName);
      Result := VAServerVersion;
      if VAPatchNumber <> '' then Result := Result + ' with patch ' + VAPatchNumber;
    finally
      Free;
    end;
end;

function CurrentExeNameAndVersion: string;
begin
  with TVersionInfo.Create(Application) do
    try
      OpenFile(Application.ExeName);
      Result := UpperCase(ExtractFileName(Application.ExeName) + ':' + FileVersion);
    finally
      Free;
    end;
end;

function CurrentExeVersion: string;
begin
  with TVersionInfo.Create(Application) do
    try
      OpenFile(Application.ExeName);
      Result := UpperCase(FileVersion);
    finally
      Free;
    end;
end;

function GetFileVersionLabel(AValue: string): string;
var
  SCurrent: string;
  I, ICount: Integer;
begin
  I := 1;
  ICount := 0;
  Result := '';
  while I <= Length(AValue) do begin
    SCurrent := Copy(AValue, I, 1);
    if SCurrent <> '.' then Result := Result + SCurrent
    else begin
      if ICount > 0 then Break
      else begin
        Inc(ICount);
        Result := Result + SCurrent;
      end;
    end;
    Inc(I);
  end;
end;

function CurrentTitle: string;
begin
  with TVersionInfo.Create(Application) do
    try
      OpenFile(Application.ExeName);
      Result := UpperCase(ProductName) + '  v.' + GetFileVersionLabel(FileVersion);
      // if Comments <> '' then
      // Result := Result + ' (' + Comments + ')';
    finally
      Free;
    end;
end;

function CurrentComments: string;
begin
  with TVersionInfo.Create(Application) do
    try
      OpenFile(Application.ExeName);
      Result := Comments;
    finally
      Free;
    end;
end;

(* === TVersionInfo Methods ================================================== *)

constructor TVersionInfo.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FLanguageID := USEnglish;
  SetFileName(EmptyStr);
end;

destructor TVersionInfo.Destroy;
begin
  if FInfoSize > 0 then FreeMem(FInfo, FInfoSize);
  inherited Destroy;
end;

procedure TVersionInfo.SetFileName(Value: string);
begin
  FFileName := Value;
  if Value = EmptyStr then FFileName := ExtractFileName(Application.ExeName);
  if CsDesigning in ComponentState then Refresh
  else OpenFile(Value);
end;

procedure TVersionInfo.OpenFile(FName: string);
var
  Vlen: DWord;
begin
  if FInfoSize > 0 then FreeMem(FInfo, FInfoSize);
  if Length(FName) <= 0 then FName := Application.ExeName;
  FInfoSize := GetFileVersionInfoSize(Pchar(Fname), Vlen);
  if FInfoSize > 0 then begin
    GetMem(FInfo, FInfoSize);
    if not GetFileVersionInfo(Pchar(Fname), Vlen, FInfoSize, FInfo) then
        raise Exception.Create('Cannot retrieve Version Information for ' + Fname);
    Refresh;
  end;
end;

procedure TVersionInfo.Close;
begin
  if FInfoSize > 0 then FreeMem(FInfo, FInfoSize);
  FInfoSize := 0;
  FFileName := EmptyStr;
end;

const
  VqvFmt = '\StringFileInfo\%8.8x\%s';

function TVersionInfo.GetKey(const KName: string): string;
var
  Vptr: Pchar;
  Vlen: DWord;
begin
  Result := EmptyStr;
  if FInfoSize <= 0 then Exit;
  if VerQueryValue(FInfo, Pchar(Format(VqvFmt, [FLanguageID, KName])), Pointer(Vptr), Vlen) then Result := Vptr;
end;

function TVersionInfo.GetIndexKey(Index: Integer): string;
begin
  Result := GetKey(VersionInfoKeys[index]);
end;

function TVersionInfo.GetVerProp(Index: Integer): TControl;
begin
  case index of
    1: Result := FCtlCompanyName;
    2: Result := FCtlFileDescription;
    3: Result := FCtlFileVersion;
    4: Result := FCtlInternalName;
    5: Result := FCtlLegalCopyRight;
    6: Result := FCtlOriginalFileName;
    7: Result := FCtlProductName;
    8: Result := FCtlProductVersion;
    9: Result := FCtlComments;
    10: Result := FCtlVAReleaseDate;
    11: Result := FCtlVANamespace;
    12: Result := FCtlVASourceInformation;
  else Result := nil;
  end;
end;

procedure TVersionInfo.SetVerProp(Index: Integer; Value: TControl);
begin
  case index of
    1: FCtlCompanyName := Value;
    2: FCtlFileDescription := Value;
    3: FCtlFileVersion := Value;
    4: FCtlInternalName := Value;
    5: FCtlLegalCopyRight := Value;
    6: FCtlOriginalFileName := Value;
    7: FCtlProductName := Value;
    8: FCtlProductVersion := Value;
    9: FCtlComments := Value;
    10: FCtlVAReleaseDate := Value;
    11: FCtlVANamespace := Value;
    12: FCtlVASourceInformation := Value;
  end;
  Refresh;
end;

procedure TVersionInfo.Notification(AComponent: TComponent; Operation: TOperation);
begin
  if Operation = OpRemove then begin
    if AComponent = FCtlCompanyName then FCtlCompanyName := nil
    else if AComponent = FCtlFileDescription then FCtlFileDescription := nil
    else if AComponent = FCtlFileVersion then FCtlFileVersion := nil
    else if AComponent = FCtlInternalName then FCtlInternalName := nil
    else if AComponent = FCtlLegalCopyRight then FCtlLegalCopyRight := nil
    else if AComponent = FCtlOriginalFileName then FCtlOriginalFileName := nil
    else if AComponent = FCtlProductName then FCtlProductName := nil
    else if AComponent = FCtlProductVersion then FCtlProductVersion := nil
    else if AComponent = FCtlComments then FCtlComments := nil
    else if AComponent = FCtlVAReleaseDate then FCtlVAReleaseDate := nil
    else if AComponent = FCtlVANamespace then FCtlVANamespace := nil
    else if AComponent = FCtlVASourceInformation then FCtlVASourceInformation := nil;
  end;
end;

procedure TVersionInfo.Refresh;
var
  PropInfo: PPropInfo;
  procedure AssignText(Actl: TComponent; Txt: string);
  begin
    if Assigned(ACtl) then begin
      PropInfo := GetPropInfo(ACtl.ClassInfo, 'Caption');
      if PropInfo <> nil then SetStrProp(ACtl, PropInfo, Txt)
      else begin
        PropInfo := GetPropInfo(ACtl.ClassInfo, 'Text');
        if PropInfo <> nil then SetStrProp(ACtl, PropInfo, Txt)
      end;
    end;
  end;

begin
  if CsDesigning in ComponentState then begin
    AssignText(FCtlCompanyName, VersionInfoKeys[1]);
    AssignText(FCtlFileDescription, VersionInfoKeys[2]);
    AssignText(FCtlFileVersion, VersionInfoKeys[3]);
    AssignText(FCtlInternalName, VersionInfoKeys[4]);
    AssignText(FCtlLegalCopyRight, VersionInfoKeys[5]);
    AssignText(FCtlOriginalFileName, VersionInfoKeys[6]);
    AssignText(FCtlProductName, VersionInfoKeys[7]);
    AssignText(FCtlProductVersion, VersionInfoKeys[8]);
    AssignText(FCtlComments, VersionInfoKeys[9]);
    AssignText(FCtlVAReleaseDate, VersionInfoKeys[10]);
    AssignText(FCtlVANamespace, VersionInfoKeys[11]);
    AssignText(FCtlVASourceInformation, VersionInfoKeys[12]);
  end else begin
    AssignText(FCtlCompanyName, CompanyName);
    AssignText(FCtlFileDescription, FileDescription);
    AssignText(FCtlFileVersion, FileVersion);
    AssignText(FCtlInternalName, InternalName);
    AssignText(FCtlLegalCopyRight, LegalCopyRight);
    AssignText(FCtlOriginalFileName, OriginalFileName);
    AssignText(FCtlProductName, ProductName);
    AssignText(FCtlProductVersion, ProductVersion);
    AssignText(FCtlComments, Comments);
    AssignText(FCtlVAReleaseDate, VAReleaseDate);
    AssignText(FCtlVANamespace, VANamespace);
    AssignText(FCtlVASourceInformation, VASourceInformation);
  end;
end;

function TVersionInfo.GetCompileDateTime: TDateTime;
begin
  FileAge(Application.ExeName, Result);
  // Result := FileDateToDateTime(FileAge(Application.ExeName));
end;

function TVersionInfo.GetVAPatchNumber: string;
begin
  // Result := 'MD * 1 * 19';
//  Result := 'MD * 1 * 40';
  Result := 'MD * 1 * 50';
end;

function TVersionInfo.GetVAServerVersion: string;
  function Piece(Value, Delimiter: string; StartPiece: Integer; EndPiece: Integer): string;
  var
    Dlen, I, Pnum: Integer;
    Buf: string;
  begin
    Result := '';
    if (Value <> '') and (StartPiece > 0) and (EndPiece >= StartPiece) then begin
      Dlen := Length(Delimiter);
      I := Pos(Delimiter, Value) - 1;
      if I >= 0 then begin
        Buf := Value;
        Pnum := 1;
        repeat
          if Pnum > EndPiece then Break;
          if I < 0 then I := Length(Buf);
          if Pnum = StartPiece then Result := Copy(Buf, 1, I)
          else if Pnum > StartPiece then Result := Result + Delimiter + Copy(Buf, 1, I);
          Delete(Buf, 1, I + Dlen);
          I := Pos(Delimiter, Buf) - 1;
          Inc(Pnum);
        until (I < 0) and (Buf = '');
      end else if StartPiece = 1 then Result := Value;
    end;
  end;

begin
  Result := Piece(FileVersion, '.', 1, 1) + '.' + Piece(FileVersion, '.', 2, 2);
  if StrToIntDef(Piece(FileVersion, '.', 3, 3), 0) = 0 then
    if StrToIntDef(Piece(FileVersion, '.', 4, 4), 0) > 0 then Result := Result + 'T' + Piece(FileVersion, '.', 4, 4);
end;

end.
