unit uMDK_RPCEvent;

interface

uses
  Classes, SysUtils;

type
  TRPCEventItem = class(TObject)
  public
    Start: TDateTime;
    Stop: TDateTime;
    RPC: string;
    Params: TStrings;
    Results: TStrings;
    constructor Create;
    destructor Destroy; override;
  end;

function NewRPCEventItem(AStart, AStop: TDateTime; RemoteProcedure: string; Parameters: array of string;
  MultList: TStringList = nil; RetList: TStrings = nil): TRPCEventItem;

implementation

function NewRPCEventItem(AStart, AStop: TDateTime; RemoteProcedure: string; Parameters: array of string;
  MultList: TStringList = nil; RetList: TStrings = nil): TRPCEventItem;
var
  Item: TRPCEventItem;
  I, J: Integer;
begin
  Item := TRPCEventItem.Create;
  Item.Start := AStart;
  Item.Stop := AStop;
  Item.RPC := RemoteProcedure;
  Item.Params.Add('Params: ---------------------------------------------------');
  I := 0;
  while I <= high(Parameters) do begin
    if (Copy(Parameters[I], 1, 1) = '@') and (Parameters[I] <> '@') then begin
      Item.Params.Add('Reference ' + IntToStr(I) + ' <' + Copy(Parameters[I], 2, Length(Parameters[I])) + '>');
    end else begin
      Item.Params.Add('Literal ' + IntToStr(I) + ' <' + Parameters[I] + '>');
    end;
    Inc(I);
  end;
  if MultList <> nil then
    if MultList.Count > 0 then begin
      Item.Params.Add('List:');
      for J := 1 to MultList.Count do Item.Params.Add(MultList[J - 1]);
    end;
  if RetList <> nil then
    if RetList.Count > 0 then begin
      Item.Results.Add('Results: --------------------------------------------------');
      for J := 0 to RetList.Count - 1 do Item.Results.Add(RetList[J]);
    end;
  Result := Item;
end;
/// /////////////////////////////////////////////////////////////////////////////

constructor TRPCEventItem.Create;
begin
  inherited;
  Params := TStringList.Create;
  Results := TStringList.Create;
end;

destructor TRPCEventItem.Destroy;
begin
  Params.Free;
  Results.Free;
  inherited;
end;

end.
