unit uXML;

{
  ================================================================================
  *
  *       Application:  CliO
  *       Revision:     $Revision: 1 $  $Modtime: 9/28/07 10:44a $
  *       Developer:    doma.user@domain.ext
  *       Site:         Hines OIFO
  *
  *       Description:  Contains global XML text utilities.
  *
  *
  *       Notes:
  *
  *
  ================================================================================
  *       $Archive: /Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/UTILS/uXML.pas $
  *
  * $History: uXML.pas $
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 1/13/09    Time: 1:16p
  * Created in $/Clinical Procedures v1.0 (CP Legacy)/CP Version 1.0 - Patch 19/HEMODIALYSIS_1_0_6_320/UTILS
  * Patch 19 source code
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/24/08    Time: 4:24p
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/HEMODIALYSIS_1_0_6_319_#18/UTILS
  *
  * *****************  Version 1  *****************
  * User: Zzzzzzandria Date: 4/02/08    Time: 9:25a
  * Created in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI Current/HEMODIALYSIS_1_0_6_319/UTILS
  *
  * *****************  Version 2  *****************
  * User: Zzzzzzandria Date: 6/20/07    Time: 3:23p
  * Updated in $/CP Version 1.0 - Patch 6 (Renal Enhancement)/GUI 1.0/UTILS
  *
  *
  ================================================================================
}
interface

uses
  SysUtils,  Classes,  XMLIntf,  XMLDoc;

function XMLHeader(const AName: string): string; overload;
function XMLHeader(const AName: string; AIdentifierNames, AIdentifierValues: array of string): string; overload;
function XMLFooter(const AName: string): string;
function XMLElement(const ATag: string; const AValue: string): string; overload;
function XMLElement(const ATag: string; const AValue: Integer): string; overload;
function XMLElement(const ATag: string; const AValue: Double; const ADecimals: Integer): string; overload;
function XMLElement(const ATag: string; const AValue: Boolean): string; overload;
function XMLElement(const ATag: string; const AValue: TDateTime): string; overload;
function XMLStream(const AXML: string): TStringStream;
function XMLDocument(const AXML: string): IXMLDocument;
function XMLDateTime(const ADateTime: TDateTime): string;
function XMLNodeValue(const AXMLDoc: IXMLDocument; const ATag: string): string;
function XMLFieldValue(AXMLCurrentRecordNode: IXMLNode; AFieldName: string): string;
function XMLFirstRecord(AXMLResultsDocument: IXMLDocument): IXMLNode;
function XMLNextRecord(AXMLCurrentRecordNode: IXMLNode): IXMLNode;
function XMLErrorMessage(const AErrorMsgText: string): string;
function XMLSuccessMessage(const AMsgText: string): string;
function XMLString(AString: string): string;
function GetXMLStatusValue(AXMLString: string): string;
function GetXMLNodeValue(AXMLString, AXMLNodeName: string): string;

const
  XML_DATE     = 'YYYY-MM-DD';
  XML_DATETIME = 'YYYY-MM-DD hh:mm:ss';
  // XML_VERSION = '<?xml version="1.0"?>';
  XML_VERSION      = '<?xml version="1.0"';
  XML_STATUS_OK    = '<RESULT><STATUS>OK</STATUS></RESULT>';
  XML_STATUS_ERROR = '<RESULT><STATUS>ERROR</STATUS><DB_ENGINE_ERROR_MESSAGE>%s</DB_ENGINE_ERROR_MESSAGE></RESULT>';

implementation

uses
  StrUtils;

function BuildElement(const ATag: string; const AValue: string): string;
{ Used internally once the element value is formatted to a string }
begin
  Result := Format('<%s>%s</%s>', [ATag, AValue, ATag]) + #13;
end;

function XMLHeader(const AName: string): string; overload;
begin
  Result := XMLHeader(AName, [], []);
end;

function XMLHeader(const AName: string; AIdentifierNames, AIdentifierValues: array of string): string;
var
  I: Integer;
begin
  Result := AName;
  if high(AIdentifierNames) > -1 then begin
    Result := Format('%s', [Result]);
    I := 0;
    while I <= high(AIdentifierNames) do begin
      Result := Format('%s %s=%s', [Result, AIdentifierNames[I], AnsiQuotedStr(AIdentifierValues[I], '''')]);
      Inc(I);
    end;
  end;
  Result := Format('<%s>', [Result]) + #13;
end;

function XMLFooter(const AName: string): string;
begin
  Result := Format('</%s>', [AName]) + #13;
end;

function XMLElement(const ATag: string; const AValue: string): string; overload;
begin
  Result := BuildElement(ATag, AValue);
end;

function XMLElement(const ATag: string; const AValue: Integer): string; overload;
begin
  Result := BuildElement(ATag, Format('%d', [AValue]));
end;

function XMLElement(const ATag: string; const AValue: Double; const ADecimals: Integer): string; overload;
begin
  Result := BuildElement(ATag, Format(Format('%%0.%df', [ADecimals]), [AValue]));
end;

function XMLElement(const ATag: string; const AValue: Boolean): string; overload;
begin
  Result := BuildElement(ATag, BoolToStr(AValue, True));
end;

function XMLElement(const ATag: string; const AValue: TDateTime): string; overload;
begin
  Result := BuildElement(ATag, FormatDateTime(XML_DATETIME, AValue));
end;

function XMLDateTime(const ADateTime: TDateTime): string;
begin
  Result := FormatDateTime(XML_DATETIME, ADateTime);
end;

function XMLStream(const AXML: string): TStringStream;
begin
  Result := TStringStream.Create(AXML);
end;

function XMLDocument(const AXML: string): IXMLDocument;
begin
  Result := NewXMLDocument;
  Result.LoadFromStream(XMLStream(AXML));
  Result.Active := True;
end;

function XMLNodeValue(const AXMLDoc: IXMLDocument; const ATag: string): string;
var
  XMLNode: IXMLNode;
begin
  Result := '';
  XMLNode := AXMLDoc.DocumentElement.ChildNodes.FindNode(ATag);
  if XMLNode <> nil then
    if XMLNode.IsTextElement then Result := XMLNode.NodeValue;
end;

function XMLFieldValue(AXMLCurrentRecordNode: IXMLNode; AFieldName: string): string;
begin
  Result := '';
  if AXMLCurrentRecordNode.ChildNodes.FindNode(AFieldName) <> nil then
    if AXMLCurrentRecordNode.ChildNodes.FindNode(AFieldName).IsTextElement then
        Result := AXMLCurrentRecordNode.ChildNodes.FindNode(AFieldName).NodeValue;
end;

function XMLFirstRecord(AXMLResultsDocument: IXMLDocument): IXMLNode;
begin
  Result := AXMLResultsDocument.DocumentElement.ChildNodes.FindNode('RECORD');
end;

function XMLNextRecord(AXMLCurrentRecordNode: IXMLNode): IXMLNode;
begin
  Result := AXMLCurrentRecordNode.NextSibling;
end;

function XMLErrorMessage(const AErrorMsgText: string): string;
begin
  Result := XMLHeader('RESULTS') + XMLElement('STATUS', 'ERROR') + XMLElement('MESSAGE', AErrorMsgText) +
    XMLFooter('RESULTS');
end;

function XMLSuccessMessage(const AMsgText: string): string;
begin
  Result := XMLHeader('RESULTS') + XMLElement('STATUS', 'OK') + XMLElement('MESSAGE', AMsgText) + XMLFooter('RESULTS');
end;

function XMLString(AString: string): string;
begin
  Result := AString;
  Result := AnsiReplaceStr(Result, '&', '&amp;');
  Result := AnsiReplaceStr(Result, '<', '&lt;');
  Result := AnsiReplaceStr(Result, '>', '&gt;');
  Result := AnsiReplaceStr(Result, '''', '&apos;');
  Result := AnsiReplaceStr(Result, '"', '&quot;');
end;

function GetXMLStatusValue(AXMLString: string): string;
begin
  Result := GetXMLNodeValue(AXMLString, 'STATUS');
end;

function GetXMLNodeValue(AXMLString, AXMLNodeName: string): string;
begin
  try
    with LoadXMLData(AXMLString).DocumentElement do Result := ChildValues[AXMLNodeName];
  except
    on E: Exception do Result := '';
  end;
end;

end.
