unit fZZ_EventLog;
{
  ================================================================================
  *
  *       Package:        development tool
  *       Date Created:   5/08/12
  *       Site:           Hines OIFO
  *       Developers:
  *                       Andrey.Andriyevskiy@va.gov
  *
  *       Description:    Event log  presentation window
  *
  ================================================================================
}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ExtCtrls, ActnList, Menus, StdActns, Buttons, ImgList, System.Actions;

type
  TZZ_EventLog = class(TObject)
  private
    FLogLimit: Integer;
    FEventLog: TStringList;
    FChangeTracker: TNotifyEvent;
    procedure SetChangeTracker(ATracker: TNotifyEvent);
    procedure SetLogLimit(ALimit: Integer);
  public
    property ChangeTracker: TNotifyEvent read FChangeTracker write SetChangeTracker;
    property EventLog: TStringList read FEventLog write FEventLog;
    property LogLimit: Integer read FLogLimit write SetLogLimit;
    constructor Create;
    destructor Destroy; override;
  end;

  TfrmEventLog = class(TForm)
    PnlTimeline: TPanel;
    PnlMainTitle: TPanel;
    PnlBottom: TPanel;
    LbLog: TListBox;
    Splitter1: TSplitter;
    Panel1: TPanel;
    Mm: TMemo;
    PnlTitle: TPanel;
    ActionList1: TActionList;
    AcRefreshLog: TAction;
    MainMenu1: TMainMenu;
    WindowClose1: TWindowClose;
    File1: TMenuItem;
    Refresh1: TMenuItem;
    N1: TMenuItem;
    Close1: TMenuItem;
    SpeedButton1: TSpeedButton;
    AcSaveAs: TAction;
    SpeedButton2: TSpeedButton;
    ImageList1: TImageList;
    Panel2: TPanel;
    OpenDialog1: TOpenDialog;
    AcSetLimit: TAction;
    AcClear: TAction;
    SpeedButton5: TSpeedButton;
    SpeedButton6: TSpeedButton;
    CkbWrap: TCheckBox;
    AcWrap: TAction;
    PnlBottomDetails: TPanel;
    PnlLimit: TPanel;
    AcLoad: TAction;
    SpeedButton3: TSpeedButton;
    EdHigh: TEdit;
    procedure LbLogClick(Sender: TObject);
    procedure AcRefreshLogExecute(Sender: TObject);
    procedure Close1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure AcSaveAsExecute(Sender: TObject);
    procedure AcSetLimitExecute(Sender: TObject);
    procedure AcClearExecute(Sender: TObject);
    procedure AcWrapExecute(Sender: TObject);
    procedure AcLoadExecute(Sender: TObject);
    procedure LbLogDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
    procedure EdHighChange(Sender: TObject);
  private
    { Private declarations }
    FEventLog: TStringList;
    Current: TObject;
    function FindObject(AnObject: TObject): Integer;
    procedure RestorePosition;
    procedure RefreshLog;
    procedure ShowLogSizeInfo;
  public
    { Public declarations }
    procedure UpdateLog(ALog: TStringList = nil);
    procedure EventLogPresenterOnChange(Sender: TObject);
    procedure SetViewMode(AParented: Boolean = False);
  end;

var
  FrmEventLog: TfrmEventLog;

function NewEventLog: TZZ_EventLog;
function AddEvent(ADescription: string; AnEvent: TObject): Boolean;
function AddRPCEvent(AnEvent: TObject): Boolean;
function AddTracker(ATracker: TNotifyEvent): Boolean;
procedure ShowEventLog(AModal: Boolean = True);
function SetParentedLog(AParent: TWinControl): Boolean;

implementation

uses
  UZZ_DescribedItem, UZZ_RPCEvent, System.UITypes;

{$R *.dfm}

var
  EL: TZZ_EventLog;

  /// /////////////////////////////////////////////////////////////////////////////
function AddRPCEvent(AnEvent: TObject): Boolean;
var
  SDescription: string;
begin
  Result := False;
  if not Assigned(EL) then begin
    // ShowMessage(TRPCEventItem(anEvent).RPC);
    FreeAndNil(AnEvent);
    Exit;
  end;
  SDescription := Format(' %-25.25s', [TRPCEventItem(AnEvent).RPC]) + FormatDateTime(' hh:mm:ss.zzz',
    TRPCEventItem(AnEvent).Start);
  EL.EventLog.InsertObject(0, SDescription, AnEvent);
  Result := True;
end;

function AddEvent(ADescription: string; AnEvent: TObject): Boolean;
begin
  Result := False;
  if not Assigned(EL) then Exit;
  EL.EventLog.InsertObject(0, ADescription, AnEvent);
  Result := True;
end;

function AddTracker(ATracker: TNotifyEvent): Boolean;
begin
  Result := False;
  if not Assigned(EL) then Exit;
  EL.ChangeTracker := ATracker;
  Result := True;
end;

function NewEventLog: TZZ_EventLog;
begin
  if not Assigned(EL) then EL := TZZ_EventLog.Create;
  Result := EL;
end;

/// /////////////////////////////////////////////////////////////////////////////
procedure ShowEventLog(AModal: Boolean = True);
begin
  if not Assigned(FrmEventLog) then Application.CreateForm(TfrmEventLog, FrmEventLog);
  if not Assigned(FrmEventLog) then Exit;
  FrmEventLog.RefreshLog;
  if FrmEventLog.Parent = nil then
    if AModal then FrmEventLog.ShowModal
    else FrmEventLog.Show;
  FrmEventLog.Left := Screen.WorkAreaRect.Left;
  FrmEventLog.Top := Screen.WorkAreaRect.Top;
end;

procedure SetFormParented(AForm: TForm; AParent: TWinControl);
begin
  if AForm.Parent <> AParent then begin
    AForm.BorderStyle := BsNone;
    AForm.Parent := AParent;
    AForm.Align := AlClient;
    AForm.Menu := nil;
    AForm.Show;
  end;
end;

function SetParentedLog(AParent: TWinControl): Boolean;
begin
  Result := False;
  if Assigned(FrmEventLog) then begin
    SetFormParented(FrmEventLog, AParent);
    FrmEventLog.SetViewMode(True);
    Result := True;
  end;
end;
/// /////////////////////////////////////////////////////////////////////////////

constructor TZZ_EventLog.Create;
begin
  inherited;
  FEventLog := TStringList.Create;
  FLogLimit := 500;
end;

destructor TZZ_EventLog.Destroy;
begin
  EventLog.OnChange := nil;
  if Assigned(EventLog) then
    while EventLog.Count > 0 do begin
      if Assigned(EventLog.Objects[0]) then
        try
          TObject(EventLog.Objects[0]).Free;
        except
        end;
      EventLog.Delete(0);
    end;
end;

procedure TZZ_EventLog.SetChangeTracker(ATracker: TNotifyEvent);
begin
  FChangeTracker := ATracker;
  EventLog.OnChange := ATracker;
end;

procedure TZZ_EventLog.SetLogLimit(ALimit: Integer);
begin
  if ALimit <= FLogLimit then begin
    while EventLog.Count > ALimit do begin
      if EventLog.Objects[EventLog.Count - 1] <> nil then EventLog.Objects[EventLog.Count - 1].Free;
      EventLog.Delete(EventLog.Count - 1);
    end;
  end;
  FLogLimit := ALimit;
end;
/// /////////////////////////////////////////////////////////////////////////////

procedure TfrmEventLog.FormCreate(Sender: TObject);
begin
  AddTracker(EventLogPresenterOnChange);
end;

procedure TfrmEventLog.RefreshLog;
begin
  if not Assigned(FEventLog) then Exit;
  LbLog.Enabled := False;
  EL.LogLimit := EL.LogLimit;
  LbLog.Items.Assign(FEventLog);
  RestorePosition;
  LbLog.Enabled := True;
  Application.ProcessMessages;
end;

procedure TfrmEventLog.LbLogClick(Sender: TObject);
var
  X: ImDescribe;
begin
  if LbLog.ItemIndex >= 0 then begin
    Current := LbLog.Items.Objects[LbLog.ItemIndex];
    if Assigned(Current) and Current.GetInterface(ImDescribe, X) then Mm.Text := X.GetDescription.Text
    else begin
      Mm.Text := '';
      PnlTitle.Caption := '';
    end;
  end else begin
    Mm.Text := '';
    PnlTitle.Caption := '';
    Current := nil;
  end;
end;

function TfrmEventLog.FindObject(AnObject: TObject): Integer;
var
  I: Integer;
begin
  Result := -1;
  for I := 0 to LbLog.Items.Count - 1 do begin
    if AnObject = LbLog.Items.Objects[I] then begin
      Result := I;
      Break;
    end;
  end;
end;

procedure TfrmEventLog.RestorePosition;
var
  I: Integer;
begin
  if not Assigned(Current) then Exit;
  I := FindObject(Current);
  if I >= 0 then LbLog.ItemIndex := I;
end;

procedure TfrmEventLog.Close1Click(Sender: TObject);
begin
  Close;
end;

procedure TfrmEventLog.ShowLogSizeInfo;
begin
  if Assigned(FEventLog) then PnlBottom.Caption := Format(' %d ', [FEventLog.Count]);
  if Assigned(EL) then PnlLimit.Caption := Format('(%d) ', [EL.LogLimit]);
end;

procedure TfrmEventLog.UpdateLog(ALog: TStringList = nil);
begin
  if Assigned(ALog) then FEventLog := ALog;
  RefreshLog;
  LbLog.ItemIndex := 0;
  LbLogClick(nil);
  ShowLogSizeInfo;
  Application.ProcessMessages;
end;

procedure TfrmEventLog.AcRefreshLogExecute(Sender: TObject);
begin
  UpdateLog(FEventLog);
end;

procedure TfrmEventLog.EventLogPresenterOnChange(Sender: TObject);
begin
  UpdateLog(TStringList(Sender));
end;

procedure TfrmEventLog.SetViewMode(AParented: Boolean = False);
begin
  PnlTitle.ParentColor := AParented;
  PnlMainTitle.ParentColor := AParented;
  PnlBottom.ParentColor := AParented;
end;

procedure TfrmEventLog.AcSaveAsExecute(Sender: TObject);
var
  I: Integer;
  S: string;
  SN: string;
  X: ImDescribe;
  SL: TStringList;
begin
  if OpenDialog1.Execute then begin
    SN := OpenDialog1.FileName;
    for I := LbLog.Items.Count - 1 downto 0 do begin
      Current := LbLog.Items.Objects[I];
      if Assigned(Current) and Current.GetInterface(ImDescribe, X) then
          S := S + '--' + LbLog.Items[I] + #13#10 + X.GetDescription.Text + #13#10
      else S := S + #13#10
    end;
    SL := TStringList.Create;
    SL.Text := S;
    SL.SaveToFile(SN);
    SL.Free;
  end;
end;

procedure TfrmEventLog.AcSetLimitExecute(Sender: TObject);
var
  InputString: string;
begin
  InputString := InputBox('Log Size Limit', 'Enter # of records to keep in the Log', IntToStr(EL.LogLimit));
  if InputString <> IntToStr(EL.LogLimit) then begin
    EL.LogLimit := StrToIntDef(InputString, EL.LogLimit);
    ShowLogSizeInfo;
  end;
end;

procedure TfrmEventLog.AcClearExecute(Sender: TObject);
begin
  if MessageDlg('Erase Log now?', MtConfirmation, [MbYes, MbNo], 0) = MrYes then begin
    LbLog.Enabled := False;
    while FEventLog.Count > 0 do begin
      if FEventLog.Objects[FEventLog.Count - 1] <> nil then FEventLog.Objects[FEventLog.Count - 1].Free;
      FEventLog.Delete(FEventLog.Count - 1);
    end;
    LbLog.Enabled := True;
  end;
  Application.ProcessMessages;
end;

procedure TfrmEventLog.AcWrapExecute(Sender: TObject);
begin
  Mm.WordWrap := CkbWrap.Checked;
  if CkbWrap.Checked then Mm.ScrollBars := SsVertical
  else Mm.ScrollBars := SsBoth;
  PnlBottomDetails.Visible := CkbWrap.Checked;
end;

procedure TfrmEventLog.AcLoadExecute(Sender: TObject);
var
  SEventName, SEventDescription, S, SN: string;
  I, ICount: Integer;
  SL: TStringList;
  procedure LogEvent;
  begin
    if SEventName + SEventDescription <> '' then AddEvent(SEventName, GetDescribedItem(SEventName, SEventDescription));
  end;

begin
  if OpenDialog1.Execute then begin
    SN := OpenDialog1.FileName;
    SL := TStringList.Create;
    SL.LoadFromFile(SN);
    I := 0;
    ICount := 0;
    SEventName := '-- File Import -- BEGIN';
    SEventDescription := 'File: ' + SN;
    while I < SL.Count do begin
      S := SL[I];
      if Pos('--', S) = 1 then begin
        LogEvent;
        SEventName := '';
        SEventDescription := '';
        SEventName := Copy(S, 3, Length(S));
        Inc(ICount);
      end
      else SEventDescription := SEventDescription + S + #13#10;
      Inc(I);
    end;
    SEventName := '-- File Import -- END';
    SEventDescription := 'Total lines: ' + IntToStr(SL.Count) + #13#10 + 'Total events: ' + IntToStr(ICount - 1);
    LogEvent;
    SL.Free;
  end;
end;

procedure TfrmEventLog.LbLogDrawItem(Control: TWinControl; Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  S: string;
begin
  inherited;
  S := EdHigh.Text;
  with (Control as TListBox).Canvas do begin
    if (Length(S) > 0) and (Pos(S, (Control as TListBox).Items[index]) > 0) then Font.Color := ClRed
    else if (OdSelected in State) then Font.Color := ClWhite
    else Font.Color := ClWindowtext;
    FillRect(Rect);
    TextOut(Rect.Left, Rect.Top, (Control as TListBox).Items[index]);
    if OdFocused in State then begin
      Brush.Color := LbLog.Color;
      DrawFocusRect(Rect);
    end;
  end
end;

procedure TfrmEventLog.EdHighChange(Sender: TObject);
begin
  inherited;
  LbLog.Invalidate;
  Application.ProcessMessages;
end;

initialization

NewEventLog;

finalization

FreeAndNil(EL);

end.
