unit uZZ_DescribedItem;

{
  ================================================================================
  *
  *       Package:        development tool
  *       Date Created:   5/08/12
  *       Site:           Hines OIFO
  *       Developers:
  *                       Andrey.Andriyevskiy@va.gov
  *
  *       Description:    General self-described object
  *
  ================================================================================
}

interface

uses
  Classes, SysUtils;

type
  ImDescribe = interface
    ['{D3CA9934-7EBA-40F9-A481-C93E301D130F}']
    function GetName: string; stdcall;
    function GetDescription: TStringList; stdcall;
    procedure Reload; stdcall;
    function GetComponents: TStringList; stdcall;
    function GetImageIndex: Integer; stdcall;
    function GetSelectedIndex: Integer; stdcall;
    function HasComponents: Integer; stdcall;
  end;

  TmDescribedItem = class(TInterfacedObject, ImDescribe, IUnknown)
  protected
    Name: string;
    Status: string;
    Description: TStringList;
    Components: TStringList;
    RawLines: TStringList;
    ImageIndex, SelectedIndex: Integer;
  public
    function _AddRef: Integer; stdcall;
    function _Release: Integer; stdcall;
    function GetName: string; stdcall;
    function GetDescription: TStringList; stdcall;
    function GetComponents: TStringList; stdcall;
    procedure Reload; stdcall;
    function GetImageIndex: Integer; stdcall;
    function GetSelectedIndex: Integer; stdcall;
    function HasComponents: Integer; stdcall;
    constructor Create(AList: TStringList);
    destructor Destroy; override;
  end;

function GetDescribedItem(AName: string; ADescription: string): TmDescribedItem;

implementation

function GetDescribedItem(AName: string; ADescription: string): TmDescribedItem;
var
  SL: TStringList;
begin
  SL := TStringList.Create;
  SL.Text := AName + #13#10 + ADescription + #13#10;
  Result := TmDescribedItem.Create(SL);
  FreeAndNil(SL);
end;

{ TDescribedItem }
function TmDescribedItem._AddRef: Integer;
begin
  Result := -1;
end;

function TmDescribedItem._Release: Integer;
begin
  Result := -1;
end;

function TmDescribedItem.GetName: string;
begin
  Result := name;
end;

function TmDescribedItem.GetDescription: TStringList;
begin
  Result := Description;
end;

function TmDescribedItem.GetComponents: TStringList;
begin
  Result := Components;
end;

procedure TmDescribedItem.Reload;
begin
end;

function TmDescribedItem.GetImageIndex: Integer;
begin
  Result := ImageIndex;
end;

function TmDescribedItem.GetSelectedIndex: Integer;
begin
  Result := SelectedIndex;
end;

function TmDescribedItem.HasComponents: Integer;
begin
  Result := Components.Count;
end;

constructor TmDescribedItem.Create(AList: TStringList);
begin
  try
    RawLines := TStringList.Create;
    Components := TStringList.Create;
    Description := TStringList.Create;
    name := '...';
    if Assigned(AList) then
      try
        RawLines.Assign(AList);
        Description.Assign(AList);
        name := Description[0];
      except
      end;
  except
  end;
end;

destructor TmDescribedItem.Destroy;
begin
  try
    if RawLines <> nil then RawLines.Free;
    if Description <> nil then Description.Free;
    if Components <> nil then Components.Free;
  except
  end;
  inherited;
end;

end.
